#pragma TextEncoding = "Windows-1252"
#pragma rtGlobals=3		// Use modern global access method.
//#pragma version=1.17d// decided to not use pragma but a string constant 

// Elemental Analysis Functions for High ToF-AMS analysis Toolkit
// Peak Integration by Key Analysis (PIKA)
//Copyright (C) 2005, 2006, 2007,2008, 2009,2010,2011, 2012, 2013, 2014, 2015, 2016, 2017
// Pete DeCarlo, Allison Aiken, Jose Jimenez, Donna Sueper University of Colorado
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
///////////////////////////////////////////////////////////////////////////////
// CURRENT STATUS
// see HR read me file for details.


CONSTANT j15NN_THEORY = 0.0073066		//1.15E

STRCONSTANT ELEM_LIST = "C;H;O;N;S"
STRCONSTANT ELEM_COLORLIST = "0,0,0;39168,26112,0;65280,0,0;0,0,39168;29440,0,58880"		
// C is black, H is brown, O is  N is blue, S is purple.

//static CONSTANT MASS_CO2 = 43.989829
//static CONSTANT MASS_CO = 27.994915
//static CONSTANT MASS_H2O = 18.010565
//static CONSTANT MASS_OH = 17.002740
//static CONSTANT MASS_O = 15.994915

// averaged isotopic abundance
static CONSTANT AVG_ATOMIC_WT_H = 1.0079
static CONSTANT AVG_ATOMIC_WT_C = 12.0107
static CONSTANT AVG_ATOMIC_WT_O = 15.9994
static CONSTANT AVG_ATOMIC_WT_N = 14.0067
static CONSTANT AVG_ATOMIC_WT_S = 32.065


Function HR_calcPlot_EAMassSpec(waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType)
string waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType

	HR_calc_EAMassSpec(waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType)
	HR_plot_EAAvgMassSpec(waveNameSuffix, myPlotName)

End


// 1.04J button function similar to the Squirrel MS spectra average button
// returns a list of waves to possibly plot  1.09 x now returns a string of created waves
Function/s HR_calc_EAMassSpec(waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType)
string waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType

	wave SpeciesMassWave = root:HR_frag:SpeciesMassWave

	svar HRResults_ionList = root:HR_panel:HRResults_ionList	// radio button 1 means in HR space
	nvar HR_checkAvgHRMS_UMR = root:HR_panel:HR_checkAvgHRMS_UMR	// radio button 1 means in HR space
	nvar NormalizeSpectra=root:HR_panel:NormalizeSpectra

	svar todoStr=root:panel:sq_toDoWvNm
	wave/z todo=$(todoStr)

	string paramStr, elemStr, myWaveList
	string nonHRSpec,HRSpec
	variable idex, num, numHRSpecIons
	
	numHRSpecIons = numpnts(SpeciesMassWave)
	
	paramStrPartial+="rie_list:1;ce_list:1;"	
	paramStrPartial+= "product_Type:14;"
 	paramstr="ug_op:0;"	// we only do EA on Hz data
//	paramStrPartial+="HRSpec_flag:1;"
// 	paramstr="specList:HROrg;fraglist:frag_HRorg;"+"ug_op:0;"+paramStrPartial		// we only do EA on Hz data
	
//	1.51Q  section added for non HROrg species
	paramStr=""	
	nonHRSpec = HR_CheckListForHRSpecies(HRResults_ionList, 3)		// get list for HR masses and families
	if (strlen(nonHRSpec)>0)
	 	HR_getParamStr(paramstr,nonHRSpec, 0, 0)	// sets fraglist:  rie_list:  ce_list:  speclist:  spec_corr:   1.06E
		paramStr+="HRSpec_flag:0;"
		paramstr+=paramStrPartial
	else		// should have single
		// do HRSpecies
		HRSpec = HR_CheckListForHRSpecies(HRResults_ionList, 4)		// get list for HR masses and families
		if (strlen(HRSpec)>0)
		 	HR_getParamStr(paramstr,HRSpec, 0, 0)	// sets fraglist:  rie_list:  ce_list:  speclist:  spec_corr:   1.06E
			paramStr+="HRSpec_flag:1;"
			paramstr+=paramStrPartial
		else
			abort "There does not seem to be any items to do Elemental Analysis on.  Aborting from HR_calc_EAMassSpec"
		endif
	endif
	
	squirrel_fetch(todo,PK_MS_concentration, paramstr, spectraType)
	
	// rename results with the waveNameSuffix
	num = itemsInList(ELEM_LIST)
	make/o/n=(numHRSpecIons) $("root:ElementalAnalysis:OM_MassSpec_"+waveNameSuffix)/Wave=OM_MassSpec = 0

	myWaveList=""
	for(idex = 0;idex< num;idex+=1)
		elemStr = stringFromList(idex, ELEM_LIST)
		wave OrigResultWave = $"root:ElementalAnalysis:"+elemStr+"_MassSpec"
		duplicate/o OrigResultWave $"root:ElementalAnalysis:"+elemStr+"_MassSpec_"+waveNameSuffix
		OM_MassSpec+=OrigResultWave
		myWaveList+="root:ElementalAnalysis:"+elemStr+"_MassSpec_"+waveNameSuffix+";"
	endfor	

	if (HR_checkAvgHRMS_UMR)		// we need to sum to unit mass
		myWaveList=""
		num = itemsInList(ELEM_LIST)
		for(idex = 0;idex< num;idex+=1)
			elemStr = stringFromList(idex, ELEM_LIST)
			wave elemMassSpecWave = $"root:ElementalAnalysis:"+elemStr+"_MassSpec"
			duplicate/o elemMassSpecWave $"root:ElementalAnalysis:"+elemStr+"_MassSpec_"+waveNameSuffix
			make/o/n=0 $("root:ElementalAnalysis:UMR_"+elemStr+"_MassSpec_"+waveNameSuffix)/WAVE=UMRresultWave// will be resized in UM_SumHRtoUMR
			UM_SumHRtoUMR(elemMassSpecWave, UMRresultWave,1)		
			myWaveList+="root:ElementalAnalysis:UMR_"+elemStr+"_MassSpec_"+waveNameSuffix+";"
		endfor	
		make/o/n=0 $("root:ElementalAnalysis:UMR_OM_MassSpec_"+waveNameSuffix)/WAVE=UMROMresultWave// will be resized in UM_SumHRtoUMR
		UM_SumHRtoUMR(OM_MassSpec, UMROMresultWave,1)		
	endif

	// take care of possible normalization
	if (NormalizeSpectra)
		sq_NormalizeSpectra(myWaveList)
		sq_NormalizeSpectra("root:ElementalAnalysis:OM_MassSpec_"+waveNameSuffix)
	endif

	return myWaveList

End


Function HR_plot_EAAvgMassSpec(waveNameSuffix, NewPlotName)
string waveNameSuffix, NewPlotName

	nvar HRResults_graphOutputType=root:HR_panel:HRResults_graphOutputType
	nvar NormalizeSpectra=root:HR_panel:NormalizeSpectra

	nvar HR_checkAvgHRMS_UMR = root:HR_panel:HR_checkAvgHRMS_UMR	// radio button 1 means in HR space
	nvar EA_MassSpecSepYAxis = root:HR_panel:EA_MassSpecSepYAxis	// "allison's" plot
	svar todoStr=root:panel:sq_toDoWvNm

	string elemStr, WaveNamePrefix, leftLabelStr, colorTriad
	variable idex, num, Csum, Hsum, Osum, Nsum, Ssum, OMsum=0, bottom, offset

	WaveNamePrefix = selectString(HR_checkAvgHRMS_UMR, "", "UMR_")
	
	// find and print elemental totals
	num = itemsInList(ELEM_LIST)

	if (HR_checkAvgHRMS_UMR)
		wave x_wave = root:HR_Sticks:amu_HR		//1.07C fix
	else
		wave x_wave = root:HR_frag:SpeciesMassWave	
	endif

// find elemental totals
	make/o/n=(numpnts(x_wave)) $("root:ElementalAnalysis:"+WaveNamePrefix+"OM_MassSpec_"+waveNameSuffix)/Wave=OM_MassSpec = 0
	for(idex = 0;idex< num;idex+=1)
		elemStr = stringFromList(idex, ELEM_LIST)
		variable/g $"tempSum"+elemStr
		nvar Xsum = $"tempSum"+elemStr
		wave resultWave = $"root:ElementalAnalysis:"+WaveNamePrefix+elemStr+"_MassSpec_"+waveNameSuffix
		OM_MassSpec +=resultWave
		Xsum=sum(resultWave)
		print "// EA analysis for "+todoStr+", total " +elemStr+" = "+num2str(Xsum)
		OMsum+=Xsum
	endfor	
	print "// EA analysis for "+todoStr+", total OM = "+num2str(OMsum)
			
	if (HRResults_graphOutputType==1 && EA_MassSpecSepYAxis)		// EA_MassSpecSepYAxis only plots new when HRResults_graphOutputType si chosen
		bottom=0 
		display/N=$NewPlotName as NewPlotName
		for(idex=0;idex<num;idex+=1)			
			elemStr = stringFromList(idex, ELEM_LIST)
			colorTriad = stringFromList(idex, ELEM_COLORLIST)
			wave resultWave = $"root:ElementalAnalysis:"+WaveNamePrefix+elemStr+"_MassSpec_"+waveNameSuffix
			appendtograph/W=$NewPlotName/L=$elemStr resultWave vs x_wave
			ModifyGraph/W=$NewPlotName axisEnab($elemStr)={bottom+.03,bottom+.03+.14}
			ModifyGraph/W=$NewPlotName rgb($NameofWave(resultWave))=(str2num(stringFromList(0, colorTriad, ",")),str2num(stringFromList(1, colorTriad, ",")),str2num(stringFromList(2, colorTriad, ",")))
			ModifyGraph/W=$NewPlotName freePos($elemStr)={0,bottom}, zero($elemStr)=1
			Label $elemStr elemStr
			ModifyGraph/W=$NewPlotName lblPos($elemStr)=100
			bottom+=.03+.14
		endfor
		// now add OM wave on top
		appendtograph/W=$NewPlotName/L=OM OM_MassSpec vs x_wave
		ModifyGraph/W=$NewPlotName axisEnab(OM)={bottom+.03,1},  freePos(OM)={0,bottom}, zero(OM)=1
		ModifyGraph/W=$NewPlotName rgb($NameofWave(OM_MassSpec))=(0,52224,0)
		Label/W=$NewPlotName OM "OM"
		ModifyGraph/W=$NewPlotName lblPos(OM)=100

		ModifyGraph mode=1,lsize=2
		Label/W=$NewPlotName bottom "mz"
		Legend/W=$NewPlotName/N=myLegend
		AppendText/W=$NewPlotName/N=myLegend  "OM normalized? "+selectstring(NormalizeSpectra, "N", "Y")

		ModifyGraph/W=$NewPlotName nticks(bottom)=9
		ModifyGraph/W=$NewPlotName minor(bottom)=1
	
	elseif(HRResults_graphOutputType!=4)		// if update we do nothing, not even bring a graph to the top window

		// bring graph to front (for multiple graphs do nothing for now)
		if (HRResults_graphOutputType==1)		// plot single
			display/N=$NewPlotName as NewPlotName
		elseif(HRResults_graphOutputType==3)	// append
			DoWindow/F $NewPlotName
		endif
		
		// start appending elemental traces
		for(idex=0;idex<num;idex+=1)			
			// get wave to append
			elemStr = stringFromList(idex, ELEM_LIST)
			colorTriad = stringFromList(idex, ELEM_COLORLIST)
			wave resultWave = $"root:ElementalAnalysis:"+WaveNamePrefix+elemStr+"_MassSpec_"+waveNameSuffix
			// if multiple plots create new plot for this species
			if(HRResults_graphOutputType==2)
				NewPlotName = "EA_MassSpecGraph_"+elemStr 
				DoWindow/K $NewPlotName
				display /N=$NewPlotName as NewPlotName
			endif

			Appendtograph/W=$NewPlotName resultWave vs x_wave		// add trace for new, multiple and append	
			ModifyGraph/W=$NewPlotName mode($NameofWave(resultWave))=1, lSize($NameofWave(resultWave))=2
			ModifyGraph/W=$NewPlotName rgb($NameofWave(resultWave))=(str2num(stringFromList(0, colorTriad, ",")),str2num(stringFromList(1, colorTriad, ",")),str2num(stringFromList(2, colorTriad, ",")))
			ModifyGraph toMode=3		//1.10A
		endfor
	
		Legend/W=$NewPlotName/N=myLegend
		AppendText/W=$NewPlotName/N=myLegend  "OM normalized? "+selectstring(NormalizeSpectra, "N", "Y")
	endif		// do nothing if update graph was selected or a new EA mass spec with separate axis was created
		
	// clean up
	for(idex = 0;idex< num;idex+=1)
		killvariables $"tempSum"+stringFromList(idex, ELEM_LIST)
	endfor	

End


//Function HR_calcPlot_EARatios(waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType)
string waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType

	// for EA analysis, we know the names of the waves
	HR_calc_EARatios(waveNameSuffix, myPlotName, spectraType, paramStrPartial, errorType)
	HR_plot_EARatioTimeSeries(waveNameSuffix, myPlotName)

End


// Calculates the Elemental ratios using Pika sticks
Function/s HR_calc_EARatios(waveNameSuffix, spectraType, paramStrPartial, errorType)
string waveNameSuffix, spectraType, paramStrPartial, errorType

	svar HRResults_ionList = root:HR_panel:HRResults_ionList	
	nvar HR_checkCalcHRErrors = root:HR_panel:HR_checkCalcHRErrors
	svar todoStr=root:panel:sq_toDoWvNm
	wave/z todo=$(todoStr)

	string paramStr, t_base, paramExtra, Wave2PlotList=""
	string nonHRSpec, HRSpec
	
	paramStrPartial+="rie_list:1;ce_list:1;"	
	paramStrPartial+= "product_Type:11;"
	paramStrPartial+="ug_op:0;"// The original apes always was done in Hz 

//	1.51Q  section added for non HROrg species
	paramStr=""	
	nonHRSpec = HR_CheckListForHRSpecies(HRResults_ionList, 3)		// get list for HR masses and families
	if (strlen(nonHRSpec)>0)
	 	HR_getParamStr(paramstr,nonHRSpec, 0, 0)	// sets fraglist:  rie_list:  ce_list:  speclist:  spec_corr:   1.06E
		paramStr+="HRSpec_flag:0;"
		paramstr+=paramStrPartial
	else		// should have single
		// do HRSpecies
		HRSpec = HR_CheckListForHRSpecies(HRResults_ionList, 4)		// get list for HR masses and families
		if (strlen(HRSpec)>0)
		 	HR_getParamStr(paramstr,HRSpec, 0, 0)	// sets fraglist:  rie_list:  ce_list:  speclist:  spec_corr:   1.06E
			paramStr+="HRSpec_flag:1;"
			paramstr+=paramStrPartial
		else
			abort "There does not seem to be any items to do Elemental Analysis on.  Aborting from HR_calc_EARatios"
		endif
	endif	
	
	squirrel_fetch(todo,PK_MS_concentration, paramstr, spectraType+";"+ errorType)		// we do not consider errors here yet
		 	
	string specTypeInitial = StringByKey("specTypeInitial",paramstr)
	Wave2PlotList = HR_EACalibrateRatios(waveNameSuffix, dataSetInitial =specTypeInitial )

	t_base=sq_tbase()	// we calc again... not ideal... it was calced once already in the sq_MS_concentration function
	if (strlen(t_base)>0)		
		wave t_baseWave = $"root:"+t_base		//1.57J
		HR_EAratioOneHRSpectra_t_base(waveNameSuffix, t_baseWave, HR_checkCalcHRErrors)
	endif
	
	killwaves/z HROrg_temp

	return Wave2PlotList
End


// Divides the Hz values found in EA_calcRatios by the calibration factors
Function/s HR_EACalibrateRatios(waveNameSuffix, [dataSetInitial] )
string waveNameSuffix, dataSetInitial

	 wave elementalMF_C = root:ElementalAnalysis:elementalMF_C 	// created in PK_MS_concentration
	 wave elementalMF_H = root:elementalAnalysis:elementalMF_H 	// created in PK_MS_concentration
	 wave elementalMF_O = root:ElementalAnalysis:elementalMF_O	// created in PK_MS_concentration
	 wave elementalMF_N = root:ElementalAnalysis:elementalMF_N	// created in PK_MS_concentration
	 wave elementalMF_S = root:ElementalAnalysis:elementalMF_S	// created in PK_MS_concentration

	nvar CalFactor_H_C=root:HR_panel:EA_CalFactor_H_C
	nvar CalFactor_O_C=root:HR_panel:EA_CalFactor_O_C
	nvar CalFactor_N_C=root:HR_panel:EA_CalFactor_N_C
	nvar CalFactor_S_C=root:HR_panel:EA_CalFactor_S_C

	nvar HR_elementalImprovedCal=root:HR_panel:HR_elementalImprovedCal		//1.14F

	variable numTime = numpnts(elementalMF_C)
	string Wave2PlotList=""
	
	make/o/n=(numTime) root:ElementalAnalysis:Total_OM/WAVE=Total_OM

	make /o/n=(numTime) $"root:ElementalAnalysis:Ratio_OM_OC_"+waveNameSuffix/WAVE= Ratio_OM_OC
	make /o/n=(numTime) $"root:ElementalAnalysis:Ratio_O_C_"+waveNameSuffix /WAVE= Ratio_O_C
	make /o/n=(numTime) $"root:ElementalAnalysis:Ratio_H_C_"+waveNameSuffix/WAVE= Ratio_H_C
	make /o/n=(numTime) $"root:ElementalAnalysis:Ratio_N_C_"+waveNameSuffix/WAVE= Ratio_N_C
	make /o/n=(numTime) $"root:ElementalAnalysis:Ratio_S_C_"+waveNameSuffix/WAVE= Ratio_S_C

	Wave2PlotList="Ratio_OM_OC_"+waveNameSuffix+";Ratio_O_C_"+waveNameSuffix+";Ratio_H_C_"+waveNameSuffix+";Ratio_N_C_"+waveNameSuffix+";Ratio_S_C_"+waveNameSuffix+";"
	
	// these are elemental ratios, not mass ratios
	Ratio_H_C = (AVG_ATOMIC_WT_C/AVG_ATOMIC_WT_H) * (elementalMF_H/elementalMF_C) / CalFactor_H_C
	Ratio_O_C = (AVG_ATOMIC_WT_C/AVG_ATOMIC_WT_O) * (elementalMF_O/elementalMF_C) / CalFactor_O_C
	Ratio_N_C = (AVG_ATOMIC_WT_C/AVG_ATOMIC_WT_N) * (elementalMF_N/elementalMF_C) / CalFactor_N_C
	Ratio_S_C = (AVG_ATOMIC_WT_C/AVG_ATOMIC_WT_S) * (elementalMF_S/elementalMF_C) / CalFactor_S_C

	Total_OM = AVG_ATOMIC_WT_C+(Ratio_H_C*(AVG_ATOMIC_WT_H) +Ratio_O_C*(AVG_ATOMIC_WT_O) + Ratio_N_C*(AVG_ATOMIC_WT_N) + Ratio_S_C*(AVG_ATOMIC_WT_S)) 

	Ratio_OM_OC = (Total_OM/AVG_ATOMIC_WT_C)

	if (HR_elementalImprovedCal )
		if (ParamIsDefault(dataSetInitial) )
			abort "Could not proceed with the improved elemental ratio calibration because of an error.  Aborting from  HR_EACalibrateRatios" 
		endif
	//1.14E  ACP 2015 Canagaratna adjustment to the cal ratios
	// Elemental ratio measurements of organic compounds using aerosol mass spectrometry: characterization, improved calibration, and implications
	// M. R. Canagaratna1, J. L. Jimenez2, J. H. Kroll3,4, Q. Chen3, S. H. Kessler4,P. Massoli1, L. Hildebrandt Ruiz5, E. Fortner1, L. R. Williams1, K. R. Wilson6,J. D. Surratt7, N. M. Donahue8, J. T. Jayne1, and D. R. Worsnop1

	// New O:C = O:C *(1.26 - 0.623  fCO2+ 2.28*fCHO)   //1.15E
	// New H:C = H:C *(1.07 + 1.07*fCHO)
	// we need fCO and fCHO

		duplicate/o Ratio_H_C  $"root:ElementalAnalysis:"+NameofWave(Ratio_H_C)+"_IA"/wave = Ratio_H_C_IA	// improved ambient
		duplicate/o Ratio_O_C  $"root:ElementalAnalysis:"+NameofWave(Ratio_O_C)+"_IA"/wave = Ratio_O_C_IA	// improved ambient

		duplicate/o Total_OM  $"root:ElementalAnalysis:"+NameofWave(Total_OM)+"_IA"/wave = Total_OM_IA	// improved ambient
		duplicate/o Ratio_OM_OC  $"root:ElementalAnalysis:"+NameofWave(Ratio_OM_OC)+"_IA"/wave =Ratio_OM_OC_IA	// improved ambient

		wave/z fCO2 = root:ElementalAnalysis:fCO2 // fCO2_"+dataSetInitial+"_"+waveNameSuffix		//1.15F
		wave/z fCHO = root:ElementalAnalysis:fCHO		// fCHO_"+dataSetInitial+"_"+waveNameSuffix
		if (  !WaveExists(fCHO) || !WaveExists(fCO2)   )
			abort "Could not find the waves fCHO and fCO2 to perform the improved O:C and H:C calculations.  Aborting from HR_EACalibrateRatios" 
		endif
		Ratio_H_C_IA = Ratio_H_C[p] * (1.07 + 1.07 * fCHO[p])
		Ratio_O_C_IA = Ratio_O_C[p] * (1.26 - 0.623 * fCO2+ 2.28 * fCHO[p])  //1.15F

		make /o/n=(numTime) $"root:ElementalAnalysis:OSc_"+waveNameSuffix+"_IA"/WAVE= Osc		//1.19C
		Osc = 2*Ratio_O_C_IA - Ratio_H_C_IA	//1.19C
		
		Total_OM_IA = AVG_ATOMIC_WT_C+(Ratio_H_C_IA*(AVG_ATOMIC_WT_H) +Ratio_O_C_IA*(AVG_ATOMIC_WT_O) + Ratio_N_C*(AVG_ATOMIC_WT_N) + Ratio_S_C*(AVG_ATOMIC_WT_S)) 

		Ratio_OM_OC_IA = (Total_OM_IA/AVG_ATOMIC_WT_C)
		Wave2PlotList="Ratio_OM_OC_"+waveNameSuffix+"_IA;Ratio_O_C_"+waveNameSuffix+"_IA;Ratio_H_C_"+waveNameSuffix+"_IA;OSc_"+waveNameSuffix+"_IA;"
	endif

	return 	Wave2PlotList
End


Function HR_EAratioOneHRSpectra_t_base(waveNameSuffix, t_base, error_flag)
string waveNameSuffix
wave t_base
variable error_flag

	// all these waves are on the DAQ time base.
	wave Ratio_OM_OC = $"root:ElementalAnalysis:Ratio_OM_OC_"+waveNameSuffix
	wave Ratio_O_C = $"root:ElementalAnalysis:Ratio_O_C_"+waveNameSuffix
	wave Ratio_H_C = $"root:ElementalAnalysis:Ratio_H_C_"+waveNameSuffix
	wave Ratio_N_C = $"root:ElementalAnalysis:Ratio_N_C_"+waveNameSuffix
	wave Ratio_S_C = $"root:ElementalAnalysis:Ratio_S_C_"+waveNameSuffix

	wave t_series = root:index:t_series
	wave imt_series = root:index:imt_series

	variable numNewTime, numTime, idex, t_step, pdex
	string newSuffix 
	
	newSuffix = "_"+waveNameSuffix+"_"+NameofWave(t_base)
	newSuffix = selectString(strlen(newSuffix)>20, newSuffix, newSuffix[0,19] )		// 20 is a guess
		
	numNewTime=numpnts(t_base)
	numTime = numpnts(t_series)

	make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_OM_OC"+newSuffix/WAVE=Ratio_OM_OC_NewTime = 0
	make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_O_C"+newSuffix/WAVE=Ratio_O_C_NewTime = 0
	make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_H_C"+newSuffix/WAVE=Ratio_H_C_NewTime = 0
	make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_N_C"+newSuffix/WAVE=Ratio_N_C_NewTime = 0
	make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_S_C"+newSuffix/WAVE=Ratio_S_C_NewTime = 0

	make/o/n=(numNewTime) dest_n = 0
	
//	if (t_base_flag)
//		make /o/d/n=(n_vals,2) t_source=imt_series[series_index[index_pos[p]]+q]
//		sq_make_map(t_source,t_base,"t_map")
		wave t_map
//	endif

	if (error_flag)
		make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_OM_OC"+newSuffix+"_err"/WAVE=Ratio_OM_OC_NewTime_err = 0
		make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_O_C"+waveNameSuffix+"_err"/WAVE=Ratio_O_C_NewTime_err = 0
		make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_H_C"+waveNameSuffix+"_err"/WAVE=Ratio_H_C_NewTime_err = 0
		make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_N_C"+waveNameSuffix+"_err"/WAVE=Ratio_N_C_NewTime_err = 0
		make/o/n=(numNewTime) $"root:ElementalAnalysis:Ratio_S_C"+waveNameSuffix+"_err"/WAVE=Ratio_S_C_NewTime_err = 0
	endif
	
	make /o/n=(1,1,1) ms_conc_tmp
	make /o/n=(2,3) mpos

	for(pdex=0;pdex<numTime;pdex+=1)
		t_step=imt_series[pdex+1]-imt_series[pdex]
		sq_map2dist(pdex,0,0,ms_conc_tmp,mpos,map1=t_map)

		Ratio_OM_OC_NewTime[mpos[0][0],mpos[1][0]]	+=Ratio_OM_OC*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
		Ratio_O_C_NewTime[mpos[0][0],mpos[1][0]]	+=Ratio_O_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
		Ratio_H_C_NewTime[mpos[0][0],mpos[1][0]]	+=Ratio_H_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
		Ratio_N_C_NewTime[mpos[0][0],mpos[1][0]]	+=Ratio_N_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
		Ratio_S_C_NewTime[mpos[0][0],mpos[1][0]]	+=Ratio_S_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]

		dest_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]

		if (error_flag)
			Ratio_OM_OC_NewTime_err [mpos[0][0],mpos[1][0]]	+=Ratio_OM_OC*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
			Ratio_O_C_NewTime_err [mpos[0][0],mpos[1][0]]	+=Ratio_O_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
			Ratio_H_C_NewTime_err [mpos[0][0],mpos[1][0]]	+=Ratio_H_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
			Ratio_N_C_NewTime_err [mpos[0][0],mpos[1][0]]	+=Ratio_N_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
			Ratio_S_C_NewTime_err [mpos[0][0],mpos[1][0]]	+=Ratio_S_C*t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
		endif
	endfor
	
	Ratio_OM_OC_NewTime /=dest_n
	Ratio_O_C_NewTime /=dest_n
	Ratio_H_C_NewTime /=dest_n
	Ratio_N_C_NewTime /=dest_n
	Ratio_S_C_NewTime /=dest_n

	if (error_flag)
		matrixop /o Ratio_OM_OC_NewTime_err = powr(Ratio_OM_OC_NewTime_err,0.5)/dest_n  
		matrixop /o Ratio_O_C_NewTime_err  = powr(Ratio_O_C_NewTime_err,0.5)/dest_n  
		matrixop /o Ratio_H_C_NewTime_err = powr(Ratio_H_C_NewTime_err,0.5)/dest_n  
		matrixop /o Ratio_N_C_NewTime_err  = powr(Ratio_N_C_NewTime_err,0.5)/dest_n  
		matrixop /o Ratio_S_C_NewTime_err = powr(Ratio_S_C_NewTime_err,0.5)/dest_n  
	endif
	
	killwaves/z ms_conc_tmp, mpos, dest_n
	
End


// note that the names of these wave do not change, regardless of todo wave, time base, etc
Function  HR_EACreateElementalTSWaves(time_points, errSuffix)
variable time_points
string errSuffix		

	make /o/n=(time_points) $"root:ElementalAnalysis:Total_OM"+errSuffix/WAVE=Total_OM =0
	make /o/n=(time_points) $"root:ElementalAnalysis:elementalMF_C"+errSuffix/WAVE=elementalMF_C =0 
	make /o/n=(time_points) $"root:ElementalAnalysis:elementalMF_H"+errSuffix/WAVE=elementalMF_H =0 
	make /o/n=(time_points) $"root:ElementalAnalysis:elementalMF_O"+errSuffix/WAVE=elementalMF_O =0
	make /o/n=(time_points) $"root:ElementalAnalysis:elementalMF_N"+errSuffix/WAVE=elementalMF_N =0
	make /o/n=(time_points) $"root:ElementalAnalysis:elementalMF_S"+errSuffix/WAVE=elementalMF_S =0
			
End


// note that the names of these wave do not change, regardless of todo wave, time base, etc
Function  HR_EACreateElementalMSWaves(n_HR, errSuffix)
variable n_HR
string errSuffix		

	make /o/n=(n_HR) $"root:ElementalAnalysis:EA_Org_MassSpec"+errSuffix/WAVE=EA_Org_MassSpec =0
	make /o/n=(n_HR) $"root:ElementalAnalysis:C_MassSpec"+errSuffix/WAVE=C_MassSpec =0 
	make /o/n=(n_HR) $"root:ElementalAnalysis:H_MassSpec"+errSuffix/WAVE=H_MassSpec =0 
	make /o/n=(n_HR) $"root:ElementalAnalysis:O_MassSpec"+errSuffix/WAVE=O_MassSpec =0
	make /o/n=(n_HR) $"root:ElementalAnalysis:N_MassSpec"+errSuffix/WAVE=N_MassSpec =0
	make /o/n=(n_HR) $"root:ElementalAnalysis:S_MassSpec"+errSuffix/WAVE=S_MassSpec =0
			
End


// It is called whether or not the data is pika -type (EA_calcEAMassSpec) or user defined
Function HR_EAparseOneHRSpectra(OneHRSpectra_temp, ElementX_temp,t_step,error_flag)
wave OneHRSpectra_temp, ElementX_temp
variable t_step, error_flag

	string errStr = selectString(error_flag, "", "_err")

	wave OM_flag = root:ElementalAnalysis:OM_flag	

// debugging
//	duplicate/o OM_Flag OM_FlagPika
//	duplicate/o OneHRSpectra_temp OneHRSpectra_tempPika	

	wave atomicMassFractionC = root:ElementalAnalysis:atomicMassFractionC			// has dimension of the number of HR fragments 
	wave atomicMassFractionH = root:ElementalAnalysis:atomicMassFractionH	
	wave atomicMassFractionO = root:ElementalAnalysis:atomicMassFractionO	
	wave atomicMassFractionN = root:ElementalAnalysis:atomicMassFractionN	
	wave atomicMassFractionS = root:ElementalAnalysis:atomicMassFractionS	
	
	 wave C_MassSpec = $"root:ElementalAnalysis:C_MassSpec"+errStr 
	 wave H_MassSpec = $"root:ElementalAnalysis:H_MassSpec"+errStr
	 wave O_MassSpec = $"root:ElementalAnalysis:O_MassSpec"+errStr
	 wave N_MassSpec = $"root:ElementalAnalysis:N_MassSpec"+errStr
	 wave S_MassSpec = $"root:ElementalAnalysis:S_MassSpec"+errStr

	// all the elemental Hz values are uncalibrated!!
	//  we do a weghted mass spec average, this is why t_step is included here
	//C 
	ElementX_temp = OM_flag[p]*atomicMassFractionC[p]*OneHRSpectra_temp[p]
	C_MassSpec += t_step*ElementX_temp
	
	//O
	ElementX_temp = OM_flag[p]*atomicMassFractionO[p]*OneHRSpectra_temp[p]
	O_MassSpec += t_step*ElementX_temp
//	O2_MassSpec += SelectNumber(O2_flag[p]==1, 0,t_step*ElementX_temp[p])		// Doug's desire to see which species have 2 or more Os.
		
	//H
	ElementX_temp = OM_flag[p]*atomicMassFractionH[p]*OneHRSpectra_temp[p]
	H_MassSpec += t_step*ElementX_temp
	
	//N
	N_MassSpec += t_step*OM_flag[p]*atomicMassFractionN[p]*OneHRSpectra_temp[p]
	
	//S
	S_MassSpec += t_step*OM_flag[p]*atomicMassFractionS[p]*OneHRSpectra_temp[p]

End


Function HR_plot_EARatioTimeSeries(Wave2PlotList, waveNameSuffix, myPlotName)
string Wave2PlotList, waveNameSuffix, myPlotName

	nvar HRResults_graphOutputType=root:HR_panel:HRResults_graphOutputType

	string elemStr, WaveNamePrefix, NewPlotName, leftLabelStr, colorTriad, waveStr
	variable idex, num, bottom, offset

	wave t_series = root:index:t_series

	num = itemsInList(ELEM_LIST)
//	Wave2PlotList = RemoveFromList("Ratio_OM_OC_"+waveNameSuffix,Wave2PlotList)

	num =itemsInList(Wave2PlotList)

	if(HRResults_graphOutputType!=4)		// if update we do nothing, not even bring a graph to the top window

		NewPlotName = myPlotName
		// bring graph to front (for multiple graphs do nothing for now)
		if (HRResults_graphOutputType==1)		// plot single
			display/N=$NewPlotName as NewPlotName
		elseif(HRResults_graphOutputType==3)	// append
			DoWindow/F $NewPlotName
		endif
		
		// start appending elemental traces
		for(idex=1;idex<num;idex+=1)			// don't start with C
			// get wave to append
			waveStr = stringFromList(idex, Wave2PlotList)
			if (!stringmatch(waveStr[0,2], "OSc"))			//1.19C
				elemStr = waveStr[6]		//  "Ratio_"+elemStr
				colorTriad = stringFromList(idex, ELEM_COLORLIST)
				wave resultWave = $"root:ElementalAnalysis:Ratio_"+elemStr+"_C"+"_"+waveNameSuffix
				// if multiple plots create new plot for this species
				if(HRResults_graphOutputType==2)
					NewPlotName = "EA_MassSpecGraph_"+elemStr 
					DoWindow/K $NewPlotName
					display /N=$NewPlotName as NewPlotName
				endif
	
				appendtograph/W=$NewPlotName resultWave vs t_series
				ModifyGraph/W=$NewPlotName rgb($NameofWave(resultWave))=(str2num(stringFromList(0, colorTriad, ",")),str2num(stringFromList(1, colorTriad, ",")),str2num(stringFromList(2, colorTriad, ",")))
				ModifyGraph/W=$NewPlotName mode($NameofWave(resultWave))=2, lSize($NameofWave(resultWave))=2
			else
				print Wave2PlotList
				wave resultWave = $"root:ElementalAnalysis:"+waveStr
				appendtograph/W=$NewPlotName/R resultWave vs t_series
				ModifyGraph mode($NameofWave(resultWave))=4,marker($NameofWave(resultWave))=18,rgb($NameofWave(resultWave))=(65535,60076,49151)
				ModifyGraph axRGB(right)=(65535,54607,32768),tlblRGB(right)=(65535,54607,32768),alblRGB(right)=(65535,54607,32768)
				Label right "~ Oxidation State = 2*O/C-H/C"
			endif
		endfor
			
		wave Ratio_OM_OC = $"root:ElementalAnalysis:Ratio_OM_OC"+"_"+waveNameSuffix
		Appendtograph/W=$NewPlotName Ratio_OM_OC vs t_series		// add trace for new, multiple and append	
		ModifyGraph/W=$NewPlotName mode($NameofWave(Ratio_OM_OC))=3,marker($NameofWave(Ratio_OM_OC))=8,rgb($NameofWave(Ratio_OM_OC))=(0,0,0)
		
		Label/W=$NewPlotName bottom " "
		Legend/C/N=text0
		ModifyGraph grid=1,zero(left)=1,gridRGB=(34816,34816,34816)
		ModifyGraph grid(bottom)=2,nticks(bottom)=9,minor(bottom)=1
		nvar HR_elementalImprovedCal = root:HR_panel:HR_elementalImprovedCal
		if (HR_elementalImprovedCal)		//1.14M
			AppendText/N=text0 "Improved ambient correction applied"
			ControlBar 30
			Checkbox ShowAiken title="Also show Aiken et al ratios", value=0, proc = EA_showHideAiken			
		endif
		Label/W=$NewPlotName left "Elemental Ratio"
		SetAxis/A/E=3 left
	endif		// do nothing if update graph was selected or a new EA mass spec with separate axis was created
		
End

////////////////////////////////////// Functions that really do the work  /////////////////////////////
//  Creates waves that counts the number of each elements in each fragment
// and the fraction of each element by mass 
// Wave names are numC, etc and atomicMassFractionC, etc
Function HR_EAatomicMassSeparation(frag_wave)	// for HROrg this wave is frag_HRorg
	wave/t frag_wave		///   = root:HR_frag:frag_HRorg  // change from EA version		//1.10C change
	
	wave/t SpeciesMassText = root:HR_frag:SpeciesMassText  // change from EA version
	wave SpeciesMassWave = root:HR_frag:SpeciesMassWave  // change from EA version
//	// these waves should have been vetted by EA_sanitizeInputs
//	wave EA_ExactMassWave = root:ElementalAnalysis:EA_ExactMassWave	
//	wave/t EA_ExactMassText = root:ElementalAnalysis:EA_ExactMassText

//	duplicate/o SpeciesMassText  root:ElementalAnalysis:EA_ExactMassText		// to keep tables and all other references updated
//	duplicate/o SpeciesMassWave  root:ElementalAnalysis:EA_ExactMassWave	

	string Fragment, NewText, str1, str2, str3, tempStr
	variable iPoint, numHRMasses, numFragChars, numAfterElement, charPos//, FragLetter
	variable numTot, jcharPos, DcharPos,numAfterIsotope, colVal, isotopeVal, fragIsotopeVal, tempVar, isPlus2
	variable containsC, containsO, containsH, containsN, containsS

	numHRMasses = numpnts(SpeciesMassWave)		// change from EA version

	NewDataFolder/o root:ElementalAnalysis		// updated experiments will need folder
	
	make/o/n=(numHRMasses,2) root:ElementalAnalysis:numC/wave=numC = 0	// each column an isotope	
	make/o/n=(numHRMasses,2) root:ElementalAnalysis:numH/wave=numH = 0	// each column an isotope	
	make/o/n=(numHRMasses,3) root:ElementalAnalysis:numO/wave=numO = 0	// each column an isotope
	make/o/n=(numHRMasses,2) root:ElementalAnalysis:numN/wave=numN = 0	// each column an isotope	
	make/o/n=(numHRMasses,4) root:ElementalAnalysis:numS/wave=numS = 0	// each column an isotope	
	make/o/t/n=(numHRMasses) root:ElementalAnalysis:nonCHONS/wave=nonCHONS = ""		// text wave where we store possible "bad" fragment names or fragments that do not have any CHOSN, or the remainder of the fragment that is not CHOSN
	make/o/n=(numHRMasses) root:ElementalAnalysis:multiCharged/wave=multiCharged = 1
	
	//  we use dimension labels to assign columns - one isotope per column
	SetDimLabel 1, 0, '12.000000', numC	// abundance = 98.93%
	SetDimLabel 1, 1, '13.003355', numC	// abundance =   1.07%

	SetDimLabel 1, 0, '1.007825', numH		// abundance = 99.9885%
	SetDimLabel 1, 1, '2.014102', numH		// abundance =   0.0115%

	SetDimLabel 1, 0, '15.994915', numO	// abundance = 99.757%
	SetDimLabel 1, 1, '17.999160', numO	// abundance =  0.205 %
	SetDimLabel 1, 2, '16.999131', numO	// abundance =  0.038%

	SetDimLabel 1, 0, '14.003074', numN	// abundance = 99.632%
	SetDimLabel 1, 1, '15.000109', numN	// abundance =   0.368%

	SetDimLabel 1, 0, '31.972071', numS	// abundance = 94.93%
	SetDimLabel 1, 1, '33.967867', numS	// abundance =  4.29%
	SetDimLabel 1, 2, '32.971458', numS	// abundance =  0.76%
	SetDimLabel 1, 3, '35.967081', numS	// abundance =  0.02%

	make/o/n=(numHRMasses) root:ElementalAnalysis:atomicMassFractionC/wave=atomicMassFractionC	 = 0	
	make/o/n=(numHRMasses) root:ElementalAnalysis:atomicMassFractionH/wave=atomicMassFractionH	 = 0	
	make/o/n=(numHRMasses) root:ElementalAnalysis:atomicMassFractionO/wave=atomicMassFractionO	 = 0	
	make/o/n=(numHRMasses) root:ElementalAnalysis:atomicMassFractionN/wave=atomicMassFractionN = 0		
	make/o/n=(numHRMasses) root:ElementalAnalysis:atomicMassFractionS/wave=atomicMassFractionS = 0		
			
	make/o/n=(numHRMasses)  root:ElementalAnalysis:OM_flag
	wave OM_flag = root:ElementalAnalysis:OM_flag	

	// go through the EA_ExactMassText and determine how many of each element is in the fragment
	For (iPoint = 0; iPoint < numHRMasses; iPoint += 1)
			
		Fragment = SpeciesMassText[iPoint]		// change from EA.  We only deal with HRorganic
		if (strlen(frag_wave[iPoint])==0)
			OM_flag[ipoint]=0
			continue
		else
			OM_flag[ipoint]=1
		endif
// debugging
//		if (stringmatch(Fragment, "Cj13CH5") )
//			print "hey"
//		endif

		containsC=0; containsH=0; containso=0; containsS=0; containsN=0;
		
		// first deal with isotopes
 		Fragment = HR_EAExtractIsotopicElements(Fragment, ipoint, numH, nonCHONS)
		Fragment = HR_EAMultpliChargedFragments(Fragment, ipoint, nonCHONS, multiCharged)
 		if (strlen(Fragment)==0)
 			continue
 		endif
		numFragChars = strlen(Fragment)
					
		charPos = strsearch(Fragment, "C", 0)
		if (charPos>=0 && !grepstring(Fragment[charPos+1], "a|u|l|r|o")  )		// changed to include o for Cobalt in 1.04A
			containsC = 1
			splitstring/E="(C)(\\d{1,3})" Fragment, str1, str2	
			if (strlen(str2) ==0)	//we do not have numbers after the letter
				numAfterElement = 1
			else	
				numAfterElement = str2num(str2)
			endif
			numC[ipoint][0]+= numAfterElement
		endif
				
		charPos = strsearch(Fragment, "H", 0)
		if (charPos>=0  && !grepstring(Fragment[charPos+1], "g|f|e")  )
			containsH = 1
			splitstring/E="(H)(\\d{1,3})" Fragment, str1, str2	
			if (strlen(str2) ==0)		//we do not have numbers after the letter
				numAfterElement = 1
			else	
				numAfterElement = str2num(str2)
			endif
			numH[ipoint][0]+= numAfterElement
		endif

		charPos = strsearch(Fragment, "O", 0)
		if (charPos>=0)
			containsO =1
			splitstring/E="(O)(\\d{1,3})" Fragment, str1, str2	
			if  (strlen(str2) ==0)	//we do not have numbers after the letter
				numAfterElement = 1
			else	
				numAfterElement = str2num(str2)
			endif
			numO[ipoint][0]+= numAfterElement
		endif
		
		charPos = strsearch(Fragment, "N", 0)
		if (charPos>=0 && !grepstring(Fragment[charPos+1], "e|a|i|b|d|o")  )		// no Na or Ne or Ni.....
			containsN =1
			splitstring/E="(N)(\\d{1,3})" Fragment, str1, str2	
			if (strlen(str2) ==0) //we do not have numbers after the letter
				numAfterElement = 1
			else	
				numAfterElement = str2num(str2)
			endif
			numN[ipoint][0]+= numAfterElement
		endif
		
		charPos = strsearch(Fragment, "S", 0)
		if (charPos>=0 && !grepstring(Fragment[charPos+1], "r|n|b|i|e|c")   )		// changed to include c for Sc in 1.04A
			containsS = 1
			splitstring/E="(S)(\\d{1,3})" Fragment, str1, str2	
			if (strlen(str2) ==0) 	//we do not have numbers after the letter
				numAfterElement = 1
			else	
				numAfterElement = str2num(str2)
			endif
			numS[ipoint][0]+= numAfterElement
		endif

		if ( !containsC && !containsH && !containsO && !containsN && !containsS)
			nonCHONS[ipoint]=Fragment 
		endif
		
	endfor
	
//  I was gonna use the Dim Label Values, but I wanted to make it more clear to any user trying to read the code.
	atomicMassFractionC +=12.000000* numC[p][0]/(SpeciesMassWave[p]*multiCharged[p])	// abundance = 98.93%
	atomicMassFractionC +=13.003355* numC[p][1]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =   1.07%

	atomicMassFractionH +=1.007825* numH[p][0]/(SpeciesMassWave[p]*multiCharged[p])	// abundance = 99.9885%
	atomicMassFractionH +=2.014102* numH[p][1]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =   0.0115%

	atomicMassFractionO +=15.994915* numO[p][0]/(SpeciesMassWave[p]*multiCharged[p])	// abundance = 99.757%
	atomicMassFractionO +=17.999160* numO[p][1]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =  0.205 %
	atomicMassFractionO +=16.999131* numO[p][2]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =  0.038%

	atomicMassFractionN +=14.003074* numN[p][0]/(SpeciesMassWave[p]*multiCharged[p])	// abundance = 99.632%
	atomicMassFractionN +=15.000109* numN[p][1]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =   0.368%

	atomicMassFractionS +=31.972071* numS[p][0]/(SpeciesMassWave[p]*multiCharged[p])	// abundance = 94.93%
	atomicMassFractionS +=33.967867* numS[p][1]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =  4.29%
	atomicMassFractionS +=32.971458* numS[p][2]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =  0.76%
	atomicMassFractionS +=35.967081* numS[p][3]/(SpeciesMassWave[p]*multiCharged[p])	// abundance =  0.02%

	// One last sanity check that the atomic Mass fraction sum = 1 or 0
	duplicate/o atomicMassFractionS Wave_temp
	duplicate/o/t SpeciesMassText TextWave_temp		// change from EA version
	wave/t TextWave_temp 
	Wave_temp = atomicMassFractionC+ atomicMassFractionH + atomicMassFractionO+atomicMassFractionN+atomicMassFractionS
	Wave_temp = selectNumber( abs(Wave_temp[p]-0)>1e-4, nan, Wave_temp[p])		// be careful of roundoff errors
	Wave_temp = selectNumber( abs(Wave_temp[p]-1)>1e-4, nan, Wave_temp[p])		// be careful of roundoff errors
	wavestats/q/M=1 Wave_temp
	if ( V_numNans != numpnts(Wave_temp))
		print "// At "+time() + " the atomic mass fraction code found these masses with some inorganic and some atoms containing C, H, O, N or S"
		TextWave_temp = SelectString(Wave_temp[p]>0, "", TextWave_temp[p])
		gen_removeEmptyStringFromTextWv(TextWave_temp)
		tempStr = sq_textwave2List(TextWave_temp)
		print "// "+tempStr[0, max (strlen(tempStr), 200)]
		print "// If you have not included these masses to be included in HRorganic, the calculations will be correct."
		if ( numpnts(TextWave_temp)>30)		// 30 is arbitrary
			abort "The number of HR ions that have C, H, O, N or S in their formula but are not Organic Mass (OM) seems excessive (>30).  See history window.  Aborting from EA_atomicMassSeparation"
		endif		//1.01A
	else
		print "// The atomic mass fraction code was performed at " + time() + " with no masses containing an inorganic fragment and H, C, O, N, or S" 
	endif
	
	Killwaves/z Wave_temp, TextWave_temp

End


// handles the part of chemical formula names that have 'j'  as an isotope flag.
//Returns the Fragment string with at most one instance of an isotopic element removed.
Function/S HR_EAextractIsotopicElements(Fragment, ipoint, numH, undetermined)
string fragment
variable ipoint
wave numH
wave/t undetermined

	variable jCharPos, DCharPos, tempVar, numAfterIsotope, colVal, isotopeVal, fragIsotopeVal
	string str1, str2, str3, tempStr, returnstr

	returnstr = Fragment

	jcharPos = strsearch(Fragment, "j", 0, 2)
	DcharPos = strsearch(Fragment, "D", 0)
	if (jcharPos>=0)
		// at this time we do not consider stange beasts such as a Cj35Cl fragment.
		if (strsearch(Fragment, "Cu", 0, 2)>=0 || strsearch(Fragment, "Cl", 0, 2)>=0 || strsearch(Fragment, "Ca", 0, 2)>=0   )
			undetermined[ipoint]=Fragment
			return ""
		endif
		// regular expression  Would split up Cj15NO into str1 = j, str2 = 15 and str3 = N.  Only check for isotopes of CONS.
		// This algorithym does not consider Ni, Ne, Na, Si, Se, Sb, Sr, or other strange beasts!! (elements that begin with the letter C, O, N, or S)
		splitstring/E="(j|J)(\\d{1,3})([CNOS])" Fragment, str1, str2, str3	
		if (strlen (str1)==0	)	// we have a strange beast, save and go on to next fragment
			undetermined[ipoint]=Fragment
			return ""
		endif
		wave/z numX = $"root:elementalAnalysis:num"+str3		// such as numS or numO, defined above
		if (!WaveExists(numX))	// we have a strange beast, save and go on to next fragment
			undetermined[ipoint]=Fragment
			return ""
		endif
		tempStr = str1+str2+str3
		tempVar = strsearch(Fragment, tempStr, 0, 2)
		tempStr = Fragment[tempVar+strlen(tempStr)]		// character after str2+str2+str3
		//splitstring/E="($str1$str2)\\d" Fragment, str3				//  Would split up Cj15NO into str3 = j15N.  I don't understand why this doesn't work
		numAfterIsotope = str2num(tempStr)		// convert the last character in str3 to a number
		numAfterIsotope = selectnumber(numtype(numAfterIsotope)==0, 1, numAfterIsotope)

		colVal=0
		do
			isotopeVal = round(str2num(GetDimLabel(numX, 1, colVal)))
			fragIsotopeVal = str2num(str2) 
			if ( isotopeVal == fragIsotopeVal )
				break
			endif
			colVal+=1
		while (colVal < dimsize(numX, 1) )
		
		if (isotopeVal!=fragIsotopeVal)		// did we get out of the for loop by the break statement or are we just at the end of the loop
			undetermined[ipoint]=Fragment
		else
			numX[ipoint][colVal]+=numAfterIsotope
		endif
		
		returnstr = ReplaceString(str1+str2+str3, returnstr, "")
	endif
		
	if(DcharPos>=0 || strsearch(Fragment, "j2H", 0) >=0)	// isotope of H 		//1.07H
		if (DcharPos==-1)
			DcharPos = strsearch(Fragment, "j2H", 0)
		endif
		numAfterIsotope = str2num(Fragment[DcharPos+1])		// assume at  most 9 D atoms
		numAfterIsotope = selectnumber(numtype(numAfterIsotope)==0, 1, numAfterIsotope)
		numH[ipoint][1]+=numAfterIsotope
		returnstr = ReplaceString(Fragment[DcharPos, DcharPos+numAfterIsotope-1], returnstr, "")
	endif
	
	return returnstr
	
End


// handles the part of chemical formula names that have 'plus2' for example.
//Returns the Fragment string with at most one instance of an isotopic element removed.
Function/S HR_EAmultpliChargedFragments(Fragment, ipoint,  nonCHONS, multiCharged)
string fragment
variable ipoint
wave/t nonCHONS
wave multiCharged

	variable plusPos, tempVar, numAfterPlus, colVal, isotopeVal, fragIsotopeVal
	string str1, str2, tempStr, returnstr

	returnstr = Fragment

	plusPos = strsearch(Fragment, "plus", 0, 2)
	if (plusPos>=0)
		// regular expression  Would split up Cj15NO into str1 = j, str2 = 15 and str3 = N.  Only check for isotopes of CONS.
		// This algorithym does not consider Ni, Ne, Na, Si, Se, Sb, Sr, or other strange beasts!! (elements that begin with the letter C, O, N, or S)
		splitstring/E="(plus)(\\d{1,3})" Fragment, str1, str2	
		if (strlen (str1)==0	 || strlen (str2)==0	)	// we have a strange beast, save and go on to next fragment
			nonCHONS[ipoint]=Fragment
			return ""
		endif
		tempStr = str1+str2
		tempVar = strsearch(Fragment, tempStr, 0, 2)
		tempStr = Fragment[tempVar+strlen(tempStr)]		// character after str2+str2+str3

		numAfterPlus = str2num(str2)		// convert the last character in str3 to a number
		numAfterPlus = selectnumber(numtype(numAfterPlus)==0, 1, numAfterPlus)
		multiCharged[ipoint] =numAfterPlus 
		
		returnstr = ReplaceString(str1+str2, returnstr, "")
	endif
			
	return returnstr
	
End


Function HR_EAratioOneHRSpectra(OneHRSpectra_temp, ElementX_temp, p1, error_flag)
wave OneHRSpectra_temp, ElementX_temp
variable p1, error_flag

	string errStr = selectString(error_flag, "", "_err")

	wave OM_flag = root:ElementalAnalysis:OM_flag	
	
	wave atomicMassFractionC = root:ElementalAnalysis:atomicMassFractionC			// has dimension of the number of HR fragments 
	wave atomicMassFractionH = root:ElementalAnalysis:atomicMassFractionH	
	wave atomicMassFractionO = root:ElementalAnalysis:atomicMassFractionO	
	wave atomicMassFractionN = root:ElementalAnalysis:atomicMassFractionN	
	wave atomicMassFractionS = root:ElementalAnalysis:atomicMassFractionS	

	 wave elementalMF_C = $"root:ElementalAnalysis:elementalMF_C"+errStr 
	 wave elementalMF_H = $"root:ElementalAnalysis:elementalMF_H"+errStr
	 wave elementalMF_O = $"root:ElementalAnalysis:elementalMF_O"+errStr
	 wave elementalMF_N = $"root:ElementalAnalysis:elementalMF_N"+errStr
	 wave elementalMF_S = $"root:ElementalAnalysis:elementalMF_S"+errStr

	// all the elemental Hz values are uncalibrated!!
	//C... order is important. Do CO2 first, then CO
	ElementX_temp = OM_flag[p]*atomicMassFractionC[p]*OneHRSpectra_temp[p]
	elementalMF_C[p1] = sum(elementX_temp)
	
	//O... order is important. Do CO2 first, then CO
	ElementX_temp = OM_flag[p]*atomicMassFractionO[p]*OneHRSpectra_temp[p]
	elementalMF_O[p1] = sum(elementX_temp)
	
	//H
	ElementX_temp = OM_flag[p]*atomicMassFractionH[p]*OneHRSpectra_temp[p]
	elementalMF_H[p1] = sum(elementX_temp)
	
	//N
	ElementX_temp = OM_flag[p]*atomicMassFractionN[p]*OneHRSpectra_temp[p]
	elementalMF_N[p1] = sum(elementX_temp)
	
	//S
	ElementX_temp = OM_flag[p]*atomicMassFractionS[p]*OneHRSpectra_temp[p]
	elementalMF_S[p1] = sum(elementX_temp)

End



// *************************************************
//  Redimensions tw so that it doesn't have an empty string in any rows.
static Function gen_removeEmptyStringFromTextWv(tw)
	Wave/t tw

	variable p, numPoints, numEmptyStrs
	string str
	
	numEmptyStrs = 0
	p = 0										// the loop index
	numPoints = numpnts(tw)						// number of times to loop

	do
		str = tw[p]
		if (strlen(str)==0)					
			numEmptyStrs += 1
		else										
			tw[p - numEmptyStrs] = str			// copy to input wave
		endif
		p += 1
	while (p < numPoints)
	
	// Truncate the wave
	DeletePoints numPoints-numEmptyStrs, numEmptyStrs, tw
	
	return(numEmptyStrs)
End


Function HR_calcPlot_EARatioDiurnal(waveNameSuffix, myPlotName, spectraType, specTypeInitial, paramStrPartial)
string waveNameSuffix, myPlotName, spectraType, specTypeInitial, paramStrPartial

	string waves2plot 
	waves2plot = HR_calc_EADiurnals(waveNameSuffix, myPlotName, spectraType, specTypeInitial, paramStrPartial)
	HR_MSplot_DiurnalBoxPlot(waves2plot,myPlotName)

End


// almost identical to sq_MSCalc_Diurnals
Function/s HR_calc_EADiurnals(waveNameSuffix, myPlotName, spectraType, specTypeInitial, paramStrPartial)
string waveNameSuffix, myPlotName, spectraType, specTypeInitial, paramStrPartial

	wave t_series = root:index:t_series
	nvar HRResults_whisker = root:HR_panel:HRResults_whisker	
	nvar HR_elementalImprovedCal = root:HR_panel:HR_elementalImprovedCal
	
	sq_journal_new(" EA DiurnalPlots")

	variable nspecies, idex, kdex, percentile_1, percentile_2, hourdex 
	variable length, timepoint, count, numberpoints
	string specStr, destWaveList, specList, specPrefix

	length = numpnts(t_series)
	if (HRResults_whisker == 2)
		percentile_1 = 5
		percentile_2 = 95
	else
		percentile_1 = 0
		percentile_2 = 100
	endif
	
	sq_diurnalIndex()

	wave DiurnalCounter = root:panel:diurnals:DiurnalCounter
	make/o/n=0 root:panel:diurnals:hourdata_temp
	wave hourdata_temp = root:panel:diurnals:hourdata_temp
	
	// we need to be careful when calling the time series function so that we don't create new waves, etc
	HR_calc_EARatios(waveNameSuffix, spectraType, paramStrPartial, "")
	if (HR_elementalImprovedCal)
		waveNameSuffix +="_IA"
		specList = "Ratio_H_C;Ratio_O_C;Ratio_OM_OC;"	
	else
		specList = "Ratio_H_C;Ratio_O_C;Ratio_N_C;Ratio_S_C;Ratio_OM_OC;"	
	endif
		
	specPrefix = "root:ElementalAnalysis:"
	destWaveList = replaceString(";", specList, "_"+waveNameSuffix+";")
	nspecies = itemsinlist(destWaveList)

	// loop through all selected species - for diurnal calcs reworked in 1.44B
	for (idex = 0; idex < nspecies; idex += 1)
		specStr = specPrefix+stringfromlist(idex, destWaveList)				
		wave/z datawave = $specStr
		specStr = ReplaceString("_temp", specStr, "_"+waveNameSuffix)
		
		// make median, min, max, percentile waves
		make /O/N=24 $(specStr + "_median") 
		wave medianw = $(specStr + "_median")
		make /O/N=24 $(specStr + "_mean") 
		wave meandata = $(specStr + "_mean")
		make /O/N=24 $(specStr + "_" + num2str(percentile_1) + "p")
		wave p1w = $(specStr + "_" + num2str(percentile_1) + "p")
		make /O/N=24 $(specStr + "_" + num2str(percentile_2) + "p")
		wave p2w = $(specStr + "_" + num2str(percentile_2) + "p")
		make /O/N=24 $(specStr + "_25p")
		wave p25w = $(specStr + "_25p")
		make /O/N=24 $(specStr + "_75p")
		wave p75w = $(specStr + "_75p")
		
		// calculate median, min, max, 25% and 75% percentiles
		for (hourdex=0; hourdex<24; hourdex+=1) 
			if (DiurnalCounter[hourdex]>0)
				// data exists for this hour
				redimension/N=(DiurnalCounter[hourdex]) hourdata_temp		// temporary wave, will be killed at the end
				wave DiurnalIndex = $("root:panel:diurnals:DiurnalIndex" +num2istr(hourdex))		// should be created in sq_diurnalIndex
				hourdata_temp = datawave[DiurnalIndex[p]]			
				sort hourdata_temp hourdata_temp		// any nans should appear at the bottom
				wavestats/q/m=1 hourdata_temp			// m=1 calcs average
				numberpoints=v_npnts
				
				medianw[hourdex] = hourdata_temp[((numberpoints+1)/2)-1]
				meandata[hourdex] = v_avg
				p25w[hourdex] = hourdata_temp[round(numberpoints*1/4)-1]
				p75w[hourdex] = hourdata_temp[round(numberpoints*3/4)-1]
				if (HRResults_whisker != 2)
					p1w[hourdex] = hourdata_temp[0]
					p2w[hourdex] = hourdata_temp[numberpoints - 1]					
				else
					p1w[hourdex] = hourdata_temp[round(numberpoints*5/100)-1]
					p2w[hourdex] = hourdata_temp[round(numberpoints*95/100)-1]
				endif
			else			// no data, make blank
				medianw[hourdex] = nan
				meandata[hourdex] = nan
				p25w[hourdex] = nan
				p75w[hourdex] = nan
				p1w[hourdex] = nan
				p2w[hourdex] = nan
			endif
		endfor				
		
		killwaves/z datawave
		destWaveList = RemoveListItem(idex, destWaveList)
		destWaveList = AddListItem(specStr, destWaveList, ";", idex)	
	endfor
	
	sq_journal_end()

	destWaveList = replaceString("_temp", destWaveList, "_"+waveNameSuffix)
	
	return destWaveList		// note that we don't actually return all the waves for diurnal plots - the XXX_median waves, etc
	
End


Function HR_view_AtomicMassFractionTable() 
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:elementalAnalysis:
	wave  OM_flag,multiCharged,atomicMassFractionC,atomicMassFractionH,atomicMassFractionO
	wave atomicMassFractionN,atomicMassFractionS,numC,numH,numO,numN,numS
	
	Edit/W=(40,80,640,380)/N=EA_AtomicMassFractionTable as "EA_AtomicMassFractionTable"
	AppendToTable ::HR_frag:SpeciesMassText,::HR_frag:SpeciesMassWave,::HR_frag:SpeciesFamilyText
	AppendToTable OM_flag,multiCharged,atomicMassFractionC,atomicMassFractionH,atomicMassFractionO
	AppendToTable atomicMassFractionN,atomicMassFractionS,numC,numH,numO,numN,numS
	ModifyTable format(Point)=1,width(Point)=27,width(::HR_frag:SpeciesMassText)=53
	ModifyTable width(::HR_frag:SpeciesMassWave)=54,width(::HR_frag:SpeciesFamilyText)=48
	ModifyTable width(OM_flag)=29,width(multiCharged)=29,format(atomicMassFractionC)=3
	ModifyTable width(atomicMassFractionC)=50,title(atomicMassFractionC)="C",rgb(atomicMassFractionC)=(0,52224,0)
	ModifyTable format(atomicMassFractionH)=3,width(atomicMassFractionH)=50,title(atomicMassFractionH)="H"
	ModifyTable format(atomicMassFractionO)=3,width(atomicMassFractionO)=50,title(atomicMassFractionO)="O"
	ModifyTable rgb(atomicMassFractionO)=(29440,0,58880),format(atomicMassFractionN)=3
	ModifyTable width(atomicMassFractionN)=50,title(atomicMassFractionN)="N",rgb(atomicMassFractionN)=(0,43520,65280)
	ModifyTable format(atomicMassFractionS)=3,width(atomicMassFractionS)=50,title(atomicMassFractionS)="S"
	ModifyTable rgb(atomicMassFractionS)=(52224,0,0),width(numC)=38,rgb(numC)=(0,52224,0)
	ModifyTable width(numH)=38,width(numO)=38,rgb(numO)=(29440,0,58880),width(numN)=38
	ModifyTable rgb(numN)=(0,43520,65280),width(numS)=38,rgb(numS)=(52224,0,0)
	SetDataFolder fldrSav0
End


//1.11L
Function/s gen_GetListOfDataFolders(dfr)
string dfr

	SetDataFolder root:		//1.15
	variable idex 
	string dfList="", dfStr = ""
	do
		dfStr = GetIndexedObjName(dfr, 4, idex)
		if (strlen(dfStr))
			dfList+=dfStr+";"
		endif
		idex+=1
	while (strlen(dfStr)>0)
	return dfList
	
End


//1.11L
Function HR_CompareAllMassedWaves(DataFolderA, DataFolderB)
string DataFolderA, DataFolderB

//  Assumes that these six wave have been imported
	wave/z BExactMassWave_All = $DataFolderB+"ExactmassWave_All"
	wave/t/z BExactMassText_All = $DataFolderB+"ExactmassText_All"
	wave/t/z BExactMassTag_All = $DataFolderB+"ExactmassTag_All"	
	wave/t/z BExactMassComments = $DataFolderB+"ExactMassComments"
	wave/z BMask_Isotope_abundance = $DataFolderB+"Mask_Isotope_abundance"
	wave/t/z BMask_Isotope_reference = $DataFolderB+"Mask_Isotope_reference"
	wave/z BMask_Stick_Integration = $DataFolderB+"Mask_Stick_Integration"
	wave/z BMask_Stick_Integration_default = $DataFolderB+"Mask_Stick_Integration_default"
	
	wave/z AExactMassWave_All = $dataFolderA+"ExactmassWave_All"
	wave/t/z AExactMassText_All = $dataFolderA+"ExactmassText_All"
	wave/t/z AExactMassTag_All = $dataFolderA+"ExactmassTag_All"
	wave/t/z AExactMassComments = $dataFolderA+"ExactMassComments"
	wave/z AMask_Isotope_abundance = $dataFolderA+"Mask_Isotope_abundance"
	wave/t/z AMask_Isotope_reference = $dataFolderA+"Mask_Isotope_reference"
	wave/z AMask_Stick_Integration = $dataFolderA+"Mask_Stick_Integration"
	wave/z AMask_Stick_Integration_default= $dataFolderA+"Mask_Stick_Integration_default"

	variable numA, numB, idexA, idexB, pos, numSum, counterAandB, counterAorB, counterAnotB, counterBnotA 
	variable idex, HRval, numAddedToA
	string HRstr

	//  check for sanity of waves
	if (!WaveExists(AExactMassWave_All) || !WaveExists(AExactMassText_All) || !WaveExists(AExactMassText_All) || !WaveExists(AMask_Isotope_abundance) || !WaveExists(AMask_Isotope_reference) || !WaveExists(AMask_Stick_Integration)|| !WaveExists(AMask_Stick_Integration_default)  )
		Abort "Please make sure the 8 waves have been loaded into the selected directory. - Aborting from HR_UpdateAllMassedWaves"
	endif

	if (!WaveExists(BExactMassWave_All) || !WaveExists(BExactMassText_All) || !WaveExists(BExactMassText_All) || !WaveExists(BMask_Isotope_abundance) || !WaveExists(BMask_Isotope_reference) || !WaveExists(BMask_Stick_Integration) || !WaveExists(BMask_Stick_Integration_default)  )
		Abort "Please make sure the 8 waves have been loaded into the selected directory. - Aborting from HR_UpdateAllMassedWaves"
	endif
	
	numA = numpnts(AExactMassWave_All)
	numB = numpnts(BExactMassWave_All)
	numSum=numA+numB		//DTS

	if  (numA != numpnts(AExactMassText_All) || numA != numpnts(AMask_Isotope_abundance) || numA != numpnts(AMask_Isotope_reference) || numA != numpnts(AMask_Stick_Integration)  || numA != numpnts(AMask_Stick_Integration_default)  )
		Abort "The 8 waves in data folder A do NOT have the same number of points. - Aborting from HR_UpdateAllMassedWaves"
	endif

	if (numB != numpnts(BExactMassText_All) || numB != numpnts(BMask_Isotope_abundance) || numB != numpnts(BMask_Isotope_reference) || numB != numpnts(BMask_Stick_Integration)  || numB != numpnts(BMask_Stick_Integration_default)  )
		Abort "The 8 waves in data folder B do NOT have the same number of points. - Aborting from HR_UpdateAllMassedWaves"
	endif

	// 1.12H addition
//	pk_PeriodicTableMassOrder()  // changes ElementIonName etc to increasing mass order
	pk_AtomOrder()				// creates ElementIonNameAtomOrder that indicates which atoms are in best CHONS  order, is needed for pk_SortAllHRionsTextByCHONS

	NewDataFolder/o/s root:HRcompareLists
	
	make/t/o/n=(numSum) root:HRcompareLists:AandBindex/wave=AandBindex		// intersection
	make/o/n=(numSum) root:HRcompareLists:AnotBindex/wave=AnotBindex
	make/o/n=(numSum) root:HRcompareLists:BnotAindex/wave=BnotAindex

	make/o/t/n=(numSum) root:HRcompareLists:AandBHRname/wave=AandBHRname		// intersection
	make/o/t/n=(numSum) root:HRcompareLists:AnotBHRname/wave=AnotBHRname			
	make/o/t/n=(numSum) root:HRcompareLists:BnotAHRname/wave=BnotAHRname
		
	gen_CompareTextWaves(AExactMassText_All, BExactMassText_All,  AandBHRname, AnotBHRname, BnotAHRname, AandBpos=AandBindex, AnotBpos=AnotBindex, BnotApos=BnotAindex)

	numAddedToA=numpnts(BnotAHRname)		 //numpnts(BnotAtext)		//1.12G
	make/o/n=(numAddedToA) positionAddToA
	
	duplicate/o  AExactMassWave_All  root:HRcompareLists:ExactMassWave_All/wave=AorB_ExactMassWave_All
	duplicate/o/t  AExactmassText_All  root:HRcompareLists:ExactmassText_All/wave=AorB_ExactmassText_All
	duplicate/o/t  AExactMassTag_All  root:HRcompareLists:ExactmassTag_All/wave=AorB_ExactmassTag_All
	duplicate/o/t  AExactMassComments  root:HRcompareLists:ExactMassComments/wave=AorB_ExactMassComments
	duplicate/o  AMask_Isotope_abundance  root:HRcompareLists:Mask_Isotope_abundance/wave=AorB_Mask_Isotope_abundance
	duplicate/o/t  AMask_Isotope_reference  root:HRcompareLists:Mask_Isotope_reference/wave=AorB_Mask_Isotope_reference
	duplicate/o  AMask_Stick_Integration  root:HRcompareLists:Mask_Stick_Integration/wave=AorB_Mask_Stick_Integration
	duplicate/o  AMask_Stick_Integration_default  root:HRcompareLists:Mask_Stick_Integration_default/wave=AorB_Mask_Stick_Integration_def
	
	for(idex=numpnts(BnotAHRname)-1;idex>=0;idex-=1)	// add from bottom to keep position values straight
		HRval = BExactMassWave_All[BnotAindex[idex]]
		pos = floor(BinarySearchinterp(AExactMassWave_All, HRval) )
		positionAddToA[idex]=pos
	endfor
	
	insertpoints numA, numAddedToA, AorB_ExactMassWave_All, AorB_ExactmassText_All, AorB_ExactMassTag_All, AorB_ExactMassComments, AorB_Mask_Isotope_abundance, AorB_Mask_Isotope_reference,AorB_Mask_Stick_Integration,AorB_Mask_Stick_Integration_DEF
		
	AorB_ExactMassWave_All[numA,] = BExactMassWave_All[BnotAindex[p-numA]]
	AorB_ExactmassText_All[numA,] =BExactmassText_All[BnotAindex[p-numA]]
	AorB_ExactMassTag_All[numA,] = BExactmassTag_All[BnotAindex[p-numA]]
	AorB_ExactMassComments[numA,] = BExactMassComments[BnotAindex[p-numA]]
	AorB_Mask_Isotope_abundance[numA,] = BMask_Isotope_abundance[BnotAindex[p-numA]]
	AorB_Mask_Isotope_reference[numA,] = BMask_Isotope_reference[BnotAindex[p-numA]]
	AorB_Mask_Stick_Integration[numA,] = BMask_Stick_Integration[BnotAindex[p-numA]]
	AorB_Mask_Stick_Integration_def[numA,] = BMask_Stick_Integration_default[BnotAindex[p-numA]]

	Sort AorB_ExactMassWave_All AorB_ExactmassText_All, AorB_ExactmassTag_All, AorB_ExactMassComments, AorB_Mask_Isotope_abundance, AorB_Mask_Isotope_reference,AorB_Mask_Stick_Integration,AorB_Mask_Stick_Integration_def
	Sort AorB_ExactMassWave_All AorB_ExactMassWave_All

	duplicate/o/t  AandBHRname   root:HRcompareLists:MaskConflictAandBHRname/wave=MaskConflictAandBHRname
	duplicate/o  AandBindex        root:HRcompareLists:MaskConflictAandBindex/wave=MaskConflictAandBindex
	make/FREE/o/n=(numpnts(AandBHRname))  MaskConflictKey
	MaskConflictKey = AMask_Stick_Integration[str2num(ReplaceString("A", stringFromList(0,AandBindex[p]), ""))]  != BMask_Stick_Integration[str2num(ReplaceString("B", stringFromList(1,AandBindex[p]), ""))]
		
	for(idex=numpnts(AandBHRname)-1;idex>=0;idex-=1)
		if (!MaskConflictKey[idex])
			deletePoints idex, 1, MaskConflictAandBHRname, MaskConflictAandBindex
		endif
	endfor
	
	ControlInfo/w=HRIonMassCalculator HRCompare_popUpMaskConflict		// 1, 0, A, B
	for(idex=0;idex<numpnts(MaskConflictAandBHRname);idex+=1)
		HRstr=MaskConflictAandBHRname[idex]
		pos = sq_findStr(AorB_ExactmassText_All,HRstr,4)
		switch(V_Value)		// found via Controlinfo
		case 1:
			AorB_Mask_Stick_Integration[pos]=1
		break
		case 2:
			AorB_Mask_Stick_Integration[pos]=0
		break
		case 3:
			// using A is the default
		break
		case 4:
			AorB_Mask_Stick_Integration[pos]=!AorB_Mask_Stick_Integration[pos]		// default is to use A
		break
		endswitch
	endfor

	SetDataFolder root:

End


//1.11L
Function HR_UpdateHRListComparison()

	wave AandBindex = root:HRcompareLists:AandBindex
	wave AnotBindex = root:HRcompareLists:AnotBindex
	wave BnotAindex = root:HRcompareLists:BnotAindex
	wave Mask_Stick_Integration = root:HRcompareLists:Mask_Stick_Integration

	ValDisplay HRCompare_valdispOr value=_NUM:numpnts(AandBindex)+numpnts(AnotBindex)+numpnts(BnotAindex), win=HRIonMassCalculator
	ValDisplay HRCompare_valdispAnotB value=_NUM:numpnts(AnotBindex), win=HRIonMassCalculator
	ValDisplay HRCompare_valdispBnotA value=_NUM:numpnts(BnotAindex), win=HRIonMassCalculator
	ValDisplay HRCompare_valdispAnd value=_NUM:numpnts(AandBindex), win=HRIonMassCalculator
//	ValDisplay HRCompare_valdispFitAorB value=_NUM:sum(Mask_Stick_Integration), win=HRIonMassCalculator

End


//1.11L
Function gen_CompareTextWaves(TextWaveA, TextWaveB, AandBtext, AnotBtext, BnotAtext [AandBpos, AnotBpos, BnotApos]  )
wave/t TextWaveA, TextWaveB,  AandBtext, AnotBtext, BnotAtext,AandBpos
wave AnotBpos, BnotApos 
		
	variable numA, numB, idexA, idexB, pos, numSum, counterAandB, counterAorB, counterAnotB, counterBnotA 
	variable posWavesExist, flag=0
	string str, Str1

	numA = numpnts(TextWaveA)
	numB = numpnts(TextWaveB)
	numSum=numA+numB	

	posWavesExist = !ParamIsDefault(AandBpos) && !ParamIsDefault(AnotBpos) && !ParamIsDefault(BnotApos)
	
	redimension/n=(numSum)  AandBtext, AnotBtext, BnotAtext
	if (posWavesExist)
		redimension/n=(numSum) AandBpos, AnotBpos, BnotApos
	endif
	
	make/FREE/o/n=(numB)	BdoneFlag=nan
	
	counterAandB=0; counterAorB=0; counterAnotB=0; counterBnotA=0
	
	// for each HR ion in AExactMassText_All check for its existence in B, must be case sensitive
	for (idexA=0;idexA<numA;idexA+=1)
		str = TextWaveA[idexA]		// exact match, look for the new, see if it is in the old
//		pos = sq_findStr(TextWaveB,str,4)		// 4 will be case sensitive
		FindValue/TEXT=str/TXOP=7/Z TextWaveB		// TXOP means case sensitive AND whole word
		pos=V_Value
		if (pos>=0)	// in the intersection
			insertpoints counterAandB, 1, AandBtext
			AandBtext[counterAandB]=str
			if (posWavesExist)
				insertpoints counterAandB, 1, AandBpos
				AandBpos[counterAandB]="A"+num2str(idexA)+";B"+num2str(pos)
			endif
			counterAandB+=1
			BdoneFlag[pos]=counterAandB
		else
			insertpoints counterAnotB, 1, AnotBtext
			AnotBtext[counterAnotB]=str
			if (posWavesExist)
				insertpoints counterAnotB, 1, AnotBpos
				AnotBpos[counterAnotB]=idexA
			endif
			counterAnotB+=1		
		endif

	endfor

	for (idexB=0;idexB<numB;idexB+=1)
		if (BdoneFlag[idexB]>=0)
			continue
		endif
		str = TextWaveB[idexB]		// exact match, look for the new, see if it is in the old
		insertpoints counterBnotA, 1, BnotAtext
		BnotAtext[counterBnotA]=str
		if (posWavesExist)
			insertpoints counterBnotA, 1, BnotApos
			BnotApos[counterBnotA]=idexB
		endif
		counterBnotA+=1
	endfor

	deletepoints counterBnotA, numSum, BnotAtext
	deletepoints counterAnotB, numSum, AnotBtext
	deletepoints counterAandB, numSum, AandBtext

	if (posWavesExist)
		deletepoints counterBnotA, numSum,  BnotApos
		deletepoints counterAnotB, numSum,  AnotBpos
		deletepoints counterAandB, numSum,  AandBpos
	endif

	//1.12H  Now check if any HR ions are really the same, just named differently.  Can use either BnotApos or 	AnotBpos
	for (idexA=0;idexA<numpnts(AnotBtext);idexA+=1)
		//  first check to see if HR ion is in CHONS form
		str = AnotBtext[idexA]
		if (strsearch(str, "inV", 0)>=0 || strsearch(str, "inW", 0)>=0 || strsearch(str, "plus", 0)>=0)
			continue
		endif
		Str = PK_SortHRionTextByCHNSO(AnotBtext[idexA])
		if (!stringmatch(Str,AnotBtext[idexA]))		//  is HR ion name in good format
			// check if B list has this ion in the correct format
			FindValue/TEXT=str/TXOP=7/Z TextWaveB		// TXOP means case sensitive AND whole word
			if (V_Value>=0)	// in the intersection..  Lo! we need to update!
				pos = V_Value
				FindValue/TEXT=(AnotBtext[idexA])/TXOP=7/Z TextWaveA		// TXOP means case sensitive AND whole word
				print "// Please fix HR ion names: "+AnotBtext[idexA]+" in row "+num2str(V_Value)+" in data set A is "+Str+" in row "+num2str(pos)+ " in data set B"			
				flag=1
			endif
		
		endif
		
	endfor

	if (flag)		
		DoAlert 0, "There may be some HR ions that are not it the CHONS preferred atomic order preference.  Look to the history window for messages.  "
	endif
	
End

//1.11L
Function HR_DataFolderPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	string dataFolderA, DFList
	if (stringMatch(popStr, "Get List"))
		DFList = gen_GetListOfDataFolders("")
		String quote = "\""	
		string popmenuList =quote+ "Get List;root;"+DFList+quote
		popupmenu $ctrlName value=#popmenuList
		
		return 0
	elseif (stringMatch(popStr, "root"))
		dataFolderA = "root:"
	else
		dataFolderA = "root:"+popStr+":"	
	endif

	string AorB = ctrlName[strlen(ctrlName)-1,strlen(ctrlName)-1]
	
	wave/z/d AExactMassWave_All = $dataFolderA+"ExactmassWave_All"
	wave/t/z AExactMassText_All = $dataFolderA+"ExactmassText_All"
	wave/t/z AExactMassTag_All = $dataFolderA+"ExactmassTag_All"
	wave/z AExactMassComments = $dataFolderA+"ExactMassComments"
	wave/z AMask_Isotope_abundance = $dataFolderA+"Mask_Isotope_abundance"
	wave/z AMask_Isotope_reference = $dataFolderA+"Mask_Isotope_reference"
	wave/z AMask_Stick_Integration = $dataFolderA+"Mask_Stick_Integration"
	wave/z AMask_Stick_IntegrationDEF = $dataFolderA+"Mask_Stick_Integration_default"

	if (!WaveExists(AExactMassWave_All) || !WaveExists(AExactMassText_All) || !WaveExists(AExactMassTag_All)  || !WaveExists(AExactMassComments) || !WaveExists(AMask_Isotope_abundance) || !WaveExists(AMask_Isotope_reference) || !WaveExists(AMask_Stick_Integration)  || !WaveExists(AMask_Stick_IntegrationDEF)   )	
		ValDisplay  $("HRCompare_valdisp"+AorB) value = _Num:nan, win=HRIonMassCalculator
		ValDisplay $("HRCompare_valdispFit"+AorB) value = _Num:nan, win=HRIonMassCalculator
		Abort "Please make sure the 8 waves have been loaded into the selected directory. - Aborting from HR_DataFolderPopMenuProc"
	endif

	variable numA = numpnts(AExactMassWave_All)

	if (numA != numpnts(AExactMassText_All) || numA != numpnts(AMask_Isotope_abundance) || numA != numpnts(AMask_Isotope_reference) || numA != numpnts(AMask_Stick_Integration) || numA != numpnts(AMask_Stick_IntegrationDEF) )
		Abort "The 8 waves in data folder do NOT have the same number of points. - Aborting from HR_DataFolderPopMenuProc"
	endif
	if (numA != numpnts(AExactMassTag_All) )		// decided not to alert the user, just overwrite the tag wave  //1.12G
		 HR_ConvertTextToTag(AExactMassText_All, AExactMassTag_All)
	endif
	if (numA != numpnts(AExactMassComments))  	// decided not to alert the user, just overwrite the tag wave
		redimension/n=(numA) AExactMassComments
	endif
	// sanity checks
	
	// monotonicity
	duplicate/o/free AExactMassWave_All delta
	delta[1,] = AExactMassWave_All[p] -AExactMassWave_All[p-1]		//1.20E
	delta[0] = delta[1]
	wavestats/q/m=1 delta
	if (V_min<=0)
		abort "The wave ExactMassWave_All"+ "is non-monotonic near row "+num2str(V_minLoc)
	endif
	
	// duplicates in text names
	
	// nans in stick integration wave
	wavestats/q/m=1 AMask_Stick_Integration
	if(V_numNaNs>0)
		abort "There are "+num2str(V_numNaNs)+" number of nans in the Mask_Stick_Integration wave" 
	endif
	
	ValDisplay  $("HRCompare_valdisp"+AorB) value = _Num:numA, win=HRIonMassCalculator

	ValDisplay $("HRCompare_valdispFit"+AorB) value = _Num:sum(AMask_Stick_Integration), win=HRIonMassCalculator

End

//1.11L
Function HR_compareHRIonLists(ctrlName) : ButtonControl
	String ctrlName

	 ControlInfo/w=HRIonMassCalculator HRCompare_popUpDataFolderA // simulate a popupMenu action
	 HR_DataFolderPopMenuProc("HRCompare_popUpDataFolderA",V_Value,S_Value)
	 string DataFolderA=selectString(!stringMatch(S_Value, "root"), "root:","root:"+S_Value+":" )

	 ControlInfo/w=HRIonMassCalculator HRCompare_popUpDataFolderB		// simulate a popupMenu action
	 HR_DataFolderPopMenuProc("HRCompare_popUpDataFolderB",V_Value,S_Value)	 
	 string DataFolderB=selectString(!stringMatch(S_Value, "root"), "root:","root:"+S_Value+":" )

	ControlInfo/w=HRIonMassCalculator HRCompare_popUpMaskConflict
	if (stringMatch(S_value, "Select option"))
		abort "Please select an option in step 1B. Aborting from HR_compareHRIonLists"
	endif
	
	 HR_CompareAllMassedWaves(DataFolderA, DataFolderB)
	 
	 HR_UpdateHRListComparison()

End


//1.11L
Function HR_Compare_ShowTableButtonProc(ctrlName) : ButtonControl
	String ctrlName

	 variable idex=0
	 string AorB = ctrlName[strlen(ctrlName)-1]
	 ControlInfo $"HRCompare_popUpDataFolder"+AorB
	 HR_DataFolderPopMenuProc("valdisp"+AorB,V_Value,S_Value)
	 string DataFolderAorB=selectString(!stringMatch(S_Value, "root"), "root:","root:"+S_Value+":" )

	DoWindow/F $"HRIonWavesDataFolder"+AorB
	if (V_flag)
//		do		// 1.12N?
//			RemoveFromTable $(StringBYKey("COLUMNNAME",TableInfo("", idex)))	
//			idex+=1
//		while(strlen(TableInfo("", idex))>0)
	else
		edit/N=$"HRIonWavesDataFolder"+AorB as  "HRIonWavesDataFolder"+AorB
	endif
	
	setdatafolder DataFolderAorB
	wave  ExactMassText_all,ExactMassWave_all,Mask_Stick_integration,ExactMassComments,Mask_Stick_Integration_Default, Mask_Isotope_abundance,Mask_Isotope_reference
	
	AppendToTable ExactMassText_all,ExactMassWave_all,Mask_Stick_integration
	AppendToTable ExactMassComments,Mask_Stick_Integration_Default		
	AppendToTable Mask_Isotope_abundance,Mask_Isotope_reference
	ModifyTable format(ExactMassWave_all)=3,digits(ExactMassWave_all)=6
	ModifyTable style(Mask_Stick_integration)=1, rgb(Mask_Stick_integration)=(65280,0,0)	
	ModifyTable rgb(Mask_Isotope_abundance)=(0,0,65280),  rgb(Mask_Isotope_reference)=(0,0,65280)
	ModifyTable width(Point)=30
	ModifyTable width(ExactMassText_All)=60,width(ExactMassWave_All)=60, width(Mask_Stick_Integration)=45
	ModifyTable title(ExactMassText_All)="HR ion",title(ExactMassWave_All)="Mass"
	ModifyTable title(Mask_Stick_Integration)="Fit Mask",title(ExactMassComments)="Comments"
	ModifyTable title(Mask_Stick_Integration_default)="Fit Mask Default"
	ModifyTable title(Mask_Isotope_abundance)="Isotope Abund."
	ModifyTable title(Mask_Isotope_reference)="Isotope Parent"

	setDataFolder root:
End

//1.11L
Function pk_HRIonWaveDescriptions()

	make/t/o/n=(8)  root:HRcompareLists:HRIonAllWaveNames/wave=HRIonAllWaveNames
	make/t/o/n=(8)  root:HRcompareLists:HRIonAllWaveDescriptions/wave=HRIonAllWaveDescriptions
	HRIonAllWaveNames[0]="ExactMassWave_All"
	HRIonAllWaveNames[1]="ExactMassText_All"
	HRIonAllWaveNames[2]="ExactMassTag_All"
	HRIonAllWaveNames[3]="ExactMassComments"
	HRIonAllWaveNames[4]="Mask_Isotope_abundance"
	HRIonAllWaveNames[5]="Mask_Isotope_Reference"
	HRIonAllWaveNames[6]="Mask_Stick_Integration"
	HRIonAllWaveNames[7]="Mask_stick_integration_default"		//1.20E
	
	HRIonAllWaveDescriptions[0]="The exact mass value of the HR ion.  For versions 1.x of pika, we do not consider the mass of the (missing) electron ion(s)."
	HRIonAllWaveDescriptions[1]="The text 'name' of the HR ion.  The code IS case sensitive, but not order sensitive.  Users are encouraged to follow existing pika naming conventions"
	HRIonAllWaveDescriptions[2]="The igor-code translation of ExactMassText_All, which allows for subscripts and superscripts."
	HRIonAllWaveDescriptions[3]="A text wave existing only for convenience.  Text in this wave has never been used.  Users may alter as they wish."
	HRIonAllWaveDescriptions[4]="The numerical ratio of an isotope's abundance relative to the 'parent' ion.  NOT the simple isotopic value.  These are used only if an isotope is constrained (i.e. NOT fitted but calculated based on the 'parent' value."
	HRIonAllWaveDescriptions[5]="The isotopic 'parent'; the HR ion for which another HR ion's value is determined."
	HRIonAllWaveDescriptions[6]="The wave indicating with ones (true) and zeros (false) whether or not an HR ion is considered in the fitting of the raw spectra."
	HRIonAllWaveDescriptions[7]="This wave is only used if a user presses the 'Set HR ions to fit to default button'.  Users may change this wave as they like."

End


//1.11L
Function HRIOnTabProc(ctrlName,tabNum) : TabControl
	String ctrlName
	Variable tabNum

	variable cont_disp,idex,num
	string cont_list, cont_item

	cont_list=controlnamelist("", ";", "*")
	num=itemsinlist(cont_list)
	for (idex=0;idex<num;idex+=1)
		cont_item=stringfromlist(idex,cont_list)
		controlinfo $cont_item
		switch (abs(v_flag))
			case 1:
				button $cont_item disable=cont_disp
				break
			case 2:
				checkbox  $cont_item disable=cont_disp
				break
			case 3:
				popupmenu $cont_item disable=cont_disp
				break
			case 4:
				valdisplay $cont_item disable=cont_disp
				break
			case 5:
				setvariable $cont_item disable=cont_disp
				break
			case 6:
				chart $cont_item disable=cont_disp
				break
			case 7:
				slider $cont_item disable=cont_disp
				break
			case 8:
				tabcontrol $cont_item disable=cont_disp
				break
			case 9:
				groupbox $cont_item disable=cont_disp
				break
			case 10:
				titlebox $cont_item disable=cont_disp
				break
			case 11:
				listbox $cont_item disable=cont_disp
				break
		endswitch	
	
	endfor

End


//revamped IN 1.11L
Function HR_HRIonMassCalculator() 
	PauseUpdate; Silent 1		// building window...

	NewPanel/N=HRIonMassCalculator /W=(92,408,880,865) as "HRIonMassCalculator"
	ModifyPanel fixedSize=1		//1.57L
	SetDrawLayer UserBack
	SetDrawEnv fsize= 14,fstyle= 1
	DrawText 5,20,"HR Ion Mass & Isotopic Calculator and HR Ion Query Tool  v"+PKVERSION
	TabControl tab0,pos={4,23},size={775,426},proc=HR_MassCalcQueryHRionTab
	TabControl tab0,tabLabel(0)="HR Mass, Isotope Calculator"
	TabControl tab0,tabLabel(1)="Verify Values in 'all' Waves"
	TabControl tab0,tabLabel(2)="Query Chemical Formulas"
	TabControl tab0,tabLabel(3)="2 Sets of HR Ion Waves"
	TabControl tab0,tabLabel(4)="Using familyWave_*",value= 0
	Button PikaGraph,pos={659.00,2.00},size={60.00,15.00},proc=HR_butt_ShowPkHtsGr,title="Pika graph"
	Button PikaGraph,fSize=10
	Button PikaPanel,pos={725.00,2.00},size={60.00,15.00},proc=pk_butt_mainPikaPanel,title="Pika panel"
	Button PikaPanel,fSize=10

	TitleBox MCalc_instructions,pos={7,50},size={320,13},disable=1,frame=0
	TitleBox MCalc_instructions,variable= root:HR_panel:MassCalcInstructions
	GroupBox MCalc_IsoRelationsGrp,pos={222,64},size={540,242},disable=1,title="Isotopic Children of HR Ion Mass"
	GroupBox MCalc_IsoRelationsGrp,fStyle=0,fColor=(1,1,1)
	GroupBox MCalc_SimpleMassCalc,pos={20,64},size={193,243},disable=1,title="One HR Ion Mass"
	GroupBox MCalc_SimpleMassCalc,fStyle=0,fColor=(1,1,1)
	Button MCalc_butt_CalcHRIonMass,pos={24,100},size={170,20},disable=1,proc=HR_butt_CalcMassOneHRIon,title="Calulate mass, iso. relations"
	Button MCalc_butt_CalcHRIonMass,fColor=(65280,43520,0)
	Button MCalc_butt_AddHRIon,pos={25,252},size={170,20},disable=1,proc=HR_butt_AddThisHRIon,title="Add HR ion to 'all' HR ion waves"
	Button MCalc_butt_AddHRIon,fSize=10
	SetVariable MCalc_NewIonText,pos={23,80},size={180,16},disable=1,title="HR ion (i.e. j13CH)"
	SetVariable MCalc_NewIonText,fSize=10,fStyle=0
	SetVariable MCalc_NewIonText,value= root:HR_panel:NewHRIonQueryText
	SetVariable MCalc_NewIonMass,pos={25,123},size={120,16},disable=1,title="Mass"
	SetVariable MCalc_NewIonMass,fSize=10,format="%1.8f"
	SetVariable MCalc_NewIonMass,limits={0,inf,0},value= root:HR_panel:NewHRIonQueryMass
	SetVariable MCalc_IsotopAbund,pos={25,150},size={145,16},disable=1,title="Iso. abundance"
	SetVariable MCalc_IsotopAbund,fSize=10,format="%1.8f"
	SetVariable MCalc_IsotopAbund,limits={0,inf,0},value= root:HR_panel:NewHRIonQueryIsoAbundance
	SetVariable MCalc_IsotopParent,pos={25,168},size={130,16},disable=1,title="Iso. Parent"
	SetVariable MCalc_IsotopParent,fSize=10
	SetVariable MCalc_IsotopParent,value= root:HR_panel:NewHRIonQueryIsoParent
	Button MCalcB_HR_IsotopChildrenTable,pos={616,181},size={30,20},disable=1,proc=sq_butt_PopThis,title="Pop"
	Button MCalcB_HR_IsotopChildrenTable,fSize=10
	SetVariable MCalc_IsotopLowerLimit,pos={234,81},size={380,16},disable=1,title="Isotopic children of HR ion whose relative abundance to parent is >"
	SetVariable MCalc_IsotopLowerLimit,fSize=10,format="%1.2e"
	SetVariable MCalc_IsotopLowerLimit,limits={0,inf,0},value= root:HR_panel:HR_isotopeLowerLimit
	Button MCalc_butt_AddToall,pos={625,249},size={100,32},disable=1,proc=HR_Butt_AddListToAll,title="Add isotopes to\r 'all' HR ion waves"
	Button MCalc_butt_AddToall,fSize=10
	CheckBox MCalc_checkChooseToFitAll,pos={624,232},size={83,14},fsize=10,disable=1,title="Choose to fit?"
	CheckBox MCalc_checkChooseToFitAll,variable= root:HR_panel:NewHRIonQueryChooseToFit
	CheckBox MCalc_checkChoseToFitOne,pos={26,231},size={83,14},fsize=10,disable=1,title="Choose to fit?"
	CheckBox MCalc_checkChoseToFitOne,variable= root:HR_panel:NewHRIonQueryChooseToFit
	TitleBox MCalc_titlchosen,pos={97,200},size={82,13},fsize=10,disable=1,frame=0
	TitleBox MCalc_titlchosen,variable= root:HR_panel:NewHRIonQueryIsoChosen
	TitleBox MCalc_titlexists,pos={27,200},size={50,13},fsize=10,disable=1,frame=0
	TitleBox MCalc_titlexists,variable= root:HR_panel:NewHRIonQueryIsoExists
	CheckBox MCalc_ck_IsoExistInAll,pos={231,101},size={127,14},disable=1,proc=HR_checkProc_massCalc,title="Child exists in 'all' wave"
	CheckBox MCalc_ck_IsoExistInAll,variable= root:HR_panel:NewHRIonQueryFindExists,mode=1
	CheckBox MCalc_ck_IsoNotExistInAll,pos={231,161},size={166,14},disable=1,proc=HR_checkProc_massCalc,title="Child does not exist in 'all' wave"
	CheckBox MCalc_ck_IsoNotExistInAll,value= 0,mode=1
	CheckBox MCalc_ck_ChildFit,pos={269,122},size={62,14},disable=1,proc=HR_checkProc_massCalc,title="Child is fit"
	CheckBox MCalc_ck_ChildFit,variable= root:HR_panel:NewHRIonQueryFindChildChosen,mode=1
	CheckBox MCalc_ck_ChildNotFit,pos={269,140},size={80,14},disable=1,proc=HR_checkProc_massCalc,title="Child is not fit"
	CheckBox MCalc_ck_ChildNotFit,value= 0,mode=1
	GroupBox MCalc_ElementalTable,pos={19,313},size={579,119},disable=1,title="FYI only - Table of Elements and Isotopes (NOT editable unless advanced user)"
	GroupBox MCalc_ElementalTable,fStyle=0,fColor=(1,1,1)
	Button MCalcB_HR_PeriodicTable,pos={535,334},size={30,20},disable=1,proc=sq_butt_PopThis,title="Pop"
	Button MCalcB_HR_PeriodicTable,fSize=10

	GroupBox Verify_AllIsoExist,pos={12,250},size={375,195},title="Opt. C. Check existence of isotopic children of each HR ion in 'all' wave"
	GroupBox Verify_AllIsoExist,fStyle=0,fColor=(1,1,1), fsize=11
	GroupBox Verify_IsotopesAreFitted,pos={395,252},size={375,195},title="Opt. D. Check that all isotopic children of fitted parents are fitted (constr.)"
	GroupBox Verify_IsotopesAreFitted,fStyle=0,fColor=(1,1,1), fsize=11
	GroupBox Verify_AllMassCalc,pos={16,50},size={375,200},title="Opt. A. Calc. mass of each HR ion in 'all' wave"
	GroupBox Verify_AllMassCalc,fStyle=0,fColor=(1,1,1), fsize=11
	GroupBox Verify_AllIsoCalc,pos={396,50},size={375,190},title="Opt. B. Calc. isotopic abundance of each HR ion isotope in 'all' wave"
	GroupBox Verify_AllIsoCalc, fsize=11
	
	SetVariable Verify_IonMassDiscrepancy,pos={21,60},size={360,16},title="Indicate when discrepancy between listed mass and calc. mass >"
	SetVariable Verify_IonMassDiscrepancy,fSize=10,format="%1.1e"
	SetVariable Verify_IonMassDiscrepancy,limits={0,inf,0},value= root:HR_panel:HRIonCheckAllMassLimit
	SetVariable Verify_IonIsotopeDiscrepancy,pos={407,61},size={330,16},title="Indicate when HR ion isootopic abundance discrepancy >"
	SetVariable Verify_IonIsotopeDiscrepancy,fSize=10,format="%1.1e"
	SetVariable Verify_IonIsotopeDiscrepancy,limits={0,inf,0},value= root:HR_panel:HRIsoAbundanceDiffLimit
	SetVariable Verify_MissingIsotope,pos={21,266},size={330,16},title="Indicate missing isotopic children with relative abundance >"
	SetVariable Verify_MissingIsotope,fSize=10,format="%1.1e"
	SetVariable Verify_MissingIsotope,limits={0,inf,0},value= root:HR_panel:HR_isotopeLowerLimit

	Button Verify_butt_CheckMassAllIons,pos={20,79},size={200,20},proc=HR_butt_massCheckAllIons,title="Check mass values for every HR ion"
	Button Verify_butt_CheckMassAllIons,fColor=(65280,43520,0)
	Button Verify_butt_CheckIsoAbund,pos={403,78},size={200,20},proc=HR_butt_FindIsoDiscrepancy,title="Check every isotopic abundance"
	Button Verify_butt_CheckIsoAbund,fColor=(65280,43520,0)
	Button Verify_butt_CkAllForIsotopChil,pos={20,285},size={200,20},proc=HR_butt_FindIsoChildrenForAll,title="Check existence of isotopic children"
	Button Verify_butt_CkAllForIsotopChil,fColor=(65280,43520,0)
	Button Verify_butt_AddtoHRAll,pos={24,420},size={140,20},fSize=10,proc=HR_butt_AddMissIsoChild,title="Add to 'all' HR ion waves"
	Button Verify_butt_ChangeToFitHRAll,pos={403,421},size={300,20},fSize=10,proc=HR_butt_UnfitIsoChange2Fit,title="Change mask wave to fit (and constrain) these isotopes "
	Button Verify_butt_CkFitForIsotopChi,pos={403,273},size={330,20},fsize=10,proc=HR_butt_findUnfitIsolTable,title="Check fitting of iso. children of fit parents (abund. > value in Opt. C)"
	Button Verify_butt_CkFitForIsotopChi,fColor=(65280,43520,0)
	Button Verify_HR_UnfitIsoTable,pos={727,420},size={30,20},fSize=10,proc=sq_butt_PopThis,title="Pop"
	Button Verify_butt_CorrectMassDIff,pos={23,213},size={240,20},fSize=10,proc=HR_butt_CorrectHRmass,title="Correct mass in table above"
	Button Verify_butt_CorrectIsoDiff,pos={398,213},size={280,20},fSize=10,proc=HR_butt_correctIsoAbund,title="Correct isotopic abundance in table above"
	Button Verify_HR_ConflictTable,pos={350,213},size={30,20},fsize=10,proc=sq_butt_PopThis,title="Pop"
	Button Verify_HR_MissIsoChildTable,pos={348,420},size={30,20},fSize=10,proc=sq_butt_PopThis,title="Pop"
	Button Verify_HR_ProblemIsoTable,pos={727,213},size={30,20},fSize=10,proc=sq_butt_PopThis,title="Pop"

	CheckBox QueryRRX_check,pos={25,149},size={21,14},disable=1,proc=HR_QueryCheckType,title=" "
	CheckBox QueryRRX_check,value= 0,mode=1
	CheckBox QueryCHONS_check,pos={25,68},size={21,14},disable=1,proc=HR_QueryCheckType,title=" "
	CheckBox QueryCHONS_check,value= 1,mode=1
	CheckBox Query_check_QueryChosen,pos={50,349},size={219,14},disable=1,proc=HR_ck_ChemFormFind,title="HR ions existing in 'all' list and chosen to fit"
	CheckBox Query_check_QueryChosen,value= 0,mode=1
	Button Query_buttonFindMasses,pos={30,404},size={220,20},disable=1,proc=HR_butt_QueryHRionChemForm,title="5.  Find matches and put results in table"
	Button Query_buttonFindMasses,fColor=(65280,43520,0)
	CheckBox Query_check_QueryNotInList,pos={50,369},size={155,14},disable=1,proc=HR_ck_ChemFormFind,title="HR ions not existing in 'all' list"
	CheckBox Query_check_QueryNotInList,value= 1,mode=1
	SetVariable QueryRRX_NewIonText,pos={300,171},size={155,16},disable=1,title="R base (i.e. C2H3)"
	SetVariable QueryRRX_NewIonText,fSize=10,fStyle=0
	SetVariable QueryRRX_NewIonText,value= root:HR_panel:IonQueryRoot
	SetVariable QueryRRX_NewIonText1,pos={287,192},size={170,16},disable=1,title="Suffix X (i.e. O, O2, NO, etc)"
	SetVariable QueryRRX_NewIonText1,fSize=10,fStyle=0
	SetVariable QueryRRX_NewIonText1,value= root:HR_panel:IonQuerySuffix
	SetVariable QueryRRX_NewIonText2,pos={225,286},size={100,16},disable=1,title="max m/z"
	SetVariable QueryRRX_NewIonText2,fSize=10,fStyle=0
	SetVariable QueryRRX_NewIonText2,value= root:HR_panel:IonQueryMaxMz
	CheckBox QueryRRX_ck_Cy,pos={351,218},size={51,14},disable=1,proc=HR_checkProc_massCalc,title="R(C)yX"
	CheckBox QueryRRX_ck_Cy,variable= root:HR_panel:Query_addC
	CheckBox QueryRRX_ck_CHy,pos={412,218},size={59,14},disable=1,title="R(CH)yX"
	CheckBox QueryRRX_ck_CHy,variable= root:HR_panel:Query_addCH
	CheckBox QueryRRX_ck_CH2y,pos={479,218},size={65,14},disable=1,title="R(CH2)yX"
	CheckBox QueryRRX_ck_CH2y,variable= root:HR_panel:Query_addCH2
	PopupMenu Query_CHONS_popNumC,pos={25,113},size={55,21},disable=1,title="# C"
	PopupMenu Query_CHONS_popNumC,mode=3,popvalue="Select option",value= #"\"0 or more;0;1;1+;2;2+;3;3+;4;4+;5;5+;6;6+;7;7+;8;8+;9;9+;10;10+;11;11+;12;12+;13;13+;14;14+;15+;15+;16+;16+;17+;17+;18+;18+;19;19+;\""
	PopupMenu Query_CHONS_popNumO,pos={136,113},size={56,21},disable=1,title="# O"
	PopupMenu Query_CHONS_popNumO,mode=3,popvalue="Select option",value= #"\"0 or more;0;1;1+;2;2+;3;3+;4;4+;5;5+;6;6+;7;7+;8;8+;9;9+;10;10+;11;11+;12;12+;13;13+;14;14+;15+;15+;16+;16+;17+;17+;18+;18+;19;19+;\""
	PopupMenu Query_CHONS_popNumH,pos={249,113},size={94,21},disable=1,title="# H"
	PopupMenu Query_CHONS_popNumH,mode=1,popvalue="Select option",value= #"\"0 or more;0;1;1+;2;2+;3;3+;4;4+;5;5+;6;6+;7;7+;8;8+;9;9+;10;10+;11;11+;12;12+;13;13+;14;14+;15+;15+;16+;16+;17+;17+;18+;18+;19;19+;\""
	PopupMenu Query_CHONS_popNumN,pos={361,113},size={94,21},disable=1,title="# N"
	PopupMenu Query_CHONS_popNumN,mode=1,popvalue="Select option",value= #"\"0 or more;0;1;1+;2;2+;3;3+;4;4+;5;5+;6;6;+7;7+;8;8+;9;9+;10;10+;11;11+;12;12+;13;13+;14;14+;15+;15+;16+;16+;17+;17+;18+;18+;19;19+;\""
	PopupMenu Query_CHONS_popNumS,pos={475,113},size={93,21},disable=1,title="# S"
	PopupMenu Query_CHONS_popNumS,mode=1,popvalue="Select option",value= #"\"0 or more;0;1;1+;2;2+;3;3+;4;4+;5;5+;6;6+;7;7+;8;8+;9;9+;10;10+;11;11+;12;12+;13;13+;14;14+;15+;15+;16+;16+;17+;17+;18+;18+;19;19+;\""
	CheckBox Query_checkChooseToFitAll1,pos={347,366},size={86,14},disable=1,title="Choose to Fit?"
	CheckBox Query_checkChooseToFitAll1,variable= root:HR_panel:NewHRIonQueryChooseToFit
	CheckBox Query_otherAtoms,pos={600,116},size={138,14},disable=1,title="Allow other atoms (P, Cl?)"
	CheckBox Query_otherAtoms,variable= root:HR_panel:HRIonMassCalcAllowOther
	TitleBox Query_title,pos={30,94},size={245,13},disable=1,title="1. Select the number of atoms of C, H,O, N, S, other"
	TitleBox Query_title,frame=0
	CheckBox Query_IncludeIsotopes,pos={50,268},size={94,14},disable=1,title="include isotopes"
	CheckBox Query_IncludeIsotopes,variable= root:HR_panel:HRIonMassCalcAllowIso
	GroupBox Query_groupBoxCHONS,pos={25,72},size={730,67},disable=1,title="Query list for CHONS containing HR ions"
	GroupBox Query_groupBoxCHONS,fStyle=0,fColor=(1,1,1)
	TitleBox Query_title0,pos={25,50},size={223,13},disable=1,title="0. Select the type of query you want to perform."
	TitleBox Query_title0,frame=0
	TitleBox Query_title1a,pos={30,174},size={233,13},disable=1,title="1A. Enter a molecular grouping of C and H atoms."
	TitleBox Query_title1a,frame=0
	TitleBox Query_title1b,pos={30,196},size={181,13},disable=1,title="1B. any other atoms (i.e. O4, OSi, etc.)"
	TitleBox Query_title1b,frame=0
	TitleBox Query_title1c,pos={30,217},size={274,13},disable=1,title="1C. Select one or more 'methyl' groupings to iteratively add"
	TitleBox Query_title1c,frame=0
	TitleBox Query_title2,pos={30,252},size={350,13},disable=1,title="2. Use the checkbox to indicate if you want the results to include isotopes."
	TitleBox Query_title2,frame=0
	TitleBox Query_title3,pos={30,288},size={157,13},disable=1,title="3. Select the maximum m/z range"
	TitleBox Query_title3,frame=0
	TitleBox Query_title4,pos={30,309},size={257,13},disable=1,title="4. Select the list of HR ions from which to find matches"
	TitleBox Query_title4,frame=0
	GroupBox Query_groupBoxRRX,pos={23,153},size={730,87},disable=1,title="Query list for R(methyl)nX HR ions"
	GroupBox Query_groupBoxRRX,fStyle=0,fColor=(1,1,1)
	CheckBox Query_check_QueryAll,pos={50,330},size={137,14},disable=1,proc=HR_ck_ChemFormFind,title="HR ions existing in 'all' list"
	CheckBox Query_check_QueryAll,value= 0,mode=1
	Button QUERY__HR_ChemFormTable,pos={419,256},size={15,50},disable=1,proc=sq_butt_popThis,title="p\ro\rp"
	Button Query_butt_AddtoHR,pos={273,384},size={160,50},disable=1,proc=HR_butt_Add2ListOrChange2Fit,title="Step 6. (Optional) Add HR ions\rin table to 'all' HR ion waves\rand/or change fitting status"
	Button Query_butt_AddtoHR,fSize=10
	GroupBox HRCompare_groupOPtional,pos={453,249},size={307,161},frame=0
	GroupBox HRCompare_groupOPtional1,pos={477,77},size={292,164}
	PopupMenu HRCompare_popUpDataFolderA,pos={15,84},size={201,21},fsize=10,proc=HR_DataFolderPopMenuProc,title="Data Folder for set \"A\" of waves"
	PopupMenu HRCompare_popUpDataFolderA,mode=5,popvalue="initialize",value= #"\"Get List;root;frag;panel;HR;HR_frag;HR_panel;Int_Sticks;HR_Sticks;HR_ms_mats;ms_mats;hoard;Packages;PToF;tw_globals;index;diagnostics;baselineFitting;StickRecalc;mzFitting;GaussFits;HR_PeakShape;HRFromTim;HRcompareLists;\""
	PopupMenu HRCompare_popUpDataFolderB,pos={245,83},size={203,21},fsize=10,proc=HR_DataFolderPopMenuProc,title="Data Folder for set \"B\" of waves"
	PopupMenu HRCompare_popUpDataFolderB,mode=2,popvalue="initialize",value= #"\"Get List;root;frag;panel;HR;HR_frag;HR_panel;Int_Sticks;HR_Sticks;HR_ms_mats;ms_mats;hoard;Packages;PToF;tw_globals;index;diagnostics;baselineFitting;StickRecalc;mzFitting;GaussFits;HR_PeakShape;HRFromTim;HRcompareLists;\""
	ValDisplay HRCompare_valdispA,pos={12,107},size={213,14},fsize=10,title="# pts in ExactMass_all etc. waves"
	ValDisplay HRCompare_valdispA,limits={0,0,0},barmisc={0,1000},value= _NUM:nan
	ValDisplay HRCompare_valdispB,pos={244,108},size={213,14},fsize=10,title="# pts in ExactMass_all etc. waves"
	ValDisplay HRCompare_valdispB,limits={0,0,0},barmisc={0,1000},value= _NUM:nan
	Button HRCompare_button0,pos={10,209},size={320,20},proc=HR_compareHRIonLists,title="Step 2. Compare HR Ion waves from A and B data folders"
	Button HRCompare_button0,fColor=(65280,43520,0)
	ValDisplay HRCompare_valdispOr,pos={14,255},size={100,14},fsize=10,title="# in A or B"
	ValDisplay HRCompare_valdispOr,limits={0,0,0},barmisc={0,1000},value= _NUM:nan
	ValDisplay HRCompare_valdispAnd,pos={124,255},size={100,14},fsize=10,title="# in A and B"
	ValDisplay HRCompare_valdispAnd,limits={0,0,0},barmisc={0,1000},fsize=10,value= _NUM:nan
	ValDisplay HRCompare_valdispAnotB,pos={234,255},size={100,14},title="# in A not B"
	ValDisplay HRCompare_valdispAnotB,limits={0,0,0},barmisc={0,1000},fsize=10
	ValDisplay HRCompare_valdispAnotB,value= _NUM:nan
	ValDisplay HRCompare_valdispBnotA,pos={345,255},size={100,14},fsize=10,fsize=10,title="# in B not A"
	ValDisplay HRCompare_valdispBnotA,limits={0,0,0},barmisc={0,1000}
	ValDisplay HRCompare_valdispBnotA,value= _NUM:nan
	Button HRComp_HR_AorBTable,pos={417,269},size={30,20},proc=sq_butt_popThis,title="pop"
	Button HRCompare_buttonTableWavesA,pos={12,146},size={204,20},fsize=10,proc=HR_Compare_ShowTableButtonProc,title="Table of HR Ion waves from data folder A"
	Button HRCompare_buttonTableWavesB,pos={245,145},size={204,20},fsize=10,proc=HR_Compare_ShowTableButtonProc,title="Table of HR Ion waves from data folder B"
	GroupBox HRCompare_groupA,pos={7,68},size={228,106},title="Set A. (A union B will start with set A)"
	GroupBox HRCompare_groupB,pos={239,68},size={232,105},title="Set B"
	ValDisplay HRCompare_valdispFitB,pos={245,127},size={213,14},fsize=10,title="# HR ions fit (mask_stick_integration)"
	ValDisplay HRCompare_valdispFitB,limits={0,0,0},barmisc={0,1000}
	ValDisplay HRCompare_valdispFitB,value= _NUM:nan
	ValDisplay HRCompare_valdispFitA,pos={12,126},size={213,14},fsize=10,title="# HR ions fit (mask_stick_integration)"
	ValDisplay HRCompare_valdispFitA,limits={0,0,0},barmisc={0,1000},value= _NUM:nan
	PopupMenu HRCompare_popUpMaskConflict,pos={10,181},size={371,21},title="Step1 B. Choose how to resolve mask_stick_integration conflicts"
	PopupMenu HRCompare_popUpMaskConflict,mode=1,popvalue="Select option",value= #"\"Set to1;Set to 0;Use Values from A;Use Values from B;\""
	Button HRCompare_Transfer,pos={15,420},size={530,20},proc=HR_CopyandOverwriteButtonProc,title="Step 3. (Optional) Copy & overwrite combined 'A or B' 8 waves above to HR data folder for HR fits"
	Button HRCompare_Transfer,fColor=(65280,43520,0)
	Button HRComp_HR_ABOptionalTable,pos={718,260},size={30,20},proc=sq_butt_popThis,title="pop"
	GroupBox HRCompare_groupResults,pos={10,236},size={753,179},title="Results from Step 2 comparison of HR ion waves"
	PopupMenu HRCompare_popOptionalTable,pos={459,267},size={226,21},proc=HRCompare_OptionalPopMenuProc,title=" "
	PopupMenu HRCompare_popOptionalTable,mode=2,popvalue="Select option",value= #"\"A and B HR ions and row positions;A and B mask_stick_integration conflicts;A and not B HR ions and row positions in A;B and not A HR ions and row positions in B;\""
	TitleBox HRCompare_Step1,pos={11,52},size={419,13},title="Step 1A.  Choose data folders from root: which contain a set of 8 waves for HR ion wave comparison"
	TitleBox HRCompare_Step1,frame=0
	TitleBox HRCompare_Ref,pos={483,83},size={268,13},title="For reference: List of 8 HR ion 'all' waves and description"
	TitleBox HRCompare_Ref,frame=0
	TitleBox HRCompare_title,pos={14,272},size={291,13},title="Table of A or B HR ions waves (the logical union, all 8 waves)"
	TitleBox HRCompare_title,frame=0
	TitleBox HRCompareOPtional,pos={464,253},size={153,13},title="Optional additional display below"
	TitleBox HRCompareOPtional,frame=0
	Button HRComp_HR_AllWaveDescrip,pos={472,185},size={15,50},proc=sq_butt_popThis,title="p\ro\rp"

// added in 1.20Y
	GroupBox Fami_CreateNewFromList,pos={11.00,114.00},size={759.00,112.00},title="Create/overwrite a new family from a list of HR chemical formulas "
	GroupBox Fami_CreateNewFromList,fStyle=0,fColor=(1,1,1)
	GroupBox Fami_OneClickFitting,pos={10.00,268.00},size={759.00,71.00},title="One click selection of fitting or not fitting all HR ions within a family"
	GroupBox Fami_OneClickFitting,fStyle=0,fColor=(1,1,1)
	Button Fami_butt_ViewFamilyWave,pos={570.00,55.00},size={200.00,30.00},proc=HR_butt_ViewFamilyWaveTable,title="View/update current HR family wave table\r(all families defined by a familyWave_*)"
	Button Fami_butt_ViewFamilyWave,fSize=10
	Button Fami_butt_batchTable,pos={152.00,55.00},size={140.00,20.00},proc=HR_butt_HRbatchTable,title="HR batch table (editable)"
	Button Fami_butt_batchTable,fSize=10
	Button Family_HR_AllMassesTable,pos={461.00,55.00},size={100.00,20.00},proc=sq_butt_PopThis,title="All HR masses table"
	Button Family_HR_AllMassesTable,fSize=10

	PopupMenu Fami_popup_viewFamilyOnly,pos={17.00,82.00},size={402.00,19.00},bodyWidth=120,proc=HR_popMenu_FamilyOnlyList,title="Select the HR family defined by root:HR:familyWave_"
	PopupMenu Fami_popup_viewFamilyOnly,mode=3,popvalue="Get List",value= #"\"Get List\""
	Button Fami_butt_ViewFamilyTable,pos={16.00,55.00},size={130.00,20.00},proc=HR_butt_ViewFamilyTable,title="HR family table (editable)"
	Button Fami_butt_ViewFamilyTable,fSize=10
	Button Fami_butt_VerifyFamilies,pos={12.00,230.00},size={250.00,30.00},proc=pk_butt_verifyFamilDisjoint,title="Verify that familyWave_* ions are NOT in other families\r(only needed if they were modified 'by hand'"
	Button Fami_butt_VerifyFamilies,fSize=10
	SetVariable Fami_CreateNewFamily,pos={16.00,131.00},size={730.00,15.00},bodyWidth=600,title="List of base HR ions in family"
	SetVariable Fami_CreateNewFamily,fSize=10,fStyle=0
	SetVariable Fami_CreateNewFamily,value= root:HR_panel:HR_familyWaveOnlyMembers
	SetVariable Fami_NewWave,pos={381.00,199.00},size={100.00,15.00},bodyWidth=100,title=" "
	SetVariable Fami_NewWave,fSize=10,fStyle=0
	SetVariable Fami_NewWave,value= root:HR_panel:HR_familyWaveOnlyName
	CheckBox Famil_ck_LessH,pos={299.00,150.00},size={49.00,15.00},title="[M-H]"
	CheckBox Famil_ck_LessH,value= 1
	CheckBox Famil_ck_Less2H,pos={372.00,152.00},size={55.00,15.00},title="[M-H2]"
	CheckBox Famil_ck_Less2H,value= 0
	CheckBox Famil_ck_isotopes,pos={226.00,151.00},size={60.00,15.00},title="Isotopes"
	CheckBox Famil_ck_isotopes,value= 1
	CheckBox Famil_ck_partOfHROrg,pos={15.00,172.00},size={211.00,15.00},proc=HR_checkProc_massCalc,title="Include this family in HROrg species?"
	CheckBox Famil_ck_partOfHROrg,value= 1
	SetVariable Fami_ColorList,pos={505.00,190.00},size={259.00,27.00},bodyWidth=100,title=" Optional list of RGB\r(RedGreenBlue) color for this family"
	SetVariable Fami_ColorList,fSize=10,fStyle=0
	SetVariable Fami_ColorList,value= root:HR_panel:HR_familyWaveOnlyCList
	Button Fami_butt_CreateNewFamily,pos={13.00,197.00},size={360.00,20.00},proc=HR_butt_newFamily,title="Make a new or overwrite a family defined by the HR list above and named "
	Button Fami_butt_CreateNewFamily,fSize=10,fColor=(65280,43520,0)
	TitleBox Fami_title1,pos={16.00,153.00},size={188.00,12.00},title="Include other related HR ions in this family:"
	TitleBox Fami_title1,fSize=10,frame=0
	Button Fami_butt_Fit,pos={15.00,287.00},size={280.00,20.00},proc=pk_butt_fitFamily,title="Choose to fit all HR ions in the selected family wave"
	Button Fami_butt_Fit,fSize=10
	Button Fami_butt_NotFit,pos={15.00,311.00},size={280.00,20.00},proc=pk_butt_fitFamily,title="Choose to NOT fit all HR ions in the selected family wave"
	Button Fami_butt_NotFit,fSize=10
	Button Fami_butt_defaultPAHs,pos={491.00,282.00},size={270.00,30.00},proc=HR_butt_AddPAHLists,title="Create 5 PAH families, SumPAH as in Herring et al 2015\rAlert: Other HR ions near PAHs must be chosen"
	Button Fami_butt_defaultPAHs,fSize=10
	Button fami_HerringPAHpaper,pos={490.00,319.00},size={270.00,15.00},proc=sq_URLButton,title="Herring et al. AS&T 2015 PAH paper (engine exhaust)"
	Button fami_HerringPAHpaper,fSize=10

	Edit/W=(226,180,608,296)/HOST=#
	HR_populate_IsotopChildrenTable()
	RenameWindow #,HRIsotopChildrenTable
	SetActiveSubwindow ##

	Edit/W=(25,328,521,424)/HOST=#
	HR_populate_PeriodicTable()
	RenameWindow #,HRPeriodicTable
	SetActiveSubwindow ##

	Edit/W=(20,100,380,210)/HOST=#
	HR_populate_ConflictTable()
	RenameWindow #,HRConflictTable
	SetActiveSubwindow ##

	Edit/W=(400,100,760,210)/HOST=#
	HR_populate_ProblemIsoTable()
	RenameWindow #,HRProblemIsoTable
	SetActiveSubwindow ##

	Edit/W=(20,310,380,420)/HOST=#
	HR_populate_MissIsoChildTable()
	RenameWindow #,HRMissIsoChildTable
	SetActiveSubwindow ##

	Edit/W=(440,255,770,435)/HOST=#
	HR_populate_ChemFormTable()
	RenameWindow #,HRChemFormTable
	SetActiveSubwindow ##
	
	Edit/W=(400,300,760,420)/HOST=# 
	HR_populate_UnfitIsoTable()
	RenameWindow #,HRUnfitIsolTable
	SetActiveSubwindow ##

	Edit/W=(15,300,420,415)/HOST=#
	HR_populate_AorBTable()
	RenameWindow #,HRAorBTable
	SetActiveSubwindow ##
	
	Edit/W=(461,288,748,403)/HOST=# 
	RenameWindow #,HRABOptionalTable
	HR_populate_ABOptionalTable()
//	RenameWindow #,HRABOptionalTable
	SetActiveSubwindow ##

	Edit/W=(490,100,760,235)/HOST=# 
	RenameWindow #,HRAllWaveDescrip
	HR_populate_AllWaveDescrip()
	SetActiveSubwindow ##

	HR_MassCalcQueryHRionTab("",0)
	HR_ck_ChemFormFind("Query_check_QueryAll",1)
	HR_QueryCheckType("",1)
End

//1.11L
Function HR_populate_AllWaveDescrip()

	pk_HRIonWaveDescriptions()
	AppendToTable root:HRcompareLists:HRIonAllWaveNames, root:HRcompareLists:HRIonAllWaveDescriptions
	ModifyTable format(Point)=1,width(Point)=20,width(root:HRcompareLists:HRIonAllWaveNames)=155,alignment(root:HRcompareLists:HRIonAllWaveDescriptions)=0
	ModifyTable width(root:HRcompareLists:HRIonAllWaveDescriptions)=450
	ModifyTable showParts=0x7B
End

//1.11L
Function HR_populate_AorBTable()

	if (!DataFolderExists("root:HRcompareLists:"))
		NewDataFolder root:HRcompareLists
	endif	
	wave/z AorB_ExactMassWave_All = root:HRcompareLists:ExactMassWave_All
	if (!WaveExists(AorB_ExactMassWave_All))
		make/n=0/o  root:HRcompareLists:ExactMassWave_All
		make/n=0/o/t  root:HRcompareLists:ExactmassText_All
		make/n=0/o/t  root:HRcompareLists:ExactmassTag_All
		make/n=0/o/t  root:HRcompareLists:ExactMassComments
		make/n=0/o  root:HRcompareLists:Mask_Isotope_abundance
		make/n=0/o/t  root:HRcompareLists:Mask_Isotope_reference
		make/n=0/o  root:HRcompareLists:Mask_Stick_Integration
		make/n=0/o  root:HRcompareLists:Mask_Stick_Integration_default
	endif
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:HRcompareLists
	wave ExactMassWave_All, ExactmassText_All, Mask_Stick_Integration, ExactMassComments, ExactMassTag_All
	wave Mask_Isotope_abundance, Mask_Isotope_reference, Mask_Stick_Integration_default
	AppendToTable 	ExactMassWave_All, ExactmassText_All, Mask_Stick_Integration, ExactMassComments, ExactMassTag_All
	AppendToTable 	Mask_Isotope_abundance, Mask_Isotope_reference, Mask_Stick_Integration_default
	ModifyTable format(Point)=1
	SetDataFolder fldrSav0

End

//1.11L
Function HR_populate_ABOptionalTable()

	wave/z AandBindex = root:HRCompareLists:AandBindex
	if (DataFolderExists("root:HRcompareLists:") && WaveExists(AandBindex))
		ControlInfo HRCompare_popOptionalTable
		if (stringmatch(S_Value, "Select Option"))
			HRCompare_OptionalPopMenuProc("1",nan,"")	
		else
			HRCompare_OptionalPopMenuProc("",V_Value,S_Value)	
		endif
	endif	

End

//1.11L
Function HR_populate_PeriodicTable()

	PK_CreateElementWaves()  //1.21B   rewrites if waves are old
	
	AppendToTable  root:HR:ElementIonName, root:HR:ElementIonMass, root:HR:ElementIsotopAbund, root:HR:ElementIsotopParentList,  root:HR:ElementIsotopParentIndexList
	AppendToTable  root:HR:ElementIonNameAtomOrder
	ModifyTable format(Point)=1,format(root:HR:ElementIonMass)=3,digits(root:HR:ElementIonMass)=8,format(root:HR:ElementIsotopAbund)=3
	ModifyTable digits(root:HR:ElementIsotopAbund)=8,width(root:HR:ElementIsotopAbund)=80

End


//1.11L
Function HR_removeNonCHONS(exactMassText, OtherWaveList)
wave/t exactMassText 
string  OtherWaveList

	variable idex, num, jdex, numOther
	
	num = numpnts(exactMassText)
	numOther = itemsInList(OtherWaveList)

	HR_OneHRIonElementalComp(exactMassText[0])		// sets root:HR_Panel:HRnameElementOrderIndex
	wave HRnameElementOrderIndex = root:HR_Panel:HRnameElementOrderIndex

	for(idex=num-1;idex>=0;idex-=1)
		HR_OneHRIonElementalComp(exactMassText[idex])
		
		HRnameElementOrderIndex = HRnameElementOrderIndex[p]==0 || HRnameElementOrderIndex[p]==1 ? 0 : HRnameElementOrderIndex[p] 		// H
		HRnameElementOrderIndex = HRnameElementOrderIndex[p]==9 || HRnameElementOrderIndex[p]==10 ? 0 : HRnameElementOrderIndex[p]	//C 
		HRnameElementOrderIndex = HRnameElementOrderIndex[p]==11 || HRnameElementOrderIndex[p]==12 ? 0 : HRnameElementOrderIndex[p] 	//N
		HRnameElementOrderIndex = HRnameElementOrderIndex[p]==13 || HRnameElementOrderIndex[p]==14  || HRnameElementOrderIndex[p]==15 ?  0 : HRnameElementOrderIndex[p] 
		HRnameElementOrderIndex = HRnameElementOrderIndex[p]==29 || HRnameElementOrderIndex[p]==30  || HRnameElementOrderIndex[p]==31   ? 0 : HRnameElementOrderIndex[p] 
		
		if (Wavemax(HRnameElementOrderIndex)>0)
			deletePoints idex, 1, exactMassText
			for(jdex=0;jdex<numOther;jdex+=1)
				deletePoints idex, 1, $StringFromList(jdex,OtherWaveList)
			endfor
		endif
	endfor

End

//1.11L
Function HR_removeIsotopesfromList(exactMassText, OtherWaveList)
wave/t exactMassText 
string  OtherWaveList

	variable idex, num, jdex, numOther
	
	num = numpnts(exactMassText)
	numOther = itemsInList(OtherWaveList)
	
	make/o/n=(num)/Free IsoFlag
	IsoFlag = strsearch(exactMassText[p], "j", 0)>=0

	for(idex=num-1;idex>=0;idex-=1)
		if (IsoFlag[idex])
			deletePoints idex, 1, exactMassText
			for(jdex=0;jdex<numOther;jdex+=1)
				deletePoints idex, 1, $StringFromList(jdex,OtherWaveList)
			endfor
		endif
	endfor

End

//1.11L
Function HRCompare_OptionalPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	string wnameList, winMenuStr
	if (Strlen(ctrlName)>0)
		winMenuStr="HRIonMassCalculator#HRABOptionalTable"
		do
			SetDataFolder root:HRCompareLists:
			wnameList=WaveList("*", ";", "WIN:"+winMenuStr )
			if (strlen(wnameList)>0)
				RemoveFromTable/w=$winMenuStr $StringFromList(0, wnameList)
			endif
			SetDataFolder root:
		while(strlen(wnameList)>0)
	else
		winMenuStr="HR_ABOptionalTable"
		doWindow/F HR_ABOptionalTable
	endif
	wave/z AandBindex = root:HRCompareLists:AandBindex
	if (DataFolderExists("root:HRCompareLists:") && WaveExists(AandBindex))
		switch (popNum)			
		case 1:
			AppendToTable/w=$winMenuStr root:HRcompareLists:AandBHRname,  root:HRcompareLists:AandBindex
		break
		case 2:
			AppendToTable/w=$winMenuStr root:HRcompareLists:MaskConflictAandBHRname, root:HRcompareLists:MaskConflictAandBindex			
		break
		case 3:
			AppendToTable/w=$winMenuStr root:HRcompareLists:AnotBHRname,  root:HRcompareLists:AnotBindex
		break
		case 4:
			AppendToTable/w=$winMenuStr root:HRcompareLists:BnotAHRname,  root:HRcompareLists:BnotAindex
		break
		endswitch
	endif

End

//1.11L
Function HR_CopyandOverwriteButtonProc(ctrlName) : ButtonControl
	String ctrlName
	
	wave  ExactMassWave_All  =root:HRcompareLists:ExactMassWave_All
	wave/t  ExactmassText_All  =root:HRcompareLists:ExactmassText_All
	wave/t  ExactMassTag_All  =root:HRcompareLists:ExactmassTag_All
	wave/t  ExactMassComments  =root:HRcompareLists:ExactMassComments
	wave  Mask_Isotope_abundance  =root:HRcompareLists:Mask_Isotope_abundance
	wave/t  Mask_Isotope_reference  =root:HRcompareLists:Mask_Isotope_reference
	wave  Mask_Stick_Integration  =root:HRcompareLists:Mask_Stick_Integration
	wave  Mask_Stick_Integration_default  =root:HRcompareLists:Mask_Stick_Integration_default

	wave/d  ExistingExactMassWave_All  =root:HR:ExactMassWave_All
	if (numpnts(ExactMassWave_All) < numpnts(ExistingExactMassWave_All) )
		DoAlert 1, "This is a check to make sure you intended to do this..  You will overwrite the 8 'all' waves in the HR folder. Click No to abort."
		if (V_flag==2)
			return 0
		endif
	endif

	duplicate/o  ExactMassWave_All  root:HR:ExactMassWave_All
	duplicate/o/t  ExactmassText_All  root:HR:ExactmassText_All
	duplicate/o/t  ExactMassTag_All  root:HR:ExactmassTag_All
	duplicate/o/t  ExactMassComments  root:HR:ExactMassComments
	duplicate/o  Mask_Isotope_abundance  root:HR:Mask_Isotope_abundance
	duplicate/o/t  Mask_Isotope_reference  root:HR:Mask_Isotope_reference
	duplicate/o  Mask_Stick_Integration  root:HR:Mask_Stick_Integration
	duplicate/o  Mask_Stick_Integration_default  root:HR:Mask_Stick_Integration_default

	DoAlert, 1, "The list of 'all' HR ion waves have been overwritten and now have "+num2str(numpnts(ExactMassWave_All)) +" number of points."		//1.12G

End


Function HR_populate_UnfitIsoTable()

	wave/z UnfitIsoChildToAllNames =  root:HR_Panel:UnfitIsoChildToAllNames
	if (!WaveExists(UnfitIsoChildToAllNames))
		make/t/o/n=0 root:HR_Panel:UnfitIsoChildToAllNames
		make/t/o/n=0 root:HR_Panel:UnfitIsoChildToAllParent
		make/o/n=0 root:HR_Panel:UnfitIsoChildToAllAbund
		make/o/n=0 root:HR_Panel:UnfitIsoChildToAllMass
	endif

	Appendtotable root:HR_Panel:UnfitIsoChildToAllNames
	Appendtotable root:HR_Panel:UnfitIsoChildToAllMass
	Appendtotable root:HR_Panel:UnfitIsoChildToAllAbund
	Appendtotable root:HR_Panel:UnfitIsoChildToAllParent	// text wave of HR isotopic parents

	ModifyTable format(Point)=1,width(Point)=36,width(root:HR_Panel:UnfitIsoChildToAllNames)=98,title(root:HR_Panel:UnfitIsoChildToAllNames)="HR Ion"
	ModifyTable format(root:HR_Panel:UnfitIsoChildToAllMass)=3,digits(root:HR_Panel:UnfitIsoChildToAllMass)=6,width(root:HR_Panel:UnfitIsoChildToAllMass)=102
	ModifyTable title(root:HR_Panel:UnfitIsoChildToAllMass)="Mass"
	ModifyTable width(root:HR_Panel:UnfitIsoChildToAllParent)=102,title(root:HR_Panel:UnfitIsoChildToAllParent)="Iso. parent"

End


Function HR_butt_findUnfitIsolTable(ctrlName) : ButtonControl
	String ctrlName

	wave/t ExactMassText = root:HR_sticks:ExactMassText
	nvar HR_isotopeLowerLimit = root:HR_panel:HR_isotopeLowerLimit
	nvar NewHRIonQueryFindExists = root:HR_panel:NewHRIonQueryFindExists
	nvar NewHRIonQueryFindChildChosen = root:HR_panel:NewHRIonQueryFindChildChosen
	NewHRIonQueryFindChildChosen=0	
	NewHRIonQueryFindExists=1

	HR_checkProc_massCalc("MCalc_ck_IsoExistInAll",1)		
	HR_checkProc_massCalc("MCalc_ck_ChildNotFit",1)		

	make/t/o/n=0 root:HR_Panel:UnfitIsoChildToAllNames/wave=UnfitIsoChildToAllNames		// text wave of HR isotopic ions that are not in the all list
	make/t/o/n=0 root:HR_Panel:UnfitIsoChildToAllParent/wave=UnfitIsoChildToAllParent	// text wave of HR isotopic parents
	make/o/n=0 root:HR_Panel:UnfitsoChildToAllAbund/wave=UnfitIsoChildToAllAbund		// isotopic abundance
	make/o/n=0 root:HR_Panel:UnfitIsoChildToAllMass/wave=UnfitIsoChildToAllMass		// isotopic abundance
	
	HR_FindMissingIsotopes(ExactMassText, HR_isotopeLowerLimit, UnfitIsoChildToAllNames, UnfitIsoChildToAllParent, UnfitIsoChildToAllAbund, UnfitIsoChildToAllMass) 	

	if (numpnts(UnfitIsoChildToAllNames)==0)
		DoAlert 0, "There are not any unfit child isotopes at this time."
	endif

End

Function HR_butt_changeUnfitIso(ctrlName) : ButtonControl
	String ctrlName

	wave/t UnfitIsoChildToAllNames = root:HR_Panel:UnfitIsoChildToAllNames
	HR_change2FitHRIon(UnfitIsoChildToAllNames, 1)

End


Function HR_butt_AddMissIsoChild(ctrlName) : ButtonControl
	String ctrlName

	wave/t MissIsoChildToAllNames = root:HR_Panel:MissIsoChildToAllNames		// text wave of HR isotopic ions that are not in the all list
	wave/t MissIsoChildToAllParent = root:HR_Panel:MissIsoChildToAllParent		// text wave of HR isotopic parents
	wave MissIsoChildToAllAbund = root:HR_Panel:MissIsoChildToAllAbund		// isotopic abundance
	wave MissIsoChildToAllMass = root:HR_Panel:MissIsoChildToAllMass		// isotopic abundance
	wave/t ExactMassText_All = root:HR:ExactMassText_All		// isotopic abundance

	variable idex, num, count=0
	string IsoStr
	
	num=numpnts(MissIsoChildToAllNames)
	
	for(idex=0;idex<num;idex+=1)
		IsoStr = MissIsoChildToAllNames[idex]
		FindValue/TEXT=IsoStr/TxOP=7 ExactMassText_All
		if (V_Value >=0)
			continue
		else
			HR_addThisMass(IsoStr, MissIsoChildToAllMass[idex], possibleIsotopAbund=MissIsoChildToAllAbund[idex],possibleIsotopParent=MissIsoChildToAllParent[idex],possibleChooseToFit =0)
			count+=1
		endif		
	endfor

	redimension/n=0 MissIsoChildToAllNames,MissIsoChildToAllParent, MissIsoChildToAllAbund, MissIsoChildToAllMass

	HR_butt_MakeExactMassWaves("")

	DoAlert 0, Num2str(num) + " many missing isotopic children were added to the 'all' waves."
End


Function HR_butt_UnfitIsoChange2Fit(ctrlName) : ButtonControl
	String ctrlName

	wave /t UnfitIsoChildToAllNames= root:HR_Panel:UnfitIsoChildToAllNames		// text wave of HR isotopic ions that are not in the all list
	wave /t UnfitIsoChildToAllParent=  root:HR_Panel:UnfitIsoChildToAllParent	// text wave of HR isotopic parents
	wave UnfitIsoChildToAllAbund= root:HR_Panel:UnfitsoChildToAllAbund		// isotopic abundance
	wave UnfitIsoChildToAllMass=  root:HR_Panel:UnfitIsoChildToAllMass	// isotopic abundance

	variable num = numpnts(UnfitIsoChildToAllNames)
	HR_change2FitHRIon(UnfitIsoChildToAllNames, 1)

	redimension/n=0 UnfitIsoChildToAllNames,UnfitIsoChildToAllParent, UnfitIsoChildToAllAbund, UnfitIsoChildToAllMass

	HR_butt_MakeExactMassWaves("")

	DoAlert 0, "You have added "+num2str(num) +" many isotopic children to fit"

End



Function HR_getIsoAbundXmany(IsoTextWave, IsoAbund, numX, ElementNameCombo, ElementAbundCombo)
wave/t IsoTextWave, ElementNameCombo
wave IsoAbund, ElementAbundCombo
variable numX

	variable numIso =numpnts(IsoTextWave)
	
	redimension/n=0 ElementNameCombo
	redimension/n=0 ElementAbundCombo
	
	variable idex0, idex1, idex2, idex3, idex4, idex5, idex6, idex7, idex8, idex9
	string str0, str1, str2, str3, str4, str5, str6, str7,  str8, str9
	variable abund0, abund1, abund2, abund3, abund4, abund5, abund6, abund7, abund8, abund9
	
	variable count = 0
	
	for (idex0=0;idex0<=numX;idex0+=1)
		//idex0 is the number of atoms of the first Iso in list
		str0= selectString(idex0>0, "", IsoTextWave[0])
		str0+= selectString(idex0>1, "", num2str(idex0)) 
		abund0=  IsoAbund[0]^idex0
		if (idex0>0)
			abund0*=binomial(numX,idex0) 
		endif		
		if (idex0== numX)
			insertPoints count, 1, ElementNameCombo, ElementAbundCombo
			ElementNameCombo[count] = str0
			ElementAbundCombo[count] = abund0
			count+=1
	//		print str0
			idex0+=1
		endif
	
		if (numIso<=1)
		continue
		else
		for (idex1=0;idex1<=numX-idex0;idex1+=1)
			//idex1 is the number of atoms of the second Iso in list
			str1= selectString(idex1>0, "", IsoTextWave[1] )
			str1+= selectString(idex1>1, "", num2str(idex1))
			abund1=  IsoAbund[1]^idex1
			if (idex1>0)
				abund1*=binomial(numX,idex0+idex1)
			endif
			if (idex0+idex1== numX)
				insertPoints count, 1, ElementNameCombo, ElementAbundCombo
				ElementNameCombo[count] = str0+str1
				ElementAbundCombo[count] = abund0*abund1
				count+=1
	//			print str0+str1
				idex1+=1
			endif
			
			if (numIso<=2)
			continue
			else
			for (idex2=0;idex2<=numX-(idex0+idex1);idex2+=1)
				//idex3 is the number of atoms of the third Iso in list
				str2= selectString(idex2>0, "", IsoTextWave[2] )
				str2+=selectString(idex2>1, "", num2str(idex2))
				abund2 =  IsoAbund[2]^idex2
				if (idex2>0)
					abund2*=binomial(numX,idex0+idex1+idex2) 
				endif
				if (idex0+idex1+idex2== numX)
					insertPoints count, 1, ElementNameCombo, ElementAbundCombo
					ElementNameCombo[count] = str0+str1+str2
					ElementAbundCombo[count] = abund0*abund1*abund2
					count+=1
	//				print  str0+str1+str2
					idex2+=1
				endif
				
				if (numIso<=3)
				continue
				else
				for (idex3=0;idex3<=numX-(idex0+idex1+idex2);idex3+=1)			// always the parent ion
					//idex4 is the number of atoms of the fourth Iso in list
					str3= selectString(idex3>0, "", IsoTextWave[3] )
					str3+=selectString(idex3>1, "", num2str(idex3))
					abund3=  IsoAbund[3]^idex3
					if (idex3>0)
						abund3*=binomial(numX,idex0+idex1+idex2+idex3)
					endif
					if (idex0+idex1+idex2+idex3 == numX)
						insertPoints count, 1, ElementNameCombo, ElementAbundCombo
						ElementNameCombo[count] = str0+str1+str2+str3
						ElementAbundCombo[count] = abund0*abund1*abund2*abund3
						count+=1
	//					print str0+str1+str2+str3
						idex3+=1
					endif
				
					if (numIso<=4)
					continue
					else
					for (idex4=0;idex4<=numX-(idex0+idex1+idex2+idex3);idex4+=1)			// always the parent ion
						str4= selectString(idex4>0, "", IsoTextWave[4] )
						str4+=selectString(idex4>1, "", num2str(idex4))
						abund4=  IsoAbund[4]^idex4
						if (idex4>0)
							abund4*=binomial(numX,idex0+idex1+idex2+idex3+idex4)
						endif
						if (idex0+idex1+idex2+idex3+idex4 == numX)
							insertPoints count, 1, ElementNameCombo, ElementAbundCombo
							ElementNameCombo[count] = str0+str1+str2+str3+str4
							ElementAbundCombo[count] = abund0*abund1*abund2*abund3*abund4
							count+=1
							idex4+=1
						endif
						
						if (numIso<=5)
						continue
						else
						for (idex5=0;idex5<=numX-(idex0+idex1+idex2+idex3+idex4);idex5+=1)			// always the parent ion
							str5= selectString(idex5>0, "", IsoTextWave[5] )
							str5+=selectString(idex5>1, "", num2str(idex5))
							abund5=  IsoAbund[5]^idex5
							if (idex5>0)	
								abund5*=binomial(numX,idex0+idex1+idex2+idex3+idex4+idex5)
							endif
							if (idex0+idex1+idex2+idex3+idex4+idex5 == numX)
								insertPoints count, 1, ElementNameCombo, ElementAbundCombo
								ElementNameCombo[count] = str0+str1+str2+str3+str4+str5
								ElementAbundCombo[count] = abund0*abund1*abund2*abund3*abund4*abund5
								count+=1
								idex5+=1
							endif
							
							if (numIso<=6)
							continue
							else
							for (idex6=0;idex6<=numX-(idex0+idex1+idex2+idex3+idex4+idex5);idex6+=1)			// always the parent ion
								str6= selectString(idex6>0, "", IsoTextWave[6] )
								str6+=selectString(idex6>1, "", num2str(idex6))
								abund6=  IsoAbund[6]^idex6
								if (idex6>0)		
									abund6*=binomial(numX,idex0+idex1+idex2+idex3+idex4+idex5+idex6)
								endif
								if (idex0+idex1+idex2+idex3+idex4+idex5+idex6  == numX)
									insertPoints count, 1, ElementNameCombo, ElementAbundCombo
									ElementNameCombo[count] = str0+str1+str2+str3+str4+str5+str6
									ElementAbundCombo[count] = abund0*abund1*abund2*abund3*abund4*abund5*abund6
									count+=1
									idex6+=1
								endif
								
								if (numIso<=7)
								continue
								else
								for (idex7=0;idex7<=numX-(idex0+idex1+idex2+idex3+idex4+idex5+idex6);idex7+=1)			// always the parent ion
									str7= selectString(idex7>0, "", IsoTextWave[7] )
									str7+=selectString(idex7>1, "", num2str(idex7))
									abund7=  IsoAbund[7]^idex7
									if (idex7>0)	
										abund7*=binomial(numX,idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7)
									endif
									if (idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7  == numX)
										insertPoints count, 1, ElementNameCombo, ElementAbundCombo
										ElementNameCombo[count] = str0+str1+str2+str3+str4+str5+str6+str7
										ElementAbundCombo[count] = abund0*abund1*abund2*abund3*abund4*abund5*abund6*abund7
										count+=1
										idex7+=1
									endif

									if (numIso<=8)
									continue
									else
									for (idex8=0;idex8<=numX-(idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7);idex8+=1)			// always the parent ion
										str8= selectString(idex8>0, "", IsoTextWave[8] )
										str8+=selectString(idex8>1, "", num2str(idex8))
										abund8=  IsoAbund[8]^idex8
										if (idex8>0)
											abund8*=binomial(numX,idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7+idex8)
										endif
										if (idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7+idex8  == numX)
											insertPoints count, 1, ElementNameCombo, ElementAbundCombo
											ElementNameCombo[count] = str0+str1+str2+str3+str4+str5+str6+str7+str8
											ElementAbundCombo[count] = abund0*abund1*abund2*abund3*abund4*abund5*abund6*abund7*abund8
											count+=1
											idex8+=1
										endif

										if (numIso<=9)
										continue
										else
										for (idex9=0;idex9<=numX-(idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7+idex8);idex9+=1)			// always the parent ion
											//idex4 is the number of atoms of the fourth Iso in list
											str9= selectString(idex9>0, "", IsoTextWave[9] )
											str9+=selectString(idex9>1, "", num2str(idex9))
											abund9=  IsoAbund[9]^idex9
											if (idex9>0)		// typo  1.20S
												abund9*=binomial(numX,idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7+idex8+idex9)
											endif
											if (idex0+idex1+idex2+idex3+idex4+idex5+idex6+idex7+idex8+idex9  == numX)
												insertPoints count, 1, ElementNameCombo, ElementAbundCombo
												ElementNameCombo[count] = str0+str1+str2+str3+str4+str5+str6+str7+str8+str9
												ElementAbundCombo[count] = abund0*abund1*abund2*abund3*abund4*abund5*abund6*abund7*abund8*abund9
												count+=1
							//					print str0+str1+str2+str3
												idex9+=1
											endif
										endfor	// this idex9	
										endif 	// this idex9


									endfor	// this idex8	
									endif 	// this idex8

								endfor	// this idex7	
								endif 	// this idex7
						
							endfor	// this idex6	
							endif 	// this idex6
							
						endfor	// this idex5	
						endif 	// this idex5
					
					endfor	// this idex4	
					endif 	// this idex4	
				
				endfor	// this idex3	
				endif //   this idex3	
			
			endfor //  this idex2	
			endif  //  this idex2	
		
		endfor	// this idex1	
		endif  //  this idex1	
	endfor	// this idex0	

End


// 1.15E
Function PK_saveReinstateHROrgfrag(copyOrReinstate)
variable copyOrReinstate
	
	wave/t HR_frag_organic = root:HR_frag:HR_frag_organic
	if (copyOrReinstate) // make a copy and then fill in with values from original Aiken at al paper
		duplicate/o/t root:HR_frag:HR_frag_organic,  root:HR_frag:HR_frag_organic_ORIG

		wave/t HR_specMassAlgebra= root:HR_frag:HR_specMassAlgebra
		variable CO_pos, H2O_pos, HO_pos, O_pos	
		FindValue/TEXT="CO"/TXOP=4 HR_specMassAlgebra
		CO_pos	=V_Value
		FindValue/TEXT="H2O"/TXOP=4 HR_specMassAlgebra
		H2O_pos =V_Value
		FindValue/TEXT="OH"/TXOP=4 HR_specMassAlgebra
		if (V_Value<0)
			FindValue/TEXT="HO"/TXOP=4 HR_specMassAlgebra
		endif
		HO_pos	=V_Value
		FindValue/TEXT="O"/TXOP=4 HR_specMassAlgebra
		O_pos	=V_Value
		
		HR_frag_organic[O_pos]= "0.04*HR_frag_organic[{H2O}]"
		HR_frag_organic[HO_pos]= "0.25*HR_frag_organic[{H2O}]"
		HR_frag_organic[H2O_pos]= "0.225*HR_frag_organic[{CO2}]"
		HR_frag_organic[CO_pos]= "HR_frag_organic[{CO2}]"
	else
		duplicate/o root:HR_frag:HR_frag_organic_ORIG, root:HR_frag:HR_frag_organic
	endif

End


//1.56F
//Function EA_smoothSetVarProc(ctrlName,varNum,varStr,varName) : SetVariableControl
//	String ctrlName
//	Variable varNum
//	String varStr
//	String varName
//
//	nvar smoothingVal = root:elementalAnalysis:smoothingVal
//	wave HROrg = root:elementalAnalysis:HROrg
//	wave HROrgCHO = root:elementalAnalysis:HROrgCHO
//	wave HROrgCO2 = root:elementalAnalysis:HROrgCO2
//	wave FHROrgCHO = root:elementalAnalysis:FHROrgCHO
//	wave FHROrgCO2 = root:elementalAnalysis:FHROrgCO2
//	wave/d t_series = root:index:t_series
//
//	smoothingVal = max(smoothingVal, 1)
//
//	// binomial smoothing
//	Duplicate/O HROrgCHO, root:elementalAnalysis:HROrgCHO_modified
//	wave HROrgCHO_modified = root:elementalAnalysis:HROrgCHO_modified
//
//	Duplicate/O HROrgCO2, root:elementalAnalysis:HROrgCO2_modified
//	wave HROrgCO2_modified = root:elementalAnalysis:HROrgCO2_modified
//
// 	if (smoothingVal>1)
// 		sq_smoothNans_acrossTime(HROrgCHO_modified, t_series, smoothingVal)	
// 		sq_smoothNans_acrossTime(HROrgCO2_modified, t_series, smoothingVal)
// 	endif
// 	
//	FHROrgCHO = HROrgCHO_modified/HROrg
//	FHROrgCHO = limit(FHROrgCHO[p],0, 1)
//
//	FHROrgCO2 = HROrgCO2_modified/HROrg
//	FHROrgCO2 = limit(FHROrgCO2[p],0, 1)
//
////	FHROrgCHO = HROrg[p] < 0 ? nan : FHROrgCHO[p]
////	FHROrgCO2 = HROrg[p] < 0 ? nan : FHROrgCO2[p]
//End


//1.56F
Function sq_CheckBatchAandB(strA, strB, strType)
string strA, strB, strType

	wave/t spec_list = root:frag:spec_list
	variable val_A, val_B, posA, posB
	FindValue/Text = strA/TXOP = 4 spec_list
	posA = V_value
	FindValue/Text = strB/TXOP = 4 spec_list
	posB = V_value
	if (posA<0 || posB < 0)
		abort "Could not find the species in the root:frag:spec_list wave.  Aborting from sq_CheckBatchAandB"
	endif

	string s_A, s_B
	strswitch (strType)
		case "RIE" :
			wave IEfac_list = root:frag:IEfac_List
			val_A = IEfac_list[posA]
			val_B = IEfac_list[posB]
			if (val_A != val_B)
				sq_butt_reviewBatch("")
				abort "The "+strType+" for "+strA+" and "+strB+" do not match.  Aborting from sq_CheckBatchAandB"
			endif
		break
		case "CE":
			wave CEfac_List = root:frag:CEfac_List
			val_A = CEfac_List[posA]
			val_B = CEfac_List[posB]
			if (val_A != val_B)
				sq_butt_reviewBatch("")
				abort "The "+strType+" for "+strA+" and "+strB+" do not match.  Aborting from sq_CheckBatchAandB"
			endif
		break
		case "CEWave_list" :
			wave/t CEWave_list = root:frag:CEWave_list
			s_A = CEWave_list[posA]
			s_B = CEWave_list[posB]
			if (!stringmatch(s_A, s_B))
				sq_butt_reviewBatch("")
				abort "The "+strType+" for "+strA+" and "+strB+" do not match.  Aborting from sq_CheckBatchAandB"
			endif
		break
		case "speccorr_List" :
			wave/t speccorr_List = root:frag:speccorr_List
			s_A = speccorr_List[posA]
			s_B = speccorr_List[posB]
			if (!stringmatch(s_A, s_B))
				sq_butt_reviewBatch("")
				abort "The "+strType+" for "+strA+" and "+strB+" do not match.  Aborting from sq_CheckBatchAandB"
			endif
		break
	endswitch
	
End

//1.56F
//Function EA_butt_fracTimeSeries(ctrlName) : ButtonControl
//	String ctrlName
//
//	string waveNameSuffixStr, dataSetInitialStr
//	waveNameSuffixStr = GetUserData(WinName(0,1), "", "waveNameSuffix" )
//	dataSetInitialStr = GetUserData(WinName(0,1) , "", "dataSetInitial" )
//	HR_EACalibrateRatios(waveNameSuffixStr, dataSetInitial=dataSetInitialStr )
//
//End



Function HROrgCheck_Plot_EA_AAvsIA(HC,OC,OM_OC, waveNameSuffix, Correlstr )
wave HC,OC,OM_OC
string waveNameSuffix, Correlstr

	nvar HRResults_graphOutputType = root:HR_panel:HRResults_graphOutputType		// 1 - new, 2 = multiple, 3 = append, 4 = update
	if (HRResults_graphOutputType==1)  // single graph

		string str = waveNameSuffix[0,strlen(waveNameSuffix)-4]
		wave/z HC_AA = $"root:elementalAnalysis:Ratio_H_C_"+str
		wave/z OC_AA = $"root:elementalAnalysis:Ratio_O_C_"+str
		wave/z OM_OC_AA = $"root:elementalAnalysis:Ratio_OM_OC_"+str
		Display /W=(80.25,80,549.75,400.25)  as "IA_vs_AA"+str
		Appendtograph OM_OC vs OM_OC_AA
		AppendToGraph HC vs HC_AA
		AppendToGraph OC vs OC_AA
		ModifyGraph mode($NameofWave(OM_OC))=3,mode($NameofWave(HC))=2,mode($NameofWave(OC))=2
		ModifyGraph marker($NameofWave(OM_OC))=8
		ModifyGraph lSize=2
		ModifyGraph rgb($NameofWave(OM_OC))=(0,0,0),rgb($NameofWave(HC))=(65280,0,0)
		ModifyGraph rgb($NameofWave(OC))=(39168,13056,0)
		ModifyGraph grid=2
		ModifyGraph gridRGB=(34816,34816,34816)
		Label left "Improved-Ambient H/C, O/C, OM/OC"
		Label bottom "Aiken-Ambient H/C, O/COM/OC"		//1.16M
		SetAxis/A/E=3 left
		SetAxis/A/E=3 bottom
		Legend/C/N=text1/J/A=RB "\\s("+NameofWave(OM_OC)+") "+NameofWave(OM_OC)
		AppendText "\\s("+NameofWave(HC)+") "+NameofWave(HC)
		AppendText "\\s("+NameofWave(OC)+") "+NameofWave(OC)
		AppendText "\\f01\\K(52224,52224,0)1.27 line\r\\K(65280,32768,58880)1.11 line"
		ShowTools/A
		variable mymin, mymax
		mymin = min( min(Wavemin(OM_OC), WaveMin(OC)), WaveMin(HC) )
		mymin = max(0, mymin)
		mymax = max(max(Wavemax(OM_OC), Wavemax(OC)), Wavemax(HC) )
		SetDrawEnv xcoord= bottom,ycoord= left
		DrawLine mymin, mymin, mymax ,mymax
		SetDrawEnv xcoord= bottom,ycoord= left, linethick= 1.5,linefgc= (65280,32768,58880)
		DrawLine mymin,mymin,mymax/1.1, mymax
		SetDrawEnv xcoord= bottom,ycoord= left, linethick= 1.5,linefgc= (52224,52224,0)
		DrawLine mymin,mymin,mymax/1.27,mymax
	endif
End

