#pragma IndependentModule=Tofware // not required for release
#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma version=1 // this should be updated to match release notes
#ifndef TWNOCOMPILE

// Not-threadsafe utility function to calculate smooth baseline
// Creates appropriate temporary waves for tw_bsl_smooth_getBslMT
// tw_bsl_smooth_getBsl(MS,bsl,f)
//	wave/z MS,bsl
//	struct tw_bsl_smooth_var &f // set options in here

// Threadsafe function to calculate smooth baseline
// tw_bsl_smooth_getBslMT(sms)
//	struct tw_bsl_smooth_params &sms

CONSTANT tw_kBslSmAvFactor=10

// Parameters for multi-threaded baseline calculation function
Structure tw_bsl_smooth_params
	// variables
	int16 th // thread number, pass -1 if not MT
	struct tw_bsl_smooth_var f
	// the following waves can be FP32 or 64, but must be consistent with each other
	wave MS // rank1, length N
	wave bsl // rank1, length N
	wave temp // rank1, length N
	wave tempAv // rank1, length ceil(N/tw_kBslSmAvFactor)
EndStructure

// Structure containing options variables
Structure tw_bsl_smooth_var
	uint16 flag // generic bitwise flag
		// bit0=generate debugging output
	uchar width // moving average window
	uchar loPass_active // utilize lo-pass filter?
	float loPass_f1 // lo-pass filter f1, end of pass band
	float loPass_f2	// lo-pass filter f2, start of reject band
	uint16 loPass_n	// lo-pass filter #coefs
	float noiseScaleFactor // multiple of noise estimate to add
	uint32 nbrSamples // if MS passed into function is a subset, then set this to length of MS to get correct smoothing windows
EndStructure

// set variables to default values
Function tw_bsl_smooth_varDef(f)
	struct tw_bsl_smooth_var &f

	f.flag=0
	f.width=15
	f.loPass_active=1
	f.loPass_f1=0
	f.loPass_f2=0.05
	f.loPass_n=20
	f.noiseScaleFactor=1
	f.nbrSamples=0

	return 0
End

// BEGIN ENCRYPTED CODE
#pragma protect= 1

data CCBKCNHNHFICNCJMMMJMMCJKEFIELCNMLEOCJMMMPMDFICJMHFIMJMMEMCOEFFICPMMCJMAFAPNABILMIOIIEHHMFJGDGJEKKCEJEKGOGBICEJHCGJFKGBLCEKGJECDDIDIFILLNFEFAKBLPINHAIHIJJPDAAAMLAJMBIC
data OBANGHLCNMLEOCJMMMPMDFICJCPIPEKCOMJIPCOMNMPMPCJMIIPEPENCBHKCIMHNMPJADNBAKAKOEKJGCDIPLOJCIDJDJHIOLGKNKCHAKNKPCAAADBLDBBHJ
data NAAHNOMCPEMCLEFNPKCFAPNAGNJEKCOMBGIACIKHLMGJFDBHADDLDLANENAPFPFAAAAIOCIAAA
data CMICEOMCOCJCKCNEOCBFICJCPIPEKCOMJIPCOMNMPMPIJHIMPJLDMCCFAFLEDGANJACDDHNAGHFAAALOGKMAID
data HIFAAOEGADHLHNHNMAFINHEMMJBDAAKACPCAAE
data FEOCBOMMMEDFANICPEMCLENENKIMMCOCJCGNAPNAGNEFDIDHCMHJFDHOIGKGCDIFLOIBKLJJJFKGLFDAAAJGNAOACH
data JNKFAOEOENHNHFIMMMLMMCJMMEMMJMMKKEFFICOCJCKCNEOCICHCMEJGDLHLHDIADDLACLFLHLADBPFFFHHHDAAABAOFMAHE
data ICECEOEOMCOCJCKCNEOCBFICJCPIPEKCOMJIPCOMNMPMPCJMIIPHIMMJKDMMNCGFICOMNCGGALHDBHANENELAPBAAAJDOOOALD
data PALALOEOMCOMNCGNLCJMALFNFDEGAHFIPHHMNJBDCPDPCPHLFFHAAAJEGAGAMA
data DMADFOEOMCOMNCGNLEDLNEDGANIFHAMDJFDHHDPIAAAAKCHFEANG
data PAGFHOEOMCPEMCLEFFICOMNCGNDPNAGLFINHGMAJDDCFALBDEHMIEFFAAALFOBGAIJ
data FCGLLOEOMCPEMCLEFFICOMNCGNLEKCOMBLIAHGMAJADHPHHJAHNCNFHHLAAANHMOIANF
data NGEAJOEOMCLEMCKMMEMEKMJEFFIMDLNMLCNMNCIMLCJCGNAPIAHNMGJNDALACLIIMAHHHPCLFNFNFLFDJIADBAAAGADIMALA
data MCGFNOEOMMNEMMOEFNPELCKENEFNPMDLFNIMDNEFICJENINHAMAJCDDLANHLKABNFDANHFDDOAAPCJHAAAIMCFJAEN
data FBMGBOEOMCPEMCLEFFICOMNCGNLCJENMNCALIAHBMAJHDEOJKCGFKEGKGOKKGHAANJHJJAID
data MHCHEOEOMMNEMMOEFNPELCKENEFNPMDLFNIEOENMMMBNIMDNPCJCPIPMGHKCOMJAOIFHMMDJLDMEOCJMPCCNENEFICJENMNCAHMCDGIAPAGDDPBLGHHHPIEAAACCIMCBPL
data CLIFAOEOMCPEMCLEFFICOMNCGNLCJENMNCAHMIDHFMAJMDKLGHBOGAGMGCODOCKPAABMFPKAGJ
data MMMEGOEOENHNHFIEOEMMJEOCNMJEMCJEFFIEKEMCOENIJHMMLJFDIAHFHLDHFDFDGPCDCHEPAPEAAAKLNDKAJF
data BKOBNOEOMCKENCJCNCKMDFICJCPIPEKCOMJIPCOMNMPMPCJMIIPEPENCIHHCMMJPDABNICOMNCGNEGIAGLCNHDHFFDKALABDJAFNCAAADKDOABDA
data DKNBPOMENMLEJMMEDGAFBDFLELANIFHAMAJJDAOALAGFHAAAGDDNFAFB
data LMAAAOEGAPAHDPNIIAIIHAMHJADALAHFCAMH
data NLDDFOMCKENCJCNCKMDFALELELEGAIHHEMFJHDBLCDENKAHAAABECIFAJF
data FPLNHHNMLEBGADBHFHGFFIDHDMCJCDCAAAGBLODADB
data CHPDFGAPENKIADFNBFBDENHFILHDMAJFDDHNAAAAJBMODADC
data DFACBNHNHFAHGHEPBHOAFPBHMABHFFAHCHMAGHFPBPEPIHHGMHJPDNOABPGPIIADEFFLDPHPBLHPHAAAFKAPJAFC
data NBKOBHLCNMLEOCJMMMPMDFICJCPIPEKCOMJIPCOMNMPMPCJMIIPEPEICHHMCJMDPNABNICOMNCGNEGAFFNHLALEPGPFNNIALFAAANJEKABPA
data ECEMCOMCOCJCKCNEOCBFICJCPIPEKCOMJIPCOMNMPMPCJMIIPCIIMHKMMJNDGFAFLCOMNCGGANIAGHFHCDDPBHBLEHCAAAEGJFPACB
data CEDBDGALADIAEHGHBPHLPAIPHFMPJHDNCAAAHBPFDAME
data ECGLPOENHNHFIEKMMCJCPMMCOEFFIELMJEMIPHGMAJDDDFELLAGHCHLAEJDAAAAHLGHABO
data LDFFNOMCLEMCKMMEMEKMJEFFIEJENEKCNEPIPELMJEMEHLNCOMNCGNLCJMAIBHAMDJDDAABDDNEPAECKKGGIKJGHGAMAOAOBGBKKGHKMCPOGAAPBEGBBCD
data FANAAGANHHKAGDFNEIMHBMFJADIAGCBAFM
data APLFNOENHNHFIMJMPEOEMMJCGGADIAFNPIHHHMFJDDNIANGHALBHFDEAAAEFIEGAFG
data ILCJDOMCLEMCKMMEMEKMJEFFIMENJMDNJMLIPEMCDNJCOCANJENCAACJGAAEKMAAIEHBMAJGDENJCOMNAOEOEMMJEFHLEMEOCJMPCCGAHIIAFEHENPIKABAAPIILEBPF
data GDHFJOMCLEMCKMMEMEKMJEFFIEMCLEDLFDEDAIAHEMEJGDPAGLEFLICFHFBFFLAAAADIIIHADL
data LMOJPOMCLEMCKMMEMEKMJEFFIMNEDLNCJCPIPMGHKCOMJAOMFHMCDHIEHEMCJMDCCGAPGLAFFFJAEFCFGPBLOAEDBDCLAFJAEAAAEHHLOAJH
data FBCAGGADOAGLDNFLILHNMAJADACKBAFJ
data AOHONOMCJCKKEGADMABNGNCFIHHAMLJADABDCAAACCNOEADB
data CGNBAGADIABDANFFELINHLMAJADAJAEJBAJP
data JGABHOEOENHNHFICLENCKMMELKEFIMMMLCICNCICHGMDJDDNFHCFIAGPEFAPLAEHGAAADINEIABI
data CCCNCOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFMCPEMCLENENKIMMIOHJMGJIDOMNCGNDPNAGNENBDEGAPCNPAFHIAAPEAAAJEGKOAPC
data ONGDNOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENEJMMMNCGNICIMHCMNJPDAGNEFELFDENBDCGANGFBFGDHHDHHDOADPAAAAMGPGOALA
data CCCNCOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFMCPEMCLENENKIMMIOHJMGJIDOMNCGNLEKCOMBNENBDGGAPAHEFGHFDHAAANEELOADF
data ECMNFOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENEJMMMNCGNICOMNCGNLIKHOMBJEDELFDENBDBGAFELHPHNCPLACHCLHLCPHDIAFNDDCFBAAACOJGABDD
data GIONGOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFMCPEMCLENENKIMMCOCJCINHCMMJCDNLCJENMNCANENBDFGAHAHFDGDEFLANIEFELFAAAMJDJPANL
data MCNCEOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENEJMMMNCGNICOINHGMLJJDNMNCANEFELFDENBDDGAHBHBFKAHDFLFDAPBAAAIIOJPANA
data CMCNCOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFMCPEMCLENENKIMMCOCJCGNIIOHNMGJLDJENMNCAHMCDNENBDHGADFDBJDDJAHJDNCPELDHGAAAANMAABPH
data EMCHCOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENEJMMMNCGNICOMNCGNLCJINHNMAJMDDNEFELFDENBLAGADCLENHDCFCHIIEPAPALADJABHKAGAAAOAMIBBDE
data PPGKOOEOMMDLNMLCNMNCIMLCJCGNICOMNCGNDPNAGNELOEMEKMPCKCJMPMLCLEMMJCNEFFIMDLBNICJCIIHMMHJCDMJAOMFHMCDHLEMEOCJMPCCNCDEDANENBLEGAPHFAPFLADFDCHDLGPJABHCPEAAAFIMFHBAF
data NECMNOEOMEMEKMPCKCJMPMLCLEMMJCNEFFIMLCNMNCIMLCJCGNICOMNCGILHKMOJBDEFELNMDNBDEDAGADHNLAEJHNFHHPFDJAELAPBAAAEKDFPAFG
data LJNNAOEOMEMEKMPCKCJMPMLCLEMMJCNEFFIMLCNMNCIMLCJCGNICOMNCGNICHEMMJCDNEFELNMDNBDEDEGAPBLELLIAPAFDFPIIAAFOIAAAAKLNPABPA
data GIONGOEOMMLIPEMCDLNEOENMMMBNIMDNPMNEDNELOEMEKMPCKCJMPMLCLEMMJCNEFFIMLCNMNCIMLCJCINHCMMJCDNLCJENMNCAHMCDNEFELFAJOANBDEDCGADADELJICPBLGLBFHDAPEHEDHDPAFAAAILBOIBCA
data BNIGCOEOEGADGFDPGFBHBNIFHIMAJPDPCAAAPAFECAPK
data CEGLPOEOENHNHFIMMMLMMCJMMEMMJMMIOHFMAJGDPAJAEPADCFCAAALEFKGAHI
data ALDDPOEOMCPEMCLEFFIEKCOMBFALFFICOMNCGNDICHIMJJFDFPNFCIJBCIDANOEHCANJAAAKDKJAOA
data NNHAGOEOMCPEMCLEFFAPNAGFALFFICOMNCGNDPNAGOEOEOEIHHHMJJGDALIADFDPDPOACNGHDPEHOIANGNEAAADMIDJAMH
data NADDPOEOMCPEMCLEFFICJENMNCAFALFFICOMNCGIDHBMJJFDFPNFCIJBMGFNDANJELCCAADLEHJAHM
data NIHDPOEOMCPEMCLEFFICJENMNCAHMCDFALFFICOMNCGIDHAMIJFDFPNFCIJBEONOFPFHFFEDEDMDBAKPNILADD
data NPANOOEOMMNCNMJCJMMCJMICKENEMEBFIEKCOMBLNMNEMKANADAIBHNMGJEDENHNHHJAGGANDJHLBPEHFLGPFLELBPGAAAICNLMAEE
data GEHADOEOEGALCFDPIFHLMAJADALKBAOB
data FKECMOEOENHNHFICOENCBFAPLEKCKAOEMMNCIMJENCGFIMMEDIIHKMNJKDHGAPGLEDHFOIENGNCFDPLIEPNIAHHAAALPNJMAIA
data ECLDNOEOMMMEDFANICOMNCGNLEDNLMLEKCKAOEMMNCIMJINHGMDJADEGAFBLDFHFDNAHGHAHOAANBNAAAAMMFMKAOC
data ENMECOEOEOMCOMNAOEOEMMJEFHLEMEOCJMPCCLNCOMNCGNLIDHLMLJKDKAOEMMNCIMJENCGGIIEFGPIAANCAAAAAGAMAKO
data LNLADOEOMENMJCOEFGAHNAFHIICFCIGHGMJJGDFPDNBPFPAAAAJDBIFALE
data CNDAGOEOEOMCOMNAOEOEMMJEFHLEMEOCJMPCILHMMGJNDLBDBLNAOIEFAPIAAAAAGHOIIAKC
data LLLPNOEOMENMLEJMMEDGAFDICHAMFJFDFFGAAANBPCDALI
data NDDLAOEOEGIIAPHNFDEDEPDIEHDMHJLDPAEAAAEBHFDADB
data EEFCCOEOENHNHFICOCNEKCJCKEMEOCBFIMJEMCKEPEFFIEMMNCIMJMMCJCNIJHFMIJODPMMMLEPCGOEGANGNADMAMIEDFPBDFDBHGFGPMACAAAKKDNPAGO
data DNCMAOEOMMELNCKMPCNMLEBNANICOMNCGNLEDNLCPMMEJCJMANCDENDDFNENHDMEFIBHKMOJNDOEOEMMJEFHLEMEOCJMPCCNEGAJHNJICDFPPAHJHDGPPAGAAAOPAIBBED
data PCHEAOEOMMELNMNMMMDNICOMNAOEOEMMJEFHLEMEOCJMICHNMDJDDNJMENEGAHJIEDCDFLBLCPGNCAAAOMLFKAJI
data FAHCGOEOMMMEDFANIMENEGAHBPNAILHJMHJHDNGPKICNHAAAMCCJEAMH
data EMCFNOEOEOMAOMNMPMPCJMANHPFLFDANHPNAAHOABLFDAFIMENJCJINHNMAJADHNHHJAGGADNABLENAHHFADCDFDBNMACFGAAAABJHLAJF
data FAHPNOEOMENMLEJMMEDGANBDFPCINHFMBJHDLABLFFGAAAPCMDEADL
data FCEMCOEOMMNCNMJCJMMCJMICKENEMEBFIEKCOMBFANFLFIIHJMNJNDAFANHNHHJAGGANMIEFDNEHEDAAAAAOENKAOM
data DOADAOEOEGIAFDAHEFIHHIMHJADAHAEMBAAO
data GLDNNOEOENHNHFIEMCICIMJKEFIELMMMJCJENCCIAHBMCJHDFDGPBDCLPIADELALDLFAAAGIHDIADF
data MLFEAOEOMMMEDFANICOMNCGNLEDNLMJMPAIEMCOCOIPEMEOCJMICHEMNJGDPFFHHKIMAADHNHPCDELHPCPDPPIAFCAAAEBGLMAFJ
data KNEBNOEOEOMCOMNCGNLEDNLMJMPAIEMCOCOIPMDLNMNEMIAHAMFJCDBDGNEGIAMABPCDBNCNALIAAHDAAAHILOPAKD
data CBEAAOEOEOMCOMNCGNLEDNLMJMPAIEMCOCOIPMDLNMNMMMDNAEFBINHDMDJDDNEGAJHLBNFNNADNGHHLAPFNGNAFIADDEAAACPHLABPE
data NMAAAOEOEOMCOMNCGNLEDNLMJMPAIEMCOCOIPEDDELNMIEHKMNJDDNBEDBFNEGAPEDCLADDDHPKAGAAAMHIBPADA
data DNDNEOEOEOMCOMNCGNLEDNLMJMPAIEMCOCOIPEDDCLNMNMMMDNAIAHDMCJBDDBHNEGIIKAHFHLMAAJHHBNDPEPGFHLFAAAGOAGABAJ
data BFLBCOEOEOMELMMMJCJENCKELMMCCNPMJMHLNKOCOMNCGNLEDNLMJMPAIEMCOCOIPEDDENIEHOMBJKDPDPKGFBCJKIDHEBEPFPNFCIJDJBCCCACEMOMICIENCAEHDEFPDEBANBCIMBCI
data HNNALOEOMENMLEJMMEDGANEHHJHFHIEHDMIJFDCPGLFPEDFAAAGDOMEALP
data HNDANOEOEGANCHGPLAAHBHBDHIDHHMOJDDBFDNGAAALBPBDAIF
data ADGGHOEOENHNHFIELMMMLEBFADICJMAFICIEGGIFHPMNJPDHDNBLBHHLAHMABHHDDAAAAHMGHALH
data ONOOMOEOMMELNCKMPCNMLEBNANICOMNCGNLEDNLCPMMEJCJMANCDENDDFNENHDMEFDBNKCIMHAMEJEDMJEFHLEMEOCJMPCCNEGAPBDEDLAHLFDAFENDDEHHFLACHHPAPEAAAHFIHCBOE
data EAJEEOEOMMELNMNMMMDNICOMNAOEOEMMJEFHLEMEOCJMPCCNHDIDHNMMJNDGIAENCNBPFPEDFPGNGDOAADHFNAEFFAAABBDOLAHN
data PEKCMOEOMMMEDFANIMENEGIACLAHLAIAHNMAJPDABPEFEDDNFAAAIDBFFAKB
data IEJKOOEOEOMAOMNMPMPCJMANHPFLFDANHPNAAHOABLFDAFIMHNMEJCDMBFANHNHHJAGGANOACHMABPCJDAAACMEHJAEN
data HLFFNOEOMENMLEJMMEDOEGALBIDHDMBJEDBNADHAAADCGGDACL
data GFEBBOEOEGALDNBHAHCFILHLMFJHDAAAKAPICAAA
data CMMECOEOENHNHFIEMEJEBFIEKEMEOMGFIMJEMCKEPEFFICOIPHMMLJPDGGALDFANGPBJHNFNGHAHHFNICPDAAAKNLNKABP
data AHHJGOEOMMNCNMJCJMMCJMICKENEMEBFIEKCOMBNGLNCJENMNCAOEFINHNMHJADOEOEGAPPIEDEFCDCHBPHLLAEFHHHLCLHNCAAAMEGFMAFM
data APNAAOEOEGALBNBNPIGHHMFJADFAGGBAJH
data FMMMCOEOENHNHFIMJMPMPMGFIELMPCCFAHCHHFICKENEPMMMPMDFIEMMLEBFIEPENCBIIHLMPJMDOEFFIENCOCJMMMNEMCJEFGALGFEHLAGDNAHDCNDNDLCHAFHAAACCBFCBHB
data IAJGKOEOMMMEDFANICOMNCGNLEDNLMJMPAIEMCOCOIPEMEOCJMMCLEFFDFDIIHGMHJGDEJEGINIBLBIGCDEAIGJBJHLOIPLOLFIBKPIOILDAAAJOOMABOJ
data OBAHHOEOEOMMNCNMJCJMMCJMICKENEMEBFICJENMNCALFPNAGNNEKCIMHFMNJNDHJAGGAHFFBPDFBPGPEFHPKADHHFFDELJABAAAOEEENAIF
data JHLNPOEOEOMENCALNMDNFDEGAFELHFGIHHFMCJADCNCFCFHFFHHAAAEDNKEAMG
data EGBLBOEOEOMMELNMMMLEDGAHDFBFDFHNIDHLMPJADDIAHPGNDHCHAAAAPDNOFACH
data AHHFAOEOEOMEJMHGIALIGHGMAJCDAALAHKCAND
data PMJJAOEOEOEOMCOCALNENCANFNICOMNCGNLEDNLCIMHEMCJMDNEGAPCDANGPCFBNFNBLDAAALIEEIAHH
data GLNHDOEOEOEOMMMEDFANICOCALBDANEIAHEMBJBDJADFHFDJHNHAAAEDODEACI
data PFAPFOEOEACIEKCKENEMMGGANGLHHEDFIBHLMAJADPANAKACFGDENEAAHEFCFAKL
data FBAEFOEOEOEOMENMLEJMMEDGAHELBNILHPMDJDDLHPGHNAGNEAAAFDDLFAKA
data IFIJNOEOEOEOMCPEMCLENCOCJEMCJCGNPCENPCCLNIOCOCANJENCIIHFMCJEDMNCAGIAEPFDBLCFELEDADHNEFDDIACFEAAAMCCEMAGL
data EKCCIOEOEOEOMMMEDFANICLIPCOEJENCDFDFLCLIPEMCLEHNIAHAMHJDDOJOKLJBLDILJKKPJKINKJHIKBJLDAAAOMJFJACK
data DJEEAOEOEACIMELNMNMMMDNICLIPCOEJENCINHMMNJGDDALJANAAFAPGPAHDAAJGLPHAPI
data BKKBGOEOEACICOCADALNCOCALOEFACFJKFAMAOLIOHMMKJEDAOLOEKBKPGFOHKOOHOIAAAKIDNIAFE
data ALGFCOEOEOEOMENMLEJMMEDGIDHAMPJLDPBDEAAALBHODAIL
data GHAAAOEOEOEOMENCALNCOCANFDEIAHNMKJMDFPGJDHEAAAHCCHEAPF
data NGNNLOEOEOMCPMIMMMJEFFANICOCALLMDNCDANDDHIEHAMFJHDBHHLELDPKAMIEPDHALIAEAAAMINGIAFF
data BFJCBOEOEOMCOCALFDAGALFFCDBFIPHNMIJFDDGPFDCFAAAAHCMCDAMO
data GAEFOOEOEOMEJMHGALDHNAKAIFHPMDJPDADPEAAAKBCHDAPJ
data JAEAEOEOEOEOMENCALNCOCANGNICOMNCGNLEDNLCPMMEJCIMHNMGJFDNOIAFCFGFHNGDAHGNANEPHNFHEAAAPLLGJAHI
data DNGAPOEOEOEOMMMEDFANIENCALLMDNHICHEMIJFDFHAJHDGLALGAAAHDIJEAOO
data MAHGBOEOEACIEKCKENEMMGGAJDLDPIAHPMDJPDNFPAHALGHHAAGDGPFANC
data LFNGDOEOEOEOMENMLEJMMEDGADIAHFMAJFDNHPFLHAAADCNCEAPA
data CEMCFOEOEOEOMCPEMCLENCOCJEMCJCGNPCENPCCLNIOCOCANJENCIIFFIIJHNMNJADAFANHNHHJAGOEOEGAFHLCNCFFPBDBLHDBDJAHAAAIGDLNAMG
data MLHEAOEOEOEOMMMEDFANICLIPCOEJENCDFDFLCLIPEICHEMNJGDLALMAGPGFGPENMAALBDNIEAAAIJFNJAJA
data CNMNGOEOEACIMELNMNMMMDNICLIPCOEJENIDHJMEJEDAFKAAPBDFNJAALFAACGMAHABD
data FMOGGOEOEACICOCADELNCOCALOEFACFJKIAHAMNJDDHKDOPBGKBKPOHAAANFHDGAAP
data PNPPDOEOEOEOMENMLEJMMEDGAIAHHMAJHDJABFDAAAPBEKDAHO
data AFLEHOEOEOEOMCOCALNENCANGDEGIDHJMAJLDNFHAHBFAAAAJCPJEAKC
data OOOGDOEOEOMCPMIMMMJEFFANIENCALJMDNCDANDDGNEIEHEMEJADCHAFGPNAEPCNCHCPDDHFGFDAAACJEDHACL
data MECEMOEOEOENHNHFIEMEJEBFIMLMPMMCOEFFIIJHNMLJNDBGALOIANLAIAKAHNGAAAPGHOIAOB
data CEAEEOEOEOMMNCNMJCJMMCJMICKENEMEBFIEKCOMBNGLNMENCDGNKCOMNCGNLEDNLMLMPMMIOHNMOJODMMJEFHLEMEOCJMPCCFANHNHHJAGOEOEOEGAFHHFDDPENBPBNCPEAAAJIGLCBIP
data HECBHOEOMENMLEJMMEDGAHFDAFIFHPMHJFDDFPFPHAAAJCJFEAMF
data EENGAOEOMMMEDFANIEJENEKCNEPIPELMJIMHHMEJIDDNAJHLGHFHCHBAAALFAKGAGP
data PMOFNOEOEOMCLEMCKMMEMEKMJEFNPEHFIALIPEKCOMJAOMFPLMIMHCMEJLDMEGAHDPCFFHJIEHBDLIENDLEHAHBNAAAAIBONMAKK
data IALDFOEOMENMLEJMMEDGAPIIKHCMEJCDAHFAAAPBGADANN
data BAFGDOEOEGAPGDBFDDCPEFILHLMLJDDHEAAAPAJGCAOH
data FMEEMOEOENHNHFIMNMMMLMMMNMMCOEFFICJMHIIHNMNJMDDGAHFPCNMACPCPANFAAAMHDLIAJH
data NPAPDOEOMEMCLEDLFDEDAGAIGHIMAJEDGLCAAAKBLKDAHG
data IPNPMOEOMCJENMNCAHMCDLNMNENEMMDNAPNAGNJCIMLCBDKKAABDCHDBLCEDMAEHKAELCHDNEHDBLMCNELKIDHLIFHCMEJKDDDDFKECDEDFDBJEFFGGOJGEOJOKBCIFNJGDGIKDNEGAHADKAENGPCDNAALFNDDBNDHHAAAGGPLBCEJ
data DEDNCOEOMMELNCKMPCNMLEBNANICOMNCGNLEDNLCPMMEJCJMANCDENDDFNENHIMHFMBJKDOMNAOEOEMMJEFHLEMEOCJMPCCNEGANGNANDDJICFFAAAKLPNPAEP
data DDNMNOEOMMELNMNMMMDNICOMNAOEOEMMJEFHLEMEOCJMPCCNHIEHAMJJEDEOEOEOEGIAGJHHALBHEFKAHFFNDHEAAAOPLCKABP
data HJNNPOEOMMENHLNMNEDGAIGHHMCJBDHAAACBOICACM
data GOEEGOEOMMMEDFANIMELDDENEGIANAILHIMNJLDFDFFFCNDPCAAAFDLNEAAL
data HCFFNOEOEOMAOMNMPMPCJMANHPFLFDANHPNAAHOABLFDAFIMENBFICJENMNCAIMHDMAJADHNHHJAGOEOEOEGAHIIADFFCNAHBNANCNDLAJHDEDDAAAMGJNMAKH
data HJIHKOEOEOMMNCNMJCJMMCJMICKENEMEBFICJENMNCAHMCDLNMNMMMDNAIADFICHKMDJCDLAELCHDNCGFDFBEBEPFPNFCIJDBIAIBEJMHNAMLNIBAAKEFKBBDJ
data JMHEAOEOEOMMMEDFANIEJENEKCNEPIPELMIEHEMNJGDFHFHHBJDJHJHHGAAAKFIMGANI
data PMMDPOEOEOEOMEJCNCIMJMMEOEMCJEFNPMHFICJENMNCAHMCDNJCKMPMPCBLKAPIPCJCHHKCOMBINHMMDJCDNENEMMDNPCPEMCLEFLFAJCGFAFANHNHHJAGOEOEGAHBHIACLJADDGPBAAAJBOFFBAN
data KDCIBOEOEOEOMCOENCJCOEOEMMJEFNPCAFIKAFIEJMMMNMPELEDAKAJGIPGALOIBIGKOAAJPJCJIJHFMJJADOKGFOGKJOICFBLCJOEKEJGBEOICICIOLOLLEADHCHCDIMHEILLJIDJCDAAGELMEBKP
data FKEABOEOEOMENMLEJMMEDGIAFPGFDHIPHIMHJHDHBLGFENCPDAAAMDONFAMG
data DDGGEOEOMENMLEJMMEDOEACAGADGFIDHLMLJLDDHFBNJAHDAAAHDOIEAJI
data ADHAEOEOEGAHBPHDIPHDMAJADACIBALF
data EGGBDOEOENHNHFICOMNMPMPCJMAGADILHHMNJNDLDLJAHNCFHAAAMDPMFAGF
data PJNFPOEOMMMEDFANIMENEGANFFCHGLEIGHHMCJBDMAGFKIAFBLHAAAGDOHEAIL
data NHAOOOEOEOMAOMNMPMPCJMANHHCFIMENJCJENMNCAHMCDFAFANHIHHJMGJEDEGALCFGPDFFHMAHLLABLNAHFLAHLHPCAAAJPPALAEA
data PNDEAOEOEOMMNCNMJCJMMCJMICKENEMEBFIEKCOMBLNCJENMNCAHMCDNICANIMHEMNJFDFANHNHHJAGOEOEGANEPEPAPADEHEPHLBNBFCNCNBAAAKKMGOAHH
data GBLELOEOMENMLEJMMEDGAPCDBNIFHNMAJHDIAHALHAAAKCIDEACI
data OCGAGOEGAPPAEHCDIAHNMAJADAPCBAMJ
data FFEGAOMEOEMCJEOMAGADDHLIIAAHBNILHHMHJPDPKIEFELDJHAAAGEDHFAMM
data EGPPLOEOMCKENCJCNCKMDFICLIPEMEKMPCKCBHOMPEJIFHAMGJDDEFEFMABDFLLICLDHFLDFDHBAAAMMNDKALJ
data EAJHGOMENMLEJCJCKKEOEOEOIGHNMAJPDDDDDAAAFCGEDAFP
data AECHGOMCKENCJCNCKMDFADAOEGIDHPMJJNDHCHEPGAAAADIDEAGJ
data DLLDAHNMLEBOEGAJDFGICHFMGJADAAOACKCAMD
data GGFCAGAFJIIAOAOAGDIDHLMDJADALAFPCALE
