#pragma TextEncoding = "Windows-1252"
#pragma rtGlobals=3
#pragma protect=1
#pragma independentmodule=tweptof
//#include "twV2_5_7g_toolsUnit"

Function deconv2d(undeconv2DMx, deconv2dMx, NewDeconvParam, normFactor, sp,ep)	 // 1.60D	//1.60A
wave undeconv2DMx, deconv2dMx,NewDeconvParam
variable normFactor //1.60A
variable sp,ep		// 1.60D startpoint, endpoint

	variable flag,noise,boxWidth,bsl
	variable idex, numMz
	
	numMz = dimsize(undeconv2DMx,1)
	flag=3
	
	for (idex=0;idex<numMz;idex+=1)
		matrixop/o rawDataWv =col(undeconv2DMx,idex)  		
		if (!tw_ePTOF_fastDeconvolve1D(rawDataWv,flag,noise,boxWidth,bsl,normFactor, sp,ep) )  // 1.60D
			deconv2dMx[][idex] =rawDataWv[p]
			NewDeconvParam[%boxW][idex]=boxWidth
			NewDeconvParam[%scalar][idex]=normFactor		//1.59D
			NewDeconvParam[%bsl][idex]=bsl
			NewDeconvParam[%noise][idex]=noise
		else
			return 1		//1.60O
		endif		
	endfor
	
	killwaves/z rawDataWv
End


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


STRCONSTANT tw_ePTOF_path="root:Packages:tw_ePTOF"
CONSTANT tw_ePTOF_startBslFrac=0.75 // baseline calculated using fractional range of X to 1.0 of distribution
CONSTANT tw_ePTOF_threshold=10 // default threshold of pk/noise, above which no pre-smoothing is applied
CONSTANT tw_ePTOF_maxBoxWidth=8 // default maximum box width for pre-smoothing (*multiplier)


// one-stop deconvolution with defaults
Function tw_ePTOF_fastDeconvolve1D(rawDataWv,flag,noise,boxWidth,bsl,normFactor,sp,ep)
	wave/z rawDataWv // 1-D FP32 input wave, will be overwritten with deconvolved result
	variable flag // inputs, see below
	variable &noise // noise level
	variable &boxWidth // box width that was applied in smoothing
	variable &bsl // baseline subtracted from profile (same units)
	variable &normFactor // normalization factor applied to profile, if applicable
	variable sp,ep		// 1.60D startpoint, endpoint

	variable smooth_flag=(flag&1)>0 // apply smoothing in raw domain before deconv
	variable norm_flag=(flag&2)>0 // normalise intensities (note that normalisation to MSSD is preferred)

	variable rv

	try
		// check input data
		abortonvalue !waveexists(rawDataWv),1
		abortonvalue wavedims(rawDataWv)!=1,2
		abortonvalue wavetype(rawDataWv)!=2,3
		
		// save raw data if normalising
		if (norm_flag)		//1.59D
			duplicate/free rawDataWv,temp
		endif
		
		// pre-smooth
		rv=tw_ePTOF_applyPreSm1D(rawDataWv,0,smooth_flag,tw_ePTOF_threshold,tw_ePTOF_maxBoxWidth,noise,boxWidth); abortonvalue rv,100+rv
		
		// deconvolve
		rv=tw_ePTOF_deconvolve1D_bsl(rawDataWv,0,bsl,sp,ep); abortonvalue rv,1000+rv		//1.60D from tw_ePTOFQ_r03e.ipf
// If sp, ep are out of range then the function will return an abort code and not perform the deconvolution.		
		// normalise
		if (norm_flag)
			normFactor=tw_ePTOF_getNormFactor(temp,rawDataWv,noise)
			noise*=normFactor
			rawDataWv*=normFactor
			bsl*=normFactor
		endif
		
	catch
		return v_abortCode
	endtry
	
	return 0
End	

// get normalisation factor based on comparison of counts in raw and deconvolved profiles
// this absolutely fails unless correlated counts >> uncorrelated counts
Function tw_ePTOF_getNormFactor(rawDataWv,deconvolvedWv,noise)
	wave/z rawDataWv // FP32 1-D input wave, no deconvolution yet applied
	wave/z deconvolvedWv // FP32 1-D input wave, deconvolved version of rawDataWv, baseline=subtracted
	variable noise

	variable normFactor=0

	try
		// check input data
		abortonvalue !waveexists(rawDataWv),1
		abortonvalue wavedims(rawDataWv)!=1,2
		abortonvalue wavetype(rawDataWv)!=2,3
		abortonvalue !waveexists(deconvolvedWv),4
		abortonvalue wavedims(deconvolvedWv)!=1,5
		abortonvalue wavetype(deconvolvedWv)!=2,6
		abortonvalue numpnts(rawDataWv)!=numpnts(deconvolvedWv),7
		
		matrixop/free i0=sum(rawDataWv) // total raw counts
		matrixop/free i1=sum(deconvolvedWv*greater(deconvolvedWv,noise))
		if (!i1[0])
			normFactor=1
		else
			normFactor=i0[0]/i1[0]
		endif
	catch
		return 0
	endtry
	
	return normFactor
End	
	

// apply pre-smoothing before doing deconvolution
Function tw_ePTOF_applyPreSm1D(rawDataWv,flag,factor,threshold,maxBoxWidth,noise,boxWidth)
	wave/z rawDataWv // FP32 1-D input wave, no deconvolution yet applied
	variable flag // settings flag, see below
	variable factor // tuneable parameter on how much pre-smoothing to apply. 0=none, 1=smoothed. Values <0 or >1 have same effect as 1
	variable threshold // pk/noise threshold above which no pre-smoothing should be applied
	variable maxBoxWidth // maximum smoothing box width to apply
	variable &noise // returned by reference
	variable &boxWidth // returned by reference
	
	variable print_flag=(flag&1)>0 // print some diagnostics to the history window
	variable useECL_flag=(flag&2)>0 // define noise using "estimated confidence limit" rather than 3-sigma on the noise. absolete.
	
	variable pkDivNoise,rv,bsl,multiplier,null,i0,minv
	
	try

		// check input data
		abortonvalue !waveexists(rawDataWv),1
		abortonvalue wavedims(rawDataWv)!=1,2
		abortonvalue wavetype(rawDataWv)!=2,3
		multiplier=floor(numpnts(rawDataWv)/127)
		
		// generate temp data
		dfref df=$tw_ePTOF_path
		if (!datafolderrefstatus(df))
			tw_createFolder(tw_ePTOF_path)
		endif		
		duplicate/o rawDataWv, df:tw_ePTOF_applyPreSm1D_temp/wave=temp
		
		// deconvolve temp
		rv=tw_ePTOF_deconvolve1D(temp,1,bsl); abortonvalue rv,100+rv
		
		// get the noise parameters
		rv=tw_ePTOF_getECL(useECL_flag,temp,bsl,noise,pkDivNoise)

		// define pre-smoothing parameters
		if (pkDivNoise<1)
			boxWidth=maxBoxWidth
		else
			boxWidth=maxBoxWidth*(1-(pkDivNoise-1)/(threshold-1))
			if (factor>=0&&factor<=1)
				// apply user-tuneable factor				
				boxWidth*=factor
				boxWidth=min(boxWidth,maxBoxWidth)
			endif			
		endif
		boxWidth=floor(boxWidth)
		boxWidth=max(0,boxWidth)

		// filter out frequencies faster than the sequence
		if (factor>0)
			filterfir/lo={0,min(0.5,1/multiplier),7} rawdatawv
		endif

		// apply median smoothing
		if (boxWidth>1&&boxWidth<51&&factor>0.1)
			Smooth/M=0/MPCT=50 boxWidth,rawDataWv
		endif
		
		if (print_flag)
			printf "* tw_ePTOF_applyPreSm1D: flag=%u, bsl=%g, noise=%g, pkDivNoise=%.1f, boxWidth=%.1f\r",flag,bsl,noise,pkDivNoise,boxWidth
		endif
		
	catch
		return v_abortCode
	endtry
	
	return 0
End


Function/wave tw_ePTOF_createSeq(multiplier)
	variable multiplier
	
	// reference/create data folder
	dfref df=$tw_ePTOF_path
	if (!datafolderrefstatus(df))
		tw_createFolder(tw_ePTOF_path)
	endif
	
	// reference/create sequence wave
	wave/z seqExp=df:$"seq_multiplier"+num2istr(multiplier)
	if (!waveexists(seqExp))
		wave/z seq=df:seq
		if (!waveexists(seq))
			make/n=127 df:seq/wave=seq=0
			seq[0,94]={1,1,1,1,1,0,0,0,1,0,1,1,0,1,1,0,0,1,1,0,0,0,1,1,0,1,0,0,1,1,0,1,1,1,1,1,1,1,0,1,1,1,0,0,1,1,1,1,0,1,0,0,0,0,0,1,0,1,0,1,1,1,1,0,0,1,0,0,1,0,0,0,1,0,0,0,0,1,0,0,1,1,1,0,1,1,0,1,0,1,0,1,0,0,1}
			seq[95,126]={0,1,1,1,0,1,0,1,1,0,0,0,0,0,0,1,1,0,0,1,0,1,0,0,0,1,1,1,0,0,0,0}
		endif
		make/o/n=(127*multiplier) df:$"seq_multiplier"+num2istr(multiplier)/wave=seqExp=seq[floor(p/multiplier)]
	endif
	
	// return wave reference
	return seqExp
End

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


STRCONSTANT tw_ksSoftwareName="Tofware"
STRCONSTANT tw_notebookName = "Tofwerk_Notebook"

Function/S tw_createFolder(DestPath [,wv])		// MJC 15th March 2005      Q r11
	String DestPath	// Full path
	Variable wv		// if DestPath includes a wave name, set to 1 to avoid creating this as a folder

	try

		if (DataFolderExists(DestPath))
			return destPath
		endif

		dfref sdf=getdatafolderdfr();abortonrte
		Variable i,n,numFolders
		String path=""
	
		if (!ParamIsDefault(wv))
			destPath=parsefilepath(1,destPath,":",1,0)
		endif
		destPath=parsefilepath(2,destPath,":",0,0) // make sure the path ends with a colon
		destPath=removeending(destPath) // and remove it
	
		numFolders = ItemsInList(DestPath, ":")
		path=""
		for (i=0; i<NumFolders; i+=1)
			path += StringFromList(i, DestPath, ":")+":"
			if (!DataFolderExists(Path))
				break
			else
				SetDataFolder $Path;abortonrte
			endif
		endfor
		for (n=i; n<NumFolders; n+=1)
			NewDataFolder /S $StringFromList(n, DestPath, ":");abortonrte
		endfor
		
		if (datafolderrefstatus(sdf))
			SetDataFolder sdf;abortonrte
		endif

		return destpath
		
	catch
		funcref tw_proto_debug ff=$"tw_debug"
		ff(v_abortCode) //tw_debug(v_abortCode)   // Q r13
		return ""
	endtry
End

Function tw_proto_debug(V_abortCode [,str]) // Q r13
	variable v_abortCode
	string str
	return v_abortCode
End

Function tw_debug(code [,str,notToFront])
	variable code
	string str
	variable notToFront
	
	variable i,j
	string caller=getrtstackinfo(2)
	string info=getrtstackinfo(3)
	i=itemsinlist(info)-1
	info=removelistitem(i,info)
	
	j=paramisdefault(notToFront) ? 0 : 8
	
	string nb="Tofware_Debug_Notebook"
	if (!wintype(nb))
		wave res=tw_getScreenRes(1,1)
		NewNotebook/F=1/K=2/N=$nb/W=(360,res[3]-220,720,res[3]-50)/opts=(2^3+2) as tw_ksSoftwareName+" | Debug output" // Q r70
		notebook $nb changeableByCommandOnly=1,showRuler=0	
	endif
	tw_panel_append2notebook("@4"+caller+" ("+num2str(code)+")",nb,1+j)
	if (!paramisdefault(str))
		tw_panel_append2notebook("@3"+str,nb,j)		
	endif
	tw_panel_append2notebook("@3"+info,nb,j)
//	hdf5dumpstate/q/z
//	tw_panel_append2notebook("@3"+S_hdf5dumpstate,nb,j) // Q r63
	
	return code
End

Function/wave tw_getScreenRes(screenNum,inPoints)
	variable screenNum, inPoints

	string info = igorInfo(0)
	variable nScr = numberByKey("nscreens",info)
	screenNum = max(1,min(nScr,screenNum))
	string infoScr = stringByKey("screen"+num2istr(screenNum),info)
	info = stringfromlist(2,infoScr,"=")+","+num2istr(screenNum)
	make/w/free/n=5 screenRes = str2num(stringfromlist(p,info,","))
	if (inPoints)
		screenRes[0,3] = tw_getpoints(screenRes)
	endif
	return screenRes
End


Function tw_getPoints(numPixels)
	variable numPixels
	return numPixels * (72/ScreenResolution)	// Convert pixels to points
End


Function tw_panel_append2notebook(str,notebkName,flag)
	String str, notebkname
	variable flag // Q r64
	
	variable insertDateTime=(flag&1)>0
	variable noCR=(flag&2)>0
	variable noStyle=(flag&4)>0
	variable notToFront=(flag&8)>0

	if (!strlen(notebkName)) // r60Q
		noteBkName=tw_notebookName
	endif

	variable notebkExists = wintype(notebkname)==5
	if (!notebkExists)
		return 0
	elseif (!notToFront)
		dowindow/f $noteBkName
	endif 
	
	// style separator = "@", followed by style number
	make/free/t/n=(15) styleWv
	String platform= UpperStr(igorinfo(2))
	Variable pos= strsearch(platform,"WINDOWS",0)
	if (!pos) // Q	r61 fix notebook for mac
		styleWv[0][] = {"calibri","12","1","calibri","10","0","calibri","9","0","calibri","8","0","calibri","10","1"}
	else
		styleWv[0][] = {"gill sans","16","1","gill sans","14","0","gill sans","12","0","arial","10","0","arial","12","1"} // Q r65
	endif

	string para,cmd
	variable n,style,numParas = itemsinlist(str,"@")
	if (insertDateTime)
		notebook $notebkname selection={endOfFile, endOfFile}
		notebook $notebkname font="arial", fsize=20, fstyle=2 // r60Q
		notebook $notebkname text = "  "
		notebook $notebkname font="arial", fsize=8+2*(pos!=0), fstyle=2 // Q r65
		notebook $notebkname text = date()+" - "+time()+":\r"
	endif
	for (n=0;n<numParas;n+=1)
		para = (stringfromlist(n,str,"@"))[1,inf]
		if (!strlen(para))
			continue
		endif
		style = str2num((stringfromlist(n,str,"@"))[0])
		notebook $notebkname selection={endOfFile, endOfFile}
		if (!noStyle) // Q r64
			notebook $notebkname font=styleWv[style*3]
			notebook $notebkname fsize=str2num(styleWv[style*3+1])
			variable gh = str2num(styleWv[style*3+1])
			notebook $notebkname fstyle=str2num(styleWv[style*3+2])
		endif
		notebook $notebkname text = para+selectstring(noCR,"\r","") // Q r64
	endfor
	return 1

End


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//		BEGIN ENCRYPTED CODE
//		r03 new functions to allow passing of baseline range
//		r02 no change
//		r01 intial version
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// tw_ePTOF_deconvolve1D(wv,dontSubtractBsl,&bsl)
	// wv: FP32 1-D input wave to be deconvolved, will be overwritten
	// dontSubtractBsl: set to non-zero to NOT do baseline subtraction
	// &bsl baseline value returned by reference, whether subtracted or not
	// Return values:
	// 0=OK
	// 1=input wave reference invalid
	// 2=input wave of wrong dimensions
	// 3=input wave of wrong type (not FP32)
	// 4=failed to create sequence
	// 5=#pnts in sequence didn't match #pnts in input wave
	
// tw_ePTOF_deconvolve1D_bsl(wv,dontSubtractBsl,&bsl,sp,ep)
	// as tw_ePTOF_deconvolve1D but additionally pass starting and ending indices for baseline region

// tw_ePTOF_getECL(useECL_flag,corrPTOFwave,bsl,&noise,&pkDivNoise)
	// useECL_flag: set to 1 to define noise as the ECL. Otherwise will use 3-sigma.
	// corrPTOFwave=input wave, correlated domain
	// bsl=baseline level in input wave (if having already subtracted baseline during deconvolution, should be zero)
	// noise=estimated noise level
	// pkDivNoise=ratio of highest peak to noise
	
// tw_ePTOF_getECL_bsl(useECL_flag,corrPTOFwave,bsl,&noise,&pkDivNoise,sp,ep)
	// as tw_ePTOF_getECL but additionally pass starting and ending indices for baseline region

data PLJONHLCNMLEOCJMMMPMDFICJCPIPENAIABPHHLIPEJENEOMPMLCLMPMJCLEFDEHBNICPCDNJEJMIMHCMAJCDEKCJCKEMEOCBHKCOMBNJEKCOMBNEGAFFFOAGDCFEJHDGHPADDLACDEFEAAABGMIHBEA
data KCEAHOMCPEMCLEFNPKCFICPCDGAHOIIAHPMLJFDNCLBDLAFNGAAAMEIGGAGC
data CHCMGOMCLEMCKMMEMEKMJEFFIEJMPMLCJAOCNEKCJCKEMEOIBHKMOJBDANDLGNIAAHBPFHBLKAHFNIAPIAGAAADBALMANO
data KDFCKOMCLEMCKMMEMEKMJEFFAFLEKCOMBFANHNHFICKENCJCNCKMDFAABPKMCBDHIEHFMBJJDCBICGJGJKFEJKJHJMJKMAKAGEGCBABEKKKBBBKFGOOEOJAAEGCDDBGD
data PHALFOEGALDLEDFDHDCLFIBHLMFJBDAAAAPACPCAPJ
data MECNCOMCLEMCKMMEMEKMJEFFIMDNJMNCNMJCJMMCIMJIMHNMCJJDOCANJENCAGIIEHHHDPHLHPHAAACOCGMAEC
data BCDGAOEGANEPDHAHKAIDHFMDJADAIANPBACO
data HCEOCOMCJCKKEGAFDPBDINHHMFJADAAAEBBBCACP
data LINBAOEOENHNHFIEOMIENEOMGFIMMMICHCMCJGDHMAEPNAFLBAAACECEGAPE
data GIPDEOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFMCPEMCLENENKIMMCOCJCINHCMCJNDNBDEGAJPIMACLALCFNAGFFDBPAFDLAHAFEFFAAALIFBOANI
data CCNFLOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENEJMMMNCGNIIPHDMEJEDFDENBDCGIAFHBFCPPAGHNAFPHHENGHBLEAAAMFNBNAPP
data ENCCNOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENCJKMCIIFHIMPJDDEFELFDCNBDGGIADHELFHLAHLALBHHAAAHCOINADB
data BCPCAOEOEGIAPABPFNIHHPMAJADAIAFPCAKB
data KGAFBOEOENHNHFIEPENCBFICOENCEGALIAHPMDJHDNFPPADHEPNIAAAAIEFAGAAE
data ADDLIOEOMMDLNMLCNMNCIMLCJCGNICPCDNEGILHHMHJADAENCHCHHFEFPIIAGAAALHGCIACC
data GFBBEOEOMENCALNMDNFDEGANIPHDMNJLDLHFHAAANBNGDAIG
data MHCEEOEOMCOCALNEOENMMMBNICJCPIPENAIABPHHLIPCOCJEMCKCBHKCOIBHLMKJMDGNKMDNEOEGALOAPADHFHFJDDAHIAIADLJIAJDAAABJFNPAMC
data ABICAOEOEGAPANPIEPINHLMIJADAHACPBAJO
data NEMMCOEOMCKENCJCNCKMDFICJCPIPENAIABPHHLIPEJENEOMPMLCLMPMJCLEFDEHJIPEKCOMBNICPCDIJHJMPJLDJAOCNEKCJCKEMEOCBHKCOMBNBAKALOEBHBIOEKGBIGCDIPJPJJJKKLKMLKCAAAELAAIBJO
data DFDEAOEOEGAFGNFLEDBNIDHLMNJHDAAAJALEBAKN
data NADPHOMEOEMCJEOMAGAFCHEDBIMHAMBJHDBHBLBAAAICENEAEG
data PJFIEOEOMCKENCJCNCKMDFICLIPEMEKMPCKCBHOMIEHEMGJIDDHDCDPACFELFHGDEDFNDAAAMKMPJALL
data LAFLNOMENMLEJCJCKKEGANGDADCIOHEMCJADJAHDEPCAAAGDCGFABB
data NFFPAHNMLEBOEOEGAHCDEICHHMBJJDHAAADBKGCAFL
data ALAAAGADDDAPELJIGHHMAJEDIPBAOH
data LJONKHLCNMLEOCJMMMPMDFICJCPIPENAIABPHHLIPEJENEOMPMLCLMPMJCLEFDEHJIPEKCOMBNICPCDNJEJMPMICHAMCJEDCJCKEMEOCBHKCOMBNBAKALOEBHBIOEKGBIGCDEALJEIIOEEIFIDKDGIJKJFJGKNIDDAAAJJJNLBEH
data GFEFHOMCPEMCLEFNPKCFICPCDGAFEPLAIAIPHPMNJNDPDHFPBFDHCDEPHAAAHGCKHAOD
data MOBKOOMCLEMCKMMEMEKMJEFFIEJMPMLCJAOCNEKCJCKEIEHCMHJCDMBGIAEHHFCHBDEFIALAANBLBAAAOOGALALO
data KCDKDOMCLEMCKMMEMEKMJEFFAFLEKCOMBFANHNHFICKENCJCNCKMDFAABPIMHBMHJEDFFBCJKCBICGJGJKFEJKJHJMJKMABCCPGKGFGMGNAAFPIBBBNM
data ECGDEOMCLEMCKMMEMEKMJEFFICOCANJENCAGALIHHNMHJLDFDDCPLIALBNBFHPHFHAAABJAIIAMH
data FADFGOEGAFFNDHADEDAHBNANIHHLMPJLDDBNBAAAGBFACANP
data MCMMEOMCLEMCKMMEMEKMJEFFIMDNJMNCNMJCJIMHIMJJMDNCCGIIEPGNDDEHEFCAAAKJIHJAHL
data LEECHOEGAPDDFPAPADFPIAHPMFJHDAAAPAHACADK
data GECDHOMCJCKKEGANIAJABDBNIFHNMJJNDNEHEAAABCOLDAKJ
data PFDANOEOENHNHFIEOMIENEOMGFIMMMLCICNCBGAIBHAMNJHDHDFDGDIADLDDEHFFCHAAAABIALIAKA
data KMCCCOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFMCPEMCLENENIIHMMOJJDGNICPCDNENBDEGADFHDNBFBDAHCAAAOAOBMABJ
data CNFLDOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENEJMMMNCGNICPIDHEMEJFDENBDCGAHAFHLPIENADBFCDGLFFAPDNGLGLEAAADHBANAKM
data DEEFCOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICPEMCLENCJKMCIEFNICPCINHFMLJDDNBDGGAFLAENHFJAGDFPCDLIEDFNFFHNMAHJHAAAEIDPOAKD
data ALDAAGAPGNIABLDHMIGHHMAJADJACFCAJA
data CEEMMOEOENHNHFIEOMIENEOMGFIEKEMCOENMJMMMLEFFIIKHNMPJMDPMDGIAAPEDJAENCNHJDLEDHPCAAAPMMNKAFO
data DEAHBOEOMMDLNMLCNMNCIMLCJCGNICPCINHGMPJHDPFFMACDPANAEAAAMFCIHABE
data ADNDBOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICOCALBDIKJKJCOCILHLMMJNDDDGAFBLHNHNHPMAFHJAFDHLBFDPDHHAAAKCKFMAAO
data KECLLOEOMEMEKMPCKCJMPMLCLEMMJCNEFFIENCALJCOCIKJIJHNMAJDDNMDNBDHGANAFOIAFCPDNAHHFEDCAAAKAHCMAHC
data DDHAAOEOEGAPFFBDGIAHHMAJADFAOCBAOF
data CGPNNOEOENHNHFIEPENCBFICOENIEHAMFJEDIABFEDBAAALCCCEABI
data CIDHEOEOMMNCNMJCJMMCIMJMMENCCLNELMJMPMPCINHMMNJDDDCDHNEGALALHNCDALDPDAAAOKDFJAOJ
data ECEAEOEOMCPEMCLEFNPKCFICOENCEHNKICALNCJCPIPENAIABPHHLIPEOCKENIMHJMNJODNCENIMNCNMJCJMMCIMJMMENCCNEGAFBLFPHHFHAFAAAACPJGCBDB
data IONENOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFMCPEMCLENENKIMMCOCJCGNICHEMCJHDKICANENBDBGIACPAHPAGLFJDFEPFNMAHLGHFHAAAAJLHFABNB
data CNLLIOEOMEMEKMPCKCJMPMLCLEMMJCNEFFAFENIMLCNMNCIMLCJCGNICOENCEHNKIIAHEMFJFDJAEJHJJGKGKOIJCDINGAJJLBLCKGKOJPLKLKKFJHJJJPBAAAALEJDBFN
data PJAAAGADIABNAFDIHHHMAJEDKJBABE
data AHJADOEOENHNHFIEJENEOMPMLCLMPMJCLEFGIJHNMIJLDLGFADDDMIKABPHNCAAAMGFPHAAJ
data HKCNCOEOMEOMPCKCKENMJEMCJEFNPEGFICOENCEINHIMAJJDPCDGAFBPCDCJHDHLPICAAABKOHJAEK
data AJAHCGANHPOADHDNFDANIHHIMNJLDAAAABIACABK
data MMMEGOEOENHNHFICOCNEKCJCKEMEOCBFIEKEMCOENIJHMMLJFDADFNGDELFHHLFPDLBHOADAAAGKHDJABN
data ECIFCOEOMEJCNCIMJMMEOEMCJEFNPELCKENEFNPCCLNIOCOCANJINHIMFJIDPCDNJCJENMNCAGALBHBLOAAHDLDDDPHAAAAEIJOAMB
data NCEMCOEOMEKCOMBLNCOCJEMCJCOMNENEJMMEMMDIIHJMNJNDANEGALDLHLELFHHDEFFAAAGKADJAFM
data OBEADOEOMMMEDFANAFMEJMPMLCJAOCNEKCJCKEMEOCBHKCIMHNMGJLDLEHAPBJLAENCLDNDNMABFEHGLBAAAKNFMKAHL
data GFHNPOEOEOMCPCDNFLNEKCOMBIAHDMAJEDBPIADAAACCNMEANC
data EDGDHOEOMENMLEJMMEDGANEDILHLMDJLDDENFAAAACANDAPL
data FNOAEOEOEGIAMAGDELIACNKAIPHAMIJFDPFDAAAAJBCPDAHI
data NCDHBOMEOEMCJEOMAGANBDIAHHMFJPDPAAAANBKBDABM
data MEEGPOEOMCKENCJCNCKMDFICLIPEMEKMPCKCBHOIPHJMFJADFLLAFJDHIAFDPADHAFGAAAEKGDJAOL
data EIPIMOMENMLEJCJCKKEGAHJIILHAMAJIDAHLDAAALCMHFAKA
data JDFAAOEGANBFFFENCIHHGMCJADEAOJBAKB
data JDALFOMCKENCJCNCKMDFADAGADAIDHLMEJDDCDNAFHHAAAGDCEFAAA
data BGNDAHNMLEBGALKADJHLOAHLDLIFHFMAJNDPFFNAFAAALCMBEAAG
data HBGHAGAFPABDKABHCNIPHLMFJADAKAGOCABC
data AJCJKHLCNMLEOCJMMMPMDFICJCPIPENAIABPHHLIPEPENCBHFHGPBNICJKMCIEFNJEOMPCKCCAJIGHHMGJFDKOECFBHOIOAAFGEHKACLNFCDLMMPCPDDLIDFFDJBEHMDMBNENDMMFLNIBAHJJPHBPP
data NEBAEOMCLEMCKMMEMEKMJEFFICJKMCIEFFANHNHFADALFDGNNCOMMEPMIEHNMFJDDLFHFHGPBGIADNCHPAHDBDOIAPAHCDOAEPDNAAAAFGDLNANJ
data HBFNDOMCPEMCLEFFIEOMPCKCKAIABPHHDABIIHEMEJMDCADMMKNHEDMMNINJBAIJACJAEB
data FCDAIOMCLEMCKMMEMEKMJEFFIEKCOMBFANHNHFIMLMPMMCGACHCBLIDFDHCDDPCFIBHDMPJIDKAKELEEBKMDBLMCFJBCANLNFMEFDELFDMDNJEDNPMCDABAAHFAFDBGA
data IKNJNOMCLEMCKMMEMEKMJEFFAFDHFHGPBNBFLCIMGHJMMCDPLMPMMCOEFFANHNHFICHEMCJCDCKMLENEBFIEKKEACLDNCFLICFDPCJDFJBEHMCFDNPFHMDBADDIOCBLN
data FANPAGAPELHFBPANFNAIJHHMHJGDAAMAIECAPD
data IGJNNOMCLEMCKMMEMEKMJEFFICIMOAJMICKENCOMIMPMJEBNJMLMPMMCOEFHMMNCIMBNJMDNJCIMHNMMJCDMJCJMMCIMJMMENCCNJCOCANJENCAGANDDANBLOABFGNGNCHENGLJAAAAAJCLEGBJH
data JNLPNOMCLEMCKMMEMEKMJEFFICICKMMMLCJIPELMJEMEHLNEJENELMMMLENEBNICJCHHIEHCMIJEDAIABPHHLEJENEOMPMLCDHJMMEMEPCGNEGALBPEDGFENKACNAAAADKAGEBON
data NADFDGADOAFFFHEPGPCFALGIIHHMCJADCDFAAAIBKDDAHB
data PFAEFOENHNHFICOENCBFICNCAFICKEMMLEIEHGMDJHDNBPGPBPNAIAFPBAAACGCFHANH
data LPMLFOMMDLNMLCNMNCIMLCJCGNIEOMPCKCKAIABPHHICHEMCJEDNEGANCLGLDNGNFPGDHFBLDAAANMKEKAMJ
data AIAAHOMENCALNMDNFDEGAJPAFFKIIHMMJJGDAPAPAPFAAAPCBJEALO
data KOBLKOMCOCALNEOENMMMBNICJCPIPENAIABPHHLIPCOCJEMCKCBHICHMMHJCDEMEGNKMDNEGALCLAPKAKADLEHGHEFMADAAAGFANOAKF
data DFHAHOEGANCHNAHHFPDNMADFDIEHDMNJEDBDADEAAANBMODAKJ
data GFKAFOMCKENCJCNCKMDFICJCPIPENAIABPHHLIPEPENCBHFHGPJIPEKCOMBNICJKMCIEFNJEOMPCKCCAJEEJHCIJHKMOJADEFHAICLKDFAJNOCJJFGOHJPJJFMJKKDFMFAKDJKFAKJMAKCCPOJKHOKGHOFGFKBOIBGAAENFLLBLM
data DFBEAHNMLEBGADGDGNGPJAELIFHDMDJHDDDDEAAABCGHDAFL
data DNNLFOEGAHIAGFDJHNBLHHGPGIBHFMDJEDGHHLEAAAKBHNDAGE
data GFKAFHLCNMLEOCJMMMPMDFICJCPIPENAIABPHHLIPEPENCBHFHGPJIPEKCOMBNICJKMCIEFNJEOMPCKCCAJEEJHCIJHKMOJADEFHAICLKDFAJNOCJJFGOHJPJJFMJKKDFMFAKDJKFAKJMAKDOMCPGKGAGMOOBKOLOOKKKLAAFCJBMBAJ
data NEBAEOMCLEMCKMMEMEKMJEFFICJKMCIEFFANHNHFADALFDGNNCOMMEPMIEHNMFJDDLFHFHGPBGAFFLDLGPMAEDBFFHLIEHCFCFMAEAAACGDKNACE
data BICENOMCPEMCLEFFIEOMPCKCKAIABPHHDAIHHBMAJJDPFFEPEPCMMPFPMJBAPIABIAHN
data FCDAIOMCLEMCKMMEMEKMJEFFIEKCOMBFANHNHFIMLMPMMCGACHCBLIDFDHCDDPCFIBHDMPJIDKAKELEEBKMDBLMCFJBENNMNEENMOMNNFMMNNFDEHNAEFAAAEFJGCBCJ
data CCMEEOMCLEMCKMMEMEKMJEFFAFDHFHGPBNBFLCIMGHJMMCDPLMPMMCOEFFANHNHFICKENCJINHKMLJNDBFIEKKEACLDNCFLICFDPCJDFJDJBEJMJCEMKMINPNPNGMMFHCICCAANLBLEBOJ
data AHMGHOMCLEMCKMMEMEKMJEFFICOCANJENCAGIIIPHDMAJDDPFNAPHDCLDHMAPANAHAAAFJKPJAOC
data HLPBEOEGIIALELKAGLAPINHAMAJHDAAAPAPHCAJM
data MNCMNOMCLEMCKMMEMEKMJEFFICIMOAJMICKENCOMIMPMJEBNJMLMPMMCOEFHMMNCIMBNJIDHJMIJGDJMNCNMJCJMMCIMJMMENCCGAFDHKACNKAFJHLBLDLIABPJAGHAAAADLNPFBNB
data CIEAAOMCLEMCKMMEMEKMJEFFICICKMMMLCJIPELMJEMEHLNEJENELMMMLENEBNICJCHHJENILHPMNJIDBPHHLEJENEOMPMLCDHJMMEMEPCGNEOEGIAEHHDEDHDDDELFFGHHAAACOHIFBIH
data MADBAOEGALIAAFBLEPIIHNMLJADAJANABAKP
data PFDDNOMCJCKKEGANMAGDEFFNHFDIHHFMAJBDGPGLJIAAAANCIIEACH
data LFDLAOEGAHBJHDEFJABIHHAMBJGDAAJAMJBAEO
data FCEEMOEOENHNHFIEOMIENEOMGFIEKEMCOENMJMMMLEFIIHKMNJPDMMPMDGAFEDHLCPJIEHHHIAGAAAJLDNKAHK
data HLPMLOEOMMDLNMLCNMNCIMLCJCGNIEOMPCKCKAIABPIHHCMEJCDEFNEGIAEHHLLIANGFELDJHAAAFMBNKANJ
data CLLMNOEOMEMEKMPCKCJMPMLCLEMMJCNEFFICOCALBDIKJKJCOIAHDMNJDDBDDGAPCPDDCPEPPIAHADHLCHEDBNAAAAACHJMACM
data NADNDOEOMEMEKMPCKCJMPMLCLEMMJCNEFFIENCALJCOCIKJKJEICHLMLJMDNBDHOEOEGAHDDGHCDFHDNLADPAPNIAAAAOCBJNALA
data HECHEOEOMMNCNMJCJMMCIMJMMENCCLNELMJMPMPCCNIMDNIDHDMDJNDGAHFDCHFHMACFPAHLGLGHGLHDEAAAOOPNLAHD
data IBNAAOEOEGAPENAPGHENIAHPMIJNDAAAMAGBCAJF
data ADAFFOEOENHNHFIEPENCBFICIENEMMGGANKIDHJMFJHDEPAPHLBDBNBFBLHAAAKFMEGAJH
data MLDFEOEOMCPEMCLENCOCJEMCJCGNPCENPCCLNIGDANJELMJMPMPCCNIMDNHDCNMIFNPIFHFMEJIDOMPCKCKAIABPHHDAJCEGAFEFBFFHBLEPFFEJHDAHCLFNDHEAAALFOMCBHC
data MMDOFOEOMCIMGLNCLIPMNEMKAGAPIIHAMPJADHAFEFLADAAAMDKLFALL
data EDBEAOEGAFOABFIAOIIDHDMHJADAIAGNBAIP
data FAAILOEOMMMEDFANICJKMCIEFNEGIAADFNEINHLMKJADGPDLGLFPCHDFFLEAAAPFDKGAHN
data NFAAAOEGANCNHLHHEIAHIMBJADFAFOBAPH
data FCMEFOEOEOENHNHFIEPENCBFIEMMNCIMJMMCJCNEJEFFIMPEDIIHJMIJFDIMLMPMMCOEFGADGFHPELKIAPJIENEAAAKPCGMAGF
data IFEMCOEOEOMCPEMCLENCOCJEMCJCGNPCENPCCLNIOCOCANJENCIIFHIMOJPDKCKAIABPHHDAJMCDIHJHEILBLGJMLMBAAAEFMLOANF
data CDLCLOEOEOMMLMPMMCOEFHMMNCIMBLNCLIPCOEJENCDNCDENDDFFANHNHFICJMIEFFAABDDDLIFBCBDPEBIFHHMAJDDICDDJDBDHEBKIBIEEGGOFKLCEKFCGCHJEOHBBDIOKMHEGIHAKOHAHALHLOLHKJCAALLIFIBEF
data HBEBBOEGAHFFEFFFBNGDIFHHMLJHDAAAKALJCAAA
data EMFCMOEOEOENHNHFIELMMMLEBFICKEMCJMMMHFIMPEDFIMJEMCKEPENCOCBFICIENIMHGMIJJDHFIMLMPMMCOEFFIMJENCLENMBGAPDHDDELHFHHEHFHJAHAAAOPCOCBHA
data OBEPDOEOEOMCIMGHJMMCDPLMPMMCOEFLFNICIMGNNEKCIMHNMNJMDABHNBDHLACHIBMOMEEPMJMGCKAADPGGMAOD
data LHNANOEOEOEGAFNAGLEFHIGHAMIJGDCAAALAOBCAEB
data FPHLFOEOEOENHNHFIEJENELMMMLEFFAHFHGPBGAIAHBMFJEDDNPAHLDNDDKIALCLLADAAAPGLCHADF
data DIFODOEOEOEHFHGPBLFNICIMGNNEKCOMBNENCDDNDDDLENCACPMFCDLMMPCPDILHDMDJEDEHNAFPMEFDFDJBMGNIMIMPMOMOFJCECKMKNOMFDEDABAKIJLOALE
data GOEGFOEOEOEGAJDFIAHFANIFHAMHJNDLHAAALAIKCAPB
data CCMMEOEOEOENHNHFICKENEJENELMMMLEFFICIENEMMGNPMLMPMMCOEFFIINHOMMJLDHFAHFHGPBFIEMCGFAALABDIFLNJDKOIPHALNBAAAEKHFABMB
data KOBEHOEOEOMCIMGHJMMCDPLMPMMCOEFLFNICIMGNNEICHMMNJNDHFHGPBGALMAFPHHBPDPIAAAAAPKMCJAJP
data FJBAFOEGIALACNEFIPHAMAJADANOBAAK
data HAPNLOEOMENMJCOEFGANBIJHHMGJFDBAAAFBPNDAHD
data DBEDBOEGAPDLADPAHNAHHLIFHDMLJFDFGAAADBDBCAFK
data CMEFMOEOEOENHNHFIEPENCBFIEMMNCIMJMMCJCNEJEFFIMPEDFIIJHIMFJIDLMPMMCOEFGAPMAEDCHAPENDLEFGHFDGAAAEBACMAKK
data CCAAPOEOEOMCPEMCLENCOCJEMCJCGNPCENPCCLNIOCOCANJENCIIFFIEOMPIKHKMIJBDHHDAJMCDINEAGEILKBLCJDLCKJKPIFEEJOIBJJDAAABNFOABFG
data ECNDGOEOEOMMLMPMMCOEFHMMNCIMBLNCLIPCOEJINHDMCJGDAHALFDFPDPEPEDBNBNFAAAGJEDJAEA
data CMNNMOEOEOMCIMGHJMMCDPLMPMMCOEFLFNICIMGNNEKIOHBMEJPDDABHNBDHLACHIBEFNFEPNDEDBAGOHOLAPM
data MGDALOEOEOEHFHGPBLNMLMPMMCOEFHMMNCIIBHAMIJCDEHBDAHELHFLABHEAAACGALHAPD
data NIAPAOEOEOEGAPHNNAIADNIAHIMHJADNHAAAABEJCAJN
data GECEOOEOMENMLEJMMEDGAJHNFLCPHLIFHFMPJHDAHHMABNNAEAAAMDGJFACG
data NAPJLOEGAHHJHFAPBLFHCIIHFMGJHDHAAAOAGPCABI
data EAIBBOEOMMMEDFANICICKMMMLCJIPELMJEMEHNIGHDMAJDDDLAFPLADNLAFPHPCFDAAAHIGKIACK
data CPAEIOEOEOMCICKMMMLCJEDFAFKCIMGLFFNEHNBFICIMOAJMICKENCOMIMPMJEBADPDPCPDDLIDFNBDHLACHAMMLEKCOMBIKKCHJMMIDHDMJJBDJLOKOMODAEKDGKIEJEHKACLFHAJMEANBILOGNBAKOENBEIOCNBAKOAGIADFAFCHDDEPDHBPDNEHFHNALIAAAAMBBMMCDK
data AHHDPOEOMENMLEJMMEDGIIAPHFLIEHHMAJGDHFKAOAGAAABDEIFAIA
data FHPDPOEGIAFPOIKAAFHFJAENGICHDMGJHDDLFPDAAAPBOODAON
data DBBIAOMEOEMCJEOMAGADEFILHPMDJIDHAAAAMBILDAEL
data PJFAAOEOMCKENCJCNCKMDFICLIPEMEKMPCKCBHOMIEHEMGJFDNGDGHCDFHGNHDDFAJHFDAAAJKCJJAND
data APDHDOMENMLEJCJCKKEGAFDDDPEHJIEHFMGJDDGPCLFFGFEAAADEDKFAOJ
data FHGAEOEGALBHEJHNINHNMAJADAJIBANC
data EDBAGOMCKENCJCNCKMDFADAGIAAHJAINHPMLJHDNCHADENLIEAAAJEKOFAGP
data IADPDHNMLEBGADELBDBFIAOIIHBMCJFDHDBAAAOBDMDACM
