#pragma TextEncoding = "Windows-1252"
#pragma rtGlobals=3		// Use modern global access method.
//#pragma version=1.17d// decided to not use pragma but a string constant 

//PeackShape for High ToF-AMS analysis Toolkit
// Peak Integration by Key Analysis (PIKA)
//Copyright (C) 2005, 2006, 2007,2008, 2009,2010,2011, 2012, 2013, 2014, 2015, 2016, 2017
// Pete DeCarlo, Allison Aiken, Jose Jimenez, Donna Sueper University of Colorado
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
///////////////////////////////////////////////////////////////////////////////
// CURRENT STATUS
// see HR read me file for details.


// These two constants were added in 1.06D		// changed in 1.07B, OH is now HO
STRCONSTANT	DEFAULTPWLIST = "CH;HO;C2H;O2;Ar;C4H2;C4H9;SO;SO2"			//these defaults are still just a guess
STRCONSTANT	GFMASSLIST = "C;N;O;HO;H2O;N2;O2;Ar;CO2;j183Wplus2;j182W;W;j186W;C8H5O3;C16H23O4;CH;C2H;C3H2;C3H3;C4H2;C4H9;C5H2;C6H5;C7H7;SO;SO2;SO3;"				// sulfates
//	GFMassList = "C;N;O;"						// single ions 
//	GFMassList += "OH;H2O;"					// water
//	GFMassList += "N2;O2;Ar;CO2;"				// major air
//	GFMassList += "j183Wplus2;j182W;W;j186W;"	// Tungstens
//	GFMassList += "C8H5O3;C16H23O4;"			// pthalates
//	GFMassList += "CH;C2H;C3H2;C3H3;C4H2;C4H9;C5H2;C6H5;C7H7;"			// organic
//	GFMassList += "SO;SO2;SO3;"				// sulfates
	

// creates and initializes globals in root:GaussFits
Function GF_createGlobals()

	NewDataFolder/o root:GaussFits

	killstrings/z root:GaussFits:TitleSpectraType 
	sq_initializeGlobalSVar("root:GaussFits:TitleSpectraType","Spectra Type: 0=don't use, 1=Diff, 2=Closed, 3=Open")
	
	sq_initializeGlobalNVar("root:GaussFits:GF_holdWidth",0)
	sq_initializeGlobalNVar("root:GaussFits:GF_holdCenter",0)
	sq_initializeGlobalNVar("root:GaussFits:GF_usePreprocessedSpectra",0)
	sq_initializeGlobalNVar("root:GaussFits:GF_useDAQmzParams",0)		// for future use?  For now always assume user did m/z calibration
	sq_initializeGlobalNVar("root:GaussFits:GF_useOpenClosedMzParams",0)

	GF_makeMassWaves()

	sq_initializeGlobalNVar("root:HR_peakShape:GF_showGFDetails", 1)  //1.06J//1.07		

	killstrings/z root:GaussFits:GFInstruct 
	string/g root:GaussFits:GFInstruct
	svar GFInstruct = root:GaussFits:GFInstruct
	GFInstruct = "The overwhelming majority of users should not change the defaults and simply proceed\r"
	GFInstruct += "to the Peak width tab if gaussian fits have already been performed.\r"
	GFInstruct += "\r"
	GFInstruct += "A subset of the HR ions for which gaussain fits were performed will be used to derive\r"
	GFInstruct += "final parameters for peak width (PW) and peak shape (PS).\r"
	GFInstruct += "\r"
	GFInstruct += "The values in the red Delta and Spectra columns are typically the only values the user should\r"
	GFInstruct += "change in this table. The number of points to fit for each mass peak is set by the Delta values\r"
	GFInstruct += " (n = 2*Delta+1). The V, W, Graph and Comments columns are not used in the Gauss Fits step\r"	
	GFInstruct += "but are presented in this table for consistency of appearance of the table in the Peak Width tab.\r"
	GFInstruct += "\r"
	GFInstruct += "Parameters for the gaussian fits y = K0+K1*exp(-((x-K2)/K3)^2) of selected HR ions for each\r"
	GFInstruct += "run are found and saved in waves such as root:GaussFits:GaussCoeff_Closed_N2.\r"
	GFInstruct += "Once gauss fit parameters for are found, they are not deleted or overwritten by subsequent\r"
	GFInstruct += "executions of the 'Perform Gaussian..' button where that ion has spectra type value of 0.\r"
	GFInstruct += "Finding gaussian fits need only be performed once, unless there has been significant\r"
	GFInstruct += "changes to the Squirrel baseline settings (if used) or the m/z calibration parameters, or the\r"
	GFInstruct += "delta values. The following somewhat arbitrary constraints were used for the gaussian peak\r"
	GFInstruct += "fitting: K0=0, K1>1e-5, K1<1e8, K2>0, K2<1e9, K3>0, K3<10\r"
	GFInstruct += "\r"
	GFInstruct += "Users can add HR ions to this table by popping the table and adding values, typically at\r"
	GFInstruct += "the bottom, or using the button below. This wave does't need to be in increasing m/z order."
	
	killstrings/z root:GaussFits:GFMainInstruct
	string/g root:GaussFits:GFMainInstruct
	svar GFMainInstruct = root:GaussFits:GFMainInstruct
	GFMainInstruct ="To change default settings for the gaussian fits of HR ions, redo gaussian fits, or see gaussian fit details deselect this checkbox.\r"
	GFMainInstruct +="To see results of gaussian fits, proceed to the peak width tab of the peak width/shape analysis."

End


// creates GFMass, etc waves
Function GF_initializeWaves()
	
	wave/z GFMass=root:GaussFits:GFMass
	wave/z/t GFName=root:GaussFits:GFName
	wave/z/i GFDeltaNum=root:GaussFits:GFDeltaNum
	wave/z/t GFComments=root:GaussFits:GFComments
	wave/z/i GFMask=root:GaussFits:GFMask

	variable num
	if ( waveExists(GFMass) && waveExists(GFname) && waveExists(GFDeltaNum) && waveExists(GFComments) && waveExists(GFMask))
		num = numpnts(GFMass)
		if (num == numpnts(GFname) && num == numpnts(GFDeltaNum) && num == numpnts(GFComments)&& num == numpnts(GFMask)   )
			return 1		// simply exit this function, don't overwrite because user may have edited
		else
			edit 
			GF_populate_MassTable()
			abort "The number of points in the waves GFMass, GFName, GFDeltaNum, GFComments, GFMask don't agree! - aborting from GF_initializeWaves()"
		endif
	endif
	
	GF_makeMasswaves()
	
End


// creates/overwrites Gaussian Fits waves; list of masses as of Pikc 1.05 may change/grow
// future version need to make this more streamlined
Function GF_makeMassWaves()

	NewDataFolder/o root:GaussFits
	
// This old way of doing things  was cumbersome.  1.06E
//	make/o/n=25 root:GaussFits:GFMass
//	make/t/o/n=25 root:GaussFits:GFName
//	make/i/o/n=25 root:GaussFits:GFDeltaNum
//	make/t/o/n=25 root:GaussFits:GFComments
//	make/i/o/n=25 root:GaussFits:GFMask=2		//1.06B
//	
//	wave GFMass=root:GaussFits:GFMass
//	wave/t GFName=root:GaussFits:GFName
//	wave GFDeltaNum=root:GaussFits:GFDeltaNum
//	wave/t GFComments=root:GaussFits:GFComments
//
//	GFMass[0]=12.000000;		GFName[0]="C";				GFDeltaNum[0]=3;	GFComments[0]="old m cal default - squirrel version 1.46" 	
//	GFMass[1]=13.007825;		GFName[1]="CH";			GFDeltaNum[1]=3;	GFComments[1]="current m cal default; possible C isotope interference"
//	GFMass[2]=14.003074;		GFName[2]="N";				GFDeltaNum[2]=3;	GFComments[2]="old m cal default - squirrel version 1.46"	
//	GFMass[3]=15.994915;		GFName[3]="O";				GFDeltaNum[3]=3;	GFComments[3]="old m cal default - squirrel version 1.46"	
//	GFMass[4]=17.002740;		GFName[4]="OH";			GFDeltaNum[4]=3;	GFComments[4]="mz cal default"	
//	GFMass[5]=18.010565;		GFName[5]="H2O";			GFDeltaNum[5]=3;	GFComments[5]="mz cal default"
//	GFMass[6]=25.007825;		GFName[6]="C2H";			GFDeltaNum[6]=3;	GFComments[6]="not always present"
//	GFMass[7]=28.006148;		GFName[7]="N2";			GFDeltaNum[7]=3;	GFComments[7]="mz cal default, possible CO interference"
//	GFMass[8]=31.989830;		GFName[8]="O2";			GFDeltaNum[8]=3;	GFComments[8]="mz cal default"
//	GFMass[9]=38.015650;		GFName[9]="C3H2";			GFDeltaNum[9]=3;	GFComments[9]="not always present"
//	GFMass[10]=39.023475;		GFName[10]="C3H3";			GFDeltaNum[10]=3;	GFComments[10]="not always present"
//	GFMass[11]=39.962384;		GFName[11]="Ar";			GFDeltaNum[11]=3;	GFComments[11]="possible CH4O interference"
////	GFMass[11]=39.962384;		GFName[11]="CO2";			GFDeltaNum[11]=3;	GFComments[11]=""		// new in 1.06D
//	GFMass[12]=47.966985;		GFName[12]="SO";			GFDeltaNum[12]=3;	GFComments[12]="not always present, possible Org interference"
//	GFMass[13]=50.015650;		GFName[13]="C4H2";			GFDeltaNum[13]=3;	GFComments[13]="not always present"
////	GFMass[12]=47.966985;		GFName[12]="C4H9";			GFDeltaNum[12]=3;	GFComments[12]=""  // new in 1.06D
//	GFMass[14]=62.015700;		GFName[14]="C5H2";			GFDeltaNum[14]=3;	GFComments[14]="not always present"
//	GFMass[15]=63.961900;		GFName[15]="SO2";			GFDeltaNum[15]=3;	GFComments[15]="not always present"
//	GFMass[16]=77.039125;		GFName[16]="C6H5";			GFDeltaNum[16]=3;	GFComments[16]="not always present"
//	GFMass[17]=79.956815;		GFName[17]="SO3";			GFDeltaNum[17]=3;	GFComments[17]="not always present"
//	GFMass[18]=91.054775;		GFName[18]="C7H7";			GFDeltaNum[18]=3;	GFComments[18]="not always present, possible inteference with neighbor"
//	GFMass[19]=91.475110;		GFName[19]="W183";		GFDeltaNum[19]=4;	GFComments[19]="doubly charged ion"
//	GFMass[20]=149.023860;		GFName[20]="C8H5O3";	 	GFDeltaNum[20]=5;	GFComments[20]="pthalate, may be noisy"
//	GFMass[21]=181.948252;		GFName[21]="W182";		GFDeltaNum[21]=7;	GFComments[21]="mz cal default, may be noisy"
//	GFMass[22]=183.950928;		GFName[22]="W184";		GFDeltaNum[22]=7;	GFComments[22]="mz cal default, may be noisy"
//	GFMass[23]=185.954357;		GFName[23]="W186";		GFDeltaNum[23]=7;	GFComments[23]="mz cal default, may be noisy"
//	GFMass[24]=279.159659;		GFName[24]="C16H23O4";	GFDeltaNum[24]=12;	GFComments[24]="pthalate, may be noisy"

	wave/t ExactMassText_All = root:HR:ExactMassText_All
	wave/d ExactMassWave_All = root:HR:ExactMassWave_All
	wave/t ExactMassComments = root:HR:ExactMassComments

	string GFMassStr, VModeList
	variable idex, pos, numList
	
	// 1.08D change We do not want to overwrite in case you user edited this table!
	if (Exists("root:GaussFits:GFMass"))		// presume if one exists they all do		
		wave GFMass=root:GaussFits:GFMass
		wave/t GFName=root:GaussFits:GFName
		wave GFDeltaNum=root:GaussFits:GFDeltaNum
		wave/t GFComments=root:GaussFits:GFComments
		wave GFMask=root:GaussFits:GFMask
		idex = numpnts(GFMass)
		if ( idex !=numpnts(GFName) || idex !=numpnts(GFDeltaNum) || idex !=numpnts(GFComments) || idex !=numpnts(GFMask) )
			abort " The number of points in the waves of the gauss fit table is not the same - aborting from GF_makeMassWaves"
 		endif
 		Sort GFMass, GFMass, GFName, GFDeltaNum, GFMask, GFComments		//1.20N
 		
 		return 0		//1.08F
	endif

	numList = itemsInList(GFMASSLIST)
	
	make/o/n=(numList) root:GaussFits:GFMass
	make/t/o/n=(numList) root:GaussFits:GFName
	make/t/o/n=(numList) root:GaussFits:GFComments
	make/i/o/n=(numList) root:GaussFits:GFDeltaNum = 3
	make/i/o/n=(numList) root:GaussFits:GFMask = 2		//1.06B

	wave GFMass=root:GaussFits:GFMass
	wave/t GFName=root:GaussFits:GFName
	wave GFDeltaNum=root:GaussFits:GFDeltaNum
	wave/t GFComments=root:GaussFits:GFComments

	for (idex=0;idex<numList;idex+=1)
		GFMassStr = stringfromList(idex, GFMASSLIST)
		pos = sq_findStr(ExactMassText_All,GFMassStr,1)		// exact match
		if (pos>0)
			GFMass[idex] = ExactMassWave_All[pos]
			GFName[idex] = ExactMassText_All[pos]
			GFComments[idex] = ExactMassComments[pos]		
		else
			Abort "Could not find the mass "+GFMassStr+" in the exact mass all list! Aborting from GF_makeMassWaves"
		endif
	endfor

	Sort GFMass, GFName, GFComments, GFMass

	GFDeltaNum = SelectNumber(GFMass[p]< 75.5 && GFMass[p]> 50.5, GFDeltaNum[p], 4)
	GFDeltaNum = SelectNumber(GFMass[p]< 100.5 && GFMass[p]> 75.5, GFDeltaNum[p], 5)
	GFDeltaNum = SelectNumber(GFMass[p]< 200.5 && GFMass[p]> 100.5, GFDeltaNum[p], 6)
	GFDeltaNum = SelectNumber(GFMass[p] > 200.5, GFDeltaNum[p], 7)

End


// code liberally stolen from HR_butt_GFFitTodo
Function GF_butt_GFsTodo(ctrlName) : ButtonControl
string ctrlName
	
	svar sq_toDoWvNm=root:panel:sq_toDoWvNm
	wave/z todo=$(sq_toDoWvNm)
	GF_findGaus(todo)

End

//1.20N
Function GF_findGaus(todo)
wave todo

	GF_createGlobals()
	if (!DataFolderExists("root:GaussFits"))	//1.06J
 		GF_createGlobals()
	endif
	
	wave GFMass=root:GaussFits:GFMass
	wave GFMask=root:GaussFits:GFMask
	wave/t GFName=root:GaussFits:GFName
	wave GFDeltaNum=root:GaussFits:GFDeltaNum

//	nvar GF_LiveUpdate = root:GaussFits:GF_LiveUpdate
//	nvar GF_UseSqBaseline = root:GaussFits:GF_UseSqBaseline		// error in 1.05
//	nvar HR_UseSqBaseline = root:HR_panel:HR_UseSqBaseline		// error in 1.05
//	nvar GF_FindOpen = root:GaussFits:GF_FindOpen
	nvar GF_useDAQmzParams = root:GaussFits:GF_useDAQmzParams
	nvar GF_useOpenClosedMzParams = root:GaussFits:GF_useOpenClosedMzParams
	nvar GF_usePreprocessedSpectra = root:GaussFits:GF_usePreprocessedSpectra
	
//	ControlInfo/w=GaussFitPanel GF_popup_ToDoWave
//	if (!stringmatch(S_value, sq_toDoWvNm))
//		abort "Please reselect the todo wave from the PeakWidthShapePanel - aborting from GF_butt_GFsTodo"
//	endif

	variable numMasses, openAlso
	string paramStr
			
	make/o/n=(numpnts(GFMask)) tempWave
	tempWave=GFMask/GFMask
	wavestats/q/m=1 tempWave
	numMasses = V_npnts
	killwaves/z tempWave	
//	nvar GF_useOpenClosedMzParams = root:GaussFits:GF_useOpenClosedMzParams

	gen_SanityTodoWaveExists()

	GF_GausFitPrelims()
	
	if (GF_useDAQmzParams)
		paramStr= "mzFolder:diagnostics;mz_Intercept:mzInt;mz_Slope:mzSlope;mz_Power:mzPower;"
	else
		if (!DataFolderExists("root:mzFitting"))
			abort "The user selected to use the m/z parameters but the mzFitting data folder does not exist - aborting from GF_butt_GFsTodo"
		else
			paramStr= "mzFolder:mzFitting;mz_Intercept:mz_FitIntercept;mz_Slope:mz_FitSlope;mz_Power:mz_FitPower;"
			if (GF_useOpenClosedMzParams)
				paramStr= "mzFolder:mzFitting;mz_Intercept:mz_FitInterceptClosed;mz_Slope:mz_FitSlopeClosed;mz_Power:mz_FitPowerClosed;"
			endif
		endif
	endif
	
	print "//** The gaussian fitting of "+num2str(numMasses)+" masses began at "+time()+" on the "+NameofWave(todo)+" todo wave containing "+num2istr(numpnts(todo))+" runs."   //  1.20O

	// 1.06B
	// wavestats/q GFMask
	openAlso = WaveMax(GFMask) > 2 ? 1 : 0

	paramStr= "specType:Closed;"+paramStr
	squirrel_fetch(todo,GF_fetch_GetGausCoefs,paramStr, "MSClosedLessBaseL_p;")
	if (openAlso)
		paramStr= ReplaceString("Closed", paramStr, "Open")
		squirrel_fetch(todo,GF_fetch_GetGausCoefs,paramStr,"MSOpenLessBaseL_p;")
	endif

	print "//** The gaussian fitting ended at "+time()+" on the "+NameofWave(todo)+" todo wave."
	
	if (Exists("root:HR_PeakShape:PW_graph")!=1)		//1.07
		PW_NewGlobals()
		PS_createGlobals()
	endif
	//1.06B we need to resize some waves for the other tabs.
	wave PW_VmodeMask = root:HR_PeakShape:PW_VmodeMask
	wave PW_WmodeMask = root:HR_PeakShape:PW_WmodeMask
	wave PW_graph = root:HR_PeakShape:PW_graph
	redimension/n=(numpnts(GFMass)) PW_VmodeMask, PW_WmodeMask, PW_graph

End


// Makes the waves that will hold the coeffs
Function GF_GausFitPrelims()

	wave rn_series = root:index:rn_series
	wave GFMass = root:GaussFits:GFMass
	wave/t GFName = root:GaussFits:GFName
	wave GFDeltaNum = root:GaussFits:GFDeltaNum
	wave GFmask= root:GaussFits:GFmask
	nvar GF_holdWidth = root:GaussFits:GF_holdWidth
	nvar GF_holdCenter = root:GaussFits:GF_holdCenter

	variable numMass, idex, numRuns
	string numRunsStr, specTypeStr, specTypeList = ";Diff;Closed;Open", goodNameStr

	numMass=numpnts(GFMass)
	numRuns = numpnts(rn_series)
	numRunsStr = num2istr(numRuns)		//1.20O
	
	if (numMass!=numpnts(GFName) || numMass!=numpnts(GFDeltaNum)  || numMass!=numpnts(GFmask)) 
		Abort "When entering values into the mass columns, make sure that all columns are the same length.  -  Aborting from GF_GausFitPrelims"
	endif

	if (WaveMax(GFmask)<=0)
		abort "No masses were chosen to perform gaussian fits on - aborting from GF_GausFitPrelims"
	endif
	
	make/o/T/N=0 root:GaussFits:GausConstraints		// redimmensioned below
	make/o/n=(4) root:GaussFits:GausCoeffsWv
	make/o/n=(numMass, 4) root:GaussFits:GausCoeffsMx=nan

	wave/t GausConstraints=root:GaussFits:GausConstraints
	wave GausCoeffsWv=root:GaussFits:GausCoeffsWv
 		
	// let baseline either be 0 or constant, peak height not be too big or too small, peak center be reasonable, peak width reasonable
	if (GF_holdCenter)
		if (GF_holdWidth)
			GausConstraints = {"K1>1e-5", "K1<1e8"}			
		else
			GausConstraints = {"K1>1e-5", "K1<1e8", "K3>0", "K3<10"}		// 10 is arbitrary
		endif
	else
		if (GF_holdWidth)
			GausConstraints = {"K1>1e-5", "K1<1e8", "K2>0", "K2<1e9"}	
		else
			GausConstraints = {"K1>1e-5", "K1<1e8", "K2>0", "K2<1e9", "K3>0", "K3<10"}		// 10 is arbitrary
		endif
	endif
	
	make/o/d/n=0 root:GaussFits:RawSpectraThisRun
	make/o/d/n=0 root:GaussFits:PeakCenterFromMassCal

	for (idex = 0;idex<numMass;idex+=1)
		specTypeStr = StringFromList(GFmask[idex], specTypeList)
		goodNameStr = CleanUPName(GFName[idex],0)		//1.19B
		hr_initializeGlobalWave("root:GaussFits:GaussCoeff_"+specTypeStr+"_"+goodNameStr, numRunsStr+";4",nan)
	endfor

End


// Squirrel fetch command for calculating peak width
Function GF_fetch_GetGausCoefs(index_pos,paramstr,acorn,n_acorns)
	wave index_pos 
	string paramstr
	variable acorn
	variable n_acorns

	wave MSSpectra=$stringbykey("acorn_0",paramstr)
	string specTypeStr = stringbykey("specType",paramstr)
	string mzFolder = stringbykey("mzFolder",paramstr)
	wave mz_Intercept=$"root:"+mzFolder+":"+stringbykey("mz_Intercept",paramstr)
	wave mz_Slope=$"root:"+mzFolder+":"+stringbykey("mz_Slope",paramstr)
	wave mz_Power=$"root:"+mzFolder+":"+stringbykey("mz_Power",paramstr)
	variable hz_op=numberbykey("hz_op",paramstr)
//	variable baseline_op=numberbykey("baseline_op",paramstr)

	variable spectType =WhichListItem(specTypeStr, ";Diff;Closed;Open")	//1.06B

	wave series_index = root:index:series_index
	wave rn_series = root:index:rn_series
	wave run_index = root:index:run_index
	wave GFMass = root:GaussFits:GFMass
	wave/t GFName = root:GaussFits:GFName
	wave GFDeltaNum = root:GaussFits:GFDeltaNum
	wave GFMask = root:GaussFits:GFMask
//	nvar GF_FindOpen = root:GaussFits:GF_FindOpen
//	nvar GF_UseDAQ = root:GaussFits:GF_UseDAQ

	wave RawSpectraThisRun = root:GaussFits:RawSpectraThisRun
	wave PeakCenterFromMassCal = root:GaussFits:PeakCenterFromMassCal

//	nvar GF_CurrentRunNum = root:GaussFits:GF_CurrentRunNum
//	nvar GF_liveUpdate = root:GaussFits:GF_liveUpdate

	// use mz parameters from mzFitting folder
	nvar HR_UseOpenClosedMzParams = root:HR_panel:HR_UseOpenClosedMzParams

	variable/g V_FitError
	variable /g V_FitOptions=4 // Suppress the fit window
	
	variable numRows, numCols, numMass, idex, intercept, slope, power, pos, jdex, runIsW  //, currentRunIndex
	string tempStr		//1.05A
	
	numRows=dimsize(MSSpectra,0)
	numCols=dimsize(MSSpectra,1)
	numMass = numpnts(GFMass)

	sq_ams_stat_upd("Gaussian fits")

	Redimension/n=(numCols) RawSpectraThisRun		//1.04B
	Redimension/n=(numMass) PeakCenterFromMassCal
		
	wave/t GausConstraints = root:GaussFits:GausConstraints		// created by 	waveGF_GausFitPrelim
	wave GausCoeffsWv =  root:GaussFits:GausCoeffsWv
	wave GausCoeffsMx =  root:GaussFits:GausCoeffsMx

	duplicate/o GausCoeffsMx GausCoeffsMx_Nan
	fastop GausCoeffsMx_Nan = (nan)
	
	if (hz_op)
		sq_Hz(index_pos,MSSpectra)
	endif

//  1.20I  Don't know why this was here to begin with
//	if (baseline_op)
//		sq_Hz(index_pos,MSSpectra)
//	endif

	for(idex=0;idex<numRows;idex+=1) // loop to deal with each run
		
		sq_ams_prog_upd(100*(acorn+(idex/numRows))/n_acorns)

		if (index_pos[idex]==-1)
			continue
		endif
		
		pos = series_index[index_pos[idex]]	

		intercept =  mz_Intercept[pos]
		slope =  mz_Slope[pos]
		power = mz_Power[pos]
		if (numtype(intercept+slope+power))	// simple way to make sure all are nonNan
			sprintf tempStr, "%d", run_index[pos]  	// 1.05A
			print "// Could not proceed with finding gauss peak fits for run "+tempStr+" for "+specTypeStr+" spectra becasue of bad m/z values."
    		GF_DumpCoeffs(GausCoeffsMx_Nan, pos, specTypeStr, GFName)
			continue
		endif
		
		PeakCenterFromMassCal =  intercept + slope*(GFMass^power)		//OneRunPeakCenter is in point space  // 1.57K 
		//  all the peak centers should be 'available.  //  1.14I
		
		RawSpectraThisRun = MSSpectra[idex][p]	
						
		if (numtype(RawSpectraThisRun[0]))	// simple way to make sure all are nonNan  //1.19B
			sprintf tempStr, "%d", run_index[pos]  
			print "// Could not proceed with finding gauss peak fits for run "+tempStr+" for "+specTypeStr+" of nonexistent raw spectra."
    		GF_DumpCoeffs(GausCoeffsMx_Nan, pos, specTypeStr, GFName)
			continue
		endif

		GF_GausFitOneRun(PeakCenterFromMassCal, RawSpectraThisRun, GausConstraints, GausCoeffsMx, GausCoeffsWv, spectType)
   
    	GausCoeffsMx = GausCoeffsMx[p][3]< 0  || GausCoeffsMx[p][3] > 10 ? nan : GausCoeffsMx[p][q]  // 1.16E
    		
    	GF_DumpCoeffs(GausCoeffsMx, pos, specTypeStr, GFName)
//    		if (GF_liveUpdate)
// 		OneRunIsolatedPC_fromGaus = GausCoeffsMx[p][2]
//    			GF_individualMassFitGraphs(GausCoeffsMx)
//  			DoUpdate
//    		endif
	endfor // run number loop	

	if (acorn == n_acorns-1)
		killwaves/z GausCoeffsMx_Nan
	endif

End


//  Finds gaussian fits for chosen ions for one run
// modified in 1.06B to allow for the fitting of closed spectra
Function GF_GausFitOneRun(PeakCenterFromMassCal, RawSpectraThisRun, GausConstraints, GausCoeffsMx, GausCoeffsWv, spectType)
wave PeakCenterFromMassCal, RawSpectraThisRun
wave/t GausConstraints
wave GausCoeffsMx, GausCoeffsWv
variable spectType

	wave/i GFMask = root:GaussFits:GFMask
	wave GFDeltaNum = root:GaussFits:GFDeltaNum
//	nvar GF_holdWidth = root:GaussFits:GF_holdWidth

	variable idex, numMass
	
	numMass = numpnts(GFDeltaNum)
	
	for(idex=0;idex<numMass;idex+=1)
		if (GFMask[idex]==spectType )
			GausCoeffsWv[0] = 0		// set baseline to 0 .. this needs to be modified at the individual spectra level
			GausCoeffsWv[1] = 1            // gaussian peak height, set to a default value, will always vary
			GausCoeffsWv[2] = PeakCenterFromMassCal[idex]
	//		if (GF_holdWidth)			 to be enabled later
	//			GausCoeffsWv[3] = 1	 	// default peak width		
	//		else
				GausCoeffsWv[3] = 1	 	// default peak width		
	//		endif
			GF_FindGausCoefs(RawSpectraThisRun, GausConstraints, GausCoeffsWv, PeakCenterFromMassCal[idex], GFDeltaNum[idex])
//			GausCoeffsWv[q] = GausCoeffsWv[3]< 0 || GausCoeffsWv[3]>10  ? nan : GausCoeffsWv[p]  // 1.16Z
			GausCoeffsMx[idex][]=GausCoeffsWv[q]
		endif
	endfor // m/z loop
		
End


//For one run, one isolated mass, find peak width
Function GF_findGausCoefs(RawSpectraThisRun, GausConstraints, CoeffWave, myCenter, delta)
wave RawSpectraThisRun, CoeffWave
wave/t GausConstraints
variable myCenter, delta		// in point space

	nvar V_FitError = root:V_FitError

	nvar GF_holdWidth = root:GaussFits:GF_holdWidth
	nvar GF_holdCenter = root:GaussFits:GF_holdCenter

	//  gauss	Gaussian peak: y = K0+K1*exp(-((x-K2)/K3)^2).
	V_FitError=0
	
	if (GF_holdCenter)		// 1.04B fix center, and if necessary, fix width
		if(GF_holdWidth)
			curveFit/NTHR=0/Q/N/h="1011"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints	
		else
			curveFit/NTHR=0/Q/N/h="1010"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints
		endif
	else
		if (GF_holdWidth)		// 1.04B do not fix center, and if necessary, fix width
			curveFit/NTHR=0/Q/N/h="1001"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints
		else
			//  	the most common - let all except the baseline be fit
			curveFit/NTHR=0/Q/N/h="1000"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints
		endif
	endif
	
	if (V_FitError!=0)
		CoeffWave=nan
	endif
		
End


// after we have found pw gauss coeffs and pw line coeffs, store results in waves, 
Function GF_dumpCoeffs(GausCoeffsMx, pos, specTypeStr, GFName)
wave GausCoeffsMx
wave/T GFName 
string specTypeStr
variable pos

	wave/i GFMask=root:GaussFits:GFMask

	string goodNameStr		//1.19B
	variable jdex, numMass = numpnts(GFName), specType
	specType = WhichLIstItem(specTypeStr, ";Diff;Closed;Open")

	for(jdex=0;jdex<numMass;jdex+=1)
		if (GFMask[jdex]==specType  )
			goodNameStr =CleanupName(GFName[jdex],0) 		//1.19B
			wave coeffw = $"root:GaussFits:GaussCoeff_"+specTypeStr+"_"+goodNameStr		//1.19B
			coeffw[pos][] = gausCoeffsMx[jdex][q]
		endif
	endfor

End


Function GF_plotAllGFplots()

	Edit/W=(10,160,415,575)/HOST=# 
	RenameWindow #,GF_MassTable
	GF_populate_MassTable()	
	SetActiveSubwindow ##

	nvar GF_showGFDetails = root:HR_peakShape:GF_showGFDetails

	GF_ck_showGFdetails("",GF_showGFDetails)

End


Function GF_butt_SetGFDefaults(ctrlName) : ButtonControl
	String ctrlName

	GF_makeMasswaves()
	 PW_SetDefaults()  //1.06x

End


Function GF_ck_OpenClosed(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	if (strsearch(lowerstr(ctrlName), "open", 0)>0)
		DoAlert 0, "Performing Gaussian fits on Open Spectra is not yet available." 
	endif

	Checkbox GF_check_Open value = 0
	Checkbox GF_check_Closed value = 1

End


Function GF_populate_MassTable()	

	AppendToTable root:GaussFits:GFName,root:GaussFits:GFMass,root:GaussFits:GFDeltaNum
	AppendToTable root:GaussFits:GFMask
	AppendToTable root:HR_PeakShape:PW_VmodeMask,root:HR_PeakShape:PW_WmodeMask,root:HR_PeakShape:PW_Graph
	AppendToTable root:GaussFits:GFComments
 
	ModifyTable format(Point)=1,width(Point)=20,title(Point)=" "
	ModifyTable width(root:GaussFits:GFName)=45,title(root:GaussFits:GFName)="Name"
	ModifyTable format(root:GaussFits:GFMass)=3,digits(root:GaussFits:GFMass)=6,width(root:GaussFits:GFMass)=70,title(root:GaussFits:GFMass)="Mass"
	ModifyTable width(root:GaussFits:GFDeltaNum)=30, title(root:GaussFits:GFDeltaNum)="Delta"
	ModifyTable width(root:GaussFits:GFMask)=30, title(root:GaussFits:GFMask)="Spec"
	ModifyTable width(root:GaussFits:GFComments)=250,title(root:GaussFits:GFComments)="Comments"
	ModifyTable rgb(root:GaussFits:GFMask)=(65280,0,0)
	ModifyTable rgb(root:GaussFits:GFDeltaNum)=(65280,0,0)
	ModifyTable width(root:HR_PeakShape:PW_VmodeMask)=20,title(root:HR_PeakShape:PW_VmodeMask)="V"
	ModifyTable width(root:HR_PeakShape:PW_WmodeMask)=20,title(root:HR_PeakShape:PW_WmodeMask)="W"
	ModifyTable width(root:HR_PeakShape:PW_Graph)=40,title(root:HR_PeakShape:PW_Graph)="Graph"
	ModifyTable alignment(root:GaussFits:GFComments)=0

End


/////////////////////////////////// PEAK WIDTH  /////////////////////////////////// 

Function PW_NewGlobals()

	NewDataFolder/o root:HR_PeakShape
	
//	nvar  HR_UseSQBaseline = root:HR_panel:HR_UseSQBaseline
	nvar  HR_UseOpenClosedMzParams = root:HR_panel:HR_UseOpenClosedMzParams
	
	sq_initializeGlobalSVar("root:HR_PeakShape:PrelimUseSqBaseline","")
	svar PrelimUseSqBaseline = root:HR_PeakShape:PrelimUseSqBaseline
	PrelimUseSqBaseline = "From Pika panel:\r"
//	PrelimUseSqBaseline += "Use SQ baseline? "+selectString(HR_UseSQBaseline,  "NO", "YES")+"\r"
	PrelimUseSqBaseline += "Use separate Open/Closed params? "+selectString(HR_UseOpenCLosedMzParams,  "NO", "YES")

	wave all = root:all
	sq_initializeGlobalNVar("root:HR_PeakShape:PW_CurrentRunNum", all[0])

	sq_initializeGlobalNVar("root:HR_PeakShape:NumPtsFinalPSWave",281)
	sq_initializeGlobalNVar("root:HR_PeakShape:MaxSigma",7)

	nvar NumPtsFinalPSWave = root:HR_PeakShape:NumPtsFinalPSWave
	nvar MaxSigma = root:HR_PeakShape:MaxSigma

	hr_initializeGlobalWave("root:HR_PeakShape:FinalPeakShape", num2str(NumPtsFinalPSWave), nan)	
	wave FinalPeakShape = root:HR_PeakShape:FinalPeakShape
	SetScale/P x -MaxSigma, 2*MaxSigma/(NumPtsFinalPSWave-1),"", FinalPeakShape
		
	sq_initializeGlobalSVar("root:HR_PeakShape:PS_runModeStr", "")
	sq_initializeGlobalNVar("root:HR_PeakShape:HR_FinalPeakShapeScalar", nan)  //THIS ONE IS FOR DISPLAY!  the one used in the calcs is HR_FinalPSScalar 
	sq_initializeGlobalSVar("root:HR_PeakShape:PS_MassesNotDisplayedList", "")
 
	sq_initializeGlobalSVar("root:HR_PeakShape:PS_nameSuffix", "")	
	
	sq_initializeGlobalSVar("root:HR_PeakShape:Step1Instruct", "Step 1. Find gaussian fits for selected masses for runs in todo wave.")	
	sq_initializeGlobalSVar("root:HR_PeakShape:Step1InstructOptions", "Step 1 Options. Users can change gauss fit settings via the tools below.")	

	sq_initializeGlobalSVar("root:HR_PeakShape:PWModeStr", "Mode:")
	sq_initializeGlobalSVar("root:HR_PeakShape:PSModeStr", "Mode:")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step4MassList", "Current mode mass list:")
	
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2Instruct","Step 2. Using the gaussian fit results from step 1, select/deselect runs and masses to generate average peak width parameters for a todo wave.")

	sq_initializeGlobalSVar("root:HR_PeakShape:Step2A","Step 2A - Confirm todo wave choice")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2B","Step 2B - Select/Deselect HR ionss")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2Bi","Enter 1s (use) and 0s (don't use) for V and W PW mask waves.")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2C","Step 2C - Deselect problematic runs")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2Ci","Step 2C i. Modify peak width run mask wave")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2Cii","Step 2C ii. Remove one run")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2Ciii","Step 2C iii. Recalculate PW averages.\r(See effect of removing runs from mass avgs, redo avg fits)")

	killstrings/z root:HR_PeakShape:Step2B
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2B","Step 2B - Select/Deselect HR ionss")

	sq_initializeGlobalSVar("root:HR_PeakShape:Step2Tool","Tool for Step 2 - View raw spectra and gaussian fits of one run.")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2ToolZ","Black=Data, Red=Fit,Yellow=Mass Cal fit")
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2ToolX","")
	svar Step2ToolX = root:HR_PeakShape:Step2ToolX
	Step2ToolX="Unlike Step 2 A, B, C, the results from this tool are only used for display purposes.\r"
	Step2ToolX+="In the graph column: 0 = don't display, >0 = display, max value = display and center here when popped."
	
	sq_initializeGlobalSVar("root:HR_PeakShape:Step3Instruct","Step 3. Finalize parameters for peak width function\r and select functional form.")

	// these are for the display of the PW tab of the panel
	sq_initializeGlobalNVar("root:HR_PeakShape:HR_displayFinalPWSlope", nan)
	 sq_initializeGlobalNVar("root:HR_PeakShape:HR_displayFinalPWIntercept", nan)

	sq_initializeGlobalNVar("root:HR_PeakShape:HR_displayFinalPowA", nan)	
	sq_initializeGlobalNVar("root:HR_PeakShape:HR_displayFinalPowB", nan)	
	sq_initializeGlobalNVar("root:HR_PeakShape:HR_displayFinalPowC", nan)	

	// these are set in HR_PeakWidthSanity() and are used in calcs
	sq_initializeGlobalNVar("root:HR_PeakShape:HR_FinalPWSlope", nan)
	sq_initializeGlobalNVar("root:HR_PeakShape:HR_FinalPWIntercept", nan)
	sq_initializeGlobalNVar("root:HR_PeakShape:HR_FinalPWPower", nan)
	sq_initializeGlobalSVar("root:HR_PeakShape:HR_FinalPSName", "")

	sq_initializeGlobalSVar("root:HR_PeakShape:Step2PopMenuB", "<;>")	
	sq_initializeGlobalSVar("root:HR_PeakShape:Step2PopMenuA", "GetList")	
	sq_initializeGlobalNVar("root:HR_PeakShape:Step2Var", 10)	

	sq_initializeGlobalNVar("root:HR_PeakShape:PWUseLineOrPow", 0)		//1.16F	
	sq_initializeGlobalSVar("root:HR_PeakShape:PS_PWParams", "")
	sq_initializeGlobalSVar("root:HR_PeakShape:PS_MassList", "")

	sq_initializeGlobalSVar("root:HR_PeakShape:PW_massAddStrHelp","Do not add/remove masses or change spec type in Step 2.")		// 1.06D

	variable initialize = !Exists("root:HR_PeakShape:PW_VmodeMask")
	hr_initializeGlobalWave("root:HR_PeakShape:PW_VmodeMask", num2str(numpnts(root:GaussFits:GFmass)), 1)
	hr_initializeGlobalWave("root:HR_PeakShape:PW_WmodeMask", num2str(numpnts(root:GaussFits:GFmass)), 1)
	if (initialize)
		PW_SetDefaults()
	endif
	
	hr_initializeGlobalWave("root:HR_PeakShape:PW_RunMask", num2istr(numpnts(root:index:t_series)), 1)  //  1.20O

	sq_initializeGlobalNVar("root:HR_PeakShape:PW_viewVmode", 1)	
	sq_initializeGlobalNVar("root:HR_PeakShape:PS_viewVmode", 1)	

	sq_initializeGlobalNVar("root:HR_PeakShape:specType", 2)
	
	sq_initializeGlobalSVar("root:HR_PeakShape:TitlePWGraph","Peak Width Graph")	
	sq_initializeGlobalSVar("root:HR_PeakShape:TitleAvgPWGraph","Average Peak Width and Fit Graph")

	sq_initializeGlobalSVar("root:HR_PeakShape:pw_massList", "")

	sq_initializeGlobalSVar("root:HR_PeakShape:PW_runModeStr", "")

	sq_initializeGlobalSVar("root:HR_PeakShape:GF_addMassList", "")		//1.06F

	// checkboxes
	sq_initializeGlobalNVar("root:HR_PeakShape:ShowPSCandidate", 1)	
	sq_initializeGlobalNVar("root:HR_PeakShape:ShowPSData", 0)	
	sq_initializeGlobalNVar("root:HR_PeakShape:ShowPSAvg", 1)	
	
	sq_initializeGlobalSVar("root:HR_PeakShape:PS_MassesNotDisplayedList", "")
	sq_initializeGlobalSVar("root:HR_PeakShape:PWTodoModeStr", "")

	sq_initializeGlobalSVar("root:HR_PeakShape:Step3Optional", "Step 3A. (Optional) View results:")

	//waves
	hr_initializeGlobalWave("root:HR_PeakShape:PeakShapeCandidate",  num2str(NumPtsFinalPSWave), nan)	

	// for display
	hr_initializeGlobalWave("root:HR_PeakShape:PW_fitAvgPW_X", "100",nan)	
	hr_initializeGlobalWave("root:HR_PeakShape:PW_fitAvgPWPowY", "100", nan)	
	hr_initializeGlobalWave("root:HR_PeakShape:PW_fitAvgPWLineY", "100", nan)	
	
	PW_makePeakWidthWaves()
	PW_makeFilledMarkerWave()
	PW_makeColorMx()	

	// pw parameter results, for use for folks upgrading experiments
	wave rn_series = root:index:rn_series
	nvar PWUseLineOrPow = root:HR_PeakShape:PWUseLineOrPow	
	nvar/z HR_VModeFinalSlope = root:HR_PeakShape:HR_VModeFinalSlope
	nvar/z HR_VModeFinalIntercept = root:HR_PeakShape:HR_VModeFinalIntercept
	nvar/z HR_WModeFinalSlope = root:HR_PeakShape:HR_WModeFinalSlope
	nvar/z HR_WModeFinalIntercept = root:HR_PeakShape:HR_WModeFinalIntercept
	variable numRuns = numpnts(rn_series)

	wave/z HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave

	// for updating from 1.04, assumes that there is only one set per V and W
	if (!WaveExists(HR_FinalPWSlopeWave)  && NVAR_Exists(HR_VModeFinalSlope) )		// assume that if one of them does not exist, none of them do
		hr_initializeGlobalWave("root:HR_PeakShape:HR_FinalPWInterceptWave", num2istr(numRuns),nan)	//  1.20O	
		hr_initializeGlobalWave("root:HR_PeakShape:HR_FinalPWSlopeWave", num2istr(numRuns), nan) //  1.20O		
		hr_initializeGlobalWave("root:HR_PeakShape:HR_FinalPWPowerWave", num2istr(numRuns), nan) //  1.20O		
		hr_initializeGlobalTextWave("root:HR_PeakShape:HR_FinalPSNameWave", numRuns, "")	
		wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
		wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
		wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave
		wave/z ToFTypeVKey = root:diagnostics:ToFTypeVKey
		wave/z ToFTypeWKey = root:diagnostics:ToFTypeWKey
		nvar/z VWSwitching= root:diagnostics:VWSwitching
		nvar/z isV = root:diagnostics:isV
		WaveStats/Q HR_FinalPWInterceptWave
		if (V_numNans ==numRuns)	// brand new wave, initialize
			if (VWSwitching==1)		//  
				HR_FinalPWSlopeWave = HR_VModeFinalSlope*ToFTypeVKey +  HR_WModeFinalSlope*ToFTypeWKey
				HR_FinalPWInterceptWave = HR_VModeFinalIntercept*ToFTypeVKey +  HR_WModeFinalIntercept*ToFTypeWKey
				HR_FinalPWPowerWave =1		// assume a line, power = 1 						
			elseif(isV)
				HR_FinalPWSlopeWave = HR_VModeFinalSlope
				HR_FinalPWInterceptWave = HR_VModeFinalIntercept
				HR_FinalPWPowerWave =1		// assume a line, power = 1 						
			else
				HR_FinalPWSlopeWave = HR_WModeFinalSlope
				HR_FinalPWInterceptWave = HR_WModeFinalIntercept
				HR_FinalPWPowerWave =1		// assume a line, power = 1 						
			endif
		endif	
			
	endif
	
 	hr_initializeGlobalWave("root:HR_PeakShape:HR_FinalPWInterceptWave", num2istr(numRuns),nan)  //  1.20O		
	hr_initializeGlobalWave("root:HR_PeakShape:HR_FinalPWSlopeWave", num2istr(numRuns), nan) //  1.20O		
	hr_initializeGlobalWave("root:HR_PeakShape:HR_FinalPWPowerWave", num2istr(numRuns), nan) //  1.20O		
	hr_initializeGlobalTextWave("root:HR_PeakShape:HR_FinalPSNameWave", numRuns, "")	
	
	PW_makePeakWidthWaves()
 
End


//  Makes all the waves that are number of masses-dependent. (i.e. someone adds a mass to the table)
Function PW_makePeakWidthWaves()

	wave/T GFName =  root:GaussFits:GFName
	wave GFMass =  root:GaussFits:GFMass
	wave rn_series =  root:index:rn_series
	
	variable numRuns, numMasses
	string numRunStr, numMassesStr
	
	numRuns = numpnts(rn_series)
	numRunStr = num2istr(numRuns)  //  1.20O
	numMasses = numpnts(GFMass)
	numMassesStr = num2str(numMasses)  	
	
	hr_initializeGlobalWave("root:HR_PeakShape:PW_Graph", numMassesStr, 1)

	hr_initializeGlobalWave("root:HR_PeakShape:IsolatedIon_PWAvg", numMassesStr, nan)
	hr_initializeGlobalWave("root:HR_PeakShape:IsolatedIon_PWStD", numMassesStr, nan)
	
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunIsolatedIonPC", numMassesStr, nan)
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunIsolatedIonPW", numMassesStr, nan)
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunIsolatedIonFitPW", numMassesStr, nan)
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunPeakWidthVmode", numMassesStr, nan)
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunPeakWidthWmode", numMassesStr, nan)
		
	hr_initializeGlobalWave("root:HR_PeakShape:RawSpectraThisRun", "0", nan)

	hr_initializeGlobalWave("root:HR_PeakShape:OneRunIsolatedPC_fromGaus", numMassesStr, nan)	// center line from gaussian fit
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunDisplayPC_fromMC",numMassesStr, nan)		// center line from mass cal
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunDisplayPC_Y	", numMassesStr, 1e9)			// arbitrary y value for plotting

	hr_initializeGlobalWave("root:HR_PeakShape:OneRunPeakWidths", numMassesStr, nan)			// arbitrary y value for plotting
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunPeakWidthsFitLine", numMassesStr, nan)			// arbitrary y value for plotting
	hr_initializeGlobalWave("root:HR_PeakShape:OneRunPeakWidthsFitPower", numMassesStr, nan)			// arbitrary y value for plotting

	hr_initializeGlobalWave("root:GaussFits:GausCoeffsMx", numMassesStr+";4"  , nan)		// for one run

	// waves for plotting on PWGraph
	hr_initializeGlobalWave("root:HR_PeakShape:DisplayPWAvg", numMassesStr+";2", nan)  // 1.04P
	make/o/n=2 root:HR_PeakShape:DisplayPWAvgX
	wave DisplayPWAvgX = root:HR_PeakShape:DisplayPWAvgX
	wave rn_series = root:index:rn_series
	DisplayPWAvgX[0] = rn_series[0]
	DisplayPWAvgX[1] = rn_series[numpnts(rn_series)-1]
	
	make/o/n=2 root:HR_PeakShape:PWCoeffs
	make/o/t/n=4 root:HR_PeakShape:PWConstraintsPower = {"K0>0","K1>0","K2>0", "K2<0.99"} 		// somewhat arbitrary, reset in 1.19E, K2<5 1.16K	
	make/o/t/n=4 root:HR_PeakShape:PWConstraintsLine = {"K0>0","K1>0","K0<10", "K1<10"} 		// somewhat arbitrary 1.16K	

End


Function PW_PeakWidthShapePanel()

	sq_initializeGlobalSVar("root:HR_PeakShape:GFTableTitle","For faster results, change the column Spec to be nonzero only for the HR ions you will use.")

	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(281,197,1209,783)/N=PeakWidthShapePanel as "PeakWidthShapePanel"
	ModifyPanel fixedSize=1
	ShowTools/A
	SetDrawLayer UserBack
	SetDrawEnv fstyle= 1, textrgb= (0,0,0)
	DrawText 320,20,"Peak Width = f(m/z)"
	SetDrawEnv fstyle= 1, textrgb= (0,0,0)
	DrawText 320,40,"Peak Shape = f(scaled distance from peak center)"
	SetDrawEnv fsize= 14,fstyle= 1
	DrawText 2,20,"Peak Width (PW), Peak Shape (PS) Panel "+PKVERSION
	
// stuff at the top of panel that always appears
	TitleBox PSPW_title_RawSpecSqBaseline pos={740,3}, variable=root:HR_PeakShape:PrelimUseSqBaseline
	TitleBox PSPW_title_RawSpecSqBaseline frame=0,fSize=10
	
//1.14M
	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	svar sq_toDoList = root:panel:sq_toDoList

	PopupMenu PSPW_popup_ToDoWave,pos={525,5},size={190,20},bodyWidth=130,proc=sq_pop_toDoList,title="ToDo Wave"
	PopupMenu PSPW_popup_ToDoWave,value= #"root:panel:sq_toDoList+\"Get List;\""		//1.14M
	PopupMenu PSPW_popup_ToDoWave,fSize=10, mode=1+ WhichListItem(sq_toDoWvNm, sq_toDoList)

	TabControl PWPS_tab,pos={5,20},size={920,560},proc=PWPSTabProc
	TabControl PWPS_tab,tabLabel(0)="Gauss Fits",tabLabel(1)="Peak Width",tabLabel(2)="Peak Shape",value= 0

// Gauss Fits
	TitleBox GF_title_Step1,pos={12,99},size={388,13},frame=0,fStyle=1
	TitleBox GF_title_Step1,variable= root:HR_PeakShape:Step1Instruct
	TitleBox GF_title_Step1A,pos={428,144},size={450,286},fSize=10,frame=0
	TitleBox GF_title_Step1A,variable= root:GaussFits:GFInstruct
	TitleBox GF_title_specType,pos={10,118},size={253,13},fSize=10,frame=0
	TitleBox GF_title_specType,variable= root:GaussFits:TitleSpectraType
	TitleBox GF_title_Step2,pos={430,441},size={338,13},frame=0,fStyle=0
	TitleBox GF_title_Step2,variable= root:HR_PeakShape:Step1InstructOptions
	TitleBox GF_titleMain,pos={9,53},size={880,40},fSize=14,frame=0, fstyle=1
	TitleBox GF_titleMain,variable=root:GaussFits:GFMainInstruct
	TitleBox GF_title_table,pos={12,139},size={128,13},frame=0,fStyle=0, fsize=9
	TitleBox GF_title_table,variable= root:HR_PeakShape:GFTableTitle

	Button GF_button_FindPeakWidth,pos={430,120},size={300,20},proc=GF_butt_GFsTodo,title="Perform Gaussian fits of HR ions using settings in table."
	Button GF_button_FindPeakWidth,help={"Pressing this button will begin the gaussian fit routine."}
	Button GF_button_FindPeakWidth,fsize=10,fStyle=0,fColor=(65280,43520,0)
	Button GF_button_SetDefault,pos={430,471},size={250,20},proc=GF_butt_SetGFDefaults,title="Recreate all gauss fit table waves; set to defaults"
	Button GF_button_SetDefault,fSize=10
	Button GF_but_GF_MassTable,pos={372,116},size={40,20},proc=sq_butt_popThis,title="Pop"
	Button GF_but_GF_MassTable,fSize=10
	Button GF_button_AddMass,pos={430,537},size={300,20},proc=GF_butt_addMass,title="Search list of all masses and add mass below to list at right"
	Button GF_button_AddMass,fSize=10
	Button GF_button_SetMaskTo0,pos={430,505},size={130,20},proc=GF_butt_setAllMaskTo0,title="Set all spectra values to"
	Button GF_button_SetMaskTo0,fSize=10
	SetVariable GF_AddList,pos={430,557},size={160,16},title="Find/add mass"
	SetVariable GF_AddList,value= root:HR_PeakShape:GF_addMassList
	PopupMenu GF_popup_setToVal,pos={576,505},size={74,21}
	PopupMenu GF_popup_setToVal,mode=1,popvalue="Initialize",value= #"\"Initialize;0 = Don't Use;1 = Diff;2 = Closed;3 = Open\""
	CheckBox GF_showdetails,pos={896,57},size={21,14},proc=GF_ck_showGFdetails,title=" "
	CheckBox GF_showdetails,variable= root:HR_PeakShape:GF_showGFDetails

// Peak widths
	TitleBox PW_title_Step2,pos={12,50},size={814,15},frame=0,fStyle=1
	TitleBox PW_title_Step2,variable= root:HR_PeakShape:Step2Instruct

//	TitleBox PW_title_Step1E,pos={15,270},size={33,15},fsize=10,frame=0
//	TitleBox PW_title_Step1E,variable= root:HR_PeakShape:PWModeStr
	
	TitleBox PW_title_Step2B,pos={20,305},size={235,15},fsize=10,frame=0,fStyle=0
	TitleBox PW_title_Step2B,variable= root:HR_PeakShape:Step2B
	TitleBox PW_title_Step2Bi,pos={20,325},size={262,13},fSize=10,frame=0
	TitleBox PW_title_Step2Bi,variable= root:HR_PeakShape:Step2Bi, fsize=10
	
	TitleBox PW_title_Step2C,pos={15,375},size={222,15},fsize=10,frame=0
	TitleBox PW_title_Step2C,variable= root:HR_PeakShape:Step2C
	TitleBox PW_title_Step2Ci,pos={15,429},size={284,13},fSize=10,frame=0
	TitleBox PW_title_Step2Ci,variable= root:HR_PeakShape:Step2Ci
	TitleBox PW_title_Step2Cii,pos={15,497},size={125,13},fSize=10,frame=0
	TitleBox PW_title_Step2Cii,variable= root:HR_PeakShape:Step2Cii
	TitleBox PW_title_Step2Ciii,pos={18,390},size={231,13},fSize=10,frame=0
	TitleBox PW_title_Step2Ciii,variable= root:HR_PeakShape:Step2Ciii

	TitleBox PW_title_Step2Tool,pos={325,75},size={388,15},fsize=10,frame=0
	TitleBox PW_title_Step2Tool,variable= root:HR_PeakShape:Step2Tool
	TitleBox PW_title_Step2ToolX,pos={325,90},size={366,13},fSize=10,frame=0
	TitleBox PW_title_Step2ToolX,variable= root:HR_PeakShape:Step2ToolX
	TitleBox PW_title_Step2Toolz variable=:HR_PeakShape:Step2ToolZ,fSize=10
	TitleBox PW_title_Step2Toolz pos={323,115},fSize=10,frame=0
	
	TitleBox PW_title_PWGraph,pos={305,282},size={88,13},fSize=10,frame=0
	TitleBox PW_title_PWGraph,variable= root:HR_PeakShape:TitlePWGraph

	GroupBox PW_groupStep2Tool,pos={320,70},size={595,190}
	
	TitleBox PW_title_specType variable=root:HR_PeakShape:PW_massAddStrHelp,frame=0,fSize=10
	TitleBox PW_title_specType pos={8,70}
	Button PW_but_PW_MassTable pos={290,68},size={30,20},proc=sq_butt_popThis,title="Pop"
	Button PW_but_PW_MassTable fSize=10,fStyle=0

//  individual run peaks
	Button PW_button_OneRun,pos={740,75},size={110,20},proc=PW_butt_showPWOneRun,title="Gauss fits for this run"
	Button PW_button_OneRun,fSize=10,fStyle=0
	SetVariable PW_setvar_CurrentRunNum,pos={850,75},size={60,16},proc=PS_SetVar_RunModeStr,title=" "
	SetVariable PW_setvar_CurrentRunNum,fSize=10
	SetVariable PW_setvar_CurrentRunNum,value= root:HR_PeakShape:PW_CurrentRunNum
	CheckBox PW_viewClosed,pos={ 635,73},size={50,14},title="Closed",fSize=10
	CheckBox PW_viewClosed,value= 1,mode=1, proc=PW_ck_OneRunOpenClosed
	CheckBox PW_viewOpen,pos={690,73},size={44,14},title="Open",fSize=10
	CheckBox PW_viewOpen,value= 0,mode=1,proc=PW_ck_OneRunOpenClosed

	TitleBox PW_title_runModeStrIndivRunRem,pos={810,95},size={75,13}
	TitleBox PW_title_runModeStrIndivRunRem,fSize=10,frame=0
	TitleBox PW_title_runModeStrIndivRunRem,variable= root:HR_PeakShape:PW_runModeStr

	Button PW_but_PW_OneRunGraph,pos={785,115},size={90,20},proc=sq_butt_popThis,title="Pop raw spectra"
	Button PW_but_PW_OneRunGraph,fSize=10,fStyle=0
	Button PW_but_PW_FitRunPWMassGraph,pos={885,115},size={30,20},proc=sq_butt_popThis,title="Pop"
	Button PW_but_PW_FitRunPWMassGraph,fSize=10,fStyle=0

	TitleBox PW_title_PWFitButNotDisplayed,pos={325,245},size={249,13},fSize=10
	TitleBox PW_title_PWFitButNotDisplayed,frame=0
	TitleBox PW_title_PWFitButNotDisplayed,variable= root:HR_PeakShape:PS_MassesNotDisplayedList

// step 2 things
	GroupBox PW_groupStep2A,pos={10,321},size={290,62}
	GroupBox PW_groupStep2B,pos={10,266},size={290,51}
	GroupBox PW_groupStep2C,pos={10,390},size={290,190}

	Button PW_but_PW_PeakWidthGraph,pos={591,272},size={30,20},proc=sq_butt_popThis,title="Pop"
	Button PW_but_PW_PeakWidthGraph,fSize=10,fStyle=0
	Button PW_but_PW_ColorLegend,pos={301,262},size={110,18},proc=PW_butt_PWColorLegend,title="HR ion color legend"
	Button PW_but_PW_ColorLegend,fSize=10
	Button PW_but_PW_TimeVaryingPW,pos={418,273},size={160,20},proc=PW_TimeVayingPW_butt,title="Adv. Users: Time Varying PW"
	Button PW_but_PW_TimeVaryingPW,fSize=10

	TitleBox PW_title_Step2A,pos={15,269},size={193,15},fsize=10,frame=0,fStyle=0
	TitleBox PW_title_Step2A,variable= root:HR_PeakShape:Step2A
	TitleBox PW_title_Step2ShowMode,pos={198,294},size={94,13},fSize=10,frame=0,fStyle=0
	TitleBox PW_title_Step2ShowMode,variable= root:HR_PeakShape:PWTodoModeStr
	TitleBox PW_title_Step2B,pos={15,323},size={168,15},fsize=10,frame=0,fStyle=0
	TitleBox PW_title_Step2B,variable= root:HR_PeakShape:Step2B
	TitleBox PW_title_Step2Bi,pos={15,342},size={168,15},fsize=10,frame=0,fStyle=0
	TitleBox PW_title_Step2Bi,variable= root:HR_PeakShape:Step2Bi
	TitleBox PW_title_Step2C,pos={15,394},size={199,15},fsize=10,frame=0
	TitleBox PW_title_Step2C,variable= root:HR_PeakShape:Step2C
	TitleBox PW_title_Step2Ci,pos={15,413},size={206,13},fSize=10,frame=0
	TitleBox PW_title_Step2Ci,variable= root:HR_PeakShape:Step2Ci
	TitleBox PW_title_Step2Cii,pos={15,486},size={131,13},fSize=10,frame=0
	TitleBox PW_title_Step2Cii,variable= root:HR_PeakShape:Step2Cii
	TitleBox PW_title_Step2Ciii,pos={15,528},size={212,26},fSize=10,frame=0
	TitleBox PW_title_Step2Ciii,variable= root:HR_PeakShape:Step2Ciii
	TitleBox PW_title_runModeStrIndivRunRe,pos={180,503},size={98,13},fSize=10
	TitleBox PW_title_runModeStrIndivRunRe,frame=0
	TitleBox PW_title_runModeStrIndivRunRe,variable= root:HR_PeakShape:PW_runModeStr
	SetVariable PW_setvar_limitValue,pos={185,428},size={50,18},title=" "
	SetVariable PW_setvar_limitValue,format="%1.2f"
	SetVariable PW_setvar_limitValue,limits={0,10,0.001},value= root:HR_PeakShape:Step2Var
	PopupMenu PW_popupMassList,pos={15,428},size={100,21},bodyWidth=100,proc=PW_PopMenuPWList, popValue = "Get List"		//1.05 fix
	PopupMenu PW_popupMassList,mode=1,value= #"root:HR_PeakShape:pw_massList+\"Get List;\""
	PopupMenu PW_popGTorLT,pos={130,428},size={43,21}
	PopupMenu PW_popGTorLT,mode=1,popvalue="<",value= #"\"<;>\""
	Button PW_button_PWSet1,pos={13,289},size={185,20},proc=PW_butt_PWSet1,title="Reset PW mask wave to 1 in todo"
	Button PW_button_PWSet1,fColor=(65280,43520,0)
	Button PW_button_UpdatePWGraph,pos={13,360},size={250,20},proc=PW_butt_UpdatePWgraph,title="Update peak width graph with m/z choices"
	Button PW_button_UpdatePWGraph,fColor=(65280,43520,0)
	Button PW_button_PWRecalcAvgs,pos={15,555},size={200,20},proc=PW_butt_RecalcPWAvgs,title="Recalculate average mass peak widths"
	Button PW_button_PWRecalcAvgs,fSize=10,fStyle=0
	Button PW_button_PWMaskModify,pos={15,453},size={190,20},proc=PW_butt_RemoveTheseRuns,title="Remove runs that make condition true"
	Button PW_button_PWMaskModify,fSize=10,fStyle=0
	SetVariable PW_setvar_CurrentRunNumRemove,pos={175,488},size={120,16},proc=PS_SetVar_RunModeStr,title="Run number"
	SetVariable PW_setvar_CurrentRunNumRemove,fSize=10
	SetVariable PW_setvar_CurrentRunNumRemove,value= root:HR_PeakShape:PW_CurrentRunNum
	Button PW_button_PWMaskRemoveRun,pos={15,498},size={155,20},proc=PW_butt_RemoveRunPW,title="Remove this run from PW mask"
	Button PW_button_PWMaskRemoveRun,fSize=10,fStyle=0

	// Step 3 things
	TitleBox PW_title_Step3,pos={630,266},size={268,15},frame=0,fStyle=1
	TitleBox PW_title_Step3,variable= root:HR_PeakShape:Step3Instruct
	TitleBox PW_title_Step3Opt,pos={634,554},size={87,15},fSize=10,frame=0
	TitleBox PW_title_Step3Opt,variable= root:HR_PeakShape:Step3Optional
//	TitleBox PW_title_Step3B,pos={325,115},size={209,13},fSize=10,frame=0
//	TitleBox PW_title_Step3B,variable= root:HR_PeakShape:Step3InstructB
	TitleBox PW_title_AvgPWGraph,pos={635,405},size={163,13},fSize=10,frame=0
	TitleBox PW_title_AvgPWGraph,variable= root:HR_PeakShape:TitleAvgPWGraph	
//	TitleBox PS_titleAreaGauss title="Area under perfect gauss = sqrt(pi) = 1.77245", pos={590,561}, frame=0, fsize=10

	GroupBox PW_groupStep3,pos={625,265},size={295,315}

	Button PW_button_declarePeakWidth,pos={630,295},size={280,20},proc=PW_butt_DeclarePWParams,title="Declare these PW parameters for this todo"
	Button PW_button_declarePeakWidth,fColor=(65280,43520,0)

	CheckBox PW_line,pos={627,317},size={218,14},proc=pw_ck_LineOrPow, title="Line: PW = a + b* mz"
	CheckBox PW_line,fSize=10,fStyle=0,value= 0,mode=1		
	CheckBox PW_pow,pos={627,332},size={280,14},proc=pw_ck_LineOrPow,title="Power: PW = A + B*mz^C; C>0.3, C<0.7"
	CheckBox PW_pow,fSize=10,fStyle=0,value= 1,mode=1		//1.12O
	SetVariable PW_setvar_Fitintercept,pos={639,350},size={100,16},disable=2,title="a"
	SetVariable PW_setvar_Fitintercept,fSize=10, format="%1.4f"
	SetVariable PW_setvar_Fitintercept,limits={0,10,0.001},value= root:HR_PeakShape:HR_displayFinalPWIntercept
	SetVariable PW_setvar_FitSlope,pos={639,367},size={100,16},disable=2,title="b"
	SetVariable PW_setvar_FitSlope,fSize=10
	SetVariable PW_setvar_FitSlope,format="%1.4f"
	SetVariable PW_setvar_FitSlope,limits={0,10,0.001},value= root:HR_PeakShape:HR_displayFinalPWSlope
	SetVariable PW_setvar_PowA,pos={748,351},size={80,16},title="A"
	SetVariable PW_setvar_PowA,fSize=10,format="%1.4f"
	SetVariable PW_setvar_PowA,limits={0,10,0.001},value= root:HR_PeakShape:HR_displayFinalPowA
	SetVariable PW_setvar_PowB,pos={748,368},size={80,16},title="B"
	SetVariable PW_setvar_PowB,fSize=10,format="%1.4f"
	SetVariable PW_setvar_PowB,limits={0,10,0.001},value= root:HR_PeakShape:HR_displayFinalPowB
	SetVariable PW_setvar_PowC,pos={748,385},size={80,16},title="C"
	SetVariable PW_setvar_PowC,fSize=10,format="%1.4f"
	SetVariable PW_setvar_PowC,limits={0,10,0.001},value= root:HR_PeakShape:HR_displayFinalPowC

	Button PW_button_recalcFinalPW,pos={840,360},size={60,35},proc=PW_butt_recalcDisplayFinalPW,title="Recalc\rFinal PW"
	Button PW_button_recalcFinalPW,fSize=10,fStyle=0
	
	Button PW_but_PW_AvgPWTable,pos={799,402},size={80,18},proc=sq_butt_popThis,title="View as Table"
	Button PW_but_PW_AvgPWTable,fSize=10,fStyle=0
	Button PW_but_PW_AvgPWGraph,pos={880,402},size={35,18},proc=sq_butt_popThis,title="Pop"
	Button PW_but_PW_AvgPWGraph,fSize=10,fStyle=0
	
	Button PW_butt_AllPWTable,pos={724,561},size={90,18},proc=PW_butt_PWParamTable,title="PW Param Table"
	Button PW_butt_AllPWTable,fSize=10,fStyle=0
	Button PW_butt_AllPWGraph,pos={822,561},size={90,18},proc=PW_butt_PWParamGraph,title="PW Param Graph"
	Button PW_butt_AllPWGraph,fSize=10,fStyle=0

//	PW_plotAllPWplots()

// 1.13C
	nvar PWUseLineOrPow = root:HR_PeakShape:PWUseLineOrPow
	if (PWUseLineOrPow)
		PW_ck_LineOrPow("PW_line",1)
	endif

	///////////////////	  now for PS stuff 	//////////////////////////////////////////////// 

	GroupBox PS_step4,pos={10,50},size={565,530}
	GroupBox PS_group4a,pos={15,172},size={555,98}
	GroupBox PS_group5,pos={580,51},size={340,530}

	TitleBox PS_titleStep4,pos={15,52},size={389,30},frame=0,fStyle=1
	TitleBox PS_titleStep4,variable= root:HR_PeakShape:Step4
	TitleBox PS_titleStep4Prelim,pos={407,56},size={163,60},fSize=10,frame=2
	TitleBox PS_titleStep4Prelim,fStyle=0,variable= root:HR_PeakShape:Step4Prelim
	TitleBox PS_titleStep4,variable= root:HR_PeakShape:Step4
	TitleBox PS_titleStep4a,pos={20,178},size={474,15},frame=0,fStyle=1
	TitleBox PS_titleStep4a,variable= root:HR_PeakShape:Step4a
	TitleBox PS_titleStep5,pos={590,52},size={261,15},frame=0,fStyle=1
	TitleBox PS_titleStep5,variable= root:HR_PeakShape:Step5

	ValDisplay PS_ValDisplayNumPnts,pos={20,85},size={220,15},title="Number of points in PS candidates:"
	ValDisplay PS_ValDisplayNumPnts,fSize=10,fStyle=2
	ValDisplay PS_ValDisplayNumPnts,limits={0,0,0},barmisc={0,1000}
	ValDisplay PS_ValDisplayNumPnts,value= #"root:HR_PEakShape:NumPtsFinalPSWave"
	ValDisplay PS_ValDisplayMaxMinSigma,pos={251,85},size={130,15},title="Max and -Min sigma:"
	ValDisplay PS_ValDisplayMaxMinSigma,fSize=10,fStyle=2
	ValDisplay PS_ValDisplayMaxMinSigma,limits={0,0,0},barmisc={0,1000}
	ValDisplay PS_ValDisplayMaxMinSigma,value= #"root:HR_PEakShape:MaxSigma"
//	CheckBox PS_check_HoldPW,pos={20,103},size={274,14},title="Hold peak width to final values found in Step 2? (rare)"		//1.09H
//	CheckBox PS_check_HoldPW,fSize=10,fStyle=2
//	CheckBox PS_check_HoldPW,variable= root:HR_PeakShape:PS_holdWidth
	Button PS_button_PSAllRun,pos={10,148},size={370,20},proc=PS_butt_PeakShapeAllMassTodo,title="Calculate peak shape data for all masses in current todo wave"
	Button PS_button_PSAllRun,fColor=(65280,43520,0)

	Button PS_button_ConfirmTodo,pos={10,122},size={160,20},proc=PS_butt_confirmTodo,title="Confirm todo wave choice"
	Button PS_button_ConfirmTodo,fColor=(65280,43520,0)		//1.05A make gold
	TitleBox PS_titlePWParams,pos={180,133},size={348,13},fSize=10,frame=0,fStyle=0
	TitleBox PS_titlePWParams,variable= root:HR_PeakShape:PS_PWParams
//	TitleBox PS_titleBox_PSModeStr,pos={162,120},size={42,13},fSize=10,frame=0,fStyle=0
//	TitleBox PS_titleBox_PSModeStr,variable= root:HR_PeakShape:PSModeStr
	TitleBox PS_title_ModeMassList,pos={180,120},size={345,13},fSize=10,frame=0
	TitleBox PS_title_ModeMassList,fStyle=0
	TitleBox PS_title_ModeMassList,variable= root:HR_PeakShape:Step4MassList
	Button PS_printHRIons,pos={437.00,146.00},size={130.00,20.00},proc=PS_Butt_Print,title="Print list of HR ions used"
	Button PS_printHRIons,fSize=10		//1.20E

	Button PS_PWLegend,pos={275,226},size={110,20},proc=PS_butt_PWColorLegend,title="Pop m/z color legend"
	Button PS_PWLegend,fSize=10
	Button PS_button_recalcPSLR,pos={392,222},size={170,40},proc=PS_butt_recalcPSLeftRight,title="Find new PS candidate with\rchosen left and right masses"
	Button PS_button_recalcPSLR,fColor=(65280,43520,0)
	SetVariable PS_leftList,pos={21,194},size={240,16},title="Left side masses:"		//1.08F
	SetVariable PS_leftList,fSize=10,value= root:HR_PeakShape:PS_massList_left
	SetVariable PS_rightList,pos={265,194},size={240,16},title=" Right side masses:"//1.08F
	SetVariable PS_rightList,fSize=10,value= root:HR_PeakShape:PS_massList_right
	CheckBox PS_check_smooth,pos={25,214},size={200,14},title="Smooth peak shape (with 3 pt boxcar)?"
	CheckBox PS_check_smooth,fSize=10,variable= root:HR_PeakShape:PS_ToSmoothOrNot
	CheckBox PS_check_monotonic,pos={24,233},size={225,14},title="Enforce y value monotonicity at x < -3, x > 3"
	CheckBox PS_check_monotonic,fSize=10
	CheckBox PS_check_monotonic,variable= root:HR_PeakShape:PS_enforceMonotonicity
	CheckBox PS_check_minmaxValuesAreGauss,pos={24,250},size={248,14},title="Minimum y values at x < -3, x > 3 be >= gaussian"
	CheckBox PS_check_minmaxValuesAreGauss,fSize=10
	CheckBox PS_check_minmaxValuesAreGauss,variable= root:HR_PeakShape:PS_enforceNotGTGauss

	CheckBox PS_check_RightAxisLog,pos={22,300},size={79,14},proc=PS_check_PSGraphLog,title="Left Axis Log"
	CheckBox PS_check_RightAxisLog,fSize=10
	CheckBox PS_check_RightAxisLog,variable= root:HR_PeakShape:PS_logLeftAxisCandidPS
	CheckBox PS_check_LeftAxisLogFinalPS,pos={599,149},size={79,14},proc=PS_check_PSGraphLog,title="Left Axis Log"
	CheckBox PS_check_LeftAxisLogFinalPS,fSize=10
	CheckBox PS_check_LeftAxisLogFinalPS,variable= root:HR_PeakShape:PS_logLeftAxisFinalPS
	Button PS_but_PS_PeakShapeGraph,pos={147,538},size={40,20},proc=sq_butt_popThis,title="Pop"
	Button PS_but_PS_PeakShapeGraph,fSize=10
	Button PS_button_editCandidPS,pos={16,411},size={160,20},proc=PS_butt_editPS,title="Edit/view peak shape candidate"
	Button PS_button_editCandidPS,fSize=10

	Button PS_button_DeclareFinalPS,pos={587,89},size={320,20},proc=PS_butt_declareFinalPeakShape,title="Declare candidate as final peak shape for runs in todo"
	Button PS_button_DeclareFinalPS,fColor=(65280,43520,0)
	SetVariable PS_SetVar_PSNameSuffix,pos={590,72},size={330,16},title="PS Name (with prefix of 'FinalPS_V_(orW)'; blank = todo"
	SetVariable PS_SetVar_PSNameSuffix,fSize=10
	SetVariable PS_SetVar_PSNameSuffix,value= root:HR_PeakShape:PS_nameSuffix
	CheckBox PS_check_showGaussData,pos={599,184},size={124,14},proc=PS_check_ShowHideFinalPS,title="Show final peak shape"
	CheckBox PS_check_showGaussData,fSize=10
	CheckBox PS_check_showGaussData,variable= root:HR_PeakShape:PS_ShowFinal
	CheckBox PS_check_showResidData,pos={599,167},size={88,14},proc=PS_check_ShowHideFinalPS,title="Show residuals"
	CheckBox PS_check_showResidData,fSize=10
	CheckBox PS_check_showResidData,variable= root:HR_PeakShape:PS_ShowResid
	CheckBox PS_check_showPSCand,pos={20,343},size={147,14},proc=PS_ck_ShowWaves,title="View peak shape candidate"
	CheckBox PS_check_showPSCand,fSize=10
	CheckBox PS_check_showPSCand,variable= root:HR_PeakShape:ShowPSCandidate
	CheckBox PS_check_showVPSNormAvg,pos={20,362},size={165,14},proc=PS_ck_ShowWaves,title=" View m/z peak shape averages"
	CheckBox PS_check_showVPSNormAvg,fSize=10
	CheckBox PS_check_showVPSNormAvg,variable= root:HR_PeakShape:ShowPSAvg
	CheckBox PS_check_showVPSNormData,pos={20,381},size={172,14},proc=PS_ck_ShowWaves,title="View m/z peak shape data points"
	CheckBox PS_check_showVPSNormData,fSize=10
	CheckBox PS_check_showVPSNormData,variable= root:HR_PeakShape:ShowPSData
	
	ValDisplay PS_setVar_psScalar,pos={719,537},size={140,14},title="Final peak shape"
	ValDisplay PS_setVar_psScalar,fsize=10,format="%1.4f"
	ValDisplay PS_setVar_psScalar,limits={0,0,0},barmisc={0,1000}
	ValDisplay PS_setVar_psScalar,value= #"root:HR_PeakShape:HR_FinalPeakShapeScalar"
	Button PS_but_PSUpdateGraph,pos={21,275},size={80,20},proc=PS_butt_updatePSGraph,title="Recreate plot"		// 1.08D
	Button PS_but_PSUpdateGraph,fSize=10
	Button PS_calcAppendResid,pos={591,123},size={220,18},proc=PS_butt_CalcAppendResid,title="Calculate, append to graph m/z residuals:"
	Button PS_calcAppendResid,fSize=10
	SetVariable PS_ResidList,pos={820,125},size={90,16},title=" ",fSize=10
	SetVariable PS_ResidList,value= root:HR_PeakShape:PS_ResidList
	Button PS_but_PS_FinalPSGraph,pos={874,179},size={40,20},proc=sq_butt_popThis,title="Pop"
	Button PS_but_PS_FinalPSGraph,fSize=10
	
	Button PS_butt_PSTable,pos={651,536},size={60,20},fsize=10,proc=PS_butt_FinalPSTable,title="PS Table"
	Button PS_button_PWPS_table,pos={595,535},size={50,40},fsize=10,proc=PW_butt_PWParamTable,title="PW / PS\rtable"
	Button PS_butt_End,pos={864,556},size={50,20},fsize=10,proc=sq_butt_End,title="End"

	PWPSTabProc("",0)

End


Function PW_ck_OneRunOpenClosed(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	if (stringMatch("PW_viewClosed",ctrlName) )
		CheckBox PW_viewClosed,value= 1
		CheckBox PW_viewOpen,value= 0
	else
		CheckBox PW_viewClosed,value= 0
		CheckBox PW_viewOpen,value= 1
	endif

End


Function PW_plotAllPWplots()

	Edit/W=(10,90,320,260)/HOST=# 
	RenameWindow #,PW_MassTable
	PW_populate_MassTable()	
	SetActiveSubwindow ##
	
	Display/W=(630,420,913,555)/HOST=# 
	RenameWindow #,PW_AvgPWGraph
	PW_populate_AvgPWGraph()
	SetActiveSubwindow ##

	Display/W=(305,295,620,573)/HOST=# 
	RenameWindow #,PWGraph
	PW_populate_PeakWidthGraph()
	SetActiveSubwindow ##

	PW_UpdatePlotsOneRun()	

	Display/W=(750,135,910,245)/HOST=# 
	RenameWindow #,PWFitOneRun
	PW_populate_FitRunPWMassGraph()
	SetActiveSubwindow ##

	// update graphs
	//PW_ck_ViewVorWmode("", nan)
	//PW_ck_LineOrPow("",nan)
		
ENd


Function PW_populate_MassTable()	

//	AppendToTable root:GaussFits:GFName,root:GaussFits:GFMass,root:GaussFits:GFDeltaNum
//	AppendToTable root:HR_peakShape:PW_VmodeMask, root:HR_peakShape:PW_WmodeMask, root:HR_peakShape:PW_Graph
//
//	ModifyTable format(Point)=1,width(Point)=20,title(Point)=" ",width(root:GaussFits:GFName)=40,title(root:GaussFits:GFName)="Name"
//	ModifyTable format(root:GaussFits:GFMass)=3,digits(root:GaussFits:GFMass)=6,width(root:GaussFits:GFMass)=72,title(root:GaussFits:GFMass)="Mass"
//	ModifyTable width(root:GaussFits:GFDeltaNum)=30, title(root:GaussFits:GFDeltaNum)="Delta"
//	ModifyTable width(root:HR_peakShape:pW_Graph)=50,title(root:HR_peakShape:PW_Graph)="Graph"
//	ModifyTable title(root:HR_peakShape:PW_VmodeMask)="V mask",width(root:HR_peakShape:PW_VmodeMask)=35
//	ModifyTable title(root:HR_peakShape:PW_WmodeMask)="W mask",width(root:HR_peakShape:PW_WmodeMask)=35
//	ModifyTable rgb(root:HR_peakShape:PW_VmodeMask)=(52224,0,0), rgb(root:HR_peakShape:PW_WmodeMask)=(52224,0,0)

	GF_populate_MassTable()
	ModifyTable rgb(root:HR_peakShape:PW_VmodeMask)=(52224,0,0)
	ModifyTable rgb(root:HR_peakShape:PW_WmodeMask)=(52224,0,0)
	ModifyTable rgb(root:GaussFits:GFDeltaNum)=(0,0,0)
	ModifyTable rgb(root:GaussFits:GFMask)=(0,0,0)
	ModifyTable rgb(root:HR_peakShape:PW_Graph)=(52224,0,0)

End


Function PW_populate_FitRunPWMassGraph()

	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:HR_PeakShape:
	wave OneRunPeakWidths, OneRunPeakWidthsFitLine, OneRunPeakWidthsFitPower
	AppendToGraph OneRunPeakWidths vs root:GaussFits:GFMass
	AppendToGraph OneRunPeakWidthsFitLine vs root:GaussFits:GFMass
	AppendToGraph OneRunPeakWidthsFitPower vs root:GaussFits:GFMass
	SetDataFolder fldrSav0
	ModifyGraph margin(left)=36,margin(bottom)=36,margin(top)=7,margin(right)=7
	ModifyGraph mode(OneRunPeakWidths)=3
	ModifyGraph marker(OneRunPeakWidths)=8
	ModifyGraph rgb(OneRunPeakWidths)=(0,0,0)
	ModifyGraph grid=2
	ModifyGraph gridRGB=(52224,52224,52224)
	SetAxis/A/E=1 // scale both axis from zero
	Label left "Peak Width"
	Label bottom "m/z"

End


Function PW_populate_PeakWidthGraph()

	wave/t GFName = root:GaussFits:GFName
	wave GFmask = root:GaussFits:GFmask
	wave FilledMarkerWave = root:HR_PeakShape:FilledMarkerWave
	wave ColorMx = root:HR_PeakShape:ColorMx
	wave DisplayPWAvg = root:HR_PeakShape:DisplayPWAvg
	wave DisplayPWAvgX = root:HR_PeakShape:DisplayPWAvgX

	variable idex, num, numMark
	string specTypeStr, specTypeList = ";Diff;Closed;Open", goodNameStr		//1.19B
	num = numpnts(GFName)
	numMark = numpnts(FilledMarkerWave)

	for (idex=0;idex<num;idex+=1)
		goodNameStr = CleanupName(GFName[idex],0)
		specTypeStr = stringFromList(GFmask[idex], specTypeList)
		wave/z pw_thismass  =$"root:GaussFits:GaussCoeff_"+specTypeStr+"_"+goodNameStr
		if (waveExists(pw_thismass))
			AppendToGraph/L=PW pw_thismass[][3] vs root:index:rn_series
			ModifyGraph rgb($NameOfWave(pw_thismass))=(ColorMx[idex][0], ColorMx[idex][1], ColorMx[idex][2])
			ModifyGraph mode($NameOfWave(pw_thismass))=3,marker($NameOfWave(pw_thismass))=FilledMarkerWave[mod(idex, numMark)]
			ModifyGraph zmrkSize($NameOfWave(pw_thismass))={root:HR_PeakShape:PW_RunMask,0,1,1,3}
		endif
		AppendtoGraph/L=PW DisplayPWAvg[idex][] vs DisplayPWAvgX
		if (idex==0)
			ModifyGraph mode(DisplayPWAvg)=0	
			ModifyGraph rgb(DisplayPWAvg)=(ColorMx[idex][0], ColorMx[idex][1], ColorMx[idex][2])
		else
			ModifyGraph mode($("DisplayPWAvg#"+num2str(idex)))=0	
			ModifyGraph rgb($"DisplayPWAvg#"+num2str(idex)) =(ColorMx[idex][0], ColorMx[idex][1], ColorMx[idex][2])
		endif
	endfor
	ModifyGraph nticks(bottom)=9
	ModifyGraph grid(bottom)=2,grid(PW)=2
	ModifyGraph gridRGB(bottom)=(56576,56576,56576)
	ModifyGraph gridRGB(PW)=(56576,56576,56576)
	ModifyGraph lblPos(PW)=30
	ModifyGraph freePos(PW)=-10
	Label bottom "Run number"
	Label PW "Peak Width"
	ModifyGraph axisEnab(bottom)={0.1,1}
	ModifyGraph freePos(PW)=-10
	
	PW_updateMaskPWGraph()
	
End


Function PW_populate_AvgPWGraph()

	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:HR_PeakShape:
	wave PW_fitAvgPWLineY, PW_fitAvgPWPowY, IsolatedIon_PWAvg, PW_fitAvgPW_X
	AppendToGraph PW_fitAvgPWLineY vs PW_fitAvgPW_X
	AppendToGraph PW_fitAvgPWPowY vs PW_fitAvgPW_X
	AppendToGraph IsolatedIon_PWAvg vs root:GaussFits:GFMass
	SetDataFolder fldrSav0
		
	ModifyGraph margin(left)=36,margin(bottom)=36,margin(top)=7,margin(right)=7
	ModifyGraph lSize(PW_fitAvgPWLineY)=2
	ModifyGraph rgb(PW_fitAvgPWLineY)=(39168,39168,0)
	ModifyGraph rgb(PW_fitAvgPWPowY)=(19456,39168,0)
	ModifyGraph marker(IsolatedIon_PWAvg)=8
	ModifyGraph rgb(IsolatedIon_PWAvg)=(65280,0,65280)
	ModifyGraph mode(IsolatedIon_PWAvg)=3
	ModifyGraph msize(IsolatedIon_PWAvg)=4

	ModifyGraph lsize(PW_fitAvgPWPowY)=2

	ModifyGraph grid=2
	ModifyGraph gridRGB=(52224,52224,52224)
	Label left "Peak Width HWHM)"
	Label bottom "m/z"
	SetAxis/A/E=1 left
	SetAxis/A/E=1 bottom
	ErrorBars IsolatedIon_PWAvg Y,wave=(root:HR_PeakShape:IsolatedIon_PWStD,root:HR_PeakShape:IsolatedIon_PWStD)

End


Function PW_populate_AvgPWTable()

	AppendToTable root:GaussFits:GFname, root:GaussFits:GFMass
	AppendToTable root:HR_PeakShape:IsolatedIon_PWAvg,root:HR_PeakShape:IsolatedIon_PWStD

End


// simple function for setting global variable PWUseLineOrPow via panel checkboxes


Function PWPSTabProc(ctrlName,tabNum) : TabControl
	String ctrlName
	Variable tabNum

	variable cont_disp,tab_1,idex,num
	string cont_list, cont_item
	
	nvar/z GF_showGFDetails = root:HR_peakShape:GF_showGFDetails  //1.06J
	if (!NVar_exists(GF_showGFDetails))
		sq_initializeGlobalNVar("root:HR_peakShape:GF_showGFDetails", 0)  //1.06J		
		nvar GF_showGFDetails = root:HR_peakShape:GF_showGFDetails  //1.06J		
	endif
	
//& The object is to store the number of the selected tab on the main tabcontrol in 'tab'
	cont_list=controlnamelist("PeakWidthShapePanel")
	num=itemsinlist(cont_list)
	for (idex=0;idex<num;idex+=1)
		cont_item=stringfromlist(idex,cont_list)
		strswitch (cont_item[0,2])
			case "GF_":	
				cont_disp=tabNum!=0
				break
			case "PW_":	
				cont_disp=tabNum!=1
				break
			case "PS_":
				cont_disp=tabNum!=2
				break
			default:
				cont_disp=0
		endswitch

		controlinfo /w=PeakWidthShapePanel $cont_item
		//  disable = 1 means hide, disable = 0 means show
		switch (abs(v_flag))
			case 1:
				button $cont_item disable=cont_disp
				break
			case 2:
				checkbox  $cont_item disable=cont_disp
				break
			case 3:
				popupmenu $cont_item disable=cont_disp
				break
			case 4:
				valdisplay $cont_item disable=cont_disp
				break
			case 5:
				setvariable $cont_item disable=cont_disp
				break
			case 6:
				chart $cont_item disable=cont_disp
				break
			case 7:
				slider $cont_item disable=cont_disp
				break
			case 8:
				tabcontrol $cont_item disable=cont_disp
				break
			case 9:
				groupbox $cont_item disable=cont_disp
				break
			case 10:
				titlebox $cont_item disable=cont_disp
				break
			case 11:
				listbox $cont_item disable=cont_disp
				break
		endswitch
	endfor

	string myWInList, winStr
	
	myWInList = ChildWindowList("PeakWidthShapePanel")
	num= ItemsInList(myWinList)
	for(idex=0;idex<num;idex+=1)
		winStr = stringFromList(idex, myWInList)
		if(tabNum==0)
			if (stringmatch("PS", winStr[0,1]) || stringmatch("PW", winStr[0,1]) )
				KillWindow $"PeakWidthShapePanel#"+winStr
			endif		
		elseif (tabNum==1)		// we want the peak width tab
			if (stringmatch("GF", winStr[0,1]) || stringmatch("PS", winStr[0,1]) )
				KillWindow $"PeakWidthShapePanel#"+winStr
			endif
		else
			if (stringmatch("GF", winStr[0,1]) || stringmatch("PW", winStr[0,1]) )
				KillWindow $"PeakWidthShapePanel#"+winStr
			else
			endif
		endif
	endfor

	if(tabNum==0)
		GF_plotAllGFplots()
	elseif (tabNum==1)		// we want the peak width tab
		PW_plotAllPWplots()
	else
		PS_plotAllPSplots()
	endif
	
End


Function PW_butt_FitPWAvgs()

	nvar HR_displayFinalPowA = root:HR_PeakShape:HR_displayFinalPowA
	nvar HR_displayFinalPowB = root:HR_PeakShape:HR_displayFinalPowB
	nvar HR_displayFinalPowC = root:HR_PeakShape:HR_displayFinalPowC

	nvar HR_displayFinalPWSlope = root:HR_PeakShape:HR_displayFinalPWSlope
	nvar HR_displayFinalPWIntercept = root:HR_PeakShape:HR_displayFinalPWIntercept

	wave IsolatedIon_PWAvg  = root:HR_PeakShape:IsolatedIon_PWAvg
	wave IsolatedIon_PWStD  = root:HR_PeakShape:IsolatedIon_PWStD
	wave/t PWConstraintsLine = root:HR_PeakShape:PWConstraintsLine
	wave/t PWConstraintsPower = root:HR_PeakShape:PWConstraintsPower
	  
	variable/g V_FitError=0
//	variable/g V_FitQuitReason=0//1.07
	
	wave GFmass = root:GaussFits:GFmass
				
	make/o/d MyCoeffs = {1,1}		// defaults for w mode - taken from baseline panel
	WaveStats/q IsolatedIon_PWAvg
	if (V_npnts>2 && V_avg!=0)		//1.06D added constraint waves
		CurveFit/G/Q line kwCWave=MyCoeffs, IsolatedIon_PWAvg /X =GFmass /W=IsolatedIon_PWStD  /I=1  /C = PWConstraintsLine  //1.05A fix, we want the reciprical of the weighting
		if (!V_FitError )//&& !V_FitQuitReason)
			HR_displayFinalPWIntercept = MyCoeffs[0]
			HR_displayFinalPWSlope = MyCoeffs[1]
		else
			HR_displayFinalPWIntercept = nan
			HR_displayFinalPWSlope = nan
		endif
	else
		HR_displayFinalPWIntercept = nan
		HR_displayFinalPWSlope = nan
	endif

	V_FitError=0
	//V_FitQuitReason=0
	if (numtype(HR_displayFinalPWIntercept) !=2)
		redimension/n=3 MyCoeffs;  MyCoeffs[2]=0.5		//1.19E	
	endif
	WaveStats/q IsolatedIon_PWAvg
	if (V_nPnts>3 && V_avg!=0 && V_sdev < 100)		//1.06B //1.06D added constraint waves
		CurveFit/G/Q Power kwCWave=MyCoeffs, IsolatedIon_PWAvg /X =GFmass /W=IsolatedIon_PWStD  /I=1  /C = PWConstraintsPower		//1.05A fix, we want the reciprical of the weighting	//1.10 I removed /G
		if (!V_FitError)//&& !V_FitQuitReason)
			HR_displayFinalPowA = MyCoeffs[0]
			HR_displayFinalPowB = MyCoeffs[1]
			HR_displayFinalPowC = MyCoeffs[2]		
		else
			HR_displayFinalPowA = nan
			HR_displayFinalPowB = nan
			HR_displayFinalPowC = nan
		endif
	else
		HR_displayFinalPowA = nan
		HR_displayFinalPowB = nan
		HR_displayFinalPowC = nan
	endif

	killwaves/z MyCoeffs
	
	PW_butt_recalcDisplayFinalPW("")

End


Function PW_butt_recalcDisplayFinalPW(ctrlName) : ButtonControl
	String ctrlName

	nvar HR_displayFinalPowA = root:HR_PeakShape:HR_displayFinalPowA
	nvar HR_displayFinalPowB = root:HR_PeakShape:HR_displayFinalPowB
	nvar HR_displayFinalPowC = root:HR_PeakShape:HR_displayFinalPowC

	nvar HR_displayFinalPWSlope = root:HR_PeakShape:HR_displayFinalPWSlope
	nvar HR_displayFinalPWIntercept = root:HR_PeakShape:HR_displayFinalPWIntercept

	wave PW_fitAvgPW_X = root:HR_peakShape:PW_fitAvgPW_X
	wave PW_fitAvgPWPowY = root:HR_peakShape:PW_fitAvgPWPowY
	wave PW_fitAvgPWLineY = root:HR_peakShape:PW_fitAvgPWLineY
	wave GFmass = root:GaussFits:GFmass
	
//	wavestats/q/m=1 GFmass
	PW_fitAvgPW_X = (round(WaveMax(GFmass)+1)/100)*p

	PW_fitAvgPWLineY = HR_displayFinalPWSlope*PW_fitAvgPW_X + HR_displayFinalPWIntercept
	PW_fitAvgPWPowY = HR_displayFinalPowA + HR_displayFinalPowB* (PW_fitAvgPW_X^HR_displayFinalPowC )

End

// for each mass-peak width wave, hide or not depending on the 1 or nan value of the entries in PW_UseThisMass
// also sets the global strings pw_massListW 
Function PW_HideChosenPSMass(isV)
variable isV

	wave DisplayPWAvg = root:HR_PeakShape:DisplayPWAvg
	wave DisplayPWAvgX = root:HR_PeakShape:DisplayPWAvgX
	wave PW_XmodeMask = $"root:hr_peakShape:PW_"+selectString(isV, "W","V")+"modeMask"
	wave/t GFName = root:GaussFits:GFName
	wave GFmask = root:GaussFits:GFmask
	wave ColorMx = root:HR_PeakShape:ColorMx
	wave FilledMarkerWave = root:HR_PeakShape:FilledMarkerWave
	svar pw_massList = root:HR_PeakShape:pw_massList

	variable idex, numMass, popWinExists
	string currentTraceList, specTypeStr, specTypeList = ";Diff;Closed;Open", goodNameStr
	
	popWinExists = ItemsInList(WinList("PW_PeakWidthGraph", ";", "WIN:1") ) == 1

	pw_massList=""
	
	numMass = numpnts(PW_XmodeMask)		// could use PW_WmodeMask
	
	currentTraceList = TraceNameList("PeakWidthShapePanel#PWGraph",";",1)
	
	for (idex = 0; idex < numMass;idex+=1)
		if (PW_XmodeMask[idex]==1)
			pw_massList+=GFName[idex]+";"
		endif
		if (GFMask[idex]>0)
			goodNameStr = CleanupName(GFName[idex],0)		//1.19B
			specTypeStr =StringFromList(GFMask[idex], specTypeList)
			wave/z PeakWidth_X = $"root:GaussFits:GaussCoeff_"+specTypeStr+"_"+goodNameStr
			if (WaveExists(PeakWidth_X))
				if (FindListItem("GaussCoeff_"+specTypeStr+"_"+goodNameStr, currentTraceList)>=0)				
					ModifyGraph/W=PeakWidthShapePanel#PWGraph hideTrace($NameofWave(PeakWidth_X)) = PW_XmodeMask[idex]!=1
					if (popWinExists)
						ModifyGraph/W=PW_PeakWidthGraph hideTrace($NameofWave(PeakWidth_X)) = PW_XmodeMask[idex]!=1
					endif	
				else
					AppendToGraph/W=PeakWidthShapePanel#PWGraph/L=PW PeakWidth_X[][3] vs root:index:rn_series
					ModifyGraph/W=PeakWidthShapePanel#PWGraph rgb($NameOfWave(PeakWidth_X))=(ColorMx[idex][0], ColorMx[idex][1], ColorMx[idex][2])
					ModifyGraph/W=PeakWidthShapePanel#PWGraph mode($NameOfWave(PeakWidth_X))=3,marker($NameOfWave(PeakWidth_X))=FilledMarkerWave[mod(idex, numpnts(FilledMarkerWave))]
					ModifyGraph/W=PeakWidthShapePanel#PWGraph zmrkSize($NameOfWave(PeakWidth_X))={root:HR_PeakShape:PW_RunMask,0,1,1,3}
				endif
			endif
		endif
	endfor
	
End


// changes the PW_runMask wave to 1
Function PW_butt_PWSet1(ctrlName) : ButtonControl
	String ctrlName

	nvar PW_viewVmode = root:HR_PeakShape:PW_viewVmode
	wave PW_RunMask = root:HR_peakShape:PW_RunMask
	wave TofTypeVKey = root:diagnostics:TofTypeVKey
	svar sq_toDoWvNm=root:panel:sq_toDoWvNm
	svar PWTodoModeStr = root:HR_PeakShape:PWTodoModeStr
	wave/z todo = $"root:"+sq_toDoWvNm

	wave rn_series = root:index:rn_series
	wave DisplayPWAvg = root:HR_peakShape:DisplayPWAvg

	wave todowave = $"root:"+sq_toDoWvNm
	PW_SanityCheckTodoWave()
	
	PW_viewVmode = TofTypeVKey[sq_findRun(rn_series, todo[0])]		//1 = V mode, 0 = W
	
	HR_TodoWaveHasOnlyModeX(PW_viewVmode)		// 
	
	PWTodoModeStr = "current ToF mode:"+selectString(PW_viewVmode, "W", "V")
	
	fastop DisplayPWAvg = (nan)		// added in 1.05B

	sq_makeMaskFromTodofindRun(root:HR_peakShape:PW_RunMask,todowave)
	
	print "// You have reset to 1 the mask wave root:HR_PeakShape:PW_RunMask for mode "+selectString(PW_viewVmode, "W", "V")+" in todo wave "+sq_toDoWvNm
	
	PW_updateMassList()

End


// changes the PW_runMask wave to nan for one run
Function PW_butt_RemoveRunPW(ctrlName) : ButtonControl
	String ctrlName

	nvar PW_CurrentRunNum = root:HR_peakShape:PW_CurrentRunNum
	wave PW_RunMask = root:HR_peakShape:PW_RunMask
	wave rn_series = root:index:rn_series	
	variable pos
	
	pos = sq_findRun(rn_series, PW_CurrentRunNum)
	if (pos>=0)
		PW_RunMask[pos]=nan
	endif

	print "// You have removed run "+num2istr(PW_CurrentRunNum)+ " from the root:HR_PEakShape:PW_RunMask mask wave"   //  1.20O

	PW_updateMaskPWGraph()

End


// updates the mask wave for displaying pws, slope avg.  also autoscales the graph
Function PW_updateMaskPWGraph()
	
//	wave PWDisplayKey = root:HR_PeakShape:PWDisplayKey
	wave TofTypeVKey = root:diagnostics:TofTypeVKey
	wave PW_RunMask = root:HR_PeakShape:PW_RunMask
//	wave PWIntercept = root:HR_PeakShape:PWIntercept
//	wave PWSlope = root:HR_PeakShape:PWSlope
	wave/t GFname = root:GaussFits:GFname
	wave GFmask= root:GaussFits:GFmask
	nvar PW_viewVmode = root:HR_PeakShape:PW_viewVmode
	
	variable popWinExists, idex, num, tempMin, tempMax
	string specTypeStr,  specTypeList = ";Diff;Closed;Open",goodNameStr
		
	popWinExists = ItemsInList(WinList("PW_PeakWIdthGraph", ";", "WIN:1") ) == 1

	duplicate/o PW_RunMask tempWave
	
	if (PW_viewVmode)
		wave PW_XmodeMask = root:HR_peakShape:PW_VmodeMask
	else
		wave PW_XmodeMask = root:HR_peakShape:PW_WmodeMask
	endif
	
	PW_HideChosenPSMass(PW_viewVmode)

	tempMin = 10
	tempMax=0
	num = numpnts(PW_XmodeMask)
	for (idex=0;idex<num;idex+=1)
		specTypeStr = StringFromList(GFmask[idex], specTypeList)
		goodNameStr = CleanUPName(GFName[idex],0)		//1.19B
		wave/z GausCoeffX = $"root:GaussFits:GaussCoeff_"+specTypeStr+"_"+goodNameStr
		if (PW_XmodeMask[idex]==1 && WaveExists(GausCoeffX))
			tempWave = PW_RunMask * GausCoeffX[p][3]
			wavestats/q/m=1 tempWave
			tempMin = min(tempMin, V_min)
			tempMax = max(tempMax, V_max)
		endif
	endfor
	if (tempMin>-10 && tempMax < 100)  // 1.16Z
		setAxis/W=PeakWidthShapePanel#PWGraph PW tempMin, tempMax
		if (popWinExists)
			setAxis/W=PW_PeakWIdthGraph PW tempMin, tempMax
		endif
	endif
	
	killwaves/z tempWave

End


// changes the PW_runMask wave to nan or 1 for fulfilling condition
Function PW_butt_RemoveTheseRuns(ctrlName) : ButtonControl
	String ctrlName

	wave PW_RunMask = root:HR_peakShape:PW_RunMask
	wave/t GFname = root:GaussFits:GFname 
	wave GFmask = root:GaussFits:GFmask 
	nvar Step2Var = root:HR_peakShape:Step2Var
	svar PW_MassList = root:HR_PeakShape:pw_MassList

	string specTypeStr,  specTypeList = ";Diff;Closed;Open"
	string tempStr1, tempStr2
	variable tempVar, pos
	
	ControlInfo/w=PeakWidthShapePanel PW_popupMassList
	tempStr1 = S_value
	tempStr2 = stringFromList(V_value-1, PW_MassList)
	if (!stringmatch(tempStr1, tempStr2))
		abort "Please update the mass list popup menu - aborting from PW_butt_RemoveTheseRuns"
	endif
	
	pos = sq_findStr(GFname,tempStr1,1)
	specTypeStr = StringFromList(GFmask[pos], specTypeList)
	
	wave/z MassWidthWave = $"root:GaussFits:GaussCoeff_"+specTypeStr+"_"+ tempStr2
	if (!WaveExists(MassWidthWave))
		abort "Having problems finding the wave for "+tempStr2+" generated in step 1 - aborting from PW_butt_RemoveTheseRuns"
	endif

	make/o/n=(dimsize(MassWidthWave,0)) root:tempWave
	wave tempwave = root:tempWave
	tempWave = PW_RunMask
	wavestats/q/m=1 PW_RunMask
	tempVar =V_numNans
	
	ControlInfo/w=PeakWidthShapePanel PW_popGTorLT
	if (V_Value==1) 
		PW_RunMask = MassWidthWave[p][3] > Step2Var ? PW_RunMask[p] : nan
	else
		PW_RunMask = 	MassWidthWave[p][3] < Step2Var ? PW_RunMask[p] : nan
	endif
	
	tempWave = PW_RunMask
	wavestats/q/m=1 PW_RunMask
	print "// You have removed "+num2str(tempVar -V_numNans)+ " more runs from the root:HR_PeakShape:PW_RunMask mask wave because the width of "+tempStr2+" was "+selectstring(V_Value-1, ">", "<")+" "+num2str(step2Var)
	// Zhaoheng Gong email with bug of V_Value
	killwaves/z tempWave

	PW_updateMaskPWGraph()

End


// if PS_holdWidth is checked put numeric values in the waves OneRunPeakWidthVmode and OneRunPeakWidthWmode
//Function PW_initializePeakWidths(OneRunPeakWidthWmode, OneRunPeakWidthVmode)
//wave OneRunPeakWidthWmode, OneRunPeakWidthVmode
//
//	nvar PS_holdWidth = root:HR_PeakShape:PS_holdWidth
//	nvar PWUseLineOrPow = root:HR_PeakShape:PWUseLineOrPow
//
//	nvar HR_VModeFinalR0 = root:HR_PeakShape:HR_VModeFinalR0	
//	nvar HR_VModeFinalM0 = root:HR_PeakShape:HR_VModeFinalM0	
//	nvar HR_VModeFinalDm = root:HR_PeakShape:HR_VModeFinalDm	
//	nvar HR_WModeFinalR0 = root:HR_PeakShape:HR_WModeFinalR0	
//	nvar HR_WModeFinalM0 = root:HR_PeakShape:HR_WModeFinalM0	
//	nvar HR_WModeFinalDm = root:HR_PeakShape:HR_WModeFinalDm	
//
//	nvar HR_VModeFinalSlope = root:HR_PeakShape:HR_VModeFinalSlope
//	nvar HR_VModeFinalIntercept = root:HR_PeakShape:HR_VModeFinalIntercept
//	nvar HR_WModeFinalSlope = root:HR_PeakShape:HR_WModeFinalSlope
//	nvar HR_WModeFinalIntercept = root:HR_PeakShape:HR_WModeFinalIntercept
//
//	wave GFMass = root:GaussFits:GFMass
//	
//	if(PS_holdWidth)
//		if (PWUseLineOrPow)			
//			OneRunPeakWidthWmode = HR_WModeFinalIntercept+HR_WModeFinalSlope*GFMass
//			OneRunPeakWidthVmode = HR_VModeFinalIntercept+HR_VModeFinalSlope*GFMass
//		else
//			OneRunPeakWidthVmode =  HR_VModeFinalR0 * (1 - (1 / (1+exp((GFMass-HR_VModeFinalm0)/HR_VModeFinaldm) ) ) )
//			OneRunPeakWidthVmode =  HR_WModeFinalR0 * (1 - (1 / (1+exp((GFMass-HR_WModeFinalm0)/HR_WModeFinaldm) ) ) )
//		endif
//	else
//		OneRunPeakWidthVmode = nan
//		OneRunPeakWidthWmode = nan
//	endif
//
//End


//  Finds gaussian fits for chosen ions for one run.  Rows = different masses, columns = intercept, center, height, width
Function PW_GausFitOneRun(OneRunDisplayPC_fromMC, OneRunPeakWidths, RawSpectraThisRun, GausConstraints, GausCoeffsMx,GausCoeffsWv) 
wave OneRunDisplayPC_fromMC, OneRunPeakWidths, RawSpectraThisRun
wave/t GausConstraints		//, LineConstraint
wave GausCoeffsMx, GausCoeffsWv

	wave GFMass=root:GaussFits:GFMass
	wave GFDeltaNum=root:GaussFits:GFDeltaNum
		
	variable idex, numMass
	
	numMass = numpnts(GFMass)
	
	for(idex=0;idex<numMass;idex+=1)
		GausCoeffsWv[0] = 0		// set baseline to 0 (using spectra w/sq baseline subtracted or not_
		GausCoeffsWv[1] = 1            // gaussian peak height, set to a default value 
		GausCoeffsWv[2] = OneRunDisplayPC_fromMC[idex]
		GausCoeffsWv[3] = SelectNumber(numtype(OneRunPeakWidths[idex]), OneRunPeakWidths[idex], 1)	 	// peak width		
		PW_FindGausCoefs(RawSpectraThisRun, GausConstraints, GausCoeffsWv, OneRunDisplayPC_fromMC[idex], GFDeltaNum[idex])
		GausCoeffsMx[idex][]=GausCoeffsWv[q]
		OneRunPeakWidths[idex]=GausCoeffsWv[3]
	endfor // m/z loop
	
End


// multiplies OneRunPeakWidths by mask, fits the widths, puts values in PWCoeffs
Function PW_findFitsOneRun(GFMass, OneRunPeakWidths, PW_modeMask, PWCoeffs, ConstraintWave, whichFitFlag)
wave GFMass, OneRunPeakWidths, PW_modeMask, PWCoeffs
wave/t ConstraintWave
variable whichFitFlag		// 1 is line

	OneRunPeakWidths *= (PW_modeMask/PW_modeMask)
			
	wavestats/q/m=1 OneRunPeakWidths
	fastop PWCoeffs=(nan)
	WaveStats/q OneRunPeakWidths		//1.16J
	if (V_npnts>=3)
		if (whichFitFlag)
			if (V_nPnts>2)
				PWCoeffs = {1,1}	// give respectable starting values	
				CurveFit/G/N/NTHR=0/Q line kwCWave=PWCoeffs, OneRunPeakWidths /X =GFMass /C = ConstraintWave		// added /G flag 1.06D
			endif	
		elseif (V_nPnts>3)
			PWCoeffs = {1,1,0.5}		// give respectable starting values
			CurveFit/G/N/NTHR=0/Q power kwCWave=PWCoeffs, OneRunPeakWidths /X =GFMass /C = ConstraintWave	// added /G flag 1.06D
		endif	
	endif
End


//For one run, one isolated mass, find peak width
Function PW_findGausCoefs(RawSpectraThisRun, GausConstraints, CoeffWave, myCenter, delta)
wave RawSpectraThisRun, CoeffWave
wave/t GausConstraints
variable myCenter, delta		// in point space

	variable /g V_FitError=0 // Deal with errors internally
	variable/g V_fitOptions=4

//	nvar PS_holdWidth = root:HR_PeakShape:PS_holdWidth
//	nvar PS_holdCenter = root:HR_PeakShape:PS_holdCenter

	//  gauss	Gaussian peak: y = K0+K1*exp(-((x-K2)/K3)^2).
	
//	if (PS_holdCenter)		// 1.04B fix center, and if necessary, fix width
//		if(PS_holdWidth)
//			curveFit/Q/N/h="1011"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints	//^ use guesses in k0, etc, do not use flag /g because if we use inital guesses, things go bad
//		else
//			curveFit/Q/N/h="1010"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints	//^ use guesses in k0, etc, do not use flag /g because if we use inital guesses, things go bad
//		endif
//	else
//		if (PS_holdWidth)		// 1.04B do not fix center, and if necessary, fix width
//			curveFit/Q/N/h="1001"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints	//^ use guesses in k0, etc, do not use flag /g because if we use inital guesses, things go bad
//		else
			curveFit/NTHR=0/Q/h="1000"  gauss kwCWave=CoeffWave RawSpectraThisRun[myCenter-delta,myCenter+delta]/C=GausConstraints	//^ use guesses in k0, etc, do not use flag /g because if we use inital guesses, things go bad
//		endif
//	endif
	
	if (V_FitError!=0)
		fastop CoeffWave=(nan)
	endif
		
End


// view raw spectra and gaussian fits for peak width fitting for one run.
Function PW_butt_showPWOneRun(ctrlName) : ButtonControl
	String ctrlName

	wave GFMass=root:GaussFits:GFMass

	wave RawSpectraThisRun = root:HR_PeakShape:RawSpectraThisRun
	wave OneRunDisplayPC_fromMC = root:HR_PeakShape:OneRunDisplayPC_fromMC
	wave OneRunIsolatedPC_fromGaus = root:HR_PeakShape:OneRunIsolatedPC_fromGaus	
	wave OneRunDisplayPC_Y = root:HR_PeakShape:OneRunDisplayPC_Y		//1.06x
	wave OneRunPeakWidthVmode = root:HR_PeakShape:OneRunPeakWidthVmode
	wave OneRunPeakWidthWmode = root:HR_PeakShape:OneRunPeakWidthWmode
	wave OneRunPeakWidths = root:HR_PeakShape:OneRunPeakWidths	
	wave OneRunPeakWidthsFitLine = root:HR_PEakshape:OneRunPeakWidthsFitLine
	wave OneRunPeakWidthsFitPower = root:HR_PEakshape:OneRunPeakWidthsFitPower

	wave TofTypeVKey = root:diagnostics:TofTypeVKey	
	wave rn_series = root:index:rn_series	

	nvar PW_CurrentRunNum = root:HR_PeakShape:PW_CurrentRunNum
//	nvar PS_holdWidth = root:HR_PeakShape:PS_holdWidth
	nvar HR_UseOpenClosedMzParams = root:HR_panel:HR_UseOpenClosedMzParams
//	nvar HR_UseSqBaseline = root:HR_panel:HR_UseSqBaseline

	wave GausCoeffsMx = root:GaussFIts:GausCoeffsMx		// created in PW_GausFitPrelims()
	wave GausCoeffsWv = root:GaussFits:GausCoeffsWv	
	wave/t GausConstraints = root:GaussFits:GausConstraints

	wave PWCoeffs = root:HR_PeakShape:PWCoeffs
	wave/t PWConstraintsLine = root:HR_PeakShape:PWConstraintsLine
	wave/t PWConstraintsPower = root:HR_PeakShape:PWConstraintsPower

	variable numMass, intercept, slope, power, pos, RunIsW

	if (HR_UseOpenClosedMzParams)
		wave mz_FitIntercept=root:mzFitting:mz_FitInterceptClosed
		wave mz_FitSlope=root:mzFitting:mz_FitSlopeClosed
		wave mz_FitPower=root:mzFitting:mz_FitPowerClosed
	else
		wave mz_FitIntercept=root:mzFitting:mz_FitIntercept
		wave mz_FitSlope=root:mzFitting:mz_FitSlope
		wave mz_FitPower=root:mzFitting:mz_FitPower	
	endif

	pos = sq_findRun(rn_series, PW_CurrentRunNum)	

	intercept =  mz_FitIntercept[pos]
	slope =  mz_FitSlope[pos]
	power = mz_FitPower[pos]
	if (numtype(slope)!=0 || numtype(intercept)!=0 || numtype(power)!=0)
		abort "Had trouble with the m/z calibration values for this run - aborting from PW_butt_findPWOneRun"
	endif

	wave PW_modeMask = $"root:HR_peakShape:PW_"+selectstring(TofTypeVKey[pos], "W", "V")+"modeMask"

	ControlInfo PW_viewClosed
	string specTypeStr = SelectString(V_Value, "Open", "Closed") 
	
//	if (HR_UseSqBaseline)
		sq_GetThisRawMS(PW_CurrentRunNum, "HR_PeakShape", "RawSpectraThisRun", "MS"+SpecTypeStr+"LessBaseL_p")
//	else	
//		sq_GetThisRawMS(PW_CurrentRunNum, "HR_PeakShape", "RawSpectraThisRun", "MS"+SpecTypeStr+"_p")
//	endif
							
	numMass = numpnts(GFMass)		//1.06B fix for when user added masses in gaussian step
	if (numMass!=numpnts(OneRunDisplayPC_fromMC)  || numMass!=numpnts(OneRunPeakWidths) )
		redimension/n=(numMass) OneRunDisplayPC_fromMC, OneRunPeakWidths, OneRunIsolatedPC_fromGaus, OneRunDisplayPC_Y		//1.06D
 		OneRunDisplayPC_Y =1e9
	endif
	
	OneRunDisplayPC_fromMC =  round(intercept + slope*(GFmass^power))		//OneRunPeakCenter is in point space

	// we need to refit the gaussians because the delta values may have changed
	PW_GausFitOneRun(OneRunDisplayPC_fromMC, OneRunPeakWidths, RawSpectraThisRun, GausConstraints, GausCoeffsMx, GausCoeffsWv)//,  PWLineCoeffs, runIsW, PWLineConstraints)

  	// update waves and plots on panel
  	OneRunIsolatedPC_fromGaus = GausCoeffsMx[p][2]		// peak centers
   	
	// find peak widths functional forms
	PW_findFitsOneRun(GFMass, OneRunPeakWidths, PW_modeMask, PWCoeffs, PWConstraintsLine, 1)
	OneRunPeakWidthsFitLine = PWCoeffs[0] + PWCoeffs[1] *GFMass
	PW_findFitsOneRun(GFMass, OneRunPeakWidths, PW_modeMask, PWCoeffs, PWConstraintsPower, 0)
	OneRunPeakWidthsFitPower = PWCoeffs[0] + PWCoeffs[1]*(GFMass^PWCoeffs[2])
	
   	PW_individualMassFitGraphs(GausCoeffsMx, PW_modeMask)
   	
End



// Plots all the x axis and x label of  individual fit windows in the HR_PeakWidthShape_Panel
Function	PW_individualMassFitGraphs(GausCoeffsMx, ModeMaskWave)
wave GausCoeffsMx, ModeMaskWave

	DoWindow/F PeakWidthShapePanel
	if (v_flag!=1)
		Abort "Is the peak width panel window renamed? - Aborting from PW_individualMassFitGraphs"
	endif
	
	wave/t GFName = root:GaussFits:GFName
	wave GFMass = root:GaussFits:GFMass
	wave GFDeltaNum = root:GaussFits:GFDeltaNum
	
	wave PW_Graph = root:HR_PeakShape:PW_Graph
	wave RawSpectraThisRun = root:HR_PeakShape:RawSpectraThisRun
	
	wave OneRunIsolatedPC_fromGaus = root:HR_PeakShape:OneRunIsolatedPC_fromGaus				// center line from gaussian fit
	wave OneRunDisplayPC_fromMC = root:HR_PeakShape:OneRunDisplayPC_fromMC		// center line from mass cal
	wave OneRunDisplayPC_Y = root:HR_PeakShape:OneRunDisplayPC_Y				// arbitrary y value for plotting
					
	variable idex, num, numMass, numMass2Display, numgaussXFitwave
	string massStr, massList, MoreThanXDisplayed, tempStr, goodNameStr
	
	svar PS_MassesNotDisplayedList = root:HR_PeakShape:PS_MassesNotDisplayedList

	numMass=numpnts(GFName)
		
	PS_MassesNotDisplayedList = ""	
	numMass2Display=0
	
	for(idex=0;idex<numMass;idex+=1)
		goodNameStr = CleanupName(GFName[idex],0)		//1.19A
		tempStr = GFName[idex]
		numgaussXFitwave = (2*GFDeltaNum[idex])* 5 + 1	// 5 is arbitrary, want a finer resolution than is fitted

		// first update all gaussian waves, whether or not this mass is displayed or used
//		wave/z gaussYFitwave = $"root:HR_PeakShape:fit_gauss_Y_" + tempStr
//		wave/z gaussXFitwave = $"root:HR_PeakShape:fit_gauss_X_" + tempStr
//		if (WaveExists(gaussYFitwave)==0 || WaveExists(gaussXFitwave)==0 )
			make/o/n=(numgaussXFitwave) $"root:HR_PeakShape:fit_gauss_Y_" + goodNameStr
			make/o/n=(numgaussXFitwave) $"root:HR_PeakShape:fit_gauss_X_" + goodNameStr
			wave gaussYFitwave = $"root:HR_PeakShape:fit_gauss_Y_" + goodNameStr
			wave gaussXFitwave = $"root:HR_PeakShape:fit_gauss_X_" + goodNameStr
//		endif
		gaussXFitwave = OneRunDisplayPC_fromMC[idex] - GFDeltaNum[idex]+p*0.2		// from the use of arbitrary 5, above
		gaussYFitwave = GausCoeffsMx[idex][0]+GausCoeffsMx[idex][1]*exp(-((gaussXFitwave-GausCoeffsMx[idex][2])/GausCoeffsMx[idex][3])^2)

		// first check if this is a mass to be used in this v/w mode
		if (ModeMaskWave[idex]==1)
			if (PW_Graph[idex]>0)
				if (numMass2Display<4)
				
					Execute "SetActiveSubwindow PeakWidthShapePanel#PWOneRunMass"+num2str(numMass2Display+1)
//					wavestats/q/m=1 gaussYFitwave
					SetAxis left 0, max(Wavemax(gaussYFitwave), RawSpectraThisRun[OneRunDisplayPC_fromMC[idex]])		//1.06D
					SetAxis bottom gaussXFitwave[0] - 3, gaussXFitwave[numgaussXFitwave-1]+3 
					Label bottom goodNameStr
						
				else //^ more than 3 masses
					PS_MassesNotDisplayedList+= tempStr + ","
				endif 	// this mass was less than <=3  so is displayed
				numMass2Display+=1				
			else
				PS_MassesNotDisplayedList+=tempStr + ","
			endif	// we try to graph this mass
		endif
	
	endfor		// go through all masses in table
	if (numMass2Display<4)	// we are displaying less than 3 masses
		SetActiveSubwindow PeakWidthShapePanel#PWOneRunMass4
		SetAxis left nan, nan
		SetAxis bottom nan, nan
		Label bottom ""
		if (numMass2Display<3)	// we are displaying less than 3 masses
			SetActiveSubwindow PeakWidthShapePanel#PWOneRunMass3
			SetAxis left nan, nan
			SetAxis bottom nan, nan
			Label bottom ""
			if (numMass2Display<2)	// we are displaying less than 3 masses
				SetActiveSubwindow PeakWidthShapePanel#PWOneRunMass2
				SetAxis left nan, nan
				SetAxis bottom nan, nan
				Label bottom ""
				if (numMass2Display<1)	// we are displaying less than 3 masses
					SetActiveSubwindow PeakWidthShapePanel#PWOneRunMass1
					SetAxis left nan, nan
					SetAxis bottom nan, nan
					Label bottom ""
				endif
			endif
		endif
	endif
	
	if (strlen(PS_MassesNotDisplayedList)>0)
		PS_MassesNotDisplayedList+= " not displayed but fitted."
	endif

	SetActiveSubwindow PeakWidthShapePanel

End


Function	PW_populate_OneRunGraph([windowNameStr])
string windowNameStr

	// Determine if the optional parameters were supplied
	if( ParamIsDefault(windowNameStr) )
		windowNameStr = "PW_OneRunGraph"
	endif

	wave/t GFname = root:GaussFits:GFname
	wave GFDeltaNum = root:GaussFits:GFDeltaNum

	variable idex, numMass, numgaussXFitwave
	string goodNameStr
	
	// values in these waves should be filled in by the first time the user find values for first run
	wave GausCoeffsMx = root:GaussFits:GausCoeffsMx
	wave OneRunIsolatedPC_fromGaus = root:HR_PeakShape:OneRunIsolatedPC_fromGaus
	nvar specType = root:HR_PeakShape:specType
	
	numMass = numpnts(GFname)
	
	if (numpnts(OneRunIsolatedPC_fromGaus)!=numMass || dimsize(GausCoeffsMx,0) !=numMass)			//1.19B
		redimension/n=(numMass) OneRunIsolatedPC_fromGaus
		redimension/n=(numMass, dimsize(GausCoeffsMx,1)) GausCoeffsMx
	endif
	
	SetActiveSubwindow $windowNameStr
	
	AppendToGraph/W=$windowNameStr root:HR_PeakShape:RawSpectraThisRun
	ModifyGraph rgb(RawSpectraThisRun)=(0,0,0), mode(RawSpectraThisRun)=4,marker(RawSpectraThisRun)=19
	
	AppendToGraph/W=$windowNameStr/r root:HR_PeakShape:OneRunDisplayPC_Y vs root:HR_PeakShape:OneRunDisplayPC_fromMC
	AppendToGraph/W=$windowNameStr/r root:HR_PeakShape:OneRunDisplayPC_Y vs root:HR_PeakShape:OneRunIsolatedPC_fromGaus
	ModifyGraph mode(OneRunDisplayPC_Y)=1,lsize(OneRunDisplayPC_Y)=2
	ModifyGraph rgb(OneRunDisplayPC_Y)=(60672,57856,5632),mode(OneRunDisplayPC_Y#1)=1
	ModifyGraph rgb(OneRunDisplayPC_Y#1)=(65280,0,0)
	
	for (idex = 0;idex<numMass;idex+=1)
		goodNameStr = CleanUPName(GFName[idex],0)		//1.19B
//		tempStr = GFName[idex]
		numgaussXFitwave = (2*GFDeltaNum[idex]+1)*5+1		// 5 is arbitrary, we want a finer resolution on the graph than is fitted
		wave/z gaussYFitwave = $"root:HR_PeakShape:fit_gauss_Y_" + goodNameStr
		wave/z gaussXFitwave = $"root:HR_PeakShape:fit_gauss_X_" + goodNameStr
		if (WaveExists(gaussYFitwave)==0 || WaveExists(gaussXFitwave)==0 )
			make/o/n=(numgaussXFitwave) $"root:HR_PeakShape:fit_gauss_Y_" + goodNameStr
			make/o/n=(numgaussXFitwave) $"root:HR_PeakShape:fit_gauss_X_" + goodNameStr
			wave gaussYFitwave = $"root:HR_PeakShape:fit_gauss_Y_" + goodNameStr
			wave gaussXFitwave = $"root:HR_PeakShape:fit_gauss_X_" + goodNameStr
		endif
		AppendToGraph/W=$windowNameStr/l gaussYFitwave vs gaussXFitwave
		ModifyGraph rgb($NameofWave(gaussYFitwave))=(65280,22016,1024)
		ModifyGraph lsize($NameofWave(gaussYFitwave))=2
		gaussXFitwave = round(OneRunIsolatedPC_fromGaus[idex]) - GFDeltaNum[idex]+p*0.2		// 0.2 from the arbitrary use of 5 above
		gaussYFitwave = GausCoeffsMx[idex][0]+GausCoeffsMx[idex][1]*exp(-((gaussXFitwave-GausCoeffsMx[idex][2])/GausCoeffsMx[idex][3])^2)
	endfor	

	ModifyGraph margin(left)=36,margin(bottom)=36,margin(top)=7,margin(right)=7
	ModifyGraph nticks(right)=0,noLabel(right)=2,standoff(right)=0
	ModifyGraph axRGB(right)=(65535,65535,65535)
	SetAxis right 0,1e+9

// set axis to values for the selected mass
	if (stringMatch(windowNameStr, "PW_OneRunGraph") )
		wave PW_graph = root:HR_PeakShape:PW_graph
		wavestats/q/m=1 PW_graph		// get biggest value
		goodNameStr = CleanUPName(GFName[V_maxLoc],0)		//1.19B
//		tempStr = GFName[V_maxLoc]
		wave/z gaussYFitwave = $"root:HR_PeakShape:fit_gauss_Y_" + goodNameStr
		wave/z gaussXFitwave = $"root:HR_PeakShape:fit_gauss_X_" + goodNameStr
		wavestats/q/m=1 gaussYFitwave
		SetAxis left 0, V_max*1.1
		SetAxis bottom gaussXFitwave[0]-3, gaussXFitwave[numpnts(gaussXFitwave)-1]+3
		Label bottom GFName[V_maxLoc]		
	endif
	
End


Function PW_updatePlotsOneRun()

//	KillWindow PeakWidthShapePanel#PWOneRunMass1
	Display/W=(330,135,430,245)/HOST=# 
	RenameWindow #,PWOneRunMass1
	PW_populate_OneRunGraph(windowNameStr = "PeakWidthShapePanel#PWOneRunMass1")
	SetAxis left nan,nan
	SetAxis bottom nan,nan
	SetActiveSubwindow ##

//	KillWindow PeakWidthShapePanel#PWOneRunMass2
	Display/W=(435,135,535,245)/HOST=# 
	RenameWindow #,PWOneRunMass2
	PW_populate_OneRunGraph(windowNameStr = "PeakWidthShapePanel#PWOneRunMass2")
	SetAxis left nan,nan
	SetAxis bottom nan,nan	
	SetActiveSubwindow ##

//	KillWindow PeakWidthShapePanel#PWOneRunMass3
	Display/W=(540,135,640,245)/HOST=# 
	RenameWindow #,PWOneRunMass3
	PW_populate_OneRunGraph(windowNameStr = "PeakWidthShapePanel#PWOneRunMass3")
	SetAxis left nan,nan
	SetAxis bottom nan,nan	
	SetActiveSubwindow ##

//	KillWindow PeakWidthShapePanel#PWOneRunMass4
	Display/W=(645,135,745,245)/HOST=# 
	RenameWindow #,PWOneRunMass4
	PW_populate_OneRunGraph(windowNameStr = "PeakWidthShapePanel#PWOneRunMass4")
	SetAxis left nan,nan
	SetAxis bottom nan,nan	
	SetActiveSubwindow ##

End


Function PW_butt_updatePWgraph(ctrlName) : ButtonControl
	String ctrlName

	nvar PW_viewVmode = root:HR_PeakShape:PW_viewVmode

	PW_HideChosenPSMass(PW_viewVmode)

	PW_updateMaskPWGraph()
	
	PW_updateMassList()

	nvar PW_viewVmode = root:HR_PeakShape:PW_viewVmode
	string TofTypeStr = selectString(PW_viewVmode, "W", "V")
	
//	PW_calcMassLegend(tofTypeStr)  1.20E

	PW_butt_RecalcPWAvgs("")
	killwaves/z W_sigma
End


// simple function for setting global variable PWUseLineOrPow via panel checkboxes
//Function PW_ck_ViewVorWmode(ctrlName,checked) : CheckBoxControl
//	String ctrlName
//	Variable checked
//	
//	nvar PW_viewVmode = root:HR_PeakShape:PW_viewVmode
//	if (strlen(ctrlName)!=0)
//		PW_viewVmode = strsearch(ctrlName, "V",0)>0
//	endif
//	
//	PW_butt_UpdatePWgraph("")
//
//	PW_updateMaskPWGraph()
//
//End


// recalcs the average pw values for each mass
Function PW_butt_RecalcPWAvgs(ctrlName) : ButtonControl
	String ctrlName

	wave GFmass = root:GaussFits:GFmass
	wave/t GFname = root:GaussFits:GFname
	wave GFmask = root:GaussFits:GFmask
	wave PW_RunMask = root:HR_peakShape:PW_RunMask
	wave DisplayPWAvg = root:HR_peakShape:DisplayPWAvg

	nvar PW_viewVmode = root:HR_peakShape:PW_viewVmode
	
	string tofTypeStr = selectString(PW_viewVmode, "W", "V")
	variable idex, numMass, numRuns, popWinExists, xMin, xMax, yMin, yMax
	string specTypeStr, specTypeList = ";Diff;Closed;Open", goodNameStr
	
	wave IsolatedIon_PWAvg  = root:HR_PeakShape:IsolatedIon_PWAvg
	wave IsolatedIon_PWStD  = root:HR_PeakShape:IsolatedIon_PWStd
	wave PW_XmodeMask = $"root:HR_peakShape:PW_"+tofTypeStr+"modeMask"

	numRuns=numpnts(PW_RunMask)
	numMass = numpnts(GFmass)
	redimension/n=(numMass) IsolatedIon_PWAvg, IsolatedIon_PWStd
	
	fastop IsolatedIon_PWAvg = (nan)
	fastop IsolatedIon_PWStd = (nan)
	fastop DisplayPWAvg = (nan)
	make/FREE/o/n=(numRuns) tempwave
	
	for (idex = 0;idex<numMass;idex+=1)
		if (PW_XmodeMask[idex]==1)
			goodNameStr = CleanupName(GFName[idex],0)		//1.19A
			specTypeStr = StringFromList(GFmask[idex], SpecTypeList)
			wave/z GausCoeffX = $"root:GaussFits:GaussCoeff_"+specTypeStr+"_"+goodNameStr
			if (!WaveExists(GausCoeffX))
				abort "The wave containing the gauss coefficients for "+specTypeStr+" "+goodNameStr+ " was not found.  Perhaps you need to redo the Gauss Fit do it button? aborting from PW_butt_RecalcPWAvgs"
			endif
			tempWave = PW_RunMask*GausCoeffX[p][3]
			WaveStats/q tempWave
			IsolatedIon_PWAvg[idex] = V_avg
			IsolatedIon_PWStd[idex] = V_sdev
			DisplayPWAvg[idex][] = V_avg
		endif
	endfor

	popWinExists = itemsInList(WInList("PW_avgPWGraph", ";", "WIN:1"))>0

	PW_butt_FitPWAvgs()

End


// button for updating the pw mass popupmenu list and other associated globals
Function PW_PopMenuPWList(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	if (stringmatch("Get List", popStr) )
		PW_updateMassList()
	endif

End


Function PW_updateMassList()

 	svar PW_MassList = root:HR_PeakShape:pw_MassList
	wave/t GFname = root:GaussFits:GFname

	nvar PW_viewVmode = root:HR_peakShape:PW_viewVmode		// 1 = V mode, 0 = W
	wave PW_modeMask = $"root:HR_PeakShape:PW_"+selectString(PW_viewVmode, "W", "V")+"modeMask"

	variable idex, num
	num = numpnts(GFname)
	PW_MassList=""
	for(idex=0;idex<num;idex+=1)
		if(PW_modeMask[idex])
			PW_MassList += GFname[idex]+";"
		endif
	endfor
	
//	PW_MassList+="Get List"

End

// generates a new window with the pw mass color legend
Function PW_butt_PWColorLegend(ctrlName) : ButtonControl
	String ctrlName

	nvar PW_viewVmode = root:HR_PeakShape:PW_viewVmode
	string TofTypeStr = selectString(PW_viewVmode, "W", "V")
	
	DoWindow/F $"PWMassLegend"+tofTypeStr
	if (v_flag ==0)
		 PW_calcMassLegend(tofTypeStr)
	 	return 1
	endif

End


//kinda lame way to make a marker legend
Function PW_displayMassLegend(tofTypeStr)
string tofTypeStr

	wave/t GFname = root:GaussFits:GFname
	wave GFmass = root:GaussFits:GFmass

	wave ColorMx = root:HR_peakShape:ColorMx
	wave FilledMarkerWave = root:HR_peakShape:FilledMarkerWave
	wave PW_modeMask = $"root:HR_peakShape:PW_"+tofTypeStr+"modeMask"
	
	variable idex, num, counter, numMarker
	string tempStr, LegendStr, suffixStr
		
	make/o/n=1 root:HR_PeakShape:symbolData = nan
	wave symbolData = root:HR_PeakShape:symbolData
	num = numpnts(GFname)
	numMarker = numpnts(FilledMarkerWave)
	LegendStr=""
	
	Display/N=$"PWMassLegend"+tofTypeStr /W=(40,40,175,275) as "PWMassLegend"+tofTypeStr
	for (idex=0;idex<num;idex+=1)
		if (PW_modeMask[idex]==1)	
			sprintf tempStr "%1.2f", GFmass[idex]
			tempStr = CleanUpName(GFname[idex],0)+" "+tempStr
			appendtograph symbolData
			suffixStr = selectString(counter>0, "", "#"+num2str(counter) )
			ModifyGraph rgb($("symbolData"+suffixStr))=(ColorMx[idex][0], ColorMx[idex][1], ColorMx[idex][2])
			ModifyGraph mode=4,marker($"symbolData"+suffixStr)=FilledMarkerWave[mod(idex, numMarker)]
			LegendStr="\s(symbolData"+suffixStr+")"+ tempStr + "\r"+LegendStr
			counter+=1
		endif
	endfor

	ModifyGraph nticks=0,axThick=0
	legendStr = legendStr[0, strlen(legendStr)-3]		// remove last return 
	legendStr = tofTypeStr+ " mode\r"+legendStr 
	Legend/X=0.00/Y=0/N=MassLegend LegendStr

End


Function PW_calcMassLegend(tofTypeStr)
string tofTypeStr

	wave/t GFname = root:GaussFits:GFname
	wave GFmass = root:GaussFits:GFmass

	wave ColorMx = root:HR_peakShape:ColorMx
	wave FilledMarkerWave = root:HR_peakShape:FilledMarkerWave
	wave PW_modeMask = $"root:HR_peakShape:PW_"+tofTypeStr+"modeMask"
	
	variable idex, num, counter, numMarker
	string tempStr, LegendStr, suffixStr, graphName, goodNameStr
		
	make/o/n=1 root:HR_PeakShape:symbolData = nan
	wave symbolData = root:HR_PeakShape:symbolData
	num = numpnts(GFname)
	numMarker = numpnts(FilledMarkerWave)
	LegendStr=""
	
	graphName = "PWMassLegend"+tofTypeStr 
	DoWindow/F $graphName
	if (V_flag==0)
		Display/N=$graphName/W=(40,40,175,275) as graphName
	endif
	for (idex=0;idex<num;idex+=1)
		if (PW_modeMask[idex]==1)	
			sprintf tempStr "%1.2f", GFmass[idex]
			goodNameStr = CleanupName(GFName[idex],0)		//1.19A
			tempStr = CleanUpName(GFname[idex],0)+" "+tempStr
			appendtograph/W=$graphName symbolData
			suffixStr = selectString(counter>0, "", "#"+num2str(counter) )
			ModifyGraph/W=$graphName rgb($("symbolData"+suffixStr))=(ColorMx[idex][0], ColorMx[idex][1], ColorMx[idex][2])
			ModifyGraph/W=$graphName mode=4,marker($"symbolData"+suffixStr)=FilledMarkerWave[mod(idex, numMarker)]
			LegendStr="\s(symbolData"+suffixStr+")"+ tempStr + "\r"+LegendStr
			counter+=1
		endif
	endfor

	ModifyGraph/W=$graphName nticks=0,axThick=0
	legendStr = legendStr[0, strlen(legendStr)-3]		// remove last return 
	legendStr = tofTypeStr+ " mode\r"+legendStr 
	Legend/K/W=$graphName/N=MassLegend 
	Legend/W=$graphName/X=0.00/Y=0/N=MassLegend LegendStr

End


Function PW_MakeFinalPWWaveForPS()

	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave	
	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave

	nvar PWUseLineOrPow = root:HR_PeakShape:PWUseLineOrPow

	wave GFmass = root:GaussFits:GFmass
		
	variable slope, intercept, power
	
	make/o/n=(numpnts(GFmass)) root:HR_PeakShape:FinalPW
	wave  FinalPW = root:HR_PeakShape:FinalPW
	
	FinalPW =intercept + slope*(GFmass^power)

End


Function PW_SanityCheckTodoWave()

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	ControlInfo/w=PeakWidthShapePanel PSPW_popup_ToDoWave
	if (V_flag ==0 || !stringmatch(S_value, sq_toDoWvNm))
		abort "Please reselect the todo wave from the PeakWidthShapePanel - aborting from PW_SanityCheckTodoWave"
	endif

End


Function HR_TodoWaveHasOnlyModeX(wantVmode)
variable wantVmode

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	if(wantVmode)
		wave TofTypeXKey = root:diagnostics:TofTypeVKey
	else
		wave TofTypeXKey = root:diagnostics:TofTypeWKey
	endif
	duplicate/o TofTypeXKey root:tempWave
	wave tempwave = root:tempWave
	tempwave/=tempwave
	sq_create_todo(sq_toDoWvNm+" and tempWave" ,"tempTodo")
	wave tempTodo
	if (numpnts(tempTodo)!=numpnts(todo))
		killwaves/z tempTodo, tempWave
		abort "There are runs of type V and W in the selected todo wave of "+sq_todoWvNm+ " - aborting from pk_TodoWaveHasOnlyModeX"
	endif
	killwaves/z tempTodo, tempWave
	
End


//Function PW_butt_RedoLinearFits(ctrlName) : ButtonControl
//	String ctrlName
//
//	wave GFMass=root:GaussFits:GFMass
//	wave/t GFname=root:GaussFits:GFname
//
//	wave PW_VmodeMask=root:HR_PeakShape:PW_VmodeMask
//	wave PW_WmodeMask=root:HR_PeakShape:PW_WmodeMask
//	wave PWSlope=root:HR_PeakShape:PWSlope
//	wave PWIntercept=root:HR_PeakShape:PWIntercept
//
//	wave PWCoeffs=root:HR_PeakShape:PWCoeffs
//	wave PWConstraintsLine = root:HR_PeakShape:PWConstraintsLine
//
//	wave ToFTypeVKey=root:diagnostics:ToFTypeVKey
//	wave rn_series = root:index:rn_series
//	svar sq_toDoWvNm=root:panel:sq_toDoWvNm
//	wave/z todo=$(sq_toDoWvNm)
//
//	variable numMasses, idex, numRuns, massDex, pos
//		
//	PW_SanityCheckTodoWave()
//
//	gen_SanityTodoWaveExists()
//
//	numMasses=numpnts(GFMass)
//	numRuns = numpnts(rn_series)
//		
//	make/o/n=(numMasses) tempWave
//	
//	// things are much easier in Matrix form.
//	make/o/n=(numRuns, numMasses) GFmx
//	for(idex=0; idex<numMasses; idex+=1)
//		wave GFwave = $"root:GaussFits:GaussCoeff_Closed_"+GFname[idex]
//		GFmx[][idex] = GFwave[p][3]
//	endfor
//	
//	for(idex=0; idex<numRuns; idex+=1)
//		tempWave[] = GFMx[idex][p]
//		if (ToFTypeVKey[idex])
//			tempWave/=PW_VmodeMask[p]
//		else
//			tempWave/=PW_WmodeMask[p]
//		endif
//		wavestats/q/m=1 tempWave
//		if (V_nPnts>2)
//			PWCoeffs = {1,1}		// defaults for w mode - taken from baseline panel
//			CurveFit/N/NTHR=0/Q line kwCWave=PWCoeffs, tempWave /X =GFmass /C = PWConstraintsLine
//		else
//			PWCoeffs = nan
//		endif	
//		PWIntercept[idex] = PWCoeffs[0]
//		PWSlope[idex] = PWCoeffs[1]
//	endfor	
//	//^ clean up panel
//
//	killwaves/z tempWave, GFmx
//	
//	PW_updateMaskPWGraph()
//
//End
//

Function PW_butt_SetPWDefaults(ctrlName) : ButtonControl
	String ctrlName

	PW_SetDefaults()

End


Function PW_SetDefaults()

	wave/t/z GFName=root:GaussFits:GFName
	if (!WaveExists(GFName))
		return 0
	endif

	variable idex, num, pos
	
	num = numpnts(GFName)
	make/o/n=(num) root:HR_peakShape:PW_Graph=1
	make/o/n=(num) root:HR_peakShape:PW_VmodeMask=0
	make/o/n=(num) root:HR_peakShape:PW_WmodeMask=0

	wave PW_VmodeMask  = root:HR_peakShape:PW_VmodeMask
	wave PW_WmodeMask  = root:HR_peakShape:PW_WmodeMask

	num = itemsInList(DEFAULTPWLIST)
	for (idex=0;idex<num;idex+=1)
		pos = sq_findStr(GFName,stringFromList(idex, DEFAULTPWLIST),1)
		if (pos>0)
			PW_VmodeMask[pos]=1
			PW_WmodeMask[pos]=1
		endif
	endfor	
	
End	


Function PW_makeFilledMarkerWave()

	make/o/i root:HR_PeakShape:FilledMarkerWave = {19,16,17,23,46,49,26,29,18,32,34,36,38,15,14}

End


// edited in 1.06D to have twice the number of colors
Function PW_makeColorMx()

	make/o/i/n=(60,3) root:HR_PeakShape:ColorMx
	wave ColorMx = root:HR_PeakShape:ColorMx

	ColorMx[0]={ {41627} , {65534} , {2621} }
	ColorMx[1]={ {29042} , {65534} , {2621} }
	ColorMx[2]={ {16462} , {65534} , {2621} }
	ColorMx[3]={ {3876} , {65534} , {2621} }
	ColorMx[4]={ {2621} , {65534} , {13945} }
	ColorMx[5]={ {2621} , {65534} , {26530} }
	ColorMx[6]={ {2621} , {65534} , {39110} }
	ColorMx[7]={ {2621} , {65534} , {51696} }
	ColorMx[8]={ {2621} , {65534} , {64276} }
	ColorMx[9]={ {2621} , {54207} , {65534} }
	ColorMx[10]={ {2621} , {41627} , {65534} }
	ColorMx[11]={ {2621} , {29042} , {65534} }
	ColorMx[12]={ {2621} , {16462} , {65534} }
	ColorMx[13]={ {2621} , {3876} , {65534} }
	ColorMx[14]={ {13945} , {2621} , {65534} }
	ColorMx[15]={ {26530} , {2621} , {65534} }
	ColorMx[16]={ {39110} , {2621} , {65534} }
	ColorMx[17]={ {51696} , {2621} , {65534} }
	ColorMx[18]={ {64276} , {2621} , {65534} }
	ColorMx[19]={ {65534} , {2621} , {54207} }
	ColorMx[20]={ {65534} , {2621} , {41627} }
	ColorMx[21]={ {65534} , {2621} , {29042} }
	ColorMx[22]={ {65534} , {2621} , {16462} }
	ColorMx[23]={ {65534} , {2621} , {3876} }
	ColorMx[24]={ {65534} , {13945} , {2621} }
	ColorMx[25]={ {65534} , {26530} , {2621} }
	ColorMx[26]={ {65534} , {39110} , {2621} }
	ColorMx[27]={ {65534} , {51696} , {2621} }
	ColorMx[28]={ {65534} , {64276} , {2621} }
	ColorMx[29]={ {54207} , {65534} , {2621} }

	ColorMx[30,59][] = (ColorMx[p-30][q] )/2
End

			
Function PS_createGlobals()

	newDataFolder/o root:HR_PeakShape

	sq_initializeGlobalSVar("root:HR_PeakShape:Step4Prelim","")
	svar Step4Prelim = root:HR_PeakShape:Step4Prelim
	Step4Prelim ="'Normalized' peak shape:\r"
	Step4Prelim +="y axis: data/gaussian peak height\r"
	Step4Prelim +="x-axis: distance from peak center\r"
	Step4Prelim +="           scaled by peak width"

	sq_initializeGlobalSVar("root:HR_PeakShape:Step4", "")
	svar Step4 = root:HR_PeakShape:Step4
	Step4 = "Step 4. Using masses, runs, and peak width parameters from Step 3,\rgenerate a peak shape candidate of current todo wave."
	sq_initializeGlobalSVar("root:HR_PeakShape:Step5","Step 5. Declare candidate as final peak shape.")
	
// "Holding the peak width will result in refitting peaks to gaussians."
// "This will take more time, but the peak shape result will reflect the same scatter as is present when used for pika fits."

	sq_initializeGlobalSVar("root:HR_PeakShape:PS_VmodeTitle","Peak width selections for "+"mode:")

	sq_initializeGlobalSVar("root:HR_PeakShape:PS_massList_left","Ar,O2,SO")	// somewhat arbitrary defaults, changed in 1.10C
	sq_initializeGlobalSVar("root:HR_PeakShape:PS_massList_right","O2")		//1.07// somewhat arbitrary defaults

	sq_initializeGlobalSVar("root:HR_PeakShape:Step4A","Step 4A. Generate a new peak shape by combining left and right mass peak shapes.")
	
	sq_initializeGlobalNVar("root:HR_peakShape:PS_ShowFinal", 1)

	sq_initializeGlobalNVar("root:HR_peakShape:PS_ShowResid", 1)

	sq_initializeGlobalNVar("root:HR_peakShape:PS_enforceMonotonicity", 1)		//1.08E
	sq_initializeGlobalNVar("root:HR_peakShape:PS_enforceNotGTGauss", 1)		//1.08E

	sq_initializeGlobalNVar("root:HR_PeakShape:PS_logLeftAxisCandidPS",0)
	sq_initializeGlobalNVar("root:HR_PeakShape:PS_logLeftAxisFinalPS",0)

	sq_initializeGlobalSVar("root:HR_PeakShape:PS_ResidList", "")

	sq_initializeGlobalNVar("root:HR_PeakShape:PS_ToSmoothOrNot",1)

	nvar NumPtsFinalPSWave =root:HR_PeakShape:NumPtsFinalPSWave
	nvar MaxSigma =root:HR_PeakShape:MaxSigma
	hr_initializeGlobalWave("root:HR_PeakShape:PeakShapeCandidate", num2str(NumPtsFinalPSWave),nan)	
	wave PeakShapeCandidate = root:HR_PeakShape:PeakShapeCandidate

	hr_initializeGlobalWave("root:HR_PeakShape:PeakShape", num2str(NumPtsFinalPSWave),nan)	
	wave PeakShape = root:HR_PeakShape:PeakShape
		
	make/o/n=(NumPtsFinalPSWave) root:HR_PeakShape:PerfectGaussY
	make/o/n=(NumPtsFinalPSWave) root:HR_PeakShape:PerfectGaussX
	wave PerfectGaussY = root:HR_PeakShape:PerfectGaussY
	wave PerfectGaussX = root:HR_PeakShape:PerfectGaussX
	variable delta = 2*MaxSigma/(NumPtsFinalPSWave-1)
	PerfectGaussX = -MaxSigma + p*delta
	PerfectGaussY = exp(-PerfectGaussX^2)
	
	SetScale/P x, -MaxSigma,delta, "", PerfectGaussY, PeakShape, PeakShapeCandidate

	sq_initializeGlobalNVar("root:HR_PeakShape:HR_FinalPSScalar",1)


End


// Creates, displays peak shape for one mode (v/w) for all masses, runs 
Function PS_butt_PeakShapeAllMassTodo(ctrlName) : ButtonControl
	String ctrlName
	
 	// we need to remove all previous plots in Final Peak shape and the peak shape scalar
	svar PS_residList = root:HR_PeakShape:PS_residList
 	nvar HR_FinalPeakShapeScalar = root:HR_PeakShape:HR_FinalPeakShapeScalar

 	wave FinalPeakShape = root:HR_PeakShape:FinalPeakShape
 	string myWaveList, myWaveStr
 	variable idex, num

 	PS_residList = ""
 	HR_FinalPeakShapeScalar = nan
 	FinalPeakShape = nan
 	
 	myWaveList = TraceNameList("PeakWidthShapePanel#PS_FinalPSGraph", ";", 1)
 	num = itemsInList(myWaveList)
 	for(idex=0;idex<num;idex+=1)
 		myWaveStr = StringFromList(idex, myWaveList)
 		if (strsearch(myWaveStr, "PeakShapeResid", 0) >=0)
 			RemoveFromGraph/W=PeakWidthShapePanel#PS_FinalPSGraph $myWaveStr
 		endif
 	endfor
 	
	//  we don't bother with using different m/z parameters for open and closed.  we just use whatever has been done last.
	svar todoStr=root:panel:sq_toDoWvNm		//1.09I change
	wave/z todo=$(todoStr)
	sq_check_NansInWaveList(todo, "root:mzFitting:mz_FitIntercept;root:mzFitting:mz_FitSlope;root:mzFitting:mz_FitPower")
	
	PS_PeakShapeAllMassTodo() 	
 	PS_populate_PeakShapeGraph()

End


// Createss peak shape for one mode (v/w) for all masses, runs 
Function PS_PeakShapeAllMassTodo() 
	
 //	nvar HR_UseSqBaseline = root:HR_panel:HR_UseSqBaseline
 	wave GFMask = root:GaussFits:GFMask
 	svar pw_massList = root:HR_PeakShape:pw_massList
 	
   	svar sq_toDoWvNm=root:panel:sq_toDoWvNm		//^ 
	wave/z todo=$"root:"+sq_toDoWvNm
  	
//	nvar PW_viewVmode = root:HR_PeakShape:PW_viewVmode		//1 = V, 0 = W
// 	wave PW_RunMask = root:HR_PeakShape:PW_RunMask

	gen_SanityTodoWaveExists()
  	PW_SanityCheckTodoWave()
 
//	sq_create_todo(sq_toDoWvNm +" and root:HR_PeakShape:PW_RunMask","todoTemp")
//	wave/z todoTemp = root:todoTemp
//	if (!WaveExists(todoTemp) || numpnts(todoTemp)==0)
//		abort "There are no runs for which there is good peak width data within the todo "+sq_toDoWvNm
//	else
//		sq_create_todo("todoTemp and root:diagnostics:tofType"+selectString(PW_viewVmode, "W", "V")+"Key","todoTemp")
//		wave/z todoTemp = root:todoTemp
//		if (!WaveExists(todoTemp) || numpnts(todoTemp)==0)
//			abort "There are no runs for which there is good peak width data of the ToF Type: "+selectString(PW_viewVmode, "W", "V")
//		endif
//	endif
  	
	print "//** Finding the peak shape for all masses began at "+time()+" on the "+sq_toDoWvNm+" todo wave."

//	wavestats/q GFMask		//1.06C
	variable openAlso = WaveMax(GFMask) > 2 ? 1 : 0

// 	if (HR_UseSqBaseline)
   		squirrel_fetch(todo,PS_fetch_PeakShapeAllMassTodo, "SpectraType:Closed","MSClosedLessBaseL_p;")
		if (openAlso)	
   			squirrel_fetch(todo,PS_fetch_PeakShapeAllMassTodo, "SpectraType:Open","MSOpenLessBaseL_p;")
		endif
//	else
//   		squirrel_fetch(todo,PS_fetch_PeakShapeAllMassTodo, "SpectraType:Closed","MSClosed_p;")
//		if (openAlso)	
//   			squirrel_fetch(todo,PS_fetch_PeakShapeAllMassTodo, "SpectraType:Open","MSOpen_p;")
//		endif
//	endif
 	
	killwaves/z todoTemp
		
	PS_CalcMassAvgPSWaves(pw_massList)
	
	PS_CalcPSCandidateFromList(pw_massList)
	
	print "//** Finding the peak shape for all masses ended at "+time()+" on the "+sq_toDoWvNm+" todo wave."

End


// squirrel fetch function
// sample usage:   squirrel_fetch(todo,PS_fetch_PeakShapeAllMassTodo, "TofType:V;SpectraType:Closed","MSClosedLessBaseL_p;")
// main purpose is to fill in the values of 3 waves: root:HR_PeakShape:PS_RawX_"+TofTypeStr+"_"+GFName[idex], 2 others
Function PS_fetch_PeakShapeAllMassTodo(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn 
	variable n_acorns

	wave MSSpectra=$stringbykey("acorn_0",paramstr)
	string SpectraTypeStr=stringbykey("SpectraType",paramstr)
	
	nvar HR_UseOpenClosedMzParams = root:HR_panel:HR_UseOpenClosedMzParams
	if (HR_UseOpenClosedMzParams)
		wave mz_FitIntercept = $"root:mzFitting:mz_FitIntercept"+SpectraTypeStr
		wave mz_FitSlope= $"root:mzFitting:mz_FitSlope"+SpectraTypeStr
		wave mz_FitPower = $"root:mzFitting:mz_FitPower"+SpectraTypeStr
	else
		wave mz_FitIntercept = root:mzFitting:mz_FitIntercept
		wave mz_FitSlope= root:mzFitting:mz_FitSlope
		wave mz_FitPower = root:mzFitting:mz_FitPower
	endif
	
	wave/t GFName = root:GaussFits:GFName
	wave GFdeltaNum = root:GaussFits:GFdeltaNum
	wave GFMass = root:GaussFits:GFMass
	wave GFMask = root:GaussFits:GFMask
	wave GausCoeffsWv = root:GaussFits:GausCoeffsWv

	wave RawSpectraThisRun = root:HR_PeakShape:RawSpectraThisRun
	
	wave rn_series=root:index:rn_series
	wave series_index=root:index:series_index

//	nvar PS_holdWidth = root:HR_PeakShape:PS_holdWidth

	variable thisAcornRows, thisAcornCols, idex, currentRunIndex, kdex, num, intercept, slope, power, pos
	string massStr, specTypeList = ";Diff;Closed;Open"

	variable specType = WhichListItem(SpectraTypeStr, specTypeList)
	
	svar  pw_massList = root:HR_PeakShape:pw_massList
	num = ItemsInList(pw_massList)

	if (acorn==0)
		PW_MakeFinalPWWaveForPS()
		for(idex=0;idex<num;idex+=1) // loop to deal with each run
			massStr = StringFromList(idex, pw_massList)
			pos=sq_findStr(GFName,massStr,1)
			if (GFMask[pos]==specType)
				massStr = StringFromList(idex, pw_massList)
				make/o/n=0 $"root:HR_PeakShape:PS_RawDataX_"+massStr
				make/o/n=0 $"root:HR_PeakShape:PS_RawDataY_"+massStr
			endif
		endfor
	endif
	
	thisAcornRows=dimsize(MSSpectra,0)
	thisAcornCols=dimsize(MSSpectra,1)
				
	if (numpnts(RawSpectraThisRun)!=thisAcornCols)
		Redimension/n=(thisAcornCols) RawSpectraThisRun
	endif
	
	for(kdex=0;kdex<thisAcornRows;kdex+=1) // loop to deal with each run
		
		sq_ams_prog_upd(100*(acorn+(kdex/thisacornrows))/n_acorns)

		if (index_pos[kdex]==-1)
			continue
		endif

		RawSpectraThisRun = MSSpectra[kdex][p]

		currentRunIndex =series_index[index_pos[kdex]]
		
		intercept = mz_FitIntercept[currentRunIndex]
		slope = mz_FitSlope[currentRunIndex]
		power = mz_FitPower[currentRunIndex]
				
		for(idex=0;idex<num;idex+=1) // loop to deal with each mass
			massStr = StringFromList(idex, pw_massList)
			pos=sq_findStr(GFName,massStr,1)
			if (GFMask[pos]==specType)
				wave/z PWGaussCoeff = $"root:GaussFits:GaussCoeff_"+SpectraTypeStr+"_"+massStr
				if (!WaveExists(PWGaussCoeff))
					abort "Could not find the wave GaussCoeff_"+SpectraTypeStr+"_"+massStr+ ".  Perhaps you need to go back to the GaussFit tab?  aborting from PS_fetch_PeakShapeAllMassTodo"
				endif
				GausCoeffsWv[] = PWGaussCoeff[currentRunIndex][p]
				wave PS_RawDataX = $"root:HR_peakShape:PS_RawDataX_"+massStr
				wave PS_RawDataY = $"root:HR_peakShape:PS_RawDataY_"+massStr
	 			PS_PeakShapeOneMass(RawSpectraThisRun, PS_RawDataX, PS_RawDataY, GausCoeffsWv,  intercept, slope,  power, GFmass[pos])	
			endif
		endfor	// mass loop			

	endfor // run number loop	
	
	if(acorn == n_acorns-1)
		for(idex=0;idex<num;idex+=1) // loop to deal with each mass
			massStr = StringFromList(idex, pw_massList)
			pos=sq_findStr(GFName,massStr,1)
			if (GFMask[pos]==specType)
				massStr = StringFromList(idex, pw_massList)
				wave PS_RawDataX = $"root:HR_peakShape:PS_RawDataX_"+massStr
				wave PS_RawDataY = $"root:HR_peakShape:PS_RawDataY_"+massStr
		 		PS_CleanUpPeakShapeWaves(PS_RawDataX, PS_RawDataY)
			endif
		endfor
		Killwaves/z thisMassSectionX, thisMassSectionY, NormalizedX, NormalizedY	// generated during PS_PeakShapeOneMass
	endif

End
 
 
Function PS_CalcMassAvgPSWaves(massList)
string massList
			
	variable kdex, num
	string massStr
	num = itemsInList(massList)
	for (kdex=0;kdex<num;kdex+=1)		// loop for each ion
		massStr = stringfromList(kdex, massList)   
		wave/z PS_RawDataX = $"root:HR_peakShape:PS_RawDataX_"+massStr
		wave/z PS_RawDataY = $"root:HR_peakShape:PS_RawDataY_"+massStr
		if (!WaveExists(PS_RawDataX) ||  !WaveExists(PS_RawDataY) )
			abort "There is no peak shape data for mass "+massStr + " -aborting from PS_MakePSWaves"
		endif
		make/o/n=0 $"root:HR_peakShape:PS_AvgX_"+massStr
		make/o/n=0 $"root:HR_peakShape:PS_StdX_"+massStr
		make/o/n=0 $"root:HR_peakShape:PS_AvgY_"+massStr
		make/o/n=0 $"root:HR_peakShape:PS_StdY_"+massStr

		wave PS_AvgX = $"root:HR_peakShape:PS_AvgX_"+massStr
		wave PS_StdX = $"root:HR_peakShape:PS_StdX_"+massStr
		wave PS_AvgY = $"root:HR_peakShape:PS_AvgY_"+massStr		
		wave PS_StdY = $"root:HR_peakShape:PS_StdY_"+massStr		

		PS_CalcPSAvgWaves(PS_RawDataX, PS_RawDataY, PS_AvgX, PS_StdX, PS_AvgY,PS_StdY)
	
	endfor 

End


Function PS_CalcPSCandidateFromList(MassList)
string MassList

	variable idex, num
	string massStr
	
	wave PeakShapeCandidate = root:HR_peakShape:PeakShapeCandidate

	make/o/n=0 $"root:HR_peakShape:PS_AvgX"
	make/o/n=0 $"root:HR_peakShape:PS_StdX"
	make/o/n=0 $"root:HR_peakShape:PS_AvgY"
	make/o/n=0 $"root:HR_peakShape:PS_StdY"

	wave PS_AvgX = $"root:HR_peakShape:PS_AvgX"
	wave PS_StdX = $"root:HR_peakShape:PS_StdX"
	wave PS_AvgY = $"root:HR_peakShape:PS_AvgY"
	wave PS_StdY = $"root:HR_peakShape:PS_StdY"
		
	make/o/n=0 $"root:HR_PeakShape:PS_RawDataX"
	make/o/n=0 $"root:HR_PeakShape:PS_RawDataY"

	wave PS_RawDataX = $"root:HR_PeakShape:PS_RawDataX"		// combines masses
	wave PS_RawDataY = $"root:HR_PeakShape:PS_RawDataY"  // combines masses

	num = ItemsInList(MassList)
	variable oldNum		//1.06A yucky bug fixwith Concatenate, bug resulted in using only the last mass
	for (idex=0;idex<num;idex+=1)		// loop for each ion
		massStr =stringFromList(idex, massList)	
		wave PS_RawDataX_mass = $"root:HR_peakShape:PS_RawDataX_"+massStr
		wave PS_RawDataY_mass = $"root:HR_peakShape:PS_RawDataY_"+massStr		
		
		//1.05A fix for Igor 6.10, found by U Hawaii folks
//		Concatenate/o {PS_RawDataX, PS_RawDataX_mass}, $"root:HR_PeakShape:PS_RawDataX"
// 		Concatenate/o {PS_RawDataY, PS_RawDataY_mass}, $"root:HR_PeakShape:PS_RawDataY"		
//		Concatenate/o {PS_RawDataX_mass}, $"root:HR_PeakShape:PS_RawDataX"
// 		Concatenate/o {PS_RawDataY_mass}, $"root:HR_PeakShape:PS_RawDataY"		
		//1.06A yucky bug fix with Concatenate, bug resulted in using only the last mass
		oldNum = numpnts(PS_RawDataX)
		redimension/n=(oldNum+numpnts(PS_RawDataX_mass)) PS_RawDataX, PS_RawDataY
		PS_RawDataX[oldNum,oldNum+numpnts(PS_RawDataX_mass)-1] = PS_RawDataX_mass[p-oldNum]
		PS_RawDataY[oldNum,oldNum+numpnts(PS_RawDataY_mass)-1] = PS_RawDataY_mass[p-oldNum]
 	endfor
 	
	PS_CalcPSAvgWaves(PS_RawDataX, PS_RawDataY, PS_AvgX, PS_StdX, PS_AvgY,PS_StdY) 

 	PS_MakePSWaveCandidate(PS_AvgX, PS_AvgY, PeakShapeCandidate) 

End


//  ion time of flight = intercept+slope*(exactMass^power)
// adds values to PeakShapeMassX, PeakShapeMass for building up a peak shape
Function PS_PeakShapeOneMass(RawSpectraThisRun, PeakShapeMassX, PeakShapeMassY, GausCoeffsWv,  intercept, slope,  power, ps_MassVal)
wave RawSpectraThisRun, PeakShapeMassX, PeakShapeMassY, GausCoeffsWv
variable  intercept, slope, power, ps_MassVal
	
	variable numptsForThisMass, t0, t_begin, t_end, num
	variable myk0, myk1, myk2, myk3
	
	nvar MaxSigma = root:HR_PeakShape:MaxSigma
	
	wavestats/q/m=1 GausCoeffsWv
	if (V_npnts !=4)
		return 0
	endif
	
	myk0 = GausCoeffsWv[0]
	myk1 = GausCoeffsWv[1] 
	myk2 = GausCoeffsWv[2]
	myk3 = GausCoeffsWv[3]
		
	t0 = intercept+slope*( (ps_MassVal)^power)		// the center of the peak 
	// 1.14I we do not care if intercept is negative
	t_begin = floor( t0 - MaxSigma*myK3)	// left most point of peak
	t_end = ceil (t0+ MaxSigma*myK3)		// right most point of peak

	numptsForThisMass = t_end - t_begin + 1		// fencepost counting
	 if (numptsForThisMass >100)		// modification suggestion by Tim O   100 is arbitrary - this value could be smaller
	        return 0
	    endif
    	
	duplicate/o/r=[t_begin,t_end]  RawSpectraThisRun thisMassSectionY, thisMassSectionX, NormalizedX, NormalizedY		// gaussianFit
	thisMassSectionX = t_begin+p
	NormalizedX = (thisMassSectionX-myk2)/myk3
		
//	thisMassSectionY-=myk0		// subtract baseline
//	gaussianFit = myk1*exp(-(NormalizedX)^2)		// recreate gaussian fit
	NormalizedY = (thisMassSectionY - myK0)/myK1 // (thisMassSectionY/gaussianFit)		// find y

	// dump results into PeakShapeMassX, PeakShapeMassY waves
	num = numpnts(PeakShapeMassX)
	InsertPoints num, numptsForThisMass, PeakShapeMassX, PeakShapeMassY

	// keep the x-axis in units of perfect gaussian - subtract peak center and divide by width
	PeakShapeMassX[num,] =NormalizedX[p-num]  ///sqrt(2))
	PeakShapeMassY[num,] = NormalizedY[p-num]   
	
//	Killwaves/z thisMassSectionX, thisMassSectionY		// let's kill these later

End


// removes any nans from x or y wave and any values too far to the left or right, top or bottom
Function PS_CleanUpPeakShapeWaves(PS_RawX, PS_RawY)
wave PS_RawX, PS_RawY
	
	nvar MaxSigma = root:HR_PeakShape:MaxSigma
	
	variable pt, num, idex, numMass
		
	num = numpnts(PS_RawY)
	sort PS_RawY PS_RawX, PS_RawY

	// remove nans (from y wave)
	//1.16G
	if (numpnts(PS_RawY)==0)
		return 0
	endif
		wavestats/q/m=1 PS_RawY
	deletepoints V_npnts, num, PS_RawX, PS_RawY
		
	// remove values that are < -1e15 or > 1e15
	pt = BinarySearch(PS_RawY, -1e15)
	if (pt>=0)
		deletepoints 0, pt, PS_RawX, PS_RawY
	endif
	pt = BinarySearch(PS_RawY,  1e15)
	if (pt>=0)
		deletepoints pt, num, PS_RawX, PS_RawY
	endif
	
	num = numpnts(PS_RawY)		
	sort PS_RawX PS_RawY, PS_RawX

	// remove nans (from x wave)
	wavestats/q/m=1 PS_RawX
	deletepoints V_npnts, num, PS_RawX, PS_RawY
	num = numpnts(PS_RawY)
	
	// remove points < -MaxSigma or > MaxSigma (from x wave)
	pt = BinarySearch(PS_RawX, -MaxSigma)
	if (pt>=0)
		deletepoints 0, pt, PS_RawX, PS_RawY
	endif
	pt = BinarySearch(PS_RawX, MaxSigma)
	if (pt>=0)
		deletepoints pt, num, PS_RawX, PS_RawY
	endif
	
End


// find values for PS_AvgX, PS_StdX, PS_AvgY,PS_StdY
Function PS_CalcPSAvgWaves(PS_RawDataX, PS_RawDataY, PS_AvgX, PS_StdX, PS_AvgY,PS_StdY) 
wave PS_RawDataX, PS_RawDataY, PS_AvgX, PS_StdX, PS_AvgY,PS_StdY

	variable beginPt, endPt, idex, num, delta
	
	nvar NumPtsFinalPSWave = root:HR_PeakShape:NumPtsFinalPSWave
	nvar MaxSigma = root:HR_PeakShape:MaxSigma
	nvar PS_ToSmoothOrNot = root:HR_PeakShape:PS_ToSmoothOrNot
			
	sort PS_RawDataX PS_RawDataY, PS_RawDataX
	redimension/n=(NumPtsFinalPSWave) PS_AvgX, PS_StdX, PS_AvgY,PS_StdY
	
	num = numpnts(PS_RawDataX)
		
	fastop PS_AvgX=(nan)
	fastop PS_StdX=(nan)

	fastop PS_AvgY=(nan)
	fastop PS_StdY=(nan)

	// we get the edges of each grouping
	beginPt = 0
	endPt = 0
	delta = 2*MaxSigma/(NumPtsFinalPSWave-1)

	for(idex=0;idex<NumPtsFinalPSWave ;idex+=1)
		do
			endPt +=1
		while( endPt<num &&  PS_RawDataX[endPt] < -MaxSigma+idex*delta + .5*delta)	
	
		endPt -=1

		if (endPt - beginPt >=1)
		
			Wavestats/q/r=[beginPt, endPt] PS_RawDataX
			PS_AvgX[idex]=V_avg
			PS_StdX[idex]=V_sdev

			Wavestats/q/r=[beginPt, endPt] PS_RawDataY
			PS_AvgY[idex]=V_avg
			PS_StdY[idex]=V_sdev
		
			beginPt = endPt+1	
		else
			beginPt = endPt	
		endif
		
	endfor
	
	if (PS_ToSmoothOrNot)
		waveStats/q/m=1 PS_AvgY
		if(V_numNans==0)
			smooth/B 3, PS_AvgY
		endif
	endif

End


// given a trio of average x, average y, average 'normalized' (gaussian) peak shapes, find a candidate for a final peak shape wave
Function  PS_MakePSWaveCandidate(PS_AvgX, PS_AvgY, PeakShapeCandidate) 
wave PS_AvgX, PS_AvgY, PeakShapeCandidate

	nvar NumPtsFinalPSWave = root:HR_PeakShape:NumPtsFinalPSWave
	nvar MaxSigma = root:HR_PeakShape:MaxSigma
	nvar PS_enforceMonotonicity = root:HR_peakShape:PS_enforceMonotonicity
	nvar PS_enforceNotGTGauss = root:HR_peakShape:PS_enforceNotGTGauss
	nvar PS_ToSmoothOrNot = root:HR_peakShape:PS_ToSmoothOrNot
	
	variable delta
	
	redimension/n=(NumPtsFinalPSWave) PeakShapeCandidate

	make/o/n=(NumPtsFinalPSWave) root:HR_PeakShape:PeakShapeXaxis
	wave PeakShapeXaxis = root:HR_PeakShape:PeakShapeXaxis
	
	delta = 2*MaxSigma/(NumPtsFinalPSWave-1)
	PeakShapeXaxis = -MaxSigma+delta*p	// final grid
	SetScale/P x -MaxSigma,delta,"", PeakShapeCandidate
	
	// before we do any interpolation across the x axis we need to remove nans, make sure to keep x order before interpolating	
	WaveStats/q PS_AvgY
	if (V_numNans>0)
		sort PS_AvgY, PS_AvgX, PS_AvgY
		deletePoints V_npnts, V_numNans, PS_AvgX, PS_AvgY, PS_AvgX
	endif	
	WaveStats/q PS_AvgX
	if (V_numNans>0)
		sort PS_AvgX, PS_AvgY, PS_AvgY, PS_AvgX
		deletePoints V_npnts, V_numNans, PS_AvgX, PS_AvgY, PS_AvgX
	endif	
	
	sort PS_AvgX, PS_AvgY, PS_AvgX

 	PeakShapeCandidate = interp(PeakShapeXaxis[p], PS_Avgx, PS_Avgy )
		
	if (PS_enforceNotGTGauss)
		PS_forceNotGTGauss(PeakShapeCandidate)
	endif

	if (PS_enforceMonotonicity)
		PS_forceMonotonicity(PeakShapeCandidate)
	endif

	if (PS_ToSmoothOrNot)
		smooth/B 3, PeakShapeCandidate
	endif
	
End


// Uses the x scaling of PeakShapeCandidate to check and enforce
Function PS_forceMonotonicity(PeakShapeCandidate)
wave PeakShapeCandidate

	wave PerfectGaussY = root:HR_PeakShape:PerfectGaussY
	nvar NumPtsFinalPSWave = root:HR_PeakShape:NumPtsFinalPSWave
	nvar MaxSigma = root:HR_PeakShape:MaxSigma

	variable delta, pos, idex
	delta = 2*MaxSigma/(NumPtsFinalPSWave-1) //	PeakShapeXaxis = -MaxSigma+delta*p	// final grid

	pos = floor((MaxSigma - 3)/delta)		// point number in x axis of PeakShapeCandidate  at -3 (similar for 3)
	for(idex=pos; idex>0; idex-=1)	// going left on the left hand side of gaussian shape "look backwards"
		if (PeakShapeCandidate[idex-1] > PeakShapeCandidate[idex])
			PeakShapeCandidate[idex-1] =PeakShapeCandidate[idex]
		endif
	endfor
	
	for(idex=NumPtsFinalPSWave-pos; idex<NumPtsFinalPSWave-1; idex+=1)	// going right on the right hand side of gaussian shape
		if (PeakShapeCandidate[idex+1] > PeakShapeCandidate[idex])
			PeakShapeCandidate[idex+1] = PeakShapeCandidate[idex]
		endif
	endfor

End


Function PS_forceNotGTGauss(PeakShapeCandidate)
wave PeakShapeCandidate

	wave PerfectGaussY = root:HR_PeakShape:PerfectGaussY
	nvar NumPtsFinalPSWave = root:HR_PeakShape:NumPtsFinalPSWave
	nvar MaxSigma = root:HR_PeakShape:MaxSigma

	variable delta, pos 
	delta = 2*MaxSigma/(NumPtsFinalPSWave-1) //	PeakShapeXaxis = -MaxSigma+delta*p	// final grid

	pos = floor((MaxSigma - 3)/delta)		// point number in x axis of PeakShapeCandidate  at -3 (similar for 3)
	PeakShapeCandidate[0,pos] = max(PeakShapeCandidate[p], PerfectGaussY[p])
	PeakShapeCandidate[NumPtsFinalPSWave-pos, NumPtsFinalPSWave-1] = max(PeakShapeCandidate[p],PerfectGaussY[p])
		
End		


Function PS_butt_declareFinalPeakShape(ctrlName) : ButtonControl
	String ctrlName

	wave PerfectGaussY = root:HR_PeakShape:PerfectGaussY
	wave PeakShapeCandidate = root:HR_peakShape:PeakShapeCandidate
	nvar HR_FinalPeakShapeScalar =  root:HR_PeakShape:HR_FinalPeakShapeScalar
	svar PS_nameSuffix = root:HR_PEakShape:PS_nameSuffix	
	nvar PS_viewVmode = root:HR_PEakShape:PS_viewVmode
	wave FinalPeakShape =  root:HR_peakShape:FinalPeakShape		// for display
	svar sq_toDoWvNm = root:panel:sq_toDoWvNm	
	wave todo = $"root:"+sq_toDoWvNm
	wave rn_series = root:index:rn_series
	wave/t HR_FinalPSNameWave = root:HR_PeakShape:HR_FinalPSNameWave	

 	HR_TodoWaveHasOnlyModeX(PS_viewVmode)
 	
	HR_PeakWidthSanity(0)

	string tempStr, finalNameStr
	// in case the user chose to edit the candidate wave and messed up.
	if (numpnts(PerfectGaussY) != numpnts(PeakShapeCandidate) )
		abort "Something is wrong with the number of points in the candidate wave - aborting from PS_declareFinalPeakShape"
	endif
		
	if(strlen(PS_nameSuffix)>0)
		finalNameStr = CleanupName(PS_nameSuffix, 0)
		finalNameStr =finalNameStr[0, 20]
	else
		finalNameStr =  sq_toDoWvNm[0, 20]
	endif
	
	finalNameStr = "FinalPS_"+ selectString(PS_viewVmode, "W", "V")+ "_"+finalNameStr
		
	// find peak shape scalar
	HR_FinalPeakShapeScalar = area (PeakShapeCandidate) / sqrt(pi)
		
	if (HR_FinalPeakShapeScalar > 2)		// 2 is arbitrary
		abort "There is something wrong with your peak shape - the peak shape scalar is too big - aborting from PS_declareFinalPeakShape"
	endif
		
	// copy values from candidate waves to final waves	
	duplicate/o PeakShapeCandidate root:HR_peakShape:FinalPeakShape
	duplicate/o PeakShapeCandidate $"root:HR_peakShape:"+ finalNameStr
	wave declaredPS = $"root:HR_peakShape:"+ finalNameStr
	Note/K declaredPS	// remove existing note
	sprintf tempStr, "%1.8f\r", HR_FinalPeakShapeScalar		// 8 digits after the decimal, too many than necessary
	Note declaredPS, "PS_scalar:"+tempStr

	make/o/n=(numpnts(rn_series)) runMask_temp				//1.57J

	sq_makeMaskFromTodofindRun(runMask_temp,todo) 
	
	HR_FinalPSNameWave = selectString(runMask_temp[p]==1, HR_FinalPSNameWave[p], finalNameStr)
	
	killwaves/z runMask_temp

	print "// The peak shape wave "+finalNameStr+" was declared at "+time() + " "+date()	

End


	// find residuals
Function PS_CalcMassResidual()
 	
	wave finalPeakShape = $"root:HR_PeakShape:finalPeakShape"
	wave PeakShapeXaxis = root:HR_PeakShape:PeakShapeXaxis

 	string massStr, tempList
 	variable idex, num
 	
 	svar PS_ResidList = $"root:HR_PeakShape:PS_ResidList"
 	PS_ResidList = ReplaceString(" ", PS_ResidList,"")
 	tempList = ReplaceString(",", PS_ResidList,";")

	num = ItemsInList(tempList)
	
	for (idex=0;idex<num;idex+=1)
	 	massStr = StringFromList(idex, tempList)
		wave PS_RawDataX = $"root:HR_peakShape:PS_RawDataX_"+massStr
		wave PS_RawDataY = $"root:HR_peakShape:PS_RawDataY_"+massStr	
	
		variable numRaw = numpnts(PS_RawDataX)
		make/o/n=(numRaw) $"root:HR_peakShape:PeakShapeResid_"+massStr
		wave PeakShapeResid = $"root:HR_peakShape:PeakShapeResid_"+massStr
			
		PeakShapeResid = PS_RawDataY -  finalPeakShape(PS_RawDataX[p])
	
	endfor
	
End


Function PS_plotAllPSplots()

	Display/W=(195,275,570,565)/HOST=PeakWidthShapePanel 
	RenameWindow #,PS_PeakShapeGraph
	PS_populate_PeakShapeGraph()
	SetActiveSubwindow ##

	Display/W=(595,205,915,530)/HOST=PeakWidthShapePanel  
	RenameWindow #,PS_FinalPSGraph
	PS_populate_FinalPSGraph()
	SetActiveSubwindow ##

End


// This function adds waves to graph; PS_showHidePeakShapes() hides the many that should not be displayed
Function PS_populate_PeakShapeGraph()

	// first append perfect gauss to panel
	string topWinName, myTraceList
	
	topWinName = WinName(0, 65)		// 65 means panels or graphs
	if (stringmatch("PeakWidthShapePanel", topWinName) )
		topWinName = "PeakWidthShapePanel#PS_PeakShapeGraph"
	else
		topWinName = "PS_PeakShapeGraph"
	endif
	
	myTraceList = TraceNameList(topWinName, ";", 1 )
	if (FindListItem("PerfectGaussY", myTraceList)<0)
		AppendToGraph/L/W=$topWinName root:HR_PeakShape:PerfectGaussY vs root:HR_PeakShape:PerfectGaussX
		ModifyGraph/W=$topWinName lsize(PerfectGaussY)=3
		ModifyGraph/W=$topWinName lstyle(PerfectGaussY)=1
		ModifyGraph/W=$topWinName rgb(PerfectGaussY)=(47872,47872,47872)
		ModifyGraph/W=$topWinName grid(left)=1,grid(bottom)=2,gridRGB(left)=(52224,52224,52224)
		ModifyGraph/W=$topWinName gridRGB(bottom)=(52224,52224,52224)
		Label/Z/W=$topWinName left "normalized peak shape" 
		Label/Z/W=$topWinName bottom "(t-t0)/peakwidth" 
	endif
	
	PS_updatePSGraph()

	// last append candidate values, whether or not they have decent values
	wave PeakShapeCandidate = root:HR_peakShape:PeakShapeCandidate
	if (FindListItem("PeakShapeCandidate", myTraceList)<0)
		AppendToGraph/L/W=$topWinName PeakShapeCandidate vs root:HR_PeakShape:PerfectGaussX
		ModifyGraph/W=$topWinName lsize(PeakShapeCandidate)=2
		ModifyGraph/W=$topWinName rgb(PeakShapeCandidate)=(1,1,1)
	endif
	
	PS_showHidePeakShapes()

	nvar PS_logLeftAxisCandidPS = root:HR_PeakShape:PS_logLeftAxisCandidPS
	PS_check_PSGraphLog("PS_PeakShapeGraph",PS_logLeftAxisCandidPS)

//	nvar PS_ShowFinal = root:HR_PeakShape:PS_ShowFinal  
//	ModifyGraph hideTrace($NameofWave(PeakShapeCandidate)) = !PS_ShowFinal

End

 
// simply shows and hides, does not append new waves
Function PS_showHidePeakShapes()
	
	variable idex, jdex, num
	string massStr, currentWin, myTracelist
	
	currentWin = WinName(0,65)		// graphs and windows
	if (strsearch(currentWin, "anel", 0)>=0)
		currentWin = "PeakWidthShapePanel#PS_PeakShapeGraph"
	endif

	myTracelist = TraceNameList(currentWin, ";", 1)

	svar ps_massList =root:HR_PeakShape:ps_massList
	num = itemsInList(ps_massList)
		
	nvar ShowPScandidate=root:HR_PeakShape:ShowPScandidate
	nvar ShowPSAvg=root:HR_PeakShape:ShowPSAvg
	nvar ShowPSData=root:HR_PeakShape:ShowPSData

	wave/z PeakShapeCandidate = root:HR_peakShape:PeakShapeCandidate

	for(idex=0;idex<num;idex+=1)
		massStr = stringFromList(idex, ps_massList)

		wave/z PS_RawDataY = $"root:HR_peakShape:PS_RawDataY_"+massStr		
		wave/z PS_AvgY = $"root:HR_peakShape:PS_AvgY_"+massStr		

		if (WaveExists(PS_RawDataY) && FindListItem(NameofWave(PS_AvgY), myTracelist)>=0 )
			ModifyGraph/W=$currentWin hideTrace($NameofWave(PS_AvgY)) = !ShowPSAvg	
			ModifyGraph/W=$currentWin hideTrace($NameofWave(PS_RawDataY)) = !ShowPSData
		endif
	endfor

	ModifyGraph/W=$currentWin hideTrace(PeakShapeCandidate) = !ShowPScandidate

End


Function PS_populate_FinalPSGraph()

	nvar PS_logLeftAxisFinalPS = root:HR_PeakShape:PS_logLeftAxisFinalPS
	string topWinName, myTraceList
	
	topWinName = WinName(0, 65)		// 65 means panels or graphs
	if (stringmatch("PeakWidthShapePanel", topWinName) )
		topWinName = "PeakWidthShapePanel#PS_FinalPSGraph"
	else
		topWinName = "PS_FinalPSGraph"
	endif
	
	myTraceList = TraceNameList(topWinName, ";", 1 )
	if (FindListItem("PerfectGaussY", myTraceList)<0)
		AppendToGraph/L/W=$topWinName root:HR_PeakShape:PerfectGaussY vs root:HR_PeakShape:PerfectGaussX
		ModifyGraph/W=$topWinName lsize(PerfectGaussY)=3
		ModifyGraph/W=$topWinName lstyle(PerfectGaussY)=1
		ModifyGraph/W=$topWinName rgb(PerfectGaussY)=(47872,47872,47872)
		ModifyGraph/W=$topWinName grid(left)=1,grid(bottom)=2,gridRGB(left)=(52224,52224,52224)
		ModifyGraph/W=$topWinName gridRGB(bottom)=(52224,52224,52224)
		Label/W=$topWinName left "Declared, normalized peak shape"
		Label/W=$topWinName bottom "(t - t0)/ peak width"
		Label/z/W=$topWinName right "Residual (normalized data - final peak shape)"
	endif
	if (FindListItem("FinalPeakShape", myTraceList)<0)
		AppendToGraph/L/W=$topWinName root:HR_PeakShape:FinalPeakShape
		ModifyGraph/W=$topWinName lsize(FinalPeakShape)=2
		ModifyGraph/W=$topWinName rgb(FinalPeakShape)=(0,0,0)
	endif

 	PS_ShowHideMassResidual()
 
	nvar PS_ShowFinal = root:HR_PeakShape:PS_ShowFinal
	PS_check_ShowHideFinalPS(topWinName, PS_ShowFinal)		//1.05A change, don't send it null

	nvar PS_logLeftAxisFinalPS = root:HR_PeakShape:PS_logLeftAxisFinalPS
	PS_check_PSGraphLog(topWinName, PS_logLeftAxisFinalPS)		//1.05A change, don't send it null
 
	
End


Function PS_check_ShowHideFinalPS(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	nvar PS_ShowFinal = root:HR_PeakShape:PS_ShowFinal
	nvar PS_ShowResid = root:HR_PeakShape:PS_ShowResid
	
	string currentWin, traceList, baseStr,Str
	variable idex, num
		
	currentWin = WinName(0,65)
	if (strsearch(currentWin, "anel", 0)>=0)	// panel or separate graph
		currentWin = "PeakWIdthShapePanel#PS_FinalPSGraph"
	endif	

	ModifyGraph/z/W=$currentWin hideTrace(FinalPeakShape)=!PS_ShowFinal
	
	traceList = TraceNameList(currentWin,";",1)
	num = itemsInList(traceList)
	for (idex = 0;idex<num;idex+=1)
		Str = stringfromlist(idex, traceList)
		if (strsearch(str, "PeakShapeResid", 0)>=0)
			ModifyGraph/z/W=$currentWin hideTrace($Str)=!PS_ShowResid
		endif
	endfor

End
	

Function PS_ck_ShowWaves(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	PS_showHidePeakShapes()

End


Function PS_check_PSGraphLog(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	string tempStr, currentWin
	
	tempStr = WinName(0,65)		//  current graphs and windows
	currentWin=""
	if (strsearch(tempStr, "anel", 0)>=0)	// panel or separate graph
		currentWin = "PeakWidthShapePanel#"
	endif
	if (strsearch(ctrlName, "inal", 0)>=0)	// candidate or final graph
		currentWin += "PS_FinalPSGraph"
	else
		currentWin += "PS_PeakShapeGraph"
	endif

	ModifyGraph/W=$currentWin log(left)=checked
	GetAxis/W=$currentWin /Q left
	SetAxis/W=$currentWin left max(1e-5, V_min), 1.1

End




Function PS_SetVar_RunModeStr(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	wave run_series = root:index:rn_series
	wave ToFTypeWKey = root:diagnostics:ToFTypeWKey
	wave ToFTypeVKey = root:diagnostics:ToFTypeVKey
	nvar PW_CurrentRunNum =  root:HR_PeakShape:PW_CurrentRunNum
	svar PW_runModeStr = root:HR_PeakShape:PW_runModeStr
	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todoWave = root:$sq_toDoWvNm
	 
	variable currentRunIndex, inToDo
	
	inToDo = sq_FindRun(todoWave, PW_CurrentRunNum)
	currentRunIndex =sq_FindRun(run_series, PW_CurrentRunNum)
	
	if(currentRunIndex<0)
		PW_runModeStr = "doesn't exist"
	else
		if(toftypeVKey[currentRunIndex]==1)
			PW_runModeStr = "v"
		elseif (toftypeWKey[currentRunIndex]==1)
			PW_runModeStr = "w"
		else
			PW_runModeStr = "c"
		endif
		if ( inToDo>=0)
			PW_runModeStr+= " mode IN "+sq_toDoWvNm
		else
			PW_runModeStr+= " mode NOT IN "+sq_toDoWvNm
		endif
	endif

End

// generates a new window with the pw mass color legend
Function PS_butt_PWColorLegend(ctrlName) : ButtonControl
	String ctrlName

	nvar PS_viewVmode = root:HR_peakShape:PS_viewVmode
	string TofTypeStr = selectString(PS_viewVmode, "W", "V")
	DoWindow/F $"PWMassLegend"+tofTypeStr
	if (v_flag ==0)
		 PW_displayMassLegend(tofTypeStr)
	 	return 1
	endif

End


Function PS_updatePSGraph()

	wave FilledMarkerWave = root:HR_PeakShape:FilledMarkerWave
	wave ColorMx = root:HR_PeakShape:ColorMx
	wave/t GFName = root:GaussFits:GFName
	svar ps_massList =root:HR_PeakShape:ps_massList
	
	variable idex, jdex, red, green, blue, pos, num, numMarker
	string massStr, currentWin, myTracelist , tempStr
			
	numMarker= numpnts(FilledMarkerWave)

	currentWin = WinName(0,65)		// graphs and windows
	if (strsearch(currentWin, "anel", 0)>=0)
		currentWin = "PeakWidthShapePanel#PS_PeakShapeGraph"
	endif
	
	myTracelist = TraceNameList(currentWin, ";", 1)
	num = itemsInList(myTracelist)
	
//  first kill any RawData or AvgY traces on graph
	for(idex=0;idex<num;idex+=1)
		tempStr = stringFromList(idex, myTracelist)
		if(strsearch(tempStr, "PS_RawData",0)>=0 || strsearch(tempStr, "PS_Avg",0)>=0 )
			RemoveFromGraph/W=$currentWin $tempStr
		endif
	endfor

	myTracelist = TraceNameList(currentWin, ";", 1)
	num = itemsInList(ps_massList)
	for(idex=0;idex<num;idex+=1)
		massStr = stringFromList(idex, ps_massList)
		pos = sq_findStr(GFName,massStr,1)
		red = ColorMx[pos][0] 
		green = ColorMx[pos][1] 
		blue = ColorMx[pos][2] 
		wave/z PS_RawDataX = $"root:HR_peakShape:PS_RawDataX_"+massStr
		wave/z PS_RawDataY = $"root:HR_peakShape:PS_RawDataY_"+massStr
		wave/z PS_AvgX = $"root:HR_peakShape:PS_AvgX_"+massStr
		wave/z PS_AvgY = $"root:HR_peakShape:PS_AvgY_"+massStr		
		wave/z PS_StdY = $"root:HR_peakShape:PS_StdY_"+massStr		
		wave/z PeakShapeCandidate = root:HR_peakShape:PeakShapeCandidate	
		
		if (!WaveExists(PS_RawDataX) || !WaveExists(PS_RawDataY)  )
			continue
		endif		
		if (FindListItem(nameOfWave(PS_AvgY), myTracelist)<0)
			AppendToGraph/L/W=$currentWin PS_AvgY vs PS_AvgX
			ModifyGraph/W=$currentWin mode($NameofWave(PS_AvgY))=4
			ModifyGraph/W=$currentWin marker($NameofWave(PS_AvgY))=FilledMarkerWave[mod(pos, numMarker)]
			ModifyGraph/W=$currentWin rgb($NameofWave(PS_AvgY))=(red, green, blue)
			if (FindListItem(nameOfWave(PeakShapeCandidate), myTracelist)>=0)
				ReorderTraces/W=$currentWin $NameofWave(PeakShapeCandidate), {$NameofWave(PS_AvgY)}
			endif
		endif

		if (FindListItem(nameOfWave(PS_RawDataY), myTracelist)<0)
			AppendToGraph/L/W=$currentWin PS_RawDataY vs PS_RawDataX
			ModifyGraph/W=$currentWin rgb($NameofWave(PS_RawDataY))=(red, green, blue)
			ModifyGraph/W=$currentWin mode($NameofWave(PS_RawDataY))=2,lsize($NameofWave(PS_RawDataY))=2
			if (FindListItem(nameOfWave(PeakShapeCandidate), myTracelist)>=0)
				ReorderTraces/W=$currentWin $NameofWave(PeakShapeCandidate), {$NameofWave(PeakShapeCandidate)}
			endif
		endif

	endfor

End


Function PS_butt_recalcPSLeftRight(ctrlName) : ButtonControl
	String ctrlName

	svar pw_massList = root:HR_PeakShape:pw_massList
	svar PS_massList_left = root:HR_PeakShape:PS_massList_left
	svar PS_massList_right = root:HR_PeakShape:PS_massList_right
	wave PeakShapeCandidate = root:HR_peakShape:PeakShapeCandidate

	nvar NumPtsFinalPSWave = root:HR_PeakShape:NumPtsFinalPSWave	

	PS_massList_left = ReplaceString(" ", ReplaceString(",", PS_massList_left,";"), "")
	PS_massList_right = ReplaceString(" ", ReplaceString(",", PS_massList_right,";"), "")

	//check to see if the masses in the list are allowed.
	
	if (!gen_isListAsubsetListB(PS_massList_left,pw_massList) || !gen_isListAsubsetListB(PS_massList_right,pw_massList) )
		abort "There is a mass in the left or right list that is not in the mode list - aborting from PS_butt_recalcPSLeftRight"
	endif
		
	if (ItemsinList(PS_massList_left)>0 && ItemsinList(PS_massList_right)>0)
		PS_CalcPSCandidateFromList(PS_massList_left)
	
		duplicate/o PeakShapeCandidate temp
	
		PS_CalcPSCandidateFromList(PS_massList_right)
	
		PeakShapeCandidate[0, NumPtsFinalPSWave/2]=temp[p]		// glue on the left side
		
	endif
	
	killwaves/z temp
	
End


Function PS_butt_CalcAppendResid(ctrlName) : ButtonControl
	String ctrlName

	
	svar ps_massList = root:HR_peakShape:ps_massList
	svar PS_ResidList = root:HR_peakShape:PS_ResidList
	string tempStr
	tempStr = ReplaceString(" ", ReplaceString(",", PS_ResidList,";"), "")

	//check to see if the masses in the list are allowed.
	
	if (!gen_isListAsubsetListB(tempStr,ps_massList) )
		abort "There is a mass in the residual list that is not in the peak width list - aborting from PS_butt_CalcAppendResid"
	endif

	PS_CalcMassResidual()
	PS_ShowHideMassResidual()

End
 

Function PS_ShowHideMassResidual()

	wave FilledMarkerWave = root:HR_PeakShape:FilledMarkerWave
	wave ColorMx = root:HR_PeakShape:ColorMx
	wave/t GFName = root:GaussFits:GFName
	
	variable idex, jdex, red, green, blue, pos, num, numMark
	string massStr, currentWin, myTracelist, tempList 
			
	numMark= numpnts(FilledMarkerWave)

	currentWin = WinName(0,65)		// graphs and windows
	if (strsearch(currentWin, "anel", 0)>=0)
		currentWin = "PeakWidthShapePanel#PS_FinalPSGraph"
	endif
	
	myTracelist = TraceNameList(currentWin, ";", 1)

 	svar PS_ResidList = root:HR_PeakShape:PS_ResidList
 	PS_ResidList = ReplaceString(" ", PS_ResidList,"")
 	tempList = ReplaceString(",", PS_ResidList,";")
	num = itemsInList(tempList)
	
	for(idex=0;idex<num;idex+=1)
		massStr = stringFromList(idex, tempList)
		pos = sq_findStr(GFName,massStr,1)
		red = ColorMx[pos][0] 
		green = ColorMx[pos][1] 
		blue = ColorMx[pos][2] 

		wave/z PS_RawDataX = $"root:HR_peakShape:PS_RawDataX_"+massStr
		wave/z PeakShapeResid = $"root:HR_peakShape:PeakShapeResid_"+massStr		
		
		if (!WaveExists(PS_RawDataX) || !WaveExists(PeakShapeResid)  )
			print "// A wave name root:HR_peakShape:PeakShapeResid_"+massStr +" was missing.  Continuing..."
			continue
		endif		
		if (FindListItem(nameOfWave(PeakShapeResid), myTracelist)<0)
			appendtoGraph/r/W=$currentWin PeakShapeResid vs PS_RawDataX
			Label/W=$currentWin right "Residual (data - peak shape)"
			ModifyGraph/W=$currentWin zero(right)=1
			ModifyGraph/W=$currentWin rgb($NameOfWave(PeakShapeResid))=(ColorMx[pos][0], ColorMx[pos][1], ColorMx[pos][2])
			ModifyGraph/W=$currentWin mode($NameOfWave(PeakShapeResid))=2,lsize($NameOfWave(PeakShapeResid))=2	
			SetAxis/W=$currentWin right -0.1,0.1
		endif
		
	endfor

End


// new in 1.04L, only 1 dimensional
Function HR_initializeGlobalTextWave(WavName, WaveLenVal, WaveStr)
string WavName
variable WaveLenVal
string WaveStr

	wave/z myWave = $WavName
	if (!WaveExists(myWave))
		make/t/n=(WaveLenVal) $WavName = WaveStr
	endif

End


// new in 1.04L
Function HR_initializeGlobalWave(WavName, WaveLenList, WaveVal)
string WavName, WaveLenList
variable WaveVal

	variable num
	wave/z myWave = $WavName
	
	num = itemsinlist(WaveLenList)		// the number of dimensions
	make/o/n=(num) tempWave
	tempWave = str2num(StringFromList(p, WaveLenList) )
	tempWave = numtype(tempwave[p])==0 ? tempWave[p] : 0
	
	if (!WaveExists(myWave))
		switch (num)
		case 1:
			make/o/n=(tempWave[0]) $WavName = WaveVal
			break
		case 2:
			make/o/n=(tempWave[0], tempWave[1]) $WavName = WaveVal
			break
		case 3:
			make/o/n=(tempWave[0], tempWave[1], tempWave[2]) $WavName = WaveVal
			break
		case 4:
			make/o/n=(tempWave[0], tempWave[1], tempWave[2], tempWave[3]) $WavName = WaveVal
			break
		endswitch
		
	else		// wave exists, now make it the correct size
		switch (num)
		case 1:
			redimension/n=(tempWave[0]) $WavName
			break
		case 2:
			redimension/n=(tempWave[0], tempWave[1]) $WavName
			break
		case 3:
			redimension/n=(tempWave[0], tempWave[1], tempWave[2]) $WavName
			break
		case 4:
			redimension/n=(tempWave[0], tempWave[1], tempWave[2], tempWave[3]) $WavName
		endswitch	
	endif

	killwaves/z tempWave			

End


Function PS_butt_editPS(ctrlName) : ButtonControl
	String ctrlName
	
	doWindow/F PeakShapeCandidateTable
	if (V_flag==0)
		Edit/N=PeakShapeCandidateTable As "PeakShapeCandidateTable"  
		appendtotable root:HR_PeakShape:PeakShapeCandidate.id
	endif

End


//  Returns a 2 char or more length string of the number x
Function/s HR_gen_Num2Str2Char(int)
	Variable int

	string returnStr = num2str(int)
	
	if ( (int>=0) && (int<=9) )
		returnStr = "0"+returnStr
	endif
	
	return returnStr

End


Function HR_PeakShapePanelWIndow()	// this calls the Peak Shape panel

	DoWindow/F PeakWidthShapePanel
	if (V_flag==0)
		GF_createGlobals()
		PW_NewGlobals()
		PS_createGlobals()
		PW_PeakWidthShapePanel()
	endif

End


// added 1.04P
Function PW_butt_PWParamGraph(ctrlName) : ButtonControl
	String ctrlName

	DoWindow/F PW_AllParamGraph
	if (V_flag!=0)
		return 0
	endif

	Display/N=PW_AllParamGraph as "PW_AllParamGraph"
	AppendToGraph/L=intercept root:HR_PeakShape:HR_FinalPWInterceptWave vs root:index:rn_series
	AppendToGraph/L=slope root:HR_PeakShape:HR_FinalPWSlopeWave vs root:index:rn_series
	AppendToGraph/L=power root:HR_PeakShape:HR_FinalPWPowerWave vs root:index:rn_series

	ModifyGraph mode=4
	ModifyGraph grid=2
	ModifyGraph gridRGB=(39168,39168,39168)
	ModifyGraph lblPos(intercept)=52,lblPos(slope)=52,lblPos(power)=52
	ModifyGraph lblLatPos(intercept)=2,lblLatPos(slope)=2
	ModifyGraph freePos(intercept)={0,bottom}
	ModifyGraph freePos(slope)={0,bottom}
	ModifyGraph freePos(power)={0,bottom}
	ModifyGraph axisEnab(intercept)={0.7,1}
	ModifyGraph axisEnab(bottom)={0.1,1}
	ModifyGraph axisEnab(slope)={0.35,0.65}
	ModifyGraph axisEnab(power)={0,0.3}
	Label intercept "intercept"
	Label bottom "run number"
	Label slope "slope"
	Label power "power"

	// added in 1.10A
	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	ControlBar 30
	PopupMenu todoPop_DF_HR_PeakShape,pos={8,4},size={250,21},bodyWidth=69,proc=sq_ShowOnlyForThisTodoPopMenu,title="Only show values from this todo wave"
	PopupMenu todoPop_DF_HR_PeakShape,fSize=10
	PopupMenu todoPop_DF_HR_PeakShape,mode=13,popvalue=sq_toDoWvNm,value= #"root:panel:sq_toDoList+\"Get List;\""
  	sq_ShowOnlyForThisTodoPopMenu("todoPop_DF_HR_PeakShape",nan,sq_toDoWvNm) 

End


// added 1.04P
Function PW_butt_PWParamTable(ctrlName) : ButtonControl
	String ctrlName

	DoWindow/F PW_AllParamTable
	if (V_flag==0)
		Edit/N=PW_AllParamTable as "PW_AllParamTable"
		AppendToTable root:index:rn_series
		AppendToTable root:diagnostics:TofTypeVKey
		AppendToTable root:diagnostics:TofTypeWKey
		AppendToTable root:HR_PeakShape:HR_FinalPWInterceptWave
		AppendToTable root:HR_PeakShape:HR_FinalPWSlopeWave
		AppendToTable root:HR_PeakShape:HR_FinalPWPowerWave
		AppendToTable root:HR_PeakShape:HR_FinalPSNameWave
	endif
	
End


Function PW_butt_DeclarePWParams(ctrlName) : ButtonControl
	String ctrlName
	
	svar sq_toDoWvNm=root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	nvar PW_viewVmode = root:HR_peakShape:PW_viewVmode		// 1 = V mode, 0 = W

	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave

	nvar HR_displayFinalPowA = root:HR_PeakShape:HR_displayFinalPowA	// 
	nvar HR_displayFinalPowB = root:HR_PeakShape:HR_displayFinalPowB
	nvar HR_displayFinalPowC = root:HR_PeakShape:HR_displayFinalPowC

	nvar HR_displayFinalPWSlope = root:HR_PeakShape:HR_displayFinalPWSlope
	nvar HR_displayFinalPWIntercept = root:HR_PeakShape:HR_displayFinalPWIntercept

	nvar PWUseLineOrPow = root:HR_PeakShape:PWUseLineOrPow
	svar pw_massList = root:HR_PeakShape:pw_massList

	variable myIntercept, mySlope, myPower
	
	HR_TodoWaveHasOnlyModeX(PW_viewVmode)

	if (PWUseLineOrPow)		// use line
		myIntercept = HR_displayFinalPWIntercept
		mySlope = HR_displayFinalPWSlope
		myPower = 1
	else
		myIntercept = HR_displayFinalPowA
		mySlope = HR_displayFinalPowB
		mypower = HR_displayFinalPowC
	endif
	
	if (numtype(myintercept)!=0 || numtype(myintercept)!=0 || numtype(myintercept)!=0)		// sanity check 1.10K
		abort "The chosen peak width values include a nan - aborting from PW_butt_DeclarePWParams"
	endif
	
	make/o/n=(numpnts(HR_FinalPWInterceptWave)) tempMask
	sq_makeMaskFromTodofindRun(tempMask,todo)

	HR_FinalPWInterceptWave = tempMask[p] == 1 ? myIntercept : HR_FinalPWInterceptWave[p]
	HR_FinalPWSlopeWave = tempMask[p] == 1 ? mySlope : HR_FinalPWSlopeWave[p] 
	HR_FinalPWPowerWave = tempMask[p] == 1 ? myPower : HR_FinalPWPowerWave[p]
	
	killwaves/z tempMask
	print "// You have declared values of intercept:"+num2str(myIntercept)+" slope:"+num2str(mySlope)+" power:"+num2str(mypower)+" for the todo wave "+sq_toDoWvNm +" using masses "+pw_massList

End


Function PW_ck_LineOrPow(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	nvar PWUseLineOrPow = root:HR_PeakShape:PWUseLineOrPow
	if (strlen(ctrlName)!=0)
		PWUseLineOrPow = strsearch(ctrlName, "line",0)>0
	endif
	
	Checkbox PW_line Win=PeakWidthShapePanel, value = PWUseLineOrPow
	Checkbox PW_Pow Win=PeakWidthShapePanel, value = !PWUseLineOrPow

End

Function PS_updateMassList()

 	svar PS_MassList = root:HR_PeakShape:pS_MassList
	wave/t GFname = root:GaussFits:GFname

	nvar PS_viewVmode = root:HR_peakShape:PS_viewVmode		// 1 = V mode, 0 = W
	wave PW_modeMask = $"root:HR_PeakShape:PW_"+selectString(PS_viewVmode, "W", "V")+"modeMask"

	variable idex, num
	num = numpnts(GFname)
	PS_MassList=""
	for(idex=0;idex<num;idex+=1)
		if(PW_modeMask[idex])
			PS_MassList += GFname[idex]+";"
		endif
	endfor
	
End


Function PS_butt_confirmTodo(ctrlName) : ButtonControl
	String ctrlName

	wave rn_series  = root:index:rn_series
	wave TofTypeVKey  = root:diagnostics:TofTypeVKey
	svar sq_toDoWvNm  = root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	
	svar PSModeStr  = root:HR_PeakShape:PSModeStr
	svar PS_PWParams  = root:HR_PeakShape:PS_PWParams
	svar PS_massList   = root:HR_PeakShape:PS_massList
	
	nvar PS_viewVmode = root:HR_PeakShape:PS_viewVmode
	svar Step4MassList = root:HR_PeakShape:Step4MassList
	
	string tempStr
	
	PW_SanityCheckTodoWave()
	
	PS_viewVmode= TofTypeVKey[sq_findRun(rn_series, todo[0])]		//1 = V mode, 0 = W

	HR_TodoWaveHasOnlyModeX(PS_viewVmode)		// 
	
	tempStr = HR_PeakWidthSanity(0)
	
	PSModeStr = "Mode: "+selectString(PS_viewVmode, "W", "V")
			
	PS_updateMassList()
	
	PS_PWParams = "Average peak width parameters: "+ tempStr	//1.10C
 
	Step4MassList = "Current mode mass list:"+PS_massList

End


Function PS_butt_FinalPSTable(ctrlName) : ButtonControl
	String ctrlName

	string myPSList, myPSStr
	variable idex, num
	
	DoWindow/k FinalPSTable 	
	
	SetDataFolder root:HR_PeakShape:
	
	myPSList =WaveList("FinalPS_*", ";", "") 

	SetDataFolder root:

	num = ItemsInList(myPSList)
	
	Edit/N=FinalPSTable as "FinalPSTable"
	for(idex=0;idex<num;idex+=1)
		wave ps= $"root:HR_PeakShape:"+stringFromList(idex, myPSList)
		AppendtoTable/W=FinalPSTable ps.id
	endfor
	
End




// a function for upgrading your 1.04 pika experiments, where there is no time-dependency for the pw and ps formulation
// to verison 1.05, where there is a set of pw parameters and the name of a ps wave, for every run.
Function PK_upgradePWPSfrom104(todo, PWIntercept, PWSlope, PSwave)
wave todo,  PSwave
variable PWIntercept, PWSlope

	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave
	wave/t HR_FinalPSNameWave = root:HR_PeakShape:HR_FinalPSNameWave
	wave DisplayPerfectGaussY= root:HR_PeakShape:DisplayPerfectGaussY
	wave DisplayPerfectGaussX= root:HR_PeakShape:DisplayPerfectGaussX

	if (numpnts(PSwave)!= numpnts(DisplayPerfectGaussY))
		abort "There appears to be a not-expected number of points in the peak shape wave "+NameofWave(PSwave)+"- aborting from Pk_upgradePWPSfrom104"
	endif
	
	string newPeakShapeName, tempStr
	variable newPSscalar
		
	make/o/n=(numpnts(HR_FinalPWInterceptWave)) tempMask
	sq_makeMaskFromTodofindRun(tempMask,todo)

	HR_FinalPWInterceptWave = tempMask[p] == 1 ? PWIntercept : HR_FinalPWInterceptWave[p]
	HR_FinalPWSlopeWave = tempMask[p] == 1 ? PWSlope : HR_FinalPWSlopeWave[p] 
	HR_FinalPWPowerWave = tempMask[p] == 1 ? 1 : HR_FinalPWPowerWave[p]
	
	newPeakShapeName = "FinalPS_"+NameofWave(todo)
	duplicate/o PSwave $"root:HR_PeakShape:"+newPeakShapeName		// duplicating means duplicating the important x scaling info of the wave
	wave PSwaveNormalized = $"root:HR_PeakShape:"+newPeakShapeName
	PSwaveNormalized*=DisplayPerfectGaussY

	newPSscalar = area(PSwaveNormalized)
	newPSscalar /= areaXY(DisplayPerfectGaussX, DisplayPerfectGaussY)
	
	Note/K PSwaveNormalized	// remove existing note
	sprintf tempStr, "%1.8f\r", newPSscalar		// 8 digits after the decimal, too many than necessary
	Note PSwaveNormalized, "PS_scalar:"+tempStr
	
	HR_FinalPSNameWave = SelectString(tempMask[p] == 1,  HR_FinalPSNameWave[p], newPeakShapeName) 
	
	killwaves/z tempMask  //, PSwaveNormalized
	print "// You have declared values of intercept:"+num2str(PWIntercept)+" slope:"+num2str(PWSlope)+" power:"+num2str(1)+" for the todo wave "+NameofWave(todo)

End


// added in 1.06D
Function GF_butt_setAllMaskTo0(ctrlName) : ButtonControl
	String ctrlName

	wave GFMask = root:GaussFits:GFMask
	ControlInfo/W=PeakWidthShapePanel GF_popup_setToVal
	if (V_value==1)
		abort "Please select a value from the menu - aborting from GF_butt_setAllMaskTo0"
	endif
	GFMask=V_Value-2 
	
End


// added in 1.06D
Function GF_butt_addMass(ctrlName) : ButtonControl
	String ctrlName

	svar GF_addMassList = root:HR_PeakShape:GF_addMassList

	wave GFMass=root:GaussFits:GFMass
	wave/t GFName=root:GaussFits:GFName
	wave/i GFDeltaNum=root:GaussFits:GFDeltaNum
	wave/t GFComments=root:GaussFits:GFComments
	wave/i GFMask=root:GaussFits:GFMask
	wave PW_VmodeMask = root:HR_PeakShape:PW_VmodeMask
	wave PW_WmodeMask = root:HR_PeakShape:PW_WmodeMask
	wave PW_Graph = root:HR_PeakShape:PW_Graph
	
	wave/d ExactMassWave_all = root:HR:ExactMassWave_all
	wave/t ExactMassText_all = root:HR:ExactMassText_all
	wave/t ExactMassComments = root:HR:ExactMassComments
	
	variable posAll, posGF, num
	
	num = numpnts(GFMass)
	posAll = sq_findStr(ExactMassText_all,GF_addMassList,1)
	if (posAll<0)
		Abort "Could not find mass '"+GF_addMassList+"' in the list of all masses.  Aborting from GF_butt_addMass"
	else
		posGF = sq_findStr(GFName,GF_addMassList,1)
		if (posGF>=0)
			Abort "The mass "+GF_addMassList+" already appears in the list of masses.  Aborting from GF_butt_addMass"
		else		// for now we will add them at the end, instead of inserting them in increasing mass order
			Insertpoints num, 1, GFMass, GFName, GFDeltaNum,GFComments,GFMask,PW_VmodeMask,PW_WmodeMask,PW_Graph
			GFMass[num] = ExactMassWave_all[posAll]
			GFName[num] = ExactMassText_all[posAll]
			GFDeltaNum[num] = 3
			GFComments[num] = ExactMassComments[posAll]
			GFMask[num] = 2
			PW_VmodeMask[num] = 0
			PW_WmodeMask[num] = 0
			PW_Graph[num] = 1
		endif
	endif
	
End




// sample lines of code for folks upgrading from 1.04 Pika to version 1.05

// for W mode data:
PK_upgradePWPSfrom104(allW, root:HR_PeakShape:HR_WModeFinalIntercept, root:HR_PeakShape:HR_WModeFinalSlope, root:HR_PeakShape:PeakShapeW)

// for V mode data:
PK_upgradePWPSfrom104(allV, root:HR_PeakShape:HR_VModeFinalIntercept, root:HR_PeakShape:HR_VModeFinalSlope, root:HR_PeakShape:PeakShapeV)






Function GF_ck_showGFdetails(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	nvar GF_showGFDetails = root:HR_peakShape:GF_showGFDetails

	variable idex, num, cont_disp
	string cont_list, cont_item
	
	cont_list=controlnamelist("PeakWidthShapePanel", ";", "GF_*")
	num=itemsinlist(cont_list)
	
	for (idex=0;idex<num;idex+=1)
		cont_item=stringfromlist(idex,cont_list)
		controlinfo /w=PeakWidthShapePanel $cont_item
		//  disable = 1 means hide, disable = 0 means show
		switch (abs(v_flag))
			case 1:
				button $cont_item disable=GF_showGFDetails
				break
			case 2:
				if (stringmatch(cont_item, "GF_showdetails"))
					checkbox $cont_item disable=0
				else
					checkbox $cont_item disable=GF_showGFDetails
				endif
				break
			case 3:
				popupmenu $cont_item disable=GF_showGFDetails
				break
			case 4:
				valdisplay $cont_item disable=GF_showGFDetails
				break
			case 5:
				setvariable $cont_item disable=GF_showGFDetails
				break
			case 6:
				chart $cont_item disable=GF_showGFDetails
				break
			case 7:
				slider $cont_item disable=GF_showGFDetails
				break
			case 8:
				tabcontrol $cont_item disable=GF_showGFDetails
				break
			case 9:
				groupbox $cont_item disable=GF_showGFDetails
				break
			case 10:
				if (stringmatch(cont_item, "GF_titleMain"))
					titlebox $cont_item disable=0
				else
					titlebox $cont_item disable=GF_showGFDetails
				endif
				break
			case 11:
				listbox $cont_item disable=GF_showGFDetails
				break
		endswitch
	endfor

	if (!GF_showGFDetails)
		if (strlen(ChildWindowList("PeakWidthShapePanel"))==0)
			Edit/W=(10,160,415,575)/HOST=# 
			RenameWindow #,GF_MassTable
			GF_populate_MassTable()	
			SetActiveSubwindow ##
		endif
	else
		if (strlen(ChildWindowList("PeakWidthShapePanel"))>0)
			KillWindow $"PeakWidthShapePanel#GF_MassTable"
		endif
	endif

End


Function PS_butt_updatePSGraph(ctrlName) : ButtonControl
	String ctrlName

// 1.08D  This was broken, and now I can't remember why we would need it.
	KillWindow PeakWidthShapePanel#PS_PeakShapeGraph 
	Display/W=(195,275,570,565)/HOST=PeakWidthShapePanel  
	RenameWindow #,PS_PeakShapeGraph
	PS_populate_PeakShapeGraph()
	PS_updatePSGraph()
	PS_showHidePeakShapes()

End


// Given a todo wave, and the selection of HR ions within the Peak Width table (1 = use, 0 = don't use)
// this function will fit the peak widths one run at a time and put the results in the waves
// root:HR_PeakShape:HR_FinalPWInterceptWave
// root:HR_PeakShape:HR_FinalPWSlopeWave
// root:HR_PeakShape:HR_FinalPWPowerWave
// Together with the PK_Backbone_1_10A ipf users can do peak fitting on todo waves that 
//  all don't have the same peak width parameters, effectively 'declaring' separate peak width parameters for each run in the todo.
//
// Users can see the result of this function by pressing the PW Param Table button in the peakwidthshape panel
// and see what the values are before and after.
Function PW_fitPWToSmoothlyVaryingPW(todo,Vflag)
wave todo
variable Vflag

	wave GFMass=root:GaussFits:GFMass
	wave/t GFName=root:GaussFits:GFName
	wave PWCoeffs = root:HR_PeakShape:PWCoeffs
	wave/t PWConstraintsLine = root:HR_PeakShape:PWConstraintsLine
	wave OneRunPeakWidths = root:HR_PeakShape:OneRunPeakWidths	
	
	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave
		
	nvar PWUseLineOrPow = root:HR_PeakShape:PWUseLineOrPow

	variable idex, numRuns, jdex, numGFions, row
	string goodNameStr
	numGFions=numpnts(GFMass)
	numRuns = numpnts(todo)
	
	if (sq_onlyOneModeInTodo(todo)!=1)
		abort "There are V and W mode runs in this todo. aborting from pw_fitPWToSmoothlyVaryingPW"
	endif

	wave PW_modeMask = $"root:HR_peakShape:PW_"+selectstring(Vflag, "W", "V")+"modeMask"

	for(idex=0;idex<numRuns;idex+=1)
		row=sq_findRun(root:index:rn_series,todo[idex])
		OneRunPeakWidths=nan
		for(jdex=0;jdex<numGFions;jdex+=1)	// for each GFion
			if (PW_modeMask[jdex])		// we use this ion
				goodNameStr = CleanUPName(GFName[jdex],0)		//1.19B
				wave GFSmoothed_X = $"root:GaussFits:SmoothedPW_"+goodNameStr
				OneRunPeakWidths[jdex]=GFSmoothed_X[row] //[3]
			endif
		endfor
		PW_findFitsOneRun(GFMass, OneRunPeakWidths, PW_modeMask, PWCoeffs, PWConstraintsLine, PWUseLineOrPow)
		HR_FinalPWInterceptWave[row]=PWCoeffs[0]
		HR_FinalPWSlopeWave[row] = PWCoeffs[1]
	    if (!PWUseLineOrPow) //user selected power fit		// 1.11O
	          HR_FinalPWPowerWave[row] = PWCoeffs[2]
	    else
	          HR_FinalPWPowerWave[row] = 1
	    endif	
	endfor
			
	sq_InterpolateWaveSameMode(HR_FinalPWPowerWave, HR_FinalPWPowerWave, 0)  //1.16J
	sq_InterpolateWaveSameMode(HR_FinalPWInterceptWave, HR_FinalPWInterceptWave, 0)
	sq_InterpolateWaveSameMode(HR_FinalPWSlopeWave, HR_FinalPWSlopeWave, 0)

End

//1.11
Function PW_smoothSelectedPWs(todo, Vflag, smoothingFactor)
wave todo
variable Vflag, smoothingFactor

	wave GFMass=root:GaussFits:GFMass
	wave/t GFName=root:GaussFits:GFName
	wave PW_modeMask = $"root:HR_peakShape:PW_"+selectstring(Vflag, "W", "V")+"modeMask"

	string str, mytraceList, HRion,goodNameStr
	variable jdex, numGFions
	
	numGFions=numpnts(GFMass)
	for(jdex=0;jdex<numGFions;jdex+=1)	// for each GFion
		if (PW_modeMask[jdex])		// we use this ion
			goodNameStr = CleanUPName(GFName[jdex],0)		//1.19B
			pw_SmoothOnePW(todo, goodNameStr, smoothingFactor)
		endif
	endfor

End


// 1.11
// Does the work of pw_smoothSelectedPWs
Function PW_smoothOnePW(todo, HRIon, smoothingFactor)
wave todo
string HRion
variable smoothingFactor

	make/o/n=0 tempMask
	wave GF_X = $"root:GaussFits:GaussCoeff_Closed_"+HRion		// for now assume user kept the default, using closed
	 wave PW_RunMask = root:HR_PeakShape:PW_RunMask
	 make/o/n=(numpnts(root:index:t_series)) $"root:GaussFits:SmoothedPW_"+HRion/Wave=Smoothed_A  
	sq_makeMaskFromTodofindRun(tempMask,todo)
	
	smoothed_A = GF_X[p][3]*tempMask*PW_RunMask
	sq_interpolateWave(Smoothed_A, Smoothed_A, 0)
	smooth smoothingFactor, Smoothed_A
	
	Killwaves/z tempMask

End


// 1.11
Function PW_displaySmoothSelectedPWs(todo, Vflag)
wave todo
variable Vflag
	
	wave rn_series=root:index:rn_series
	wave GFMass=root:GaussFits:GFMass
	wave/t GFName=root:GaussFits:GFName
	wave PW_modeMask = $"root:HR_peakShape:PW_"+selectstring(Vflag, "W", "V")+"modeMask"

	string str, mytraceList, HRion,goodNameStr
	variable jdex, numGFions
	
	numGFions=numpnts(GFMass)
	for(jdex=0;jdex<numGFions;jdex+=1)	// for each GFion
		if (PW_modeMask[jdex])		// we use this ion
			goodNameStr = CleanUPName(GFName[jdex],0)		//1.19B
//			HRion=GFName[jdex]
			wave Smoothed_A  = $"root:GaussFits:SmoothedPW_"+goodNameStr 
			Appendtograph/L=PW Smoothed_A vs rn_series
			ModifyGraph rgb($NameOfWave(Smoothed_A))=(0,0,0)
		endif
	endfor

End


// button on Peak Width tab of PeakWidthShapePanel
Function PW_TimeVayingPW_butt(ctrlName) : ButtonControl
	String ctrlName

 	DoWindow/F PW_PeakWidthGraph
 	if (V_flag && strsearch(TraceNameList("", ";", 1), "smooth", 0)<0)		//window exists but is old one, doesn't have peak smoothing controls
 		DoWindow/K PW_PeakWidthGraph
	endif
	display/N=PW_PeakWidthGraph as "PW_PeakWidthGraph"
	pw_populate_PeakWidthGraph()
 	pw_AddControlsToPWGraph()
 	pw_SmoothPWButtonProc("")	 
	
	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	variable Vflag
 	wave toftypeVKey = root:diagnostics:toftypeVKey
	Vflag = toftypeVKey[sq_FindRun(root:index:rn_series, todo[0])]==1
	pw_DisplaySmoothSelectedPWs(todo, Vflag)

End


//1.11
Function PW_smoothPWButtonProc(ctrlName) : ButtonControl
	String ctrlName

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	variable Vflag
	ControlInfo setvarSmoothingFactor	
 	
 	wave toftypeVKey = root:diagnostics:toftypeVKey
	Vflag = toftypeVKey[sq_FindRun(root:index:rn_series, todo[0])]==1
	pw_smoothSelectedPWs(todo, Vflag, V_Value)
 
End


// 1.11
Function PW_SetHRionPWtoAvgButtonProc(ctrlName) : ButtonControl
	String ctrlName
	
	wave DisplayPWAvg = root:HR_PeakShape:DisplayPWAvg
	wave/t GFName=root:GaussFits:GFName
	
	ControlInfo setvarHRionName
	string HRion = S_value
	wave/z SmoothedPW_A = $"root:GaussFits:SmoothedPW_"+HRion
	FindValue/Text=HRion/TXOP=7 GFName
	if (V_Value<0)
		abort "Could not find the HR ion as one that was chosen for PW analysis"
	endif
	if (!WaveExists(SmoothedPW_A))
		abort "Please reselect your HR ions for PW analysis"			//1.21E
	endif
	if (numtype( DisplayPWAvg[V_Value][0])!=0)
	 	PW_butt_updatePWgraph("")		// get averages
	endif
	SmoothedPW_A = DisplayPWAvg[V_Value][0]

End


Function PW_DoFitsForTimeVaryingPW_butt(ctrlName) : ButtonControl
	String ctrlName
	
	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	variable Vflag
 	wave toftypeVKey = root:diagnostics:toftypeVKey
	Vflag = toftypeVKey[sq_FindRun(root:index:rn_series, todo[0])]==1
 	pw_fitPWToSmoothlyVaryingPW(todo,Vflag)
 
	// showtime trend of pw param grph
	pw_butt_PWParamGraph("")

End


Function PW_AddControlsToPWGraph()
	ControlBar 60
	SetVariable setvarSmoothingFactor,pos={12,3},size={150,16},title="Smoothing factor"
	SetVariable setvarSmoothingFactor,value= _NUM:100
	Button SmoothPWButton,pos={7,21},size={180,20},proc=pw_SmoothPWButtonProc,title="Smooth all chosen HR PW ions"
	SetVariable setvarHRionName,pos={207,5},size={200,16},title="HR ion name (i.e. C7H7)"
	SetVariable setvarHRionName,value= _STR:""
	Button buttonSetHRionPWToAvg,pos={200,23},size={280,20},proc=pw_SetHRionPWtoAvgButtonProc,title="Set smoothed PW value of HR ion to average PW"
	Button buttonDoPWFitsOnSmoothedPW,pos={500,2},size={200,50},proc=pw_DoFitsForTimeVaryingPW_butt,title="Use HR ion smoothed PW values\rto find PW parameters (will also\r'declare' PW params as in step3)"
End



root:HR_PeakShape:Step4MassList

Function PS_Butt_Print(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			// click code here
			svar Step4MassList = root:HR_PeakShape:Step4MassList
			print "// "+Step4MassList
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End
